@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)blockmean.c	2.88  10/18/99
+ *    $Id: blockmean.c,v 1.5 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 
 /*  
@@ -32,6 +32,8 @@
    Modified:	3 Jan 1995 by PW for gmt 3.0
    Modified:	3 May 1998 by PW for gmt 3.1
    Modified:	18 Oct 1999 by PW to add -S
+   Modified:	3.3.5: 10 Jul 2000 by PW to add -L
+   Version:	3.4: 01-MAR-2001 by PW, Use -F instead of -N, and add -C
 */
 
 #include "gmt.h"
@@ -51,22 +53,21 @@ struct LIST {
 }	**hash, *this_box;
 
 int	set_up_arrays(int n_x, int n_y, int *method);
-void	write_output_and_free_space(int n_x, int n_y, int method, int report_weight, int *n_cells_filled);
+void	write_output_and_free_space (int n_x, int n_y, int method, int report_weight, BOOLEAN mean_xy, double west, double delta_x, double south, double delta_y, double xy_off, int *n_cells_filled);
 struct	LIST *find_box(int i, int j, int method);
 BOOLEAN sum_only = FALSE;
 
 main (int argc, char **argv)
 {
 
-	BOOLEAN	error, weighted, offset, report_weight, nofile = TRUE, done = FALSE, first = TRUE;
-	BOOLEAN skip;
+	BOOLEAN	error, weighted, offset, report_weight, nofile = TRUE, done = FALSE, first = TRUE, mean_xy = TRUE;
 	
 	FILE *fp = NULL;
 	
-	double	west, east, south, north, delta_x, delta_y, del_off, *in;
+	double	west, east, south, north, delta_x, delta_y, del_off, xy_off, *in;
 	double	idx, idy, weight;
 	
-	int	i, j, method, n_x, n_y, ij, x, y, n_expected_fields, n_fields, n_req;
+	int	i, j, method, n_x, n_y, ij, n_expected_fields, n_fields, n_req;
 	int	n_lost, n_read, n_pitched, n_cells_filled, n_files = 0, fno, n_args;
 	
 	char	modifier, line[BUFSIZ], format[BUFSIZ];
@@ -97,10 +98,17 @@ main (int argc, char **argv)
 					error += GMT_io_selection (&argv[i][2]);
 					break;
 					
+				case 'C':
+					mean_xy = FALSE;	/* Report center of block instead */
+					break;
 				case 'I':
 					GMT_getinc (&argv[i][2], &delta_x, &delta_y);
 					break;
-				case 'N':
+				case 'L':
+					GMT_geographic_in = TRUE;
+					break;
+				case 'N':	/* Backward compatible with 3.3.6 */
+				case 'F':
 					offset = TRUE;
 					break;
 				case 'S':
@@ -132,7 +140,7 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "blockmean %s - Block averaging by L2 norm\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: blockmean [infile(s)] -I<xinc[m]>[/<yinc>[m]] -R<west/east/south/north>\n");
-		fprintf (stderr, "\t[-H[<nrec>]] [-N] [-S] [-V] [-W[i][o] ] [-:] [-bi[s][<n>]] [-bo[s]]\n\n");
+		fprintf (stderr, "\t[-C] [-F] [-H[<nrec>]] [-L] [-S] [-V] [-W[i][o] ] [-:] [-bi[s][<n>]] [-bo[s]]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
@@ -141,8 +149,10 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t   e.g., -I10m/5m grids longitude every 10 minutes, latitude every 5 minutes.\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
+		fprintf (stderr, "\t-C Output center of block and mean z-value.  [Default outputs mean x-y location]\n");
+		fprintf (stderr, "\t-F Offsets registration so block edges are on gridlines (pixel reg.).  [Default: grid reg.]\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "\t-N Offsets registration so block edges are on gridlines.\n");
+		fprintf (stderr, "\t-L means that x is longitude, i.e. assumed to be periodic in 360\n");
 		fprintf (stderr, "\t-S report block sums rather than mean values [Default is mean values].\n");
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t-W sets Weight options.  -WI reads Weighted Input (4 cols: x,y,z,w) but writes only (x,y,z) Output.\n");
@@ -198,6 +208,7 @@ main (int argc, char **argv)
 		n_y--;
 		del_off = 0.0;
 	}
+	xy_off = 0.5 - del_off;	/* Use to calculate mean location of block */
 	
 	if ( (set_up_arrays(n_x, n_y, &method) ) ) exit (EXIT_FAILURE);
 	
@@ -214,11 +225,9 @@ main (int argc, char **argv)
 		}
 	}
 
-	x = (gmtdefs.xy_toggle) ? 1 : 0;        y = 1 - x;              /* Set up which columns have x and y */
-	
 	n_read = n_pitched = 0;
 
-	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : 3 + weighted;
 	
 	if (n_files > 0)
 		nofile = FALSE;
@@ -246,62 +255,59 @@ main (int argc, char **argv)
 		
 		if (gmtdefs.io_header) {
 			for (i = 0; i < gmtdefs.n_header_recs; i++) {
-				fgets (line, BUFSIZ, fp);
+				GMT_fgets (line, BUFSIZ, fp);
 				line[strlen(line)-1] = 0;
-				if (first) (report_weight && !(weighted)) ? printf ("%s weights\n", line) : printf ("%s\n", line);
+				if (first) (report_weight && !(weighted)) ? fprintf (GMT_stdout, "%s weights\n", line) : fprintf (GMT_stdout, "%s\n", line);
 			}
 			first = FALSE;
 		}
-	
-		n_fields = GMT_input (fp,  &n_expected_fields, &in);
 				
-		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
-			skip = FALSE;
-			
 			if (GMT_io.status & GMT_IO_MISMATCH) {
 				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 
-			if (GMT_is_dnan (in[2])) skip = TRUE;	/* Skip when z = NaN */
+			if (GMT_is_dnan (in[2])) continue;	/* Skip when z = NaN */
 			
-			if (!skip) {	/* Check if point is inside area */
-				n_read++;
+			n_read++;
 			
-				i = (int)floor(((in[x] - west) * idx) + del_off);
-				if ( i < 0 || i >= n_x ) skip = TRUE;
-				j = (int)floor(((in[y] - south) * idy) + del_off);
-				if ( j < 0 || j >= n_y ) skip = TRUE;
-			}
-			
-			if (!skip) {
-				weight = (weighted) ? in[3] : 1.0;
-				if (method) {
-					this_box = find_box(i, j, method);
-					this_box->key = (n_x > n_y) ? j : i;
-					this_box->sum.w += weight;
-					this_box->sum.x += (in[x]*weight);
-					this_box->sum.y += (in[y]*weight);
-					this_box->sum.z += (in[2]*weight);
+			i = (int)floor(((in[0] - west) * idx) + del_off);
+			if ( i < 0 || i >= n_x ) continue;
+			j = (int)floor(((in[1] - south) * idy) + del_off);
+			if ( j < 0 || j >= n_y ) continue;
+			weight = (weighted) ? in[3] : 1.0;
+			if (method) {
+				this_box = find_box(i, j, method);
+				this_box->key = (n_x > n_y) ? j : i;
+				if (mean_xy) {
+					this_box->sum.x += (in[0]*weight);
+					this_box->sum.y += (in[1]*weight);
 				}
-				else {
-					ij = i * n_y + j;
-					wxyz[ij].w += weight;
-					wxyz[ij].x += (in[x]*weight);
-					wxyz[ij].y += (in[y]*weight);
-					wxyz[ij].z += (in[2]*weight);
+				else if (this_box->sum.w == 0.0) {	/* So we only do this once */
+					this_box->sum.x = west  + (i + xy_off) * delta_x;
+					this_box->sum.y = south + (j + xy_off) * delta_y;
 				}
-				n_pitched++;
+				this_box->sum.w += weight;
+				this_box->sum.z += (in[2]*weight);
 			}
-			
-			n_fields = GMT_input (fp, &n_expected_fields, &in);
+			else {
+				ij = i * n_y + j;
+				wxyz[ij].w += weight;
+				if (mean_xy) {
+					wxyz[ij].x += (in[0]*weight);
+					wxyz[ij].y += (in[1]*weight);
+				}
+				wxyz[ij].z += (in[2]*weight);
+			}
+			n_pitched++;
 		}
 		if (fp != GMT_stdin) GMT_fclose(fp);
 		
 	}
 
-	write_output_and_free_space (n_x, n_y, method, report_weight, &n_cells_filled);
+	write_output_and_free_space (n_x, n_y, method, report_weight, mean_xy, west, delta_x, south, delta_y, xy_off, &n_cells_filled);
 
 	n_lost = n_read - n_pitched;
 	if (gmtdefs.verbose) {
@@ -335,13 +341,11 @@ int	set_up_arrays(int n_x, int n_y, int *method)
 	return(0);
 }
 
-void	write_output_and_free_space (int n_x, int n_y, int method, int report_weight, int *n_cells_filled)
+void	write_output_and_free_space (int n_x, int n_y, int method, int report_weight, BOOLEAN mean_xy, double west, double delta_x, double south, double delta_y, double xy_off, int *n_cells_filled)
 {
-	int	loop, limit, x, y, n_out;
+	int	loop, limit, n_out, i, j;
 	double	out[4], iw;
 	
-	x = (gmtdefs.xy_toggle) ? 1 : 0;        y = 1 - x;              /* Set up which columns have x and y */
-
 	n_out = (report_weight) ? 4 : 3;
 
 	*n_cells_filled = 0;
@@ -371,8 +375,14 @@ void	write_output_and_free_space (int n_x, int n_y, int method, int report_weigh
 				(*n_cells_filled)++;
 				out[3] = this_box->sum.w;
 				iw = 1.0 / out[3];
-				out[x] = this_box->sum.x * iw;
-				out[y] = this_box->sum.y * iw;
+				if (mean_xy) {
+					out[0] = this_box->sum.x * iw;
+					out[1] = this_box->sum.y * iw;
+				}
+				else {	/* This was only set once to the center location */
+					out[0] = this_box->sum.x;
+					out[1] = this_box->sum.y;
+				}
 				out[2] = (sum_only) ? this_box->sum.z : this_box->sum.z * iw;
 
 				GMT_output (GMT_stdout, n_out, out);
@@ -401,8 +411,16 @@ void	write_output_and_free_space (int n_x, int n_y, int method, int report_weigh
 			(*n_cells_filled)++;
 			out[3] = wxyz[loop].w;
 			iw = 1.0 / out[3];
-			out[x] = wxyz[loop].x * iw;
-			out[y] = wxyz[loop].y * iw;
+			if (mean_xy) {
+				out[0] = wxyz[loop].x * iw;
+				out[1] = wxyz[loop].y * iw;
+			}
+			else {	/* Use block center */
+				j = loop % n_y;
+				i = loop / n_y;
+				out[0] = west  + (i + xy_off) * delta_x;
+				out[1] = south + (j + xy_off) * delta_y;
+			}
 			out[2] = (sum_only) ? wxyz[loop].z : wxyz[loop].z * iw;
 
 			GMT_output (GMT_stdout, n_out, out);
