@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)blockmode.c	1.14  09/21/99
+ *    $Id: blockmode.c,v 1.6 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 
 /*
@@ -27,6 +27,8 @@
    Version:	1st version based on GMT3.1b blockmedian.c, but with changes
    		to struct DATA and to sort comparison routines to facilitate
    		mode coding.
+   Modified:	3.3.5: 10-JUL-2000 PW: Added -L option
+   Version:	3.4 01-MAR-2001 by PW, Replace -N with -F, and added -C
 */
 
 #include "gmt.h"
@@ -45,17 +47,17 @@ double	weighted_mode (struct DATA *d, double wsum, int n, int k);
 main (int argc, char **argv)
 {	
 	
-	BOOLEAN	error, weighted, offset, report, nofile = TRUE, done = FALSE, first = TRUE, go_quickly = FALSE;
+	BOOLEAN	error, weighted, offset, report, nofile = TRUE, done = FALSE, first = TRUE;
+	BOOLEAN use_center = FALSE, go_quickly = FALSE, mode_xy = TRUE;
 	/* Default value for go_quickly = FALSE for backward compatibility with 3.0  */
-	BOOLEAN skip;
 	
 	FILE *fp = NULL;
 	
 	double	west, east, south, north, delta_x, delta_y, del_off;
-	double	*in, out[4], idx, idy, i_n_in_cell;
+	double	*in, out[4], idx, idy, i_n_in_cell, xy_off;
 	
-	int	i, n_x, n_y, ix, iy, index, first_in_cell, first_in_new_cell, fno, n_files = 0, n_args, n_req;
-	int	n_lost, n_read, n_pitched, n_cells_filled, n_alloc, x, y, n_expected_fields, n_fields, n_out;
+	int	i, j, n_x, n_y, ix, iy, index, first_in_cell, first_in_new_cell, fno, n_files = 0, n_args, n_req;
+	int	n_lost, n_read, n_pitched, n_cells_filled, n_alloc, n_expected_fields, n_fields, n_out;
 	int	n_in_cell;
 	
 	char	modifier, buffer[BUFSIZ], format[BUFSIZ];
@@ -86,10 +88,18 @@ main (int argc, char **argv)
 					error += GMT_io_selection (&argv[i][2]);
 					break;
 					
+				case 'C':
+					mode_xy = FALSE;
+					use_center = TRUE;
+					break;
 				case 'I':
 					GMT_getinc (&argv[i][2], &delta_x, &delta_y);
 					break;
-				case 'N':
+				case 'L':
+					GMT_geographic_in = TRUE;
+					break;
+				case 'N':	/* Backward compatible with 3.3.6 */
+				case 'F':
 					offset = TRUE;
 					break;
 				case 'Q':
@@ -121,7 +131,7 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "blockmode %s - Block averaging by mode estimates\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: blockmode [infile(s)] -I<xinc[m]>[/<yinc>[m]] -R<west/east/south/north>\n");
-		fprintf (stderr, "\t[-H[<nrec>]] [-N] [-Q] [-V] [-W[i][o] ] [-:] [-bi[s][<n>]] [-bo[s]]\n\n");
+		fprintf (stderr, "\t[-C] [-F] [-H[<nrec>]] [-L] [-Q] [-V] [-W[i][o] ] [-:] [-bi[s][<n>]] [-bo[s]]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
@@ -130,8 +140,10 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t   e.g., -I10m/5m grids longitude every 10 minutes, latitude every 5 minutes.\n");
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
+		fprintf (stderr, "\t-C Output center of block and mode z-value  [Default is mode location (but see -Q)]\n");
+		fprintf (stderr, "\t-F Offsets registration so block edges are on gridlines (pixel reg.).  [Default: grid reg.]\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "\t-N Offsets registration so block edges are on gridlines (pixel reg.).  [Default: grid reg.]\n");
+		fprintf (stderr, "\t-L means that x is longitude, i.e. assumed to be periodic in 360\n");
 		fprintf (stderr, "\t-Q Quicker; get mode z and mean x,y.  [Default gets mode x, mode y, mode z.]\n");
 		GMT_explain_option ('V');
 		fprintf (stderr, "\t-W sets Weight options.  -WI reads Weighted Input (4 cols: x,y,z,w) but writes only (x,y,z) Output.\n");
@@ -146,6 +158,10 @@ main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 	
+	if (!mode_xy && go_quickly == 1) {
+		fprintf (stderr, "%s: GMT WARNING:  -C overrides -Q\n", GMT_program);
+		go_quickly = 0;
+	}
 	if (!project_info.region_supplied) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option\n", GMT_program);
 		error++;
@@ -188,6 +204,8 @@ main (int argc, char **argv)
 		n_y--;
 		del_off = 0.0;
 	}
+	xy_off = 0.5 - del_off;
+	
 	
 	if (gmtdefs.verbose) {
 		sprintf (format, "%%s: W: %s E: %s S: %s N: %s nx: %%d ny: %%d\n\0", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
@@ -200,9 +218,7 @@ main (int argc, char **argv)
 	
 	/* Read the input data  */
 	
-	x = (gmtdefs.xy_toggle) ? 1 : 0;        y = 1 - x;              /* Set up which columns have x and y */
-
-	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : 3 + weighted;
 
 	if (n_files > 0)
 		nofile = FALSE;
@@ -229,52 +245,44 @@ main (int argc, char **argv)
 		
 		if (gmtdefs.io_header) {
 			for (i = 0; i < gmtdefs.n_header_recs; i++) {
-				fgets (buffer, BUFSIZ, fp);
+				GMT_fgets (buffer, BUFSIZ, fp);
 				buffer[strlen(buffer)-1] = 0;
-				if (first) (report && !(weighted)) ? printf ("%s weights\n", buffer) : printf ("%s\n", buffer);
+				if (first) (report && !(weighted)) ? fprintf (GMT_stdout, "%s weights\n", buffer) : fprintf (GMT_stdout, "%s\n", buffer);
 			}
 			first = FALSE;
 		}
-
-		n_fields = GMT_input (fp,  &n_expected_fields, &in);
 				
-		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 	
-			skip = FALSE;
-			
 			if (GMT_io.status & GMT_IO_MISMATCH) {
 				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_read);
 				exit (EXIT_FAILURE);
 			}
 
-			if (GMT_is_dnan (in[2])) skip = TRUE;	/* Skip when z = NaN */
+			if (GMT_is_dnan (in[2])) continue;	/* Skip when z = NaN */
 
-			if (!skip) {	/* Check if point is inside area */
-				n_read++;
+			n_read++;
 		
-				ix = (int)floor(((in[x] - west) * idx) + del_off);
-				if ( ix < 0 || ix >= n_x ) skip = TRUE;
-				iy = (int)floor(((in[y] - south) * idy) + del_off);
-				if ( iy < 0 || iy >= n_y ) skip = TRUE;
+			ix = (int)floor(((in[0] - west) * idx) + del_off);
+			if ( ix < 0 || ix >= n_x ) continue;
+			iy = (int)floor(((in[1] - south) * idy) + del_off);
+			if ( iy < 0 || iy >= n_y ) continue;
+		
+			index = iy * n_x + ix;
+		
+			data[n_pitched].i = index;
+			if (mode_xy) {
+				data[n_pitched].a[0] = in[0];
+				data[n_pitched].a[1] = in[1];
 			}
-		
-			if (!skip) {
-				index = iy * n_x + ix;
-		
-				data[n_pitched].i = index;
-				data[n_pitched].a[0] = in[x];
-				data[n_pitched].a[1] = in[y];
-				data[n_pitched].a[2] = in[2];
-				data[n_pitched].a[3] = (weighted) ? in[3] : 1.0;
-		
-				n_pitched++;
-				if (n_pitched == n_alloc) {
-					n_alloc += GMT_CHUNK;
-					data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
-				}
+			data[n_pitched].a[2] = in[2];
+			data[n_pitched].a[3] = (weighted) ? in[3] : 1.0;
+
+			n_pitched++;
+			if (n_pitched == n_alloc) {
+				n_alloc += GMT_CHUNK;
+				data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 			}
-			
-			n_fields = GMT_input (fp, &n_expected_fields, &in);
 		}
 		if (fp != GMT_stdin) GMT_fclose(fp);
 		
@@ -300,13 +308,23 @@ main (int argc, char **argv)
 	n_cells_filled = 0;
 	while (first_in_cell < n_pitched) {
 		out[3] = data[first_in_cell].a[3];
-		out[x] = data[first_in_cell].a[0];
-		out[y] = data[first_in_cell].a[1];
+		if (use_center) {
+			j = data[first_in_cell].i / n_x;
+			i = data[first_in_cell].i % n_x;
+			out[0] = west  + (i + xy_off) * delta_x;
+			out[1] = south + (j + xy_off) * delta_y;
+		}
+		else {
+			out[0] = data[first_in_cell].a[0];
+			out[1] = data[first_in_cell].a[1];
+		}
 		first_in_new_cell = first_in_cell + 1;
 		while ( (first_in_new_cell < n_pitched) && (data[first_in_new_cell].i == data[first_in_cell].i) ) {
-			out[3] += data[first_in_new_cell].a[3];
-			out[x] += data[first_in_new_cell].a[0];
-			out[y] += data[first_in_new_cell].a[1];
+			out[3] += data[first_in_new_cell].a[3];	/* Summing up weights */
+			if (mode_xy) {
+				out[0] += data[first_in_new_cell].a[0];
+				out[1] += data[first_in_new_cell].a[1];
+			}
 			first_in_new_cell++;
 		}
 		n_in_cell = first_in_new_cell - first_in_cell;
@@ -318,12 +336,12 @@ main (int argc, char **argv)
 				out[0] *= i_n_in_cell;
 				out[1] *= i_n_in_cell;
 			}
-			else {
+			else if (mode_xy) {
 				qsort((void *)&data[first_in_cell], (size_t)n_in_cell, sizeof (struct DATA), compare_x);
-				out[x] = weighted_mode (&data[first_in_cell], out[3], n_in_cell, 0);
+				out[0] = weighted_mode (&data[first_in_cell], out[3], n_in_cell, 0);
 				
 				qsort((void *)&data[first_in_cell], (size_t)n_in_cell, sizeof (struct DATA), compare_y);
-				out[y] = weighted_mode (&data[first_in_cell], out[3], n_in_cell, 1);
+				out[1] = weighted_mode (&data[first_in_cell], out[3], n_in_cell, 1);
 			}
 		}
 		else if (n_in_cell == 2) {
@@ -333,9 +351,9 @@ main (int argc, char **argv)
 					out[0] *= 0.5;
 					out[1] *= 0.5;
 				}
-				else {
-					out[x] = data[first_in_cell].a[0];
-					out[y] = data[first_in_cell].a[1];
+				else if (mode_xy) {
+					out[0] = data[first_in_cell].a[0];
+					out[1] = data[first_in_cell].a[1];
 				}
 			}
 			else if (data[first_in_cell].a[3] < data[first_in_cell+1].a[3]) {
@@ -344,18 +362,21 @@ main (int argc, char **argv)
 					out[0] *= 0.5;
 					out[1] *= 0.5;
 				}
-				else {
-					out[x] = data[first_in_cell+1].a[0];
-					out[y] = data[first_in_cell+1].a[1];
+				else if (mode_xy) {
+					out[0] = data[first_in_cell+1].a[0];
+					out[1] = data[first_in_cell+1].a[1];
 				}
 			}
 			else {
-				out[0] *= 0.5;
-				out[1] *= 0.5;
-				out[2] *= 0.5;
+				if (mode_xy) {	/* Need average location */
+					out[0] *= 0.5;
+					out[1] *= 0.5;
+				}
+				out[2] = 0.5 * (data[first_in_cell].a[2] + data[first_in_cell+1].a[2]);
 			}
 		}
-		/* else n_in_cell == 1, which means out[] is fine as is.  */		
+		else
+			out[2] = data[first_in_cell].a[2];
 		
 		GMT_output (GMT_stdout, n_out, out);
 		
@@ -383,7 +404,7 @@ double	weighted_mode (struct DATA *d, double wsum, int n, int k)
 	   of range from i to j is the mode estimate.  Choose 
 	   to move either i or j depending on which one will
 	   cause greatest increase in pdf estimate.  If a tie,
-	   alternate between i and j to avoid bias.
+	   move both.
 	   
 	   Strictly, the pdf estimated this way would need to be
 	   scaled by (1/wsum), but this is constant so we don't
@@ -422,15 +443,10 @@ double	weighted_mode (struct DATA *d, double wsum, int n, int k)
 				j--;
 				top = topj;
 			}
-			else {	/* flip a coin  */
-				if (i%2 == 1) {
-					i++;
-					top = topi;
-				}
-				else {
-					j--;
-					top = topj;
-				}
+			else {
+				top -= (d[i].a[3] + d[j].a[3]);
+                                i++;
+                                j--;
 			}
 		}
 	}
