@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)filter1d.c	2.62  09/21/99
+ *    $Id: filter1d.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * filter1d will read N colums of data from file/GMT_stdin and return
@@ -30,8 +30,7 @@
  *
  * Author:	Walter H. F. Smith
  * Date:	13 January, 1989
- * Version:	2.0, Developmental
- * Modified:	PW: 13-MAY-1998 for GMT 3.1
+ * Version:	3.4
  */
 
 #include "gmt.h"
@@ -177,7 +176,7 @@ main (int argc, char **argv)
 						case 'F':
 							robust = TRUE;
 						case 'f':
-							if ((fp_wt = GMT_fopen(&argv[i][3],"r")) == NULL) {
+							if ((fp_wt = fopen(&argv[i][3],"r")) == NULL) {
 								fprintf (stderr, "%s: Cannot open file %s\n", GMT_program, &argv[i][3]);
 								exit (EXIT_FAILURE);
 							}
@@ -376,7 +375,7 @@ int load_data_and_check_extrema (void)
 	}
 	
 	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) {
-		fgets (buffer, BUFSIZ, fp_in);
+		GMT_fgets (buffer, BUFSIZ, fp_in);
 		fputs (buffer, GMT_stdout);
 	}
 
@@ -394,9 +393,7 @@ int load_data_and_check_extrema (void)
 	else
 		n_expected_fields = n_cols;
 	
-	n_fields = GMT_input (fp_in,  &n_expected_fields, &in);
-	
-	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 	
 		if (GMT_io.status & GMT_IO_MISMATCH) {
 			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_rows);
@@ -432,8 +429,6 @@ int load_data_and_check_extrema (void)
 			n_row_alloc += GMT_CHUNK;
 			allocate_more_data_space ();
 		}
-		
-		n_fields = GMT_input (fp_in, &n_expected_fields, &in);
 	}
 	
 	if (fp_in != GMT_stdin) GMT_fclose (fp_in);
@@ -480,7 +475,7 @@ int	set_up_filter(void)
 				allocate_more_work_space ();
 			}
 		}
-		GMT_fclose (fp_wt);
+		fclose (fp_wt);
 		f_operator = (w_sum == 0.0);	/* If weights sum to zero it is an operator like {-1 1] or [1 -2 1] */
 		half_n_f_wts = n_f_wts / 2;
 		half_width = half_n_f_wts * dt;
