@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)fitcircle.c	2.50  09/21/99
+ *    $Id: fitcircle.c,v 1.3 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * fitcircle <lonlatfile> [-L1] [-L2] [-S]
@@ -54,8 +54,7 @@
  *
  * Author:	W. H. F. Smith
  * Date:	16 September 1991.
- * Version:	2.0, for GMT release.
- * Modified:	PW: 21-MAY-1998, for GMT 3.1.
+ * Version:	3.4
  *
  */
 
@@ -67,10 +66,10 @@ struct	DATA {
 
 main (int argc, char **argv)
 {
-	int	i, j, k, ix, iy, imin, imax, n_alloc, n_data, n, np, nrots, norm = -1, n_fields, n_expected_fields;
+	int	i, j, k, imin, imax, n_alloc, n_data, n, np, nrots, norm = -1, n_fields, n_expected_fields;
 	BOOLEAN	error = FALSE, greenwich = FALSE, find_small_circle = FALSE;
 
-	double	lonsum, latsum, latlon[2], *in;
+	double	lonsum, latsum, *in;
 	double	meanv[3], cross[3], cross_sum[3], gcpole[3], scpole[3];		/* Extra vectors  */
 	double	*a, *lambda, *v, *b, *z;	/* Matrix stuff */
 	double	get_small_circle(struct DATA *data, int ndata, double *center, double *gcpole, double *scpole, int norm, double *work), rad, *work;
@@ -177,34 +176,29 @@ main (int argc, char **argv)
 	
 	data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (buffer, BUFSIZ, fp);
+	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
 		
-	ix = (gmtdefs.xy_toggle) ? 1 : 0;	iy = 1 - ix;		/* Set up which columns have x and y */
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 2;
 
-	n_fields = GMT_input (fp, &n_expected_fields, &in);
-
-	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
 			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_data);
 			exit (EXIT_FAILURE);
 		}
 
-		latlon[0] = in[ix];
-        	latlon[1] = in[iy];
-		lonsum += latlon[0];
-		latsum += latlon[1];
-		GMT_geo_to_cart (&latlon[1], &latlon[0], data[n_data].x, TRUE);
+		lonsum += in[0];
+		latsum += in[1];
+		GMT_geo_to_cart (&in[1], &in[0], data[n_data].x, TRUE);
 		n_data++;
 
 		if (n_data == n_alloc) {
 			n_alloc += GMT_CHUNK;
       			data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 	        }
-		n_fields = GMT_input (fp, &n_expected_fields, &in);
         }
         if (fp != GMT_stdin) GMT_fclose(fp);
+        
       	data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_data, sizeof(struct DATA), GMT_program);
 	if (find_small_circle && (norm%2) ) work = (double *) GMT_memory (VNULL, (size_t)n_data, sizeof(double), GMT_program);
 
@@ -230,8 +224,8 @@ main (int argc, char **argv)
 	if (norm%2) {
 		GMT_cart_to_geo (&latsum, &lonsum, meanv, TRUE);
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
-		printf (format, lonsum, latsum);
-		printf("\tL1 Average Position (Fisher's Method)\n");
+		fprintf (GMT_stdout, format, lonsum, latsum);
+		fprintf (GMT_stdout, "\tL1 Average Position (Fisher's Method)\n");
 
 		cross_sum[0] = cross_sum[1] = cross_sum[2] = 0.0;
 		for (i = 0; i < n_data; i++) {
@@ -252,23 +246,23 @@ main (int argc, char **argv)
 		
 		GMT_cart_to_geo (&latsum, &lonsum, cross_sum, TRUE);
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
-		printf (format, lonsum, latsum);
-		printf("\tL1 N Hemisphere Great Circle Pole (Cross-Averaged)\n");
+		fprintf (GMT_stdout, format, lonsum, latsum);
+		fprintf (GMT_stdout, "\tL1 N Hemisphere Great Circle Pole (Cross-Averaged)\n");
 		latsum = -latsum;
 		lonsum = d_atan2(-cross_sum[1], -cross_sum[0]) * R2D;
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
-		printf (format, lonsum, latsum);
-		printf("\tL1 S Hemisphere Great Circle Pole (Cross-Averaged)\n");
+		fprintf (GMT_stdout, format, lonsum, latsum);
+		fprintf (GMT_stdout, "\tL1 S Hemisphere Great Circle Pole (Cross-Averaged)\n");
 		if (find_small_circle) {
-			if (gmtdefs.verbose) fprintf(stderr,"Fitting small circle using L1 norm.\n");
+			if (gmtdefs.verbose) fprintf (stderr,"Fitting small circle using L1 norm.\n");
 			rad = get_small_circle (data, n_data, meanv, gcpole, scpole, 1, work);
 			if (rad >= 0.0) {
 				GMT_cart_to_geo (&latsum, &lonsum, scpole, TRUE);
 				if (greenwich && lonsum < 0.0) lonsum += 360.0;
-				printf (format, lonsum, latsum);
-				printf("\tL1 Small Circle Pole.  ");
+				fprintf (GMT_stdout, format, lonsum, latsum);
+				fprintf (GMT_stdout, "\tL1 Small Circle Pole.  ");
 				sprintf(format, "Distance from Pole to L1 Small Circle (degrees):     %s\n\0", gmtdefs.d_format);
-				printf(format, rad);
+				fprintf (GMT_stdout, format, rad);
 			}
 		}
 	}
@@ -301,8 +295,8 @@ main (int argc, char **argv)
 			for (i = 0; i < 3; i++) meanv[i] = v[imax*np+i];
 		GMT_cart_to_geo (&latsum, &lonsum, meanv, TRUE);
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
-		printf (format, lonsum, latsum);
-		printf("\tL2 Average Position (Eigenval Method)\n");
+		fprintf (GMT_stdout, format, lonsum, latsum);
+		fprintf (GMT_stdout, "\tL2 Average Position (Eigenval Method)\n");
 		
 		if (v[imin*np+2] < 0.0)	/* Eigvec is in S Hemisphere  */
 			for (i = 0; i < 3; i++) gcpole[i] = -v[imin*np+i];
@@ -311,13 +305,13 @@ main (int argc, char **argv)
 		
 		GMT_cart_to_geo (&latsum, &lonsum, gcpole, TRUE);		
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
-		printf (format, lonsum, latsum);
-		printf("\tL2 N Hemisphere Great Circle Pole (Eigenval Method)\n");
+		fprintf (GMT_stdout, format, lonsum, latsum);
+		fprintf (GMT_stdout, "\tL2 N Hemisphere Great Circle Pole (Eigenval Method)\n");
 		latsum = -latsum;
 		lonsum = d_atan2(-gcpole[1], -gcpole[0]) * R2D;
 		if (greenwich && lonsum < 0.0) lonsum += 360.0;
-		printf (format, lonsum, latsum);
-		printf("\tL2 S Hemisphere Great Circle Pole (Eigenval Method)\n");
+		fprintf (GMT_stdout, format, lonsum, latsum);
+		fprintf (GMT_stdout, "\tL2 S Hemisphere Great Circle Pole (Eigenval Method)\n");
 
 		GMT_free ((void *)v);
 		GMT_free ((void *)z);
@@ -331,10 +325,10 @@ main (int argc, char **argv)
 				/* True when small circle fits better than great circle */
 				GMT_cart_to_geo (&latsum, &lonsum, scpole, TRUE);
 				if (greenwich && lonsum < 0.0) lonsum += 360.0;
-				printf (format, lonsum, latsum);
-				printf("\tL2 Small Circle Pole.  ");
+				fprintf (GMT_stdout, format, lonsum, latsum);
+				fprintf (GMT_stdout, "\tL2 Small Circle Pole.  ");
 				sprintf(format, "Distance from Pole to L2 Small Circle (degrees):     %s\n\0", gmtdefs.d_format);
-				printf(format, rad);
+				fprintf (GMT_stdout, format, rad);
 			}
 		}
 	}
