@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *   		   The GMT-system:  @(#)fourt.c	2.31  11/22/99
+ *	$Id: fourt.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*	Translation of old fourt.f FORTRAN code to C using the automatic
  *	translator f2c written by S.I. Feldman, David M. Gay, Mark W. Maimone,
