@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)gmt.h	2.154  12/03/99
+ *	$Id: gmt.h,v 1.7 2001/03/22 17:26:33 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * gmt.h is the main include file for GMT.  It contains definitions
@@ -21,12 +21,18 @@
  *
  * Author:	Paul Wessel
  * Date:	01-MAR-1991
- * Revised:	10-NOV-1999
+ * Revised:	22-MAR-2001
+ * Version:	3.4
  */
  
+#ifndef _GMT_H
+#define _GMT_H
+
 /*  GMT is POSIX.1 COMPLIANT  */
 
+#ifndef _POSIX_SOURCE
 #define _POSIX_SOURCE 1
+#endif
 
 /* Declaration modifiers for DLL support (MSC et al) */
 
@@ -109,7 +115,7 @@
 #define GMT_CHUNK	2000
 #define GMT_SMALL_CHUNK	50
 #define GMT_TINY_CHUNK	5
-#define GMT_VERSION	"3.3.3"
+#define GMT_VERSION	"3.4"
 #define CNULL		((char *)NULL)
 #define VNULL		((void *)NULL)
 #define GMT_CM		0
@@ -173,7 +179,7 @@ typedef double (*PFD) ();	/* PFD declares a pointer to a function returning a do
  *--------------------------------------------------------------------*/
 
 #define N_UNIQUE 59		/* Number of unique options */
-#define N_KEYS 57		/* Number of gmt defaults */
+#define N_KEYS 58		/* Number of gmt defaults */
 #define N_FONTS 39		/* Number of fonts in the PS_font_names.h include file */
 #define GMT_N_MEDIA 29		/* Number of standard paper formats in the GMT_media_names.h include file */
 #define HASH_SIZE 61		/* Used in get_gmtdefaults, should be ~> N_KEYS */
@@ -194,6 +200,7 @@ struct GMT_PEN {	/* Holds pen attributes */
 
 struct GMTDEFAULTS {
 	double anot_min_angle;		/* If angle between map boundary and anotation is less, no anot is drawn [20] */
+	double anot_min_spacing;	/* If an anotation is closer that this to an older anotation, the anot is skipped [0.0] */
 	int anot_font;			/* Font for anotations [Helvetica] */
 	int anot_font_size;		/* Font size for anotations in points [14] */
 	double anot_offset;		/* Distance between anotation and tickmarks [0.075] */
@@ -205,7 +212,7 @@ struct GMTDEFAULTS {
 	int nan_rgb[3];			/* Color of NaNs [0/0/0] */
 	int color_image;		/* 0 = Adobe's colorimage, 1 = Tiles, 2 = RGB-separation */
 	int color_model;		/* 0 = RGB, 1 = HSV [0] */
-	char d_format[10];		/* Default double output format [%lg] */
+	char d_format[32];		/* Default double output format [%lg] */
 	int degree_format;		/* 0 = <0/360/-90/90>, 1 = <-180/180/-90/90>, 2 = <0/180/0/90>
 					   3 = 0E/180E/180W/0W/90S/90N> */
 	int dpi;			/* Dots pr. inch plotter resolution [300] */
@@ -238,8 +245,7 @@ struct GMTDEFAULTS {
 	int n_lat_nodes;		/* No of points to use for drawing a latitudal line [50] */
 	int n_lon_nodes;		/* No of points to use for drawing a longitudal line [50] */
 	double dlon, dlat;		/* Corresponding increment in lon/lat */
-	int oblique_anotation;		/* Anotate all lon/lat marks (1),  or only longitudes on top/bottom boundary 
-					 * and latitudes on left/right boundary (0) [1] */
+	int oblique_anotation;		/* Controls anotations and tick angles etc. [0] */
 	BOOLEAN overlay;		/* Make plot in overlay mode [FALSE] */
 	int page_rgb[3];		/* Color of the page [255/255/255 white] */
 	int page_orientation;		/* Orientation of page [0 = Landscape, 1 = Portrait] */
@@ -248,7 +254,7 @@ struct GMTDEFAULTS {
 	double tick_length;		/* Length of tickmarks [0.075] */
 	struct GMT_PEN tick_pen;	/* Pen attributes for tickmarks [2] */
 	BOOLEAN unix_time;		/* Plot time and map projection on map [FALSE] */
-	char unix_time_label[512];	/* Label to plot after time-stamp instead of command line */
+	char unix_time_label[BUFSIZ];	/* Label to plot after time-stamp instead of command line */
 	double unix_time_pos[2];	/* Where to plot timestamp relative to origin */
 	double vector_shape;		/* 0.0 = straight vectorhead, 1.0 = arrowshape, with continuous range in between */
 	BOOLEAN verbose;		/* Give info during execution [FALSE] */
@@ -261,7 +267,7 @@ struct GMTDEFAULTS {
 	int y_axis_type;		/* Select y-axis with horizontal (0) or vertical (1) annotations  [0] */
 	struct ELLIPSOID {	/* Information about a particular ellipsoid */
 		/* Table taken from Snyder "Map projection - a working manual", p 12 Table 1 */
-		char name[20];
+		char name[32];
 		int date;
 		double eq_radius;
 		double pol_radius;
@@ -319,6 +325,7 @@ struct BCR {	/* Used mostly in gmt_support.c */
 	int	my;			/* Ditto  */
 };
 
+
 /*--------------------------------------------------------------------*/
 /*	External variables for misc purposes */
 /*--------------------------------------------------------------------*/
@@ -344,7 +351,8 @@ EXTERN_MSC BOOLEAN GMT_quick;		/* TRUE if short usage message is desired (must s
 EXTERN_MSC char *GMT_program;		/* Name of current GMT program */
 EXTERN_MSC int GMT_no_rgb[];
 EXTERN_MSC int GMT_oldargc;
-EXTERN_MSC char *GMT_oldargv[];	/* Pointers to old common arguments */
+EXTERN_MSC char *GMT_oldargv[];		/* Pointers to old common arguments */
+EXTERN_MSC char *GMT_degree_symbol[2];	/* Contains the two octal codes for small and large degree symbols */
 
 /*--------------------------------------------------------------------*/
 /*	For i/o purposes */
@@ -353,6 +361,8 @@ EXTERN_MSC char *GMT_oldargv[];	/* Pointers to old common arguments */
 
 EXTERN_MSC FILE *GMT_stdin, *GMT_stdout;
 EXTERN_MSC PFI GMT_input, GMT_output;
+EXTERN_MSC BOOLEAN GMT_geographic_in;	/*TRUE if input data is long/lat */
+EXTERN_MSC BOOLEAN GMT_geographic_out;	/*TRUE if output data is long/lat */
 EXTERN_MSC double GMT_data[BUFSIZ];
 EXTERN_MSC double GMT_grd_in_nan_value, GMT_grd_out_nan_value;
 EXTERN_MSC int GMT_n_file_suffix;
@@ -399,6 +409,7 @@ EXTERN_MSC PFV GMT_get_crossings;		/* Returns map crossings in x or y */
 EXTERN_MSC PFI GMT_truncate;			/* Truncate polygons agains boundaries */
 EXTERN_MSC BOOLEAN GMT_meridian_straight;	/* TRUE if meridians plot as straight lines */
 EXTERN_MSC BOOLEAN GMT_parallel_straight;	/* TRUE if parallels plot as straight lines */
+EXTERN_MSC int GMT_3D_mode;			/* Determines if we draw fore and/or back 3-D box lines */
 
 /*--------------------------------------------------------------------*/
 /*	For projection purposes */
@@ -413,7 +424,12 @@ EXTERN_MSC PFI GMT_z_forward, GMT_z_inverse;	/*	Pointers to the selected linear
 EXTERN_MSC PFD GMT_scan_time_string;		/*	pointer to functions that converts timestring to secs */
 
 #include "gmt_math.h"		/* Machine-dependent macros for non-POSIX math functions */
+#ifdef WIN32
+#define GMT_make_fnan(x) (((unsigned int *) &x)[0] = 0x7fffffff)
+#define GMT_make_dnan(x) (((unsigned int *) &x)[0] = 0xffffffff, ((unsigned int *) &x)[1] = 0x7fffffff)
+#else
 #include "gmt_nan.h"		/* Machine-dependent macros for making and testing NaNs */
+#endif
 #include "gmt_project.h"        /* Define project_info and frame_info structures */
 #include "gmt_grd.h"            /* Define grd file header structure */
 #include "gmt_io.h"		/* Defines structurs and macros for table i/o */
@@ -422,3 +438,7 @@ EXTERN_MSC PFD GMT_scan_time_string;		/*	pointer to functions that converts time
 #include "pslib.h"		/* Defines pslib function prototypes */
 #include "gmt_shore.h"		/* Defines structures used when reading shore database */
 #include "gmt_funcnames.h"      /* List of functions */
+#include "gmt_boundcond.h"	/* Boundary conditions for grids */
+#include "gmt_bcr.h"		/* Grid resampling functions */
+
+#endif  /* _GMT_H */
