@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)gmt_customio.c	2.55  11/18/99
+ *	$Id: gmt_customio.c,v 1.4 2001/03/22 17:12:44 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  *
@@ -34,8 +34,8 @@
  *
  * Author:	Paul Wessel
  * Date:	9-SEP-1992
- * Modified:	10-SEP-1999
- * Version:	3.3.2
+ * Modified:	05-MAR-2001
+ * Version:	3.4
  *
  * Functions include:
  *
@@ -70,11 +70,18 @@
 int GMT_read_rasheader (FILE *fp, struct rasterfile *h);
 int GMT_write_rasheader (FILE *fp, struct rasterfile *h);
 float GMT_native_decode (void *vptr, int k, int type);
+double GMT_native_encode (float z, int type);
 size_t GMT_native_write_one (FILE *fp, float z, int type);
 int GMT_native_read_grd_info (char *file, struct GRD_HEADER *header);
 int GMT_native_write_grd_info (char *file, struct GRD_HEADER *header);
 int GMT_native_read_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex, int type);
 int GMT_native_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex, int type);
+int GMT_cdf_byte_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex);
+int GMT_cdf_char_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex);
+int GMT_cdf_short_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex);
+int GMT_cdf_int_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex);
+int GMT_cdf_float_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex);
+int GMT_cdf_double_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex);
 
 int GMT_native_size[GMT_N_NATIVE_FORMATS] = {
 	sizeof(char),
@@ -102,7 +109,7 @@ void GMT_grdio_init (void) {
 	GMT_io_readinfo[0]  = (PFI) GMT_cdf_read_grd_info;
 	GMT_io_writeinfo[0] = (PFI) GMT_cdf_write_grd_info;
 	GMT_io_readgrd[0]   = (PFI) GMT_cdf_read_grd;
-	GMT_io_writegrd[0]  = (PFI) GMT_cdf_write_grd;
+	GMT_io_writegrd[0]  = (PFI) GMT_cdf_float_write_grd;
 	
 	/* FORMAT # 1: GMT native binary (float) grdio */
 	
@@ -152,8 +159,46 @@ void GMT_grdio_init (void) {
 	GMT_io_readgrd[id]   = (PFI) GMT_srf_read_grd;
 	GMT_io_writegrd[id]  = (PFI) GMT_srf_write_grd;
 
-	/* FORMAT # 7:  */
+	/* FORMAT # 7: GMT netCDF-based grdio (byte) */
+ 	
+	id = 7;
+	GMT_io_readinfo[id]  = (PFI) GMT_cdf_read_grd_info;
+	GMT_io_writeinfo[id] = (PFI) GMT_cdf_write_grd_info;
+	GMT_io_readgrd[id]   = (PFI) GMT_cdf_read_grd;
+	GMT_io_writegrd[id]  = (PFI) GMT_cdf_byte_write_grd;
 	
+	/* FORMAT # 8: GMT netCDF-based grdio (char) */
+ 	
+	id = 8;
+	GMT_io_readinfo[id]  = (PFI) GMT_cdf_read_grd_info;
+	GMT_io_writeinfo[id] = (PFI) GMT_cdf_write_grd_info;
+	GMT_io_readgrd[id]   = (PFI) GMT_cdf_read_grd;
+	GMT_io_writegrd[id]  = (PFI) GMT_cdf_char_write_grd;
+	
+	/* FORMAT # 9: GMT netCDF-based grdio (short) */
+ 	
+	id = 9;
+	GMT_io_readinfo[id]  = (PFI) GMT_cdf_read_grd_info;
+	GMT_io_writeinfo[id] = (PFI) GMT_cdf_write_grd_info;
+	GMT_io_readgrd[id]   = (PFI) GMT_cdf_read_grd;
+	GMT_io_writegrd[id]  = (PFI) GMT_cdf_short_write_grd;
+	
+	/* FORMAT # 10: GMT netCDF-based grdio (int) */
+ 	
+	id = 10;
+	GMT_io_readinfo[id]  = (PFI) GMT_cdf_read_grd_info;
+	GMT_io_writeinfo[id] = (PFI) GMT_cdf_write_grd_info;
+	GMT_io_readgrd[id]   = (PFI) GMT_cdf_read_grd;
+	GMT_io_writegrd[id] = (PFI) GMT_cdf_int_write_grd;
+	
+	/* FORMAT # 11: GMT netCDF-based grdio (double) */
+ 	
+	id = 11;
+	GMT_io_readinfo[id]  = (PFI) GMT_cdf_read_grd_info;
+	GMT_io_writeinfo[id] = (PFI) GMT_cdf_write_grd_info;
+	GMT_io_readgrd[id]   = (PFI) GMT_cdf_read_grd;
+	GMT_io_writegrd[id] = (PFI) GMT_cdf_double_write_grd;
+
 	/*
 	 * ----------------------------------------------
 	 * ADD CUSTOM FORMATS BELOW AS THEY ARE NEEDED */
@@ -739,7 +784,7 @@ int GMT_bit_read_grd (char *file, struct GRD_HEADER *header, float *grid, double
 		i_0_out *= 2;
 		inc = 2;
 	}
-	tmp = (unsigned int *) GMT_memory (VNULL, (size_t)mx, sizeof (unsigned char), "GMT_bit_read_grd");
+	tmp = (unsigned int *) GMT_memory (VNULL, (size_t)mx, sizeof (unsigned int), "GMT_bit_read_grd");
 	
 	if (piping) {	/* Skip data by reading it */
 		for (j = 0; j < first_row; j++) fread ((void *) tmp, sizeof (unsigned int), (size_t)mx, fp);
@@ -1079,7 +1124,7 @@ int GMT_native_write_grd (char *file, struct GRD_HEADER *header, float *grid, do
 		exit (EXIT_FAILURE);
 	}
 	
-	check = !GMT_is_dnan (GMT_grd_in_nan_value);
+	check = !GMT_is_dnan (GMT_grd_out_nan_value);
 
 	k = GMT_grd_prep_io (header, &w, &e, &s, &n, &width_out, &height_out, &first_col, &last_col, &first_row, &last_row);
 
@@ -1101,7 +1146,7 @@ int GMT_native_write_grd (char *file, struct GRD_HEADER *header, float *grid, do
 		for (i = first_col, i2 = pad[0]; i <= last_col; i++, i2++) {
 			ij = (j2 * width_in + i2) * inc;
 			if (GMT_is_fnan (grid[ij])) {
-				if (check) grid[i] = (float)GMT_grd_out_nan_value;
+				if (check) grid[ij] = (float)GMT_grd_out_nan_value;
 			}
 			else {
 				if ((double)grid[ij] < header->z_min) header->z_min = (double)grid[ij];
@@ -1109,6 +1154,10 @@ int GMT_native_write_grd (char *file, struct GRD_HEADER *header, float *grid, do
 			}
 		}
 	}
+	/* Round off to chosen type */
+	
+	header->z_min = GMT_native_encode ((float)header->z_min, type);
+	header->z_max = GMT_native_encode ((float)header->z_max, type);
 	
 	/* store header information and array */
 	
@@ -1154,12 +1203,48 @@ float GMT_native_decode (void *vptr, int k, int type)
 			fval = (float)(((double *)vptr)[k]);
 			break;
 		default:
+			fprintf (stderr, "GMT: Bad call to GMT_native_decode (gmt_customio.c)\n");
 			break;
 	}
 
 	return (fval);
 }
 
+double GMT_native_encode (float z, int type)
+{
+	char c;
+	unsigned char u;
+	short int h;
+	int i;
+
+	switch (type) {
+		case GMT_NATIVE_CHAR:
+			c = (char)irint ((double)z);
+			return ((double)c);
+			break;
+		case GMT_NATIVE_UCHAR:
+			u = (unsigned char)irint ((double)z);
+			return ((double)u);
+			break;
+		case GMT_NATIVE_SHORT:
+			h = (short int)irint ((double)z);
+			return ((double)h);
+			break;
+		case GMT_NATIVE_INT:
+			i = (int)irint ((double)z);
+			return ((double)i);
+			break;
+		case GMT_NATIVE_FLOAT:
+		case GMT_NATIVE_DOUBLE:
+			return ((double)z);
+			break;
+		default:
+			fprintf (stderr, "GMT: Bad call to GMT_native_encode (gmt_customio.c)\n");
+			return (0.0);
+			break;
+	}
+}
+
 size_t GMT_native_write_one (FILE *fp, float z, int type)
 {
 	char c;
@@ -1170,19 +1255,19 @@ size_t GMT_native_write_one (FILE *fp, float z, int type)
 
 	switch (type) {
 		case GMT_NATIVE_CHAR:
-			c = (char)z;
+			c = (char)irint ((double)z);
 			return (fwrite ((void *)&c, GMT_native_size[type], (size_t)1, fp));
 			break;
 		case GMT_NATIVE_UCHAR:
-			u = (unsigned char)z;
+			u = (unsigned char)irint ((double)z);
 			return (fwrite ((void *)&u, GMT_native_size[type], (size_t)1, fp));
 			break;
 		case GMT_NATIVE_SHORT:
-			h = (short int)z;
+			h = (short int)irint ((double)z);
 			return (fwrite ((void *)&h, GMT_native_size[type], (size_t)1, fp));
 			break;
 		case GMT_NATIVE_INT:
-			i = (int)z;
+			i = (int)irint ((double)z);
 			return (fwrite ((void *)&i, GMT_native_size[type], (size_t)1, fp));
 			break;
 		case GMT_NATIVE_FLOAT:
@@ -1227,6 +1312,7 @@ int GMT_srf_read_grd_info (char *file, struct GRD_HEADER *header)
 	FILE *fp;
 	struct srf_header h;
 	int GMT_read_srfheader (FILE *fp, struct srf_header *h);
+	char id[5];
 	
 	if (!strcmp (file, "=")) {
 #ifdef SET_IO_MODE
@@ -1243,7 +1329,8 @@ int GMT_srf_read_grd_info (char *file, struct GRD_HEADER *header)
 		fprintf (stderr, "GMT Fatal Error: Error reading file %s!\n", file);
 		exit (EXIT_FAILURE);
 	}
-	if (strcmp (h.id, "DSBBz")) {
+	sprintf (id, "%.4s", h.id);
+	if (strcmp (id, "DSBB")) {
 		fprintf (stderr, "GMT Fatal Error: %s is not a valid surfer grid\n", file);
 		exit (EXIT_FAILURE);
 	}
@@ -1327,11 +1414,11 @@ int GMT_write_srfheader (FILE *fp, struct srf_header *h)
 
 int GMT_surfer_read_grd (char *file, struct GRD_HEADER *header, float *grid, int type)
 {	/* file:	File name	*/
-	/* h:     	grid structure header */
+	/* header:     	grid structure header */
 	/* grid:	array with final grid */
 	/* type:	Data type (int, short, float, etc) */
 
-	int kk, i, j, j2, ij;         	/* Misc. counters */
+	int kk, i, j2, ij;         	/* Misc. counters */
 	int *k;				/* Array with indeces */
 	FILE *fp;			/* File pointer to data or pipe */
 	BOOLEAN piping = FALSE;		/* TRUE if we read input pipe instead of from file */
@@ -1359,9 +1446,8 @@ int GMT_surfer_read_grd (char *file, struct GRD_HEADER *header, float *grid, int
 
 	tmp = (void *) GMT_memory (VNULL, (size_t)header->nx, GMT_native_size[type], "GMT_native_read");
 
-	for (j = 0, j2 = (header->ny - 1); j <= (header->nx - 1); j++, j2--) {
+	for (j2 = (header->ny - 1); j2 >= 0; j2--) {
 		fread (tmp, GMT_native_size[type], (size_t)header->nx, fp);	/* Get one row */
-		if (j2 < 0) break;	/* Might be stupid but works	*/
 		ij = j2 * header->nx;
 		for (i = 0; i < header->nx; i++) {
 			kk = ij + i;
@@ -1462,13 +1548,42 @@ int GMT_surfer_write_grd (char *file, struct GRD_HEADER *header, float *grid, do
 /* Add custom code here */
 
 /*-----------------------------------------------------------
- * Format # :	7
- * Type :	
- * Prefix :	
- * Author :	
- * Date :	
- * Purpose:	
- * Functions :	
+ * Format # :	7 - 11
+ * Type :	netCDF-based format
+ * Prefix :	GMT_cdf_*_ (byte, char, short, int, float, double)
+ * Date :	21-MAY-2000
+ * Author :	Masakazu Higaki (m-higaki@naps.kishou.go.jp)
+ * Purpose:	Read/Write netcdf with non-float data types
+ * Functions :	GMT_cdf_*_write_grd
  *		
  *-----------------------------------------------------------*/
-
+ 
+int GMT_cdf_byte_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
+{
+	return (GMT_cdf_write_grd (file, header, grid, w, e, s, n, pad, complex, NC_BYTE));
+}
+ 
+int GMT_cdf_char_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
+{
+	return (GMT_cdf_write_grd (file, header, grid, w, e, s, n, pad, complex, NC_CHAR));
+}
+ 
+int GMT_cdf_short_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
+{
+	return (GMT_cdf_write_grd (file, header, grid, w, e, s, n, pad, complex, NC_SHORT));
+}
+ 
+int GMT_cdf_int_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
+{
+	return (GMT_cdf_write_grd (file, header, grid, w, e, s, n, pad, complex, NC_INT));
+}
+ 
+int GMT_cdf_float_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
+{
+	return (GMT_cdf_write_grd (file, header, grid, w, e, s, n, pad, complex, NC_FLOAT));
+}
+ 
+int GMT_cdf_double_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex)
+{
+	return (GMT_cdf_write_grd (file, header, grid, w, e, s, n, pad, complex, NC_DOUBLE));
+}
