@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)gmt_customio.h	2.20  09/10/99
+ *	$Id: gmt_customio.h,v 1.3 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
  
 /*
@@ -21,10 +21,13 @@
  *
  * Author:	Paul Wessel
  * Date:	10-AUG-1995
- * Revised:	10-SEPT-1999
+ * Version:	3.4
  *
  */
 
+#ifndef GMT_CUSTOMIO_H
+#define GMT_CUSTOMIO_H
+
 /* List groups of 4 integer functions for each custom i/o grd format */
 
 /* Format # 1 */
@@ -62,3 +65,5 @@ EXTERN_MSC int GMT_srf_read_grd_info (char *file, struct GRD_HEADER *header);
 EXTERN_MSC int GMT_srf_write_grd_info (char *file, struct GRD_HEADER *header);
 EXTERN_MSC int GMT_srf_read_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex);
 EXTERN_MSC int GMT_srf_write_grd (char *file, struct GRD_HEADER *header, float *grid, double w, double e, double s, double n, int *pad, BOOLEAN complex);
+
+#endif /* GMT_CUSTOMIO_H */
