@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)gmt_io.c	3.37  09/22/99
+ *	$Id: gmt_io.c,v 1.3 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * Table input/output in GMT can be either ascii or binary (where supported)
@@ -33,7 +33,7 @@
  *	GMT_multisegment:	Decode the -M switch
  *	GMT_write_segmentheader	Write header record for multisegment files
  *	GMT_ascii_input:	Decode ascii input record
- *	GMT_scanf:		Robust scanf function
+ *	GMT_scanf:		Robust scanf function with optional dd:mm:ss conversion
  *	GMT_bin_double_input:	Decode binary double precision record
  *	GMT_bin_float_input:	Decode binary single precision record
  *	GMT_ascii_output:	Write ascii record
@@ -67,12 +67,15 @@
  *	GMT_row_ij:		Convert index to row format
  *
  * Author:	Paul Wessel
- * Date:	29-JUN-1998
- * Version:	3.3.1
+ * Date:	14-JUL-2000
+ * Version:	3.4
  */
  
 #include "gmt.h"
 
+#define GMT_I_60	(1.0 / 60.0)	/* Convert minutes to degrees */
+#define GMT_I_3600	(1.0 / 3600.0)	/* Convert seconds to degrees */
+
 BOOLEAN GMT_do_swab = FALSE;	/* Used to indicate swab'ing during binary read */
 
 int GMT_a_read (FILE *fp, double *d);
@@ -103,12 +106,13 @@ int GMT_bin_float_input (FILE *fp, int *n, double **ptr);	/* Decode binary float
 int GMT_ascii_output (FILE *fp, int n, double *ptr);		/* Write Ascii output records */
 int GMT_bin_double_output (FILE *fp, int n, double *ptr);	/* Write binary double output records */
 int GMT_bin_float_output (FILE *fp, int n, double *ptr);	/* Write binary float output records */
+void GMT_adjust_periodic ();					/* Add/sub 360 as appropriate */
 
 /* Table I/O routines for ascii and binary io */
 
 FILE *GMT_fopen (const char* filename, const char* mode)
 {
-        return fopen (filename, mode);
+        return (fopen (filename, mode));
 }
 
 int GMT_fclose (FILE *stream)
@@ -118,6 +122,8 @@ int GMT_fclose (FILE *stream)
 
 void GMT_io_init (void)
 {
+	/* No need to init the structure as this is done in gmt_init.h directoy */
+	
 	GMT_input = GMT_ascii_input;
 	GMT_output = GMT_ascii_output;
 
@@ -204,7 +210,7 @@ void GMT_multisegment (char *text)
 int GMT_ascii_input (FILE *fp, int *n, double **ptr)
 {
 	char line[BUFSIZ], *p;
-	int i, len, n_convert, GMT_scanf (char *p, double *val);
+	int i, len, n_convert;
 	BOOLEAN done = FALSE, bad_record;
 	double val;
 	
@@ -219,7 +225,7 @@ int GMT_ascii_input (FILE *fp, int *n, double **ptr)
 		/* First read until we get a non-blank, non-comment record or reach EOF */
 
 		GMT_io.rec_no++;
-		while ((p = fgets (line, BUFSIZ, fp)) && (line[0] == '\0' || (line[0] == '#' && GMT_io.EOF_flag != '#'))) GMT_io.rec_no++;
+		while ((p = fgets (line, BUFSIZ, fp)) && (line[0] == '\n' || (line[0] == '#' && GMT_io.EOF_flag != '#'))) GMT_io.rec_no++;
 
 		if (!p) {
 			GMT_io.status = GMT_IO_EOF;
@@ -252,9 +258,10 @@ int GMT_ascii_input (FILE *fp, int *n, double **ptr)
 		line[++i] = '\n';	line[++i] = '\0';
  
 		bad_record = FALSE;
-		i = 0;
 		strcpy (GMT_io.current_record, line);
+		line[i-1] = '\0';	/* Chop off newline */
 		p = strtok(line, " \t,");
+		i = 0;
 		while (!bad_record && p && i < *n) {
 			if ((n_convert = GMT_scanf (p, &val)) == 1) {	/* Decoded string to a number */
 				GMT_data[i] = val;
@@ -278,32 +285,104 @@ int GMT_ascii_input (FILE *fp, int *n, double **ptr)
 	*ptr = GMT_data;
 	GMT_io.status = (i == *n || *n == BUFSIZ) ? 0 : GMT_IO_MISMATCH;
 	if (*n == BUFSIZ) *n = i;
-	return (i);
 
+	if (gmtdefs.xy_toggle) d_swap (GMT_data[0], GMT_data[1]);	/* Got lat/lon instead of lon/lat */
+	if (GMT_geographic_in) {
+		GMT_adjust_periodic ();	/* Must account for periodicity in 360 */
+	}
+	
+	return (i);
 }
 
 int GMT_scanf (char *p, double *val)
 {
-	/* Converts text to double under if certain conditions are met:
-	    p must be of the form [+|-][.][0-9]*
-	    If it is we look for d or D which is FORTRASH for exponent
-	    and convert to e and then call atof
+	/* Converts text to double if certain conditions are met:
+	   p must be of the form [+|-][0-9][.][0-9][E|e|D|e][+|-][0-9] OR [+|-][dd:mm[:ss][WESN]
+	   If it is the former we look for d or D which is FORTRASH
+	   for exponent and convert to e and then call atof.  If it
+	   is the latter we do a straight conversion.
 	*/
+	int i, k, c, colons, suffix;
+	BOOLEAN error, period, exponent, sign, flip_sign;
+	double degree, minute, second;
 
-	int i = 0;
-
-	if (p[i] == '-' || p[i] == '+') i++;
-	if (p[i] == '.' || (p[i] >= '0' && p[i] <= '9')) {
-		while (p[i]) {
-			if (p[i] == 'D' || p[i] == 'd') p[i] = 'e';	/* Fortran Double Precision Fix */
+	for (i = colons = 0; p[i]; i++) if (p[i] == ':') colons++;	/* Colons indicate dd:mm:ss format */
+	k = --i;
+	c = (int)p[k];		/* Last character in string */
+	suffix = toupper (c);	/* Last character in string, forced upper case */
+	
+	if (colons == 0) {	/* Regular ASCII representation of an integer or floating point number */
+		if (suffix == 'W' || suffix == 'S') {	/* Sign was given implicitly as -ve */
+			p[k] = 0;	/* Temporarily hide the suffix - will restore below */
+			flip_sign = TRUE;
+		}
+		else if (suffix == 'E' || suffix == 'N') {	/* Sign was given implicitly as +ve */
+			p[k] = 0;	/* Temporarily hide the suffix - will restore below */
+			flip_sign = FALSE;
+		}
+		else	/* No suffix or unrecognized suffix */
+			flip_sign = FALSE;
+			
+		i = 0;					/* Reset to 1st character */
+		while (p[i] == ' ') i++;		/* Skip leading blanks */
+		if (p[i] == '-' || p[i] == '+') i++;	/* Leading sign is OK */
+		error = period = exponent = sign = FALSE;
+		while (p[i] && !error) {
+			if (p[i] == '.') {	/* One period is OK */
+				if (period) error = TRUE;
+				period = TRUE;
+			}
+			else if (p[i] == 'D' || p[i] == 'd') {	/* Fortran Double Precision Fix */
+				p[i] = 'e';	/* 'd' is not understood outside Fortran i/o */
+				if (exponent) error = TRUE;
+				exponent = TRUE;
+			}
+			else if (p[i] == 'E' || p[i] == 'e') {	/* Normal exponentional notation */
+				if (exponent) error = TRUE;
+				exponent = TRUE;
+			}
+			else if (p[i] == '-' || p[i] == '+') {	/* One more sign only ok after exponential */
+				if (sign || !exponent) error = TRUE;
+				sign = TRUE;
+			}
+			else if (p[i] < '0' || p[i] > '9')	/* Other non-digits are not allowed */
+				error = TRUE;
 			i++;
 		}
-		*val = atof (p);
+		if (error) return (0);	/* Failed format check */
+		*val = atof (p);	/* Safe to convert */
+		if (flip_sign) {	/* Flip sign and restore missing suffix in text string */
+			*val = -(*val);
+			p[k] = suffix;
+		}
+			
 		return (1);
 	}
-	return (0);
+
+	/* Here we know we need to deal with dd:mm[:ss] strings */
+	
+	if (colons == 2) {	/* dd:mm:ss format */
+		sscanf (p, "%lf:%lf:%lf", &degree, &minute, &second);
+		if (suffix == 'W' || suffix == 'w' || suffix == 'S' || suffix == 's') degree = -degree;	/* Sign was given implicitly */
+		*val = degree + copysign (minute * GMT_I_60 + second * GMT_I_3600, degree);
+	}
+	else if (colons == 1) {	/* dd:mm format */
+		sscanf (p, "%lf:%lf", &degree, &minute);
+		if (suffix == 'W' || suffix == 'w' || suffix == 'S' || suffix == 's') degree = -degree;	/* Sign was given implicitly */
+		*val = degree + copysign (minute * GMT_I_60, degree);
+	}
+	else	/* Unrecognized */
+		return (0);
+
+	return (1);
 }
 
+char *GMT_fgets (char *record, int maxlength, FILE *fp)
+{
+	return (fgets (record, maxlength, fp));
+}
+
+
 int GMT_bin_double_input (FILE *fp, int *n, double **ptr)
 {
 	int n_read;
@@ -327,6 +406,10 @@ int GMT_bin_double_input (FILE *fp, int *n, double **ptr)
 			return (0);
 		}
 	}
+	if (gmtdefs.xy_toggle) d_swap (GMT_data[0], GMT_data[1]);	/* Got lat/lon instead of lon/lat */
+	if (GMT_geographic_in) {
+		GMT_adjust_periodic ();	/* Must account for periodicity in 360 */
+	}
 
 	return (n_read);
 }
@@ -358,14 +441,28 @@ int GMT_bin_float_input (FILE *fp, int *n, double **ptr)
 			return (0);
 		}
 	}
+	if (gmtdefs.xy_toggle) d_swap (GMT_data[0], GMT_data[1]);	/* Got lat/lon instead of lon/lat */
+	if (GMT_geographic_in) {
+		GMT_adjust_periodic ();	/* Must account for periodicity in 360 */
+	}
 
 	return (n_read);
 }
+
+void GMT_adjust_periodic (void) {
+	/* while (GMT_data[0] > project_info.e) GMT_data[0] -= 360.0;
+	while (GMT_data[0] < project_info.w) GMT_data[0] += 360.0; */
+	while (GMT_data[0] > project_info.e && (GMT_data[0] - 360.0) >= project_info.w) GMT_data[0] -= 360.0;
+	while (GMT_data[0] < project_info.w && (GMT_data[0] + 360.0) <= project_info.w) GMT_data[0] += 360.0;
+	/* If data is not inside the given range it will satisfy (lon > east) */
+	/* Now it will be outside the region on the same side it started out at */
+}
 	
 int GMT_ascii_output (FILE *fp, int n, double *ptr)
 {
 	int i, e = 0, wn = 0;
 	
+	if (gmtdefs.xy_toggle) d_swap (ptr[0], ptr[1]);	/* Write lat/lon instead of lon/lat */
 	n--;
 	for (i = 0; i < n && e >= 0; i++) {
 		(GMT_is_dnan (ptr[i])) ? (e = fprintf (fp, "NaN\t")) : (e = fprintf (fp, gmtdefs.d_format, ptr[i]), putc ('\t', fp));
@@ -378,6 +475,7 @@ int GMT_ascii_output (FILE *fp, int n, double *ptr)
 
 int GMT_bin_double_output (FILE *fp, int n, double *ptr)
 {
+	if (gmtdefs.xy_toggle) d_swap (ptr[0], ptr[1]);	/* Write lat/lon instead of lon/lat */
 	return (fwrite ((void *) ptr, sizeof (double), (size_t)n, fp));
 }
 	
@@ -386,6 +484,7 @@ int GMT_bin_float_output (FILE *fp, int n, double *ptr)
 	int i;
 	static float GMT_f[BUFSIZ];
 	
+	if (gmtdefs.xy_toggle) d_swap (ptr[0], ptr[1]);	/* Write lat/lon instead of lon/lat */
 	for (i = 0; i < n; i++) GMT_f[i] = (float)ptr[i];
 	return (fwrite ((void *) GMT_f, sizeof (float), (size_t)n, fp));
 }
@@ -420,12 +519,12 @@ void GMT_init_z_io (struct GMT_Z_IO *r, BOOLEAN input)
 
 int GMT_parse_z_io (char *txt, struct GMT_Z_IO *r, BOOLEAN input)
 {
-	BOOLEAN first = TRUE, error = FALSE;
+	BOOLEAN first = TRUE;
 	int i;
 
 	/* BOOLEAN input:  currently unused */
 
-	for (i = 0; !error && txt[i]; i++) {	/* Loop over flags */
+	for (i = 0; txt[i]; i++) {	/* Loop over flags */
 
 		switch (txt[i]) {
 
@@ -534,7 +633,8 @@ int GMT_parse_z_io (char *txt, struct GMT_Z_IO *r, BOOLEAN input)
 
 
 			default:
-				error = TRUE;
+				fprintf (stderr, "%s: GMT SYNTAX ERROR -Z: %c not a valid modifier!\n", GMT_program, txt[i]);
+				exit (EXIT_FAILURE);
 				break;
 		}
 	}
@@ -544,7 +644,7 @@ int GMT_parse_z_io (char *txt, struct GMT_Z_IO *r, BOOLEAN input)
 		strcpy (GMT_io.w_mode, "wb");
 	}
 
-	return (error);
+	return (FALSE);
 }
 
 void GMT_set_z_io (struct GMT_Z_IO *r, struct GRD_HEADER *h)
@@ -577,8 +677,8 @@ void GMT_check_z_io (struct GMT_Z_IO *r, float *a)
 
 int GMT_a_read (FILE *fp, double *d)
 {
-	char line[50];
-	if (fgets (line, 50, fp)) {
+	char line[64];
+	if (fgets (line, 64, fp)) {
 		*d = (line[0] == 'N' || line[0] == 'n') ? GMT_d_NaN : atof (line);
 		return (1);
 	}
@@ -695,7 +795,7 @@ int GMT_d_read (FILE *fp, double *d)
 int GMT_a_write (FILE *fp, double d)
 {
 	int n = 0;
-	n = (GMT_is_dnan (d)) ? fprintf (fp, "NaN\n") : fprintf (fp, gmtdefs.d_format, d);
+	n = (GMT_is_dnan (d)) ? fprintf (fp, "NaN") : fprintf (fp, gmtdefs.d_format, d);
 	fprintf (fp, "\n"); 
 	return (n);
 }
