@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)gmt_shore.c	3.78  12/01/99
+ *	$Id: gmt_shore.c,v 1.3 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 
 #include "gmt.h"
@@ -37,8 +37,8 @@
  * GMT_br_cleanup :		Frees up main river/border structure memory
  *
  * Author:	Paul Wessel
- * Date:	13-JUN-1995
- * Modified:	16-AUG-1999
+ * Date:	13-JUN-2000
+ * Version:	3.4
  *
  */
 
@@ -443,8 +443,8 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 
 {
 	struct POL *p;
-	int start_side, next_side, id, P = 0, more, p_alloc, wet_or_dry, use_this_level;
-	int n_alloc, cid, nid, add, first_pos, entry_pos, n, low_level, high_level;
+	int start_side, next_side, id, P = 0, more, p_alloc, wet_or_dry, use_this_level, high_seg_level = MAX_LEVEL;
+	int n_alloc, cid, nid, add, first_pos, entry_pos, n, low_level, high_level, nseg_at_level[MAX_LEVEL+1];
 	BOOLEAN completely_inside;
 	double *xtmp, *ytmp, plon, plat;
 	
@@ -465,7 +465,18 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 		return (c->ns);
 	}
 	
-	for (n = high_level = 0; n < 4; n++) high_level = MAX (c->node_level[n], high_level);
+	/* Check the consistancy of node levels in case some features are dropped */
+	
+	memset ((void *)nseg_at_level, 0, (size_t)((MAX_LEVEL + 1) * sizeof (int)));
+	for (id = 0; id < c->ns; id++) if (c->seg[id].entry != 4) nseg_at_level[c->seg[id].level]++;	/* Only count segments that crosses the bin */
+	for (n = 0; n <= MAX_LEVEL; n++) if (nseg_at_level[n]) high_seg_level = n;
+	
+	if (c->ns == 0) for (n = 0; n < 4; n++) high_seg_level = MIN (c->node_level[n], high_seg_level);	/* Initialize to lowest when there are no segments */
+	for (n = high_level = 0; n < 4; n++) {
+		c->node_level[n] = MIN (c->node_level[n], high_seg_level);
+		high_level = MAX (c->node_level[n], high_level);
+	}
+	
 	wet_or_dry = (dir == 1) ? 1 : 0;
 	use_this_level = (high_level%2 == wet_or_dry && high_level >= first_level);
 
@@ -474,7 +485,7 @@ int GMT_assemble_shore (struct GMT_SHORE *c, int dir, int first_level, BOOLEAN a
 	/* Here we must assemble [at least one] polygons in the correct order */
 	
 	for (n = 0, completely_inside = TRUE; completely_inside && n < c->ns; n++) if (c->seg[n].entry != 4) completely_inside = FALSE;
-
+	
 	shore_prepare_sides (c, dir);
 	
 	p_alloc = (c->ns == 0) ? 1 : GMT_SMALL_CHUNK;
