@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)gmt_stat.c	2.40  11/01/99
+ *	$Id: gmt_stat.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,13 +13,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  *  some stuff for significance tests.
  *
  * Author:	Walter H. F. Smith
  * Date:	12-JUN-1995
+ * Version:	3.4
  * Revised:	12-AUG-1998
  *		06-JAN-1999 P Wessel: Added GMT_ber, GMT_bei, GMT_ker, GMT_kei
  *		12-JAN-1999 P Wessel: Added GMT_plm
@@ -39,6 +40,13 @@
  *					Added GMT_j0, GMT_j1, GMT_jn, GMT_y0,
  *					GMT_y1, GMT_yn for platforms that do no
  *					have the bessel functions in the math lib.
+ *		07-DEC-1999 P. Wessel: Added GMT_rand and GMT_nrand
+ *		29-MAR-2000 W Smith: Added GMT_hypot, GMT_log1p, and GMT_atanh for
+ *					platforms without native library support.
+ *		09-MAY-2000 P. Wessel: Added GMT_chi2 and GMT_cumpoission
+ *		18-AUG-2000 P. Wessel: Moved GMT_mode and GMT_median from gmt_support to here.
+ *					Added float versions of these two functions for grd data.
+ *					Added 
  *
  * PUBLIC functions:
  *
@@ -66,6 +74,8 @@
  *	GMT_yn:		Bessel function 2nd kind order N
  *	GMT_dilog:	The dilog function
  *	GMT_erfinv:	The inverse error function
+ *	GMT_rand:	Uniformly distributed random numbers 0 < x < 1
+ *	GMT_nrand:	Normally distributed random numbers from N(0,1)
  */
 
 #include "gmt.h"
@@ -73,7 +83,8 @@
 int GMT_inc_beta (double a, double b, double x, double *ibeta);
 int GMT_ln_gamma_r (double x, double *lngam);
 int GMT_f_q (double chisq1, int nu1, double chisq2, int nu2, double *prob);	
-int GMT_student_t_a(double t, int n, double *prob);
+int GMT_f_test_new (double chisq1, int nu1, double chisq2, int nu2, double *prob, int iside);
+int GMT_student_t_a (double t, int n, double *prob);
 double GMT_ln_gamma (double xx);
 double GMT_cf_beta (double a, double b, double x);
 double GMT_bei (double x);
@@ -92,6 +103,12 @@ double GMT_dilog (double x);
 double GMT_mag3v (double *a);		/*	Returns magnitude of 3-D vector	*/
 double GMT_ln_gamma (double xx);		/*	Computes natural log of the gamma function	*/
 double GMT_erfinv (double y);
+double GMT_gammq (double a, double x);
+void GMT_gamma_cf (double *gammcf, double a, double x, double *gln);
+void GMT_gamma_ser (double *gamser, double a, double x, double *gln);
+void GMT_chi2 (double chi2, double nu, double *prob);
+void GMT_cumpoission (double k, double mu, double *prob);
+
 
 #if HAVE_J0 == 0
 double GMT_j0 (double x);
@@ -394,6 +411,89 @@ double	GMT_cf_beta (double a, double b, double x)
 	return(az);
 }
 
+#define ITMAX 100
+
+void	GMT_gamma_ser (double *gamser, double a, double x, double *gln) {
+	/* Returns the incomplete gamma function P(a,x) by series rep.
+	 * Press et al, gser() */
+	 
+	int n;
+	double sum, del, ap;
+	 
+	GMT_ln_gamma_r (a, gln);
+	 
+	if (x < 0.0) {
+		fprintf (stderr, "GMT DOMAIN ERROR:  x < 0 in GMT_gamma_ser(x)\n");
+		*gamser = GMT_d_NaN;
+		return;
+	}
+	if (x == 0.0) {
+		*gamser = 0.0;
+		return;
+	}
+	ap = a;
+	del = sum = 1.0 / a;
+	for (n = 1; n <= ITMAX; n++) {
+	 	ap += 1.0;
+	 	del *= x / ap;
+	 	sum += del;
+	 	if (fabs (del) < fabs (sum) * DBL_EPSILON) {
+	 		*gamser = sum * exp (-x  + a * log (x) - (*gln));
+	 		return;
+	 	}
+	}
+	fprintf (stderr, "GMT DOMAIN ERROR:  a too large, ITMAX too small in GMT_gamma_ser(x)\n");
+}
+
+void	GMT_gamma_cf (double *gammcf, double a, double x, double *gln) {
+	/* Returns the incomplete gamma function P(a,x) by continued fraction.
+	 * Press et al, gcf() */
+	int n;
+	double gold = 0.0, g, fac = 1.0, b1 = 1.0;
+	double b0 = 0.0, anf, ana, an, a1, a0 = 1.0;
+	
+	GMT_ln_gamma_r (a, gln);
+
+	a1 = x;
+	for (n = 1; n <= ITMAX; n++) {
+		an = (double) n;
+		ana = an - a;
+		a0 = (a1 + a0 * ana) * fac;
+		b0 = (b1 + b0 * ana) * fac;
+		anf = an * fac;
+		a1 = x * a0 + anf * a1;
+		b1 = x * b0 + anf * b1;
+		if (a1 != 0.0) {
+			fac = 1.0 / a1;
+			g = b1 * fac;
+			if (fabs ((g - gold) / g) < DBL_EPSILON) {
+				*gammcf = exp (-x + a * log (x) - (*gln)) * g;
+				return;
+			}
+			gold = g;
+		}
+	}
+	fprintf (stderr, "GMT DOMAIN ERROR:  a too large, ITMAX too small in GMT_gamma_cf(x)\n");
+}
+
+double GMT_gammq (double a, double x) {
+	/* Returns Q(a,x) = 1 - P(a,x) Inc. Gamma function */
+	
+	double G, gln;
+	
+	if (x < 0.0 || a <= 0.0) {
+		fprintf (stderr, "GMT DOMAIN ERROR:  Invalid arguments to GMT_gammaq\n");
+		return (GMT_d_NaN);
+	}
+	
+	if (x < (a + 1.0)) {
+		GMT_gamma_ser (&G, a, x, &gln);
+		return (1.0 - G);
+	}
+	GMT_gamma_cf (&G, a, x, &gln);
+	return (G);
+}
+
 /*
  * Kelvin-Bessel functions, ber, bei, ker, kei.  ber(x) and bei(x) are even;
  * ker(x) and kei(x) are defined only for x > 0 and x >=0, respectively.  
@@ -953,7 +1053,7 @@ double GMT_erf (double y)
 #endif
 
 #if HAVE_ERFC == 0
-double erfc (double y)
+double GMT_erfc (double y)
 {
 	int i, sign = 1;
 	double x, res, xsq, xnum, xden, xi;
@@ -1031,6 +1131,10 @@ int     GMT_f_q (double chisq1, int nu1, double chisq2, int nu2, double *prob)
 		a finite limit on the sum for their expression.
 
 		W H F Smith, August, 1999.
+		
+		REVISED by W H F Smith, October 27, 2000 after GMT 3.3.6 release.
+		I found that the A&S methods overflowed for large nu1 and nu2, so
+		I decided to go back to the GMT_inc_beta way of doing things.
 
 	*/
 		double	t, p, x, y, theta, term, sum, c, s, csq, ssq;
@@ -1056,6 +1160,14 @@ int     GMT_f_q (double chisq1, int nu1, double chisq2, int nu2, double *prob)
 		return (0);
 	}
 	
+	/* REVISION of Oct 27, 2000:  This inc beta call here returns
+		the value.  All subsequent code is not used.  */
+	
+	if (GMT_inc_beta(0.5*nu2, 0.5*nu1, chisq2/(chisq2+chisq1), prob) ) {
+		fprintf(stderr,"GMT_q_p:  Trouble in GMT_inc_beta call.\n");
+		return(-1);
+	}
+	return(0);	/* REVISION Oct 27, 2000:  All code below here is not used.  */
 
 	/* If nu1 == 1, sqrt(F) is distributed as Student's t;
 		if nu2 == 1, use reflexive property to swap 1 and 2.  */
@@ -1523,3 +1635,581 @@ void sincos (double a, double *s, double *c)
 }
 
 #endif
+
+#define GMT_RAND_IQ 127773
+#define GMT_RAND_IA 16807
+#define GMT_RAND_IM 2147483647
+#define GMT_RAND_AM (1.0 / GMT_RAND_IM)
+#define GMT_RAND_IR 2836
+#define GMT_RAND_NTAB 32
+#define GMT_RAND_NDIV (1 + (GMT_RAND_IM - 1) / GMT_RAND_NTAB)
+#define GMT_RAND_EPS 2.2204460492503131E-16
+
+double GMT_rand ()
+{
+	/* Uniform random number generator based on ran1 of
+	 * Press et al, Numerical Recipes, 2nd edition,
+	 * converted from Fortran to C.  Will return values
+	 * x so that 0.0 < x < 1.0 occurs with equal probablilty.
+	 */
+	
+	static int GMT_rand_iy = 0;
+	static int GMT_rand_seed = 0;
+	static int GMT_rand_iv[GMT_RAND_NTAB];
+	
+	int j, k;
+	
+	if (GMT_rand_iy == 0) {	/* First time initialization */
+		GMT_rand_seed = (int) time (NULL);		/* Seed value is positive sec since 1970 */
+		if (GMT_rand_seed <= 0) GMT_rand_seed = 1;	/* Make sure to prevent seed = 0 */
+		for (j = GMT_RAND_NTAB + 8 - 1; j >= 0; j--) {	/* Load shuffle table after 8 warm-ups */
+			k = GMT_rand_seed / GMT_RAND_IQ;
+			GMT_rand_seed = GMT_RAND_IA * (GMT_rand_seed - k * GMT_RAND_IQ) - GMT_RAND_IR * k;
+			if (GMT_rand_seed < 0) GMT_rand_seed += GMT_RAND_IM;
+			if (j < GMT_RAND_NTAB) GMT_rand_iv[j] = GMT_rand_seed;
+		}
+		GMT_rand_iy = GMT_rand_iv[0];
+	}
+
+	/* Starts here when not initializing */
+
+	k = GMT_rand_seed / GMT_RAND_IQ;
+
+	/* Compute GMT_rand_seed = mod (GMT_rand_seed * GMT_RAND_IA, GMT_RAND_IM) without overflowing using Schrage's method */
+
+	GMT_rand_seed = GMT_RAND_IA * (GMT_rand_seed - k * GMT_RAND_IQ) - GMT_RAND_IR * k;
+	if (GMT_rand_seed < 0) GMT_rand_seed += GMT_RAND_IM;
+
+	j = GMT_rand_iy / GMT_RAND_NDIV;	/* Will be in the range 0:NTAB-1 */
+	GMT_rand_iy = GMT_rand_iv[j];		/* Output previous shuffle value and fill table again */
+	GMT_rand_iv[j] = GMT_rand_seed;
+	return (MIN (GMT_RAND_AM * GMT_rand_iy, (1.0 - GMT_RAND_EPS)));	/* so we dont return 1.0 */
+}
+
+double GMT_nrand (void) {
+	/* Gaussian random number generator based on gasdev of
+	 * Press et al, Numerical Recipes, 2nd edition.  Will
+	 * return values that zero mean and unit variance.
+	 */
+	 
+	static int iset = 0;
+	static double gset;
+	double fac, r, v1, v2;
+	double GMT_rand ();
+
+	if (iset == 0) {	/* We don't have an extra deviate handy, so */
+		do {
+			v1 = 2.0 * GMT_rand () - 1.0;	/* Pick two uniform numbers in the -1/1/-1/1 square */
+			v2 = 2.0 * GMT_rand () - 1.0;
+			r = v1 * v1 + v2 * v2;
+		} while (r >= 1.0 || r == 0.0);	/* Keep trying until v1,v2 is inside unit circle */
+		
+		fac = sqrt (-2.0 * log (r) / r);
+		
+		/* Now make Box-Muller transformation to get two normal deviates.  Return
+		 * one and save the other for the next time GMT_nrand is called */
+	 
+		gset = v1 * fac;
+		iset = 1;	/* Set flag for next time */
+		return (v2 * fac);
+	}
+	else {
+		iset = 0;	/* Take old value, reset flag */
+		return (gset);
+	}
+}
+
+void GMT_chi2 (double chi2, double nu, double *prob) {
+	/* Evalulate probability that chi2 will exceed the
+	 * theoretical chi2 by chance. */
+ 
+ 	*prob = GMT_gammq (0.5 * nu, 0.5 * chi2);
+}
+
+void GMT_cumpoission (double k, double mu, double *prob) {
+	/* evaluate Cumulative Poisson Distribution */
+	
+	*prob = GMT_gammq (k, mu);
+}
+
+#if HAVE_HYPOT == 0
+
+double	GMT_hypot (double x, double y) {
+/* 	Return sqrt(x*x + y*y), guarding against
+	some overflows where possible.  If a local
+	library hypot() is available, it should be
+	used instead; it could be faster and more
+	accurate.  WHFS 30 March 2000  */
+	
+	double	a, b, c, d, r, s, t;
+
+	if (GMT_is_dnan (x) || GMT_is_dnan (y) ) return (GMT_d_NaN);
+	
+	/* A complete implementation of IEEE exceptional values
+	would also return +Inf if either x or y is +/- Inf  */
+	
+	
+	if (x == 0.0) return (fabs(y));
+	if (y == 0.0) return (fabs(x));
+	
+	a = fabs(x);
+	b = fabs(y);
+	
+	/* If POSIX defined M_SQRT2 (= sqrt(2.0) to machine
+	precision) then we could say
+		if (a == b) return (a * M_SQRT2);
+	*/
+	
+	if (a < b) {
+		c = b;
+		d = a/b;
+	}
+	else {
+		c = a;
+		d = b/a;
+	}
+	
+	/* DBL_EPSILON is defined in POSIX float.h
+	as the smallest positive e such that 1+e > 1
+	in floating point.  */
+	
+	if (d < DBL_EPSILON) return (c);
+	
+	s = d*d;
+	if (s > DBL_EPSILON) return (c * sqrt (1.0 + s) );
+	
+	t = 1.0 + d;
+	s = (2 * (d/t)) / t;
+	
+	r = t * sqrt (1.0 - s);
+	
+	if (r <= 1.0) return (c);
+	
+	return (c * r);
+}
+
+#endif
+
+#if HAVE_LOG1P == 0
+
+double	GMT_log1p (double x) {
+/* 	Approximate log(1 + x) fairly well for small x.
+	This should be used only if there isn't a better
+	library routine available.  WHFS 30 March 2000  */
+
+	double 	u;
+	
+	if (GMT_is_dnan(x) || x <= -1.0) return (GMT_d_NaN);
+	
+	u = 1.0 + x;
+	
+	if (u == 1.0) 
+		return (x);
+	else
+		return ( log(u) * (x/(u - 1.0) ) );
+}
+
+#endif
+
+#if HAVE_ATANH == 0
+
+double	GMT_atanh (double x) {
+/* 	Return hyperbolic arctangent of x.
+	This should be used only if there isn't a better
+	library routine available.  WHFS 30 March 2000  */
+	
+	if (GMT_is_dnan(x) || fabs(x) >= 1.0) return (GMT_d_NaN);
+	
+	return (0.5 * ( log1p(x) - log1p(-x) ) );
+}
+
+#endif
+
+int GMT_median (double *x, int n, double xmin, double xmax, double m_initial, double *med)
+{
+	double	lower_bound, upper_bound, m_guess, t_0, t_1, t_middle;
+	double	lub, glb, xx, temp;
+	int	i, n_above, n_below, n_equal, n_lub, n_glb;
+	int	finished = FALSE;
+	int	iteration = 0;
+	
+	m_guess = m_initial;
+	lower_bound = xmin;
+	upper_bound = xmax;
+	t_0 = 0;
+	t_1 = n - 1;
+	t_middle = 0.5 * (n - 1);
+	
+	do {
+
+		n_above = n_below = n_equal = n_lub = n_glb = 0;
+		lub = xmax;
+		glb = xmin;
+		
+		for (i = 0; i < n; i++) {
+		
+			xx = x[i];
+			if (xx == m_guess) {
+				n_equal++;
+			}
+			else if (xx > m_guess) {
+				n_above++;
+				if (xx < lub) {
+					lub = xx;
+					n_lub = 1;
+				}
+				else if (xx == lub) {
+					n_lub++;
+				}
+			}
+			else {
+				n_below++;
+				if (xx > glb) {
+					glb = xx;
+					n_glb = 1;
+				}
+				else if (xx == glb) {
+					n_glb++;
+				}
+			}
+		}
+		
+		iteration++;
+
+		/* Now test counts, watch multiple roots, think even/odd:  */
+		
+		if ( (abs(n_above - n_below)) <= n_equal) {
+			
+			if (n_equal) {
+				*med = m_guess;
+			}
+			else {
+				*med = 0.5 * (lub + glb);
+			}
+			finished = TRUE;
+		}
+		else if ( (abs( (n_above - n_lub) - (n_below + n_equal) ) ) < n_lub) {
+		
+			*med = lub;
+			finished = TRUE;
+		}
+		else if ( (abs( (n_below - n_glb) - (n_above + n_equal) ) ) < n_glb) {
+		
+			*med = glb;
+			finished = TRUE;
+		}
+		/* Those cases found the median; the next two will forecast a new guess:  */
+		
+		else if ( n_above > (n_below + n_equal) ) {  /* Guess is too low  */
+		
+			lower_bound = m_guess;
+			t_0 = n_below + n_equal - 1;
+			temp = lower_bound + (upper_bound - lower_bound) * (t_middle - t_0) / (t_1 - t_0);
+			m_guess = (temp > lub) ? temp : lub;	/* Move guess at least to lub  */
+		}
+		else if ( n_below > (n_above + n_equal) ) {  /* Guess is too high  */
+		
+			upper_bound = m_guess;
+			t_1 = n_below + n_equal - 1;
+			temp = lower_bound + (upper_bound - lower_bound) * (t_middle - t_0) / (t_1 - t_0);
+			m_guess = (temp < glb) ? temp : glb;	/* Move guess at least to glb  */
+		}
+		else {	/* If we get here, I made a mistake!  */
+			fprintf (stderr,"%s: GMT Fatal Error: Internal goof - please report to developers!\n", GMT_program);
+			exit (EXIT_FAILURE);
+		}
+			
+	} while (!finished);
+	
+	/* That's all, folks!  */
+	return (iteration);
+}
+
+int GMT_median_f (float *x, int n, double xmin, double xmax, double m_initial, double *med)
+{
+	double	lower_bound, upper_bound, m_guess, t_0, t_1, t_middle;
+	double	lub, glb, xx, temp;
+	int	i, n_above, n_below, n_equal, n_lub, n_glb;
+	int	finished = FALSE;
+	int	iteration = 0;
+	
+	m_guess = m_initial;
+	lower_bound = xmin;
+	upper_bound = xmax;
+	t_0 = 0;
+	t_1 = n - 1;
+	t_middle = 0.5 * (n - 1);
+	
+	do {
+
+		n_above = n_below = n_equal = n_lub = n_glb = 0;
+		lub = xmax;
+		glb = xmin;
+		
+		for (i = 0; i < n; i++) {
+		
+			xx = x[i];
+			if (xx == m_guess) {
+				n_equal++;
+			}
+			else if (xx > m_guess) {
+				n_above++;
+				if (xx < lub) {
+					lub = xx;
+					n_lub = 1;
+				}
+				else if (xx == lub) {
+					n_lub++;
+				}
+			}
+			else {
+				n_below++;
+				if (xx > glb) {
+					glb = xx;
+					n_glb = 1;
+				}
+				else if (xx == glb) {
+					n_glb++;
+				}
+			}
+		}
+		
+		iteration++;
+
+		/* Now test counts, watch multiple roots, think even/odd:  */
+		
+		if ( (abs(n_above - n_below)) <= n_equal) {
+			
+			if (n_equal) {
+				*med = m_guess;
+			}
+			else {
+				*med = 0.5 * (lub + glb);
+			}
+			finished = TRUE;
+		}
+		else if ( (abs( (n_above - n_lub) - (n_below + n_equal) ) ) < n_lub) {
+		
+			*med = lub;
+			finished = TRUE;
+		}
+		else if ( (abs( (n_below - n_glb) - (n_above + n_equal) ) ) < n_glb) {
+		
+			*med = glb;
+			finished = TRUE;
+		}
+		/* Those cases found the median; the next two will forecast a new guess:  */
+		
+		else if ( n_above > (n_below + n_equal) ) {  /* Guess is too low  */
+		
+			lower_bound = m_guess;
+			t_0 = n_below + n_equal - 1;
+			temp = lower_bound + (upper_bound - lower_bound) * (t_middle - t_0) / (t_1 - t_0);
+			m_guess = (temp > lub) ? temp : lub;	/* Move guess at least to lub  */
+		}
+		else if ( n_below > (n_above + n_equal) ) {  /* Guess is too high  */
+		
+			upper_bound = m_guess;
+			t_1 = n_below + n_equal - 1;
+			temp = lower_bound + (upper_bound - lower_bound) * (t_middle - t_0) / (t_1 - t_0);
+			m_guess = (temp < glb) ? temp : glb;	/* Move guess at least to glb  */
+		}
+		else {	/* If we get here, I made a mistake!  */
+			fprintf (stderr,"%s: GMT Fatal Error: Internal goof - please report to developers!\n", GMT_program);
+			exit (EXIT_FAILURE);
+		}
+			
+	} while (!finished);
+	
+	/* That's all, folks!  */
+	return (iteration);
+}
+
+int GMT_mode (double *x, int n, int j, int sort, double *mode_est)
+{
+	int	i, istop, multiplicity = 0;
+	double	mid_point_sum = 0.0, length, short_length = 1.0e+30;
+
+	if (sort) qsort((void *)x, (size_t)n, sizeof(double), GMT_comp_double_asc);
+
+	istop = n - j;
+	
+	for (i = 0; i < istop; i++) {
+		length = x[i + j] - x[i];
+		if (length < 0.0) {
+			fprintf(stderr,"GMT_mode: Array not sorted in non-decreasing order.\n");
+			return (-1);
+		}
+		else if (length == short_length) {
+			multiplicity++;
+			mid_point_sum += (0.5 * (x[i + j] + x[i]));
+		}
+		else if (length < short_length) {
+			multiplicity = 1;
+			mid_point_sum = (0.5 * (x[i + j] + x[i]));
+			short_length = length;
+		}
+	}
+
+	if (multiplicity - 1) mid_point_sum /= multiplicity;
+	
+	*mode_est = mid_point_sum;
+	return (0);
+}
+
+int GMT_mode_f (float *x, int n, int j, int sort, double *mode_est)
+{
+	int	i, istop, multiplicity = 0;
+	double	mid_point_sum = 0.0, length, short_length = 1.0e+30;
+
+	if (sort) qsort((void *)x, (size_t)n, sizeof(float), GMT_comp_float_asc);
+
+	istop = n - j;
+	
+	for (i = 0; i < istop; i++) {
+		length = x[i + j] - x[i];
+		if (length < 0.0) {
+			fprintf (stderr,"GMT_mode: Array not sorted in non-decreasing order.\n");
+			return (-1);
+		}
+		else if (length == short_length) {
+			multiplicity++;
+			mid_point_sum += (0.5 * (x[i + j] + x[i]));
+		}
+		else if (length < short_length) {
+			multiplicity = 1;
+			mid_point_sum = (0.5 * (x[i + j] + x[i]));
+			short_length = length;
+		}
+	}
+
+	if (multiplicity - 1) mid_point_sum /= multiplicity;
+	
+	*mode_est = mid_point_sum;
+	return (0);
+}
+
+
+void GMT_getmad (double *x, int n, double location, double *scale)
+{
+	/* Compute MAD (Median Absolute Deviation) for a double data set */
+	
+	double	e_low, e_high, error, last_error;
+	int	i_low, i_high, n_dev, n_dev_stop;
+	
+	i_low = 0;
+	while (i_low < n && x[i_low] <= location) i_low++;
+	i_low--;
+	
+	i_high = n - 1;
+	while (i_high >= 0 && x[i_high] >= location) i_high--;
+	i_high++;
+
+	n_dev_stop = n / 2;
+	error = 0.0;
+	n_dev = 0;
+
+
+	while (n_dev < n_dev_stop) {
+	
+		last_error = error;
+
+		if (i_low < 0) {
+			error = x[i_high] - location;
+			i_high++;
+			n_dev++;
+		}
+		
+		else if (i_high == n) {
+			error = location - x[i_low];
+			i_low--;
+			n_dev++;
+		}
+		else {
+			e_low = location - x[i_low];
+			e_high = x[i_high] - location;
+			
+			if (e_low < e_high) {
+				error = e_low;
+				i_low--;
+				n_dev++;
+			}
+			else if (e_high < e_low) {
+				error = e_high;
+				i_high++;
+				n_dev++;
+			}
+			else {
+				error = e_high;
+				i_low--;
+				i_high++;
+				if ( !(n_dev)) n_dev--; /* First time count only once  */
+				n_dev += 2;
+			}
+		}
+	}
+	
+	*scale = (n%2) ? (1.4826 * error) : (0.7413 * (error + last_error));
+}
+
+void GMT_getmad_f (float *x, int n, double location, double *scale)
+{
+	/* Compute MAD (Median Absolute Deviation) for a float data set */
+	
+	double	e_low, e_high, error, last_error;
+	int	i_low, i_high, n_dev, n_dev_stop;
+	
+	i_low = 0;
+	while (i_low < n && x[i_low] <= location) i_low++;
+	i_low--;
+	
+	i_high = n - 1;
+	while (i_high >= 0 && x[i_high] >= location) i_high--;
+	i_high++;
+
+	n_dev_stop = n / 2;
+	error = 0.0;
+	n_dev = 0;
+
+
+	while (n_dev < n_dev_stop) {
+	
+		last_error = error;
+
+		if (i_low < 0) {
+			error = x[i_high] - location;
+			i_high++;
+			n_dev++;
+		}
+		
+		else if (i_high == n) {
+			error = location - x[i_low];
+			i_low--;
+			n_dev++;
+		}
+		else {
+			e_low = location - x[i_low];
+			e_high = x[i_high] - location;
+			
+			if (e_low < e_high) {
+				error = e_low;
+				i_low--;
+				n_dev++;
+			}
+			else if (e_high < e_low) {
+				error = e_high;
+				i_high++;
+				n_dev++;
+			}
+			else {
+				error = e_high;
+				i_low--;
+				i_high++;
+				if ( !(n_dev)) n_dev--; /* First time count only once  */
+				n_dev += 2;
+			}
+		}
+	}
+	
+	*scale = (n%2) ? (1.4826 * error) : (0.7413 * (error + last_error));
+}
