@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)gmtconvert.c	3.19  09/21/99
+ *	$Id: gmtconvert.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,15 +13,15 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * gmtconvert.c
  * reads a GMT table and writes it out in a different format.
   
    Author: 	P. Wessel
-   Date:	25 FEB, 1998.
-   version:	3.0
+   Date:	13-JUL-2000.
+   version:	3.4
 
 */
 
@@ -142,7 +142,7 @@ main (int argc, char **argv)
 		
 		if (!GMT_io.binary[0] && gmtdefs.io_header) {
 			for (i = 0; i < gmtdefs.n_header_recs; i++) {
-				fgets (record_str, BUFSIZ, fp);
+				GMT_fgets (record_str, BUFSIZ, fp);
 				n_total_read ++;
 				if (!GMT_io.binary[1]) fputs (record_str, GMT_stdout);
 			}
