@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)gmtdefaults.c	2.32  11/03/99
+ *	$Id: gmtdefaults.c,v 1.3 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,16 +13,15 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * gmtdefaults will list the users default settings for the GMT-SYSTEM or
  * (by using the -D option), get the GMT-SYSTEM's default settings.
  *
  * Author:	Paul Wessel
- * Date:	1-MAY-1991
- * Version:	2.0
- * Modified:	12-JUN-1998, for GMT 3.1
+ * Date:	12-JUL-2000
+ * Version:	3.4
  */
  
 #include "gmt.h"
@@ -74,10 +73,10 @@ main (int argc, char **argv)
 		fprintf (stderr, "gmtdefaults %s - List GMT-SYSTEM default parameters\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: gmtdefaults [-D[s|u] | -L]\n\n");
 		if (GMT_quick) exit (EXIT_FAILURE);
-		fprintf (stderr, "	-D prints the default settings for the GMT system\n");
-		fprintf (stderr, "	   Append s to see the SI version of defaults\n");
-		fprintf (stderr, "	   Append u to see the US version of defaults\n");
-		fprintf (stderr, "	-L prints the users current GMT default settings\n");
+		fprintf (stderr, "\t-D prints the default settings for the GMT system\n");
+		fprintf (stderr, "\t   Append s to see the SI version of defaults\n");
+		fprintf (stderr, "\t   Append u to see the US version of defaults\n");
+		fprintf (stderr, "\t-L prints the users current GMT default settings\n");
 		exit (EXIT_FAILURE);
 	}
 	
