@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)gmtselect.c	3.60  09/22/99
+ *	$Id: gmtselect.c,v 1.5 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * gmtselect is a general-purpose spatial filter.  Data pass or fail based
@@ -40,8 +40,11 @@
  *		3.3, 13-APR-1999.	Added quick check for inside a square before checking
  *					for inside a circle (suggested by Joaquim Luis)
  *					Also added option -Cf for faster, approximate distances
- *		3.3.2 8-SEPT=1999.	Sorting -C points in x to speed up search as
+ *		3.3.2 8-SEPT-1999.	Sorting -C points in x to speed up search as
  *				 	suggested by Joaquim Luis.
+ *		3.3.4 17-FEB-2000.	Polygons containing either S or N pole will work correctly
+ *		3.3.5 10-JUL-2000	Added plain -L for periodicity
+ * Version:	3.4
  */
  
 #include "gmt.h"
@@ -56,6 +59,7 @@ struct LINES {
 	double *lon, *lat;
 	double dist;
 	int np;
+	int polar;
 };
 
 struct DATA {
@@ -80,7 +84,7 @@ PFD distance_func;
 
 main (int argc, char **argv)
 {
-	int i, j, k, fno, n_read = 0, n_pass = 0, n_files = 0, x, y, n_args, n_points, n_lines, n_pols;
+	int i, j, k, fno, n_read = 0, n_pass = 0, n_files = 0, n_args, n_points, n_lines, n_pols;
 	int n_fields, n_expected_fields, out_edge_in[5], ind, bin, last_bin = -1;
 	int min_level = 0, max_level = MAX_LEVEL, np[2], base = 3, wd[2], id, this_node;
 	int side, is_inside = 1, row, col;
@@ -92,7 +96,7 @@ main (int argc, char **argv)
 	
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, xx, yy, min_area = 0.0;
 	double *xp, *yp, *dp, pt_dist = 0.0, ln_dist = 0.0, *in, west_border, east_border;
-	double xmin, xmax, ymin, ymax, lon;
+	double xmin, xmax, ymin, ymax, lon, r, sa, ca;
 	
 	char buffer[BUFSIZ], *pt_file, *ln_file, *pol_file, *ptr, res = 'l';
 	
@@ -150,16 +154,20 @@ main (int argc, char **argv)
 					base = GMT_set_resolution (&res, 'D');
 					break;
 				case 'L':
-					limit_line_distance = TRUE;
-					for (j = 2; argv[i][j] && argv[i][j] != '/'; j++);
-					if (!argv[i][j]) {
-						fprintf (stderr, "%s: GMT SYNTAX ERROR -L:  Expects -Ldist/file\n", GMT_program);
-						error++;
-					}
-					else {
-						ln_file = &argv[i][j+1];
-						ln_dist = atof (&argv[i][2]);
+					if (argv[i][2]) {	/* Set line options */
+						limit_line_distance = TRUE;
+						for (j = 2; argv[i][j] && argv[i][j] != '/'; j++);
+							if (!argv[i][j]) {
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -L:  Expects -Ldist/file\n", GMT_program);
+							error++;
+						}
+						else {
+							ln_file = &argv[i][j+1];
+							ln_dist = atof (&argv[i][2]);
+						}
 					}
+					else	/* Flag to process geographic data */
+						GMT_geographic_in = GMT_geographic_out = TRUE;
 					break;
 				case 'F':
 					limit_polygon = TRUE;
@@ -236,41 +244,42 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "gmtselect %s - Select data subsets based on multiple spatial criteria\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: gmtselect <infiles> [-A<min_area>[/<min_level>/<max_level>]] [-C[f]<dist>/<ptfile>] [-D<resolution>]\n");
-		fprintf (stderr, "	[-F<polygon>] [-J<parameters>] [-L<dist>/<lfile>] [-H[<nrec>]] [-I[cflrs]\n");
-		fprintf (stderr, "	[-M[<flag>]]  [-N<maskvalues>[o]] [-R<west/east/south/north>] [-V] [-:] [-bi[s][<n>]] [-bo[s]]\n\n");
+		fprintf (stderr, "\t[-F<polygon>] [-J<parameters>] [-L[<dist>/<lfile>]] [-H[<nrec>]] [-I[cflrs]\n");
+		fprintf (stderr, "\t[-M[<flag>]]  [-N<maskvalues>[o]] [-R<west/east/south/north>] [-V] [-:] [-bi[s][<n>]] [-bo[s]]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr, "	infiles (in ASCII or binary) have 2 or more columns with (x,y) or (y,x) in first columns.\n");
-		fprintf (stderr, "	  If no file(s) is given, standard input is read.\n");
+		fprintf (stderr, "\tinfiles (in ASCII or binary) have 2 or more columns with (x,y) or (y,x) in first columns.\n");
+		fprintf (stderr, "\t  If no file(s) is given, standard input is read.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-A coastline features smaller than <min_area> (in km^2) or of levels (0-4) outside the min-max levels\n");
 		fprintf (stderr, "\t   will be skipped [0/4] (see pscoast for details)].  Ignored unless -N is set\n");
-		fprintf (stderr, "	-C pass locations that are within <dist> of any point in ASCII <ptfile>\n");
-		fprintf (stderr, "	   Give 0 distance if 3rd column of ptfile has individual distances.\n");
-		fprintf (stderr, "	   Distances are in users units [or km if -R -J are used].\n");
-		fprintf (stderr, "	   Prepend f to use approximate (flat earth) rather than exact great circle distances.\n");
-		fprintf (stderr, "	-D Choose one of the following resolutions: (Ignored unless -N is set)\n");
-		fprintf (stderr, "	   f - full resolution (may be very slow for large regions)\n");
-		fprintf (stderr, "	   h - high resolution (may be slow for large regions)\n");
-		fprintf (stderr, "	   i - intermediate resolution\n");
-		fprintf (stderr, "	   l - low resolution [Default]\n");
-		fprintf (stderr, "	   c - crude resolution, for tasks that need crude continent outlines only\n");
-		fprintf (stderr, "	-L pass locations that are within <dist> of any line in ASCII <linefile>\n");
-		fprintf (stderr, "	   Give 0 distance if 2nd column of segment headers have individual distances.\n");
-		fprintf (stderr, "	   Distances are in users units [or km if -R -J are used].\n");
-		fprintf (stderr, "	-F pass locations that are inside the polygons in the ASCII <polygon> file\n");
+		fprintf (stderr, "\t-C pass locations that are within <dist> of any point in ASCII <ptfile>\n");
+		fprintf (stderr, "\t   Give 0 distance if 3rd column of ptfile has individual distances.\n");
+		fprintf (stderr, "\t   Distances are in users units [or km if -R -J are used].\n");
+		fprintf (stderr, "\t   Prepend f to use approximate (flat earth) rather than exact great circle distances.\n");
+		fprintf (stderr, "\t-D Choose one of the following resolutions: (Ignored unless -N is set)\n");
+		fprintf (stderr, "\t   f - full resolution (may be very slow for large regions)\n");
+		fprintf (stderr, "\t   h - high resolution (may be slow for large regions)\n");
+		fprintf (stderr, "\t   i - intermediate resolution\n");
+		fprintf (stderr, "\t   l - low resolution [Default]\n");
+		fprintf (stderr, "\t   c - crude resolution, for tasks that need crude continent outlines only\n");
+		fprintf (stderr, "\t-L with no arguments indicates that x is longitude and periodic in 360.\n");
+		fprintf (stderr, "\t   With arguments, pass locations that are within <dist> of any line in ASCII <linefile>\n");
+		fprintf (stderr, "\t   Give 0 distance if 2nd column of segment headers have individual distances.\n");
+		fprintf (stderr, "\t   Distances are in users units [or km if -R -J are used].\n");
+		fprintf (stderr, "\t-F pass locations that are inside the polygons in the ASCII <polygon> file\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "	-I Used to reverse the tests, i.e. pass locations outside the region\n");
-		fprintf (stderr, "	   Supply a combination of cflr where each flag means:\n");
-		fprintf (stderr, "	   c will pass locations beyond the minimum distance to the points in -C\n");
-		fprintf (stderr, "	   f will pass locations outside the polygons in -F\n");
-		fprintf (stderr, "	   l will pass locations beyond the minimum distance to the lines in -L\n");
-		fprintf (stderr, "	   r will pass locations outside the region given in -R [and -J]\n");
-		fprintf (stderr, "	   s will pass locations that otherwise would be skipped in -N\n");
+		fprintf (stderr, "\t-I Used to reverse the tests, i.e. pass locations outside the region\n");
+		fprintf (stderr, "\t   Supply a combination of cflr where each flag means:\n");
+		fprintf (stderr, "\t   c will pass locations beyond the minimum distance to the points in -C\n");
+		fprintf (stderr, "\t   f will pass locations outside the polygons in -F\n");
+		fprintf (stderr, "\t   l will pass locations beyond the minimum distance to the lines in -L\n");
+		fprintf (stderr, "\t   r will pass locations outside the region given in -R [and -J]\n");
+		fprintf (stderr, "\t   s will pass locations that otherwise would be skipped in -N\n");
 		GMT_explain_option ('J');
 		GMT_explain_option ('M');
-		fprintf (stderr, "\t-N set if a point outside or inside a geographic feature should be skipped or kept (requires -R).\n");
+		fprintf (stderr, "\t-N set if a point outside or inside a geographic feature should be s(kipped) or k(ept) (requires -R).\n");
 		fprintf (stderr, "\t   Append o to let feature boundary be considered outside [Default is inside].\n");
 		fprintf (stderr, "\t   Specify this information with s or k using 1 of 2 formats:\n");
 		fprintf (stderr, "\t   -N<wet>/<dry>.\n");
@@ -281,7 +290,7 @@ main (int argc, char **argv)
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "	   Default is 2 input columns\n");
+		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
@@ -291,6 +300,14 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option when -N is set\n", GMT_program);
 		error++;
 	}
+	if (GMT_geographic_in && !project_info.region_supplied) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify -R option when plain -L is set\n", GMT_program);
+		error++;
+	}
+	if (limit_coast && !MAPPING) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR:  Must specify a map projection when -N is set\n", GMT_program);
+		error++;
+	}
 	if (GMT_io.binary[0] && gmtdefs.io_header) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data cannot have header -H\n", GMT_program);
 		error++;
@@ -321,11 +338,13 @@ main (int argc, char **argv)
 			project_info.pars[0] = project_info.pars[1] = 1.0;
 		}
 		do_project = TRUE;
-		if (west < 0.0 && east < 0.0) {
-			west += 360.0;
-			east += 360.0;
+		if (MAPPING) {
+			if (west < 0.0 && east < 0.0) {
+				west += 360.0;
+				east += 360.0;
+			}
+			greenwich = (west < 0.0 && east > 0.0);
 		}
-		greenwich = (west < 0.0 && east > 0.0);
 		GMT_map_setup (west, east, south, north);
 	}
 
@@ -376,10 +395,20 @@ main (int argc, char **argv)
 		n_pols = lines_init (pol_file, &pol, -9999.0, greenwich);
 		if (do_project) {	/* Convert all the polygons */
 			for (i = 0; i < n_pols; i++) {
-				for (j = 0; j < pol[i].np; j++) {
-					GMT_geo_to_xy (pol[i].lon[j], pol[i].lat[j], &xx, &yy);
-					pol[i].lon[j] = xx;
-					pol[i].lat[j] = yy;
+				if (pol[i].polar) {	/* Polygon is a polar cap, use simple polar (r-theta) transformation */
+					for (j = 0; j < pol[i].np; j++) {
+						r = 90.0 - fabs (pol[i].lat[j]);
+						sincos (pol[i].lon[j] * D2R, &sa, &ca);
+						pol[i].lon[j] = r * ca;
+						pol[i].lat[j] = r * sa;
+					}
+				}
+				else {
+					for (j = 0; j < pol[i].np; j++) {
+						GMT_geo_to_xy (pol[i].lon[j], pol[i].lat[j], &xx, &yy);
+						pol[i].lon[j] = xx;
+						pol[i].lat[j] = yy;
+					}
 				}
 			}
 		}
@@ -387,8 +416,6 @@ main (int argc, char **argv)
 
 	/* Now we are ready to take on some input values */
 	
-	x = (gmtdefs.xy_toggle) ? 1 : 0;	y = 1 - x;		/* Set up which columns have x and y */
-	
 	if (n_files > 0)
 		nofile = FALSE;
 	else
@@ -415,12 +442,12 @@ main (int argc, char **argv)
 		
 		if (gmtdefs.io_header) {
 			for (i = 0; i < gmtdefs.n_header_recs; i++) {
-				fgets (buffer, BUFSIZ, fp);
-				if (first) printf ("%s", buffer);
+				GMT_fgets (buffer, BUFSIZ, fp);
+				if (first) fprintf (GMT_stdout, "%s", buffer);
 			}
 			first = FALSE;
 		}
-		n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+		n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 2;
 	
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && ! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
@@ -437,40 +464,48 @@ main (int argc, char **argv)
 
 			n_read++;
 
-			lon = in[x];
-			if (greenwich && lon > 180.0) lon -= 360.0;
-			if (!greenwich && lon < 0.0) lon += 360.0;
-
+			lon = in[0];
 			if (project_info.region_supplied) {
-				inside = !GMT_map_outside (lon, in[y]);
+				inside = !GMT_map_outside (lon, in[1]);
 				if (inside != want_inside[0]) continue;
 			}
 
 			if (limit_point_distance_cart) {
-				inside = near_a_point_cart (lon, in[y], xp, yp, dp, n_points); 
+				inside = near_a_point_cart (lon, in[1], xp, yp, dp, n_points); 
 				if (inside != want_inside[1]) continue;
 			}
 			else if (limit_point_distance) {
-				inside = near_a_point (lon, in[y], xp, yp, dp, n_points);
+				inside = near_a_point (lon, in[1], xp, yp, dp, n_points);
 				if (inside != want_inside[1]) continue;
 			}
 
 			if (limit_line_distance) {
-				inside = near_a_line (lon, in[y], line, n_lines);
+				inside = near_a_line (lon, in[1], line, n_lines);
 				if (inside != want_inside[2]) continue;
 			}
 
 			if (limit_polygon) {
 				if (do_project)
-					GMT_geo_to_xy (lon, in[y], &xx, &yy);
+					GMT_geo_to_xy (lon, in[1], &xx, &yy);
 				else {
 					xx = lon;
-					yy = in[y];
+					yy = in[1];
 				}
 				i = 0;
 				inside = FALSE;
 				while (i < n_pols && !inside) {
-					inside = (GMT_non_zero_winding (xx, yy, pol[i].lon, pol[i].lat, pol[i].np) >= is_inside);
+					if (pol[i].polar == -1 && in[1] <= 0.0) {	/* Point on S hemisphere, check */
+						r = 90.0 + in[1];
+						sincos (lon * D2R, &sa, &ca);
+						inside = (GMT_non_zero_winding (r * ca, r * sa, pol[i].lon, pol[i].lat, pol[i].np) >= is_inside);
+					}
+					else if (pol[i].polar == +1 && in[1] >= 0.0) {	/* Point on N hemisphere, check */
+						r = 90.0 - in[1];
+						sincos (lon * D2R, &sa, &ca);
+						inside = (GMT_non_zero_winding (r * ca, r * sa, pol[i].lon, pol[i].lat, pol[i].np) >= is_inside);
+					}
+					else
+						inside = (GMT_non_zero_winding (xx, yy, pol[i].lon, pol[i].lat, pol[i].np) >= is_inside);
 					i++;
 				}
 				if (inside != want_inside[3]) continue;
@@ -479,7 +514,7 @@ main (int argc, char **argv)
 			if (limit_coast) {
 				xx = lon;
 				while (xx < 0.0) xx += 360.0;
-				row = ((int)floor ((90.0 - in[y]) / c.bsize));
+				row = ((int)floor ((90.0 - in[1]) / c.bsize));
 				if (row >= c.bin_ny) row = c.bin_ny - 1;	/* Presumably only kicks in for south pole */
 				col = (int)floor (xx / c.bsize);
 				bin = row * c.bin_nx + col;
@@ -504,7 +539,7 @@ main (int argc, char **argv)
 				}
 				else {
 					this_node = 0;
-					GMT_geo_to_xy (lon, in[y], &xx, &yy);
+					GMT_geo_to_xy (lon, in[1], &xx, &yy);
 					for (id = 0; id < 2; id++) {
 		
 						for (k = 0; k < np[id]; k++) {
@@ -545,7 +580,7 @@ main (int argc, char **argv)
 			if (binary)
 				GMT_output (GMT_stdout, n_expected_fields, in);
 			else
-				printf ("%s", GMT_io.current_record);
+				fprintf (GMT_stdout, "%s", GMT_io.current_record);
 			n_pass++;
 			
 			if (gmtdefs.verbose && n_read%5000 == 0) fprintf (stderr, "%s: Read %d records\r", GMT_program, n_read);
@@ -567,10 +602,8 @@ main (int argc, char **argv)
 int points_init (char *file, double **xp, double **yp, double **dp, double dist, BOOLEAN greenwich)
 {
 	FILE *fp;
-	double *x, *y, *d;
-	char  buffer[BUFSIZ];
-	int i = 0, n, n_alloc = GMT_CHUNK;
-	double dd;
+	double *x, *y, *d, *in;
+	int i = 0, n_alloc = GMT_CHUNK, n_fields, n_expected_fields = BUFSIZ;
 
 	x = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	y = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
@@ -581,18 +614,30 @@ int points_init (char *file, double **xp, double **yp, double **dp, double dist,
 		exit (EXIT_FAILURE);
 	}
 
-	while (fgets (buffer, BUFSIZ, fp) != NULL) {
+	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && ! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
-		if (buffer[0] == '#' || buffer[0] == GMT_io.EOF_flag) continue;
+		while (GMT_io.status & GMT_IO_SEGMENT_HEADER && !(GMT_io.status & GMT_IO_EOF)) {
+			n_fields = GMT_input (fp, &n_expected_fields, &in);
+		}
+		if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
-		if ((n = sscanf (buffer, "%lf %lf %lf", &x[i], &y[i], &dd)) < 2) {
+		if (GMT_io.status & GMT_IO_MISMATCH) {
+			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d in file %s\n", GMT_program, n_fields,  n_expected_fields, i, file);
+			exit (EXIT_FAILURE);
+		}
+
+		if (n_expected_fields < 2) {
 			fprintf (stderr, "%s: Failure to read file %s near line %d\n", GMT_program, file, i);
 			exit (EXIT_FAILURE);
 		}
-		if (greenwich && x[i] > 180.0) x[i] -= 360.0;
-		if (!greenwich && x[i] < 0.0) x[i] += 360.0;
+		
+		x[i] = in[0];	y[i] = in[1];
+		d[i] = (n_expected_fields >= 3 && dist == 0.0) ? in[2] : dist;
+		if (MAPPING) {
+			if (greenwich  && x[i] > 180.0) x[i] -= 360.0;
+			if (!greenwich && x[i] < 0.0)   x[i] += 360.0;
+		}
 
-		d[i] = (n == 3 && dist == 0.0) ? dd : dist;
 		i++;
 		if (i == n_alloc) {
 			n_alloc += GMT_CHUNK;
@@ -602,6 +647,13 @@ int points_init (char *file, double **xp, double **yp, double **dp, double dist,
 		}
 
 	}
+	GMT_fclose (fp);
+	
+	if (i == 0) {
+		fprintf (stderr, "%s: File %s is empty!\n", GMT_program, file);
+		exit (EXIT_FAILURE);
+	}
+	
 	x = (double *) GMT_memory ((void *)x, (size_t)i, sizeof (double), GMT_program);
 	y = (double *) GMT_memory ((void *)y, (size_t)i, sizeof (double), GMT_program);
 	d = (double *) GMT_memory ((void *)d, (size_t)i, sizeof (double), GMT_program);
@@ -616,10 +668,14 @@ int lines_init (char *file, struct LINES **p, double dist, BOOLEAN greenwich)
 {
 	FILE *fp;
 	struct LINES *e;
-	char  buffer[BUFSIZ], *more;
 	int i = -1, j = 0, n, i_alloc = GMT_CHUNK, n_read = 0, j_alloc = GMT_CHUNK;
-	double d;
+	int n_fields, n_expected_fields = BUFSIZ;
+	BOOLEAN poly = FALSE, check_cap, save;
+	double d, dlon, lon_sum, *in;
 
+	if (fabs (dist + 9999.0) < GMT_CONV_LIMIT) poly = TRUE;	/* A polygon */
+	check_cap = (poly && MAPPING);
+	
 	e = (struct LINES *) GMT_memory (VNULL, (size_t)i_alloc, sizeof (struct LINES), GMT_program);
 
 	if ((fp = GMT_fopen (file, "r")) == NULL) {
@@ -627,55 +683,93 @@ int lines_init (char *file, struct LINES **p, double dist, BOOLEAN greenwich)
 		exit (EXIT_FAILURE);
 	}
 
-	more = fgets (buffer, BUFSIZ, fp);
-	if (buffer[0] != GMT_io.EOF_flag) {	/* Quick check to let user know */
+	save = GMT_io.multi_segments;	/* Must set this to TRUE temporarily */
+	GMT_io.multi_segments = TRUE;
+	
+	n_fields = GMT_input (fp, &n_expected_fields, &in);
+	if (GMT_io.status & GMT_IO_EOF) {
+		fprintf (stderr, "%s: File %s is empty!\n", GMT_program, file);
+		exit (EXIT_FAILURE);
+	}
+	if (!(GMT_io.status & GMT_IO_SEGMENT_HEADER)) {	/* Quick check up front that it is a -M kind of file */
 		fprintf (stderr, "%s: Files for -F or -L must be in multisegment format!\n", GMT_program);
 		exit (EXIT_FAILURE);
 	}
-	while (more != NULL) {
-		if (buffer[0] == GMT_io.EOF_flag) {	/* Multisegment header record */
+
+	while (n_fields >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+		while (GMT_io.status & GMT_IO_SEGMENT_HEADER && !(GMT_io.status & GMT_IO_EOF)) {
 			/* To use different line-distances for each segment, place the distance in the segment header */
 			i++;
 			n_read++;
-			n = sscanf (&buffer[1], "%lg", &d);
+			n = sscanf (&GMT_io.segment_header[1], "%lg", &d);
 			e[i].dist = (n == 1 && dist == 0.0) ? d : dist;
 			j_alloc = GMT_CHUNK;
 			j = 0;
+			lon_sum = 0.0;
+			n_fields = GMT_input (fp, &n_expected_fields, &in);
 		}
+		if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
+		
 		e[i].lon = (double *) GMT_memory (VNULL, (size_t)j_alloc, sizeof (double), GMT_program);
 		e[i].lat = (double *) GMT_memory (VNULL, (size_t)j_alloc, sizeof (double), GMT_program);
 
-		while ((more = fgets (buffer, BUFSIZ, fp)) != NULL && buffer[0] != GMT_io.EOF_flag) {
+		while (! (GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE or = 2 segment header */
+			if (GMT_io.status & GMT_IO_MISMATCH) {
+				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, i);
+				exit (EXIT_FAILURE);
+			}
+			
 			n_read++;
-			if ((n = sscanf (buffer, "%lf %lf", &e[i].lon[j], &e[i].lat[j])) < 2) {
+			if (n_expected_fields < 2) {
 				fprintf (stderr, "%s: Failure to read file %s near line %d\n", GMT_program, file, n_read);
 				exit (EXIT_FAILURE);
 			}
-			if (greenwich && e[i].lon[j] > 180.0) e[i].lon[j] -= 360.0;
-			if (!greenwich && e[i].lon[j] < 0.0) e[i].lon[j] += 360.0;
+			e[i].lon[j] = in[0];
+			e[i].lat[j] = in[1];
+			if (MAPPING) {
+				if (greenwich && e[i].lon[j] > 180.0) e[i].lon[j] -= 360.0;
+				if (!greenwich && e[i].lon[j] < 0.0) e[i].lon[j] += 360.0;
+			}
 
+			if (check_cap && j > 0) {	/* Keep track of sum (dlon) */
+				dlon = e[i].lon[j] - e[i].lon[j-1];
+				if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
+				lon_sum += dlon;
+			}
 			j++;
 			if (j == (j_alloc-1)) {
 				j_alloc += GMT_CHUNK;
 				e[i].lon = (double *) GMT_memory ((void *)e[i].lon, (size_t)j_alloc, sizeof (double), GMT_program);
 				e[i].lat = (double *) GMT_memory ((void *)e[i].lat, (size_t)j_alloc, sizeof (double), GMT_program);
 			}
+			n_fields = GMT_input (fp, &n_expected_fields, &in);
 		}
 		e[i].np = j;
 
 		/* If dist = -9999.0 then file is a polygon and we must close it if needed */
 
-		if (dist == -9999.0 && !(e[i].lon[0] == e[i].lon[j-1] && e[i].lat[0] == e[i].lat[j-1])) {
+		if (poly && !(e[i].lon[0] == e[i].lon[j-1] && e[i].lat[0] == e[i].lat[j-1])) {
 			e[i].lon[j] = e[i].lon[0];
 			e[i].lat[j] = e[i].lat[0];
 			e[i].np++;
+			if (check_cap) {	/* Keep track of sum (dlon) */
+				dlon = e[i].lon[j] - e[i].lon[j-1];
+				if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
+				lon_sum += dlon;
+			}
 		}
+		if (check_cap && fabs (fabs (lon_sum) - 360.0) < GMT_CONV_LIMIT) {	/* Contains a pole */
+			e[i].polar = (e[i].lat[0] < 0.0) ? -1 : +1;	/* S or N pole */
+		}
+		
 		if (i == (i_alloc-1)) {
 			i_alloc += GMT_CHUNK;
 			e = (struct LINES *) GMT_memory ((void *)e, (size_t)i_alloc, sizeof (struct LINES), GMT_program);
 		}
-
 	}
+	GMT_fclose (fp);
+	GMT_io.multi_segments = save;
+
 	i++;
 	e = (struct LINES *) GMT_memory ((void *)e, (size_t)i, sizeof (struct LINES), GMT_program);
 	*p = e;
@@ -707,7 +801,7 @@ int near_a_line (double lon, double lat, struct LINES *p, int np)
 		}
 		else {
 			a = (p[i].lat[j1] - p[i].lat[j0]) / dx;
-			a_inv = 1.0 / a;
+			a_inv = -1.0 / a;
 			xc = (lat - p[i].lat[j0] + a_inv * lon - a * p[i].lon[j0]) / (a_inv - a);
 			yc = lat - a_inv * (xc - lon);
 		}
