@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)gmtset.c	2.28  09/21/99
+ *	$Id: gmtset.c,v 1.3 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,15 +13,14 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * gmtset will set the specified options to the argument that follows them.
  *
  * Author:	Paul Wessel
- * Date:	1-JUN-1993
- * Version:	2.0
- * Revised:	07-SEP-1999, for GMT 3.3.2
+ * Date:	10-JUN-2000
+ * Version:	3.4
  */
  
 #include "gmt.h"
@@ -36,22 +35,22 @@ main (int argc, char **argv)
 	
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "gmtset %s - To set individual default parameters\n\n", GMT_VERSION);
-		fprintf (stderr, "usage: gmtset [-G<defaultsfile> PARAMETER1 value1 PARAMETER2 value2 PARAMETER3 value3 ...\n");
-		fprintf (stderr, "	For available PARAMETERS, see gmtdefaults man page\n");
+		fprintf (stderr, "usage: gmtset [-G<defaultsfile> PARAMETER1 [=] value1 PARAMETER2 [=] value2 PARAMETER3 [=] value3 ...\n");
+		fprintf (stderr, "\tFor available PARAMETERS, see gmtdefaults man page\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "	-G sets name of specific .gmtdefaults file to modify\n");
-		fprintf (stderr, "	   [Default looks for file in current directory,  If not fould,\n");
-		fprintf (stderr, "	   it looks in the home directory, if not found it uses GMT defaults.\n");
-		fprintf (stderr, "	   The modified defaults are written to the current directory]\n");
+		fprintf (stderr, "\t-G sets name of specific .gmtdefaults file to modify\n");
+		fprintf (stderr, "\t   [Default looks for file in current directory,  If not fould,\n");
+		fprintf (stderr, "\t   it looks in the home directory, if not found it uses GMT defaults.\n");
+		fprintf (stderr, "\t   The modified defaults are written to the current directory]\n");
 		
 		exit (EXIT_FAILURE);
 	}
 	
 	for (i = strlen(argv[0]); i >= 0 && argv[0][i] != '/'; i--);
-	GMT_program = &argv[0][i+1];
+	GMT_program = &argv[0][i+1];	/* Name without full path */
 
 	for (i = 1, j = 0; i < argc && j == 0; i++) {
 		if (!strncmp (argv[i], "-G", 2)) {
