@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system: 	@(#)grd2cpt.c	2.43  11/18/99
+ *	$Id: grd2cpt.c,v 1.3 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grd2cpt reads a 2d binary gridded grdfile and creates a continous-color-
@@ -31,6 +31,8 @@
  * Date:	12-JAN-1994
  * Revised:	PW: 12-MAY-1998, for GMT 3.1
  *		PW: 08-MAR-1998, for GMT 3.2 to allow use of master cptfiles
+ *		PW: 08-JUL-2000, for GMT 3.3.5
+ * Version:	3.4
  * 
  */
 
@@ -46,7 +48,7 @@ main (int argc, char **argv)
 
 	int i, j, nxy, nfound, ngood, ncdf;
 	
-	BOOLEAN error = FALSE, set_limits = FALSE, set_z_vals = FALSE, reverse = FALSE, continuous = FALSE;
+	BOOLEAN error = FALSE, set_limits = FALSE, set_z_vals = FALSE, reverse = FALSE, ok = FALSE, continuous = FALSE;
 	
 	double *z, min_limit, max_limit, z_start, z_stop, z_inc, mean, sd;
 	
@@ -56,7 +58,7 @@ main (int argc, char **argv)
 
 	float	*zdata;
 
-	FILE	*fpc = NULL, *fp = NULL;
+	FILE	*fpc = NULL;
 		
 	argc = GMT_begin (argc, argv);
 
@@ -64,7 +66,7 @@ main (int argc, char **argv)
 
 	sprintf (CPT_lis, "%s%cshare%cGMT_CPT.lis\0", GMTHOME, DIR_DELIM, DIR_DELIM);
 
-	if ((fpc = GMT_fopen (CPT_lis, "r")) == NULL) {
+	if ((fpc = fopen (CPT_lis, "r")) == NULL) {
 		fprintf (stderr, "%s: ERROR: Cannot open file %s\n", GMT_program, CPT_lis);
 		exit (EXIT_FAILURE);
 	}
@@ -139,27 +141,28 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"grd2cpt %s - Make a histogram-equalized color palette table from a grdfile\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grd2cpt <grdfile> [-C<table>] [-I] [-L<min_limit>/<max_limit>]\n");
-		fprintf (stderr, "	[-S<z_start>/<z_stop>/<z_inc>] [-V] [-Z]\n");
+		fprintf (stderr, "\t[-S<z_start>/<z_stop>/<z_inc>] [-V] [-Z]\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr, "	<grdfile> is the name of the 2-D binary data set\n");
+		fprintf (stderr, "\t<grdfile> is the name of the 2-D binary data set\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf(stderr,"\t-C  Specify a colortable [Default is rainbow]:\n");
 		fprintf(stderr,"\t   [Original z-range is given in brackets]\n");
 		fprintf(stderr,"\t   ---------------------------------\n");
 		while (fgets (format, BUFSIZ, fpc)) if (!(format[0] == '#' || format[0] == 0)) fprintf (stderr, "\t   %s", format);
+		fclose (fpc);
 		fprintf(stderr,"\t   ---------------------------------\n");
 		fprintf(stderr,"\t-I reverses the sense of the color table\n");
-		fprintf (stderr, "	-L Limit the range of the data [Default uses actual min,max of data].\n");
-		fprintf (stderr, "	-S Sample points should Step from z_start to z_stop by z_inc [Default guesses some values].\n");
+		fprintf (stderr, "\t-L Limit the range of the data [Default uses actual min,max of data].\n");
+		fprintf (stderr, "\t-S Sample points should Step from z_start to z_stop by z_inc [Default guesses some values].\n");
 		GMT_explain_option ('V');
 		fprintf(stderr,"\t-Z will create a continuous color palette.\n");
 		fprintf(stderr,"\t   [Default is discontinuous, i.e., constant color intervals]\n");
 		exit (EXIT_FAILURE);
 	}
 	
-	GMT_fclose (fpc);
+	fclose (fpc);
 
 	/* Open the specified master color table */
 
@@ -171,17 +174,17 @@ main (int argc, char **argv)
 		else
 			sprintf (CPT_file, "%s.cpt\0", table);
 
-		fp = GMT_fopen (CPT_file, "r");
-		if (fp && gmtdefs.verbose) fprintf (stderr, "%s: Reading %s in current directory\n", GMT_program, CPT_file);
+		ok = !access (CPT_file, R_OK);
+		if (ok && gmtdefs.verbose) fprintf (stderr, "%s: Reading %s in current directory\n", GMT_program, CPT_file);
 	}
 
-	if (fp == NULL)	{	/* No table in current dir, try /share */
+	if (!ok)	{	/* No table in current dir, try /share */
 		if (table)
 			sprintf (CPT_file, "%s%cshare%cGMT_%s.cpt\0", GMTHOME, DIR_DELIM, DIR_DELIM, table);
 		else	/* Default to rainbow colors */
 			sprintf (CPT_file, "%s%cshare%cGMT_rainbow.cpt\0", GMTHOME, DIR_DELIM, DIR_DELIM);
 
-		if ((fp = GMT_fopen (CPT_file, "r")) == NULL) {
+		if (access (CPT_file, R_OK)) {
 			fprintf (stderr, "%s: ERROR: Cannot find colortable %s\n", GMT_program, CPT_file);
 			error++;
 		}
@@ -280,6 +283,19 @@ main (int argc, char **argv)
 			of 0.1 for a Gaussian CDF:  */
 		ncdf = 11;
 		cdf_cpt = (struct CDF_CPT *)GMT_memory (VNULL, (size_t)ncdf, sizeof(struct CDF_CPT), GMT_program);
+                /* Stupid bug fix here:  
+                        If (mean-1.28155*sd <= grd.z_min || mean+1.28155*sd >= grd.z_max) then
+                          reset mean and sd so they fit inside available range:  */
+                          
+		if ((mean - 1.28155*sd) <= grd.z_min || (mean + 1.28155*sd) >= grd.z_max) {
+			mean = 0.5 * (grd.z_min + grd.z_max);
+			sd = (grd.z_max - mean) / 1.5;
+			if (sd <= 0.0) {
+				fprintf (stderr, "%s:  ERROR.  Min and Max data values are equal.\n", GMT_program);
+				exit (EXIT_FAILURE);
+			}
+		}	/* End of stupid bug fix  */
+		
 		cdf_cpt[0].z = grd.z_min;
 		cdf_cpt[1].z = mean - 1.28155 * sd;
 		cdf_cpt[2].z = mean - 0.84162 * sd;
@@ -315,15 +331,15 @@ main (int argc, char **argv)
 
 	/* Write to GMT_stdout.  */
 
-	printf("#\tcpt file created by: %s", GMT_program);
-	for (i = 1; i < argc; i++) printf (" %s", argv[i]);
-	printf("\n");
+	fprintf (GMT_stdout, "#\tcpt file created by: %s", GMT_program);
+	for (i = 1; i < argc; i++) fprintf (GMT_stdout, " %s", argv[i]);
+	fprintf (GMT_stdout, "\n");
 
 	
 	z = (double *) GMT_memory (VNULL, (size_t)ncdf, sizeof (double), GMT_program);
 	for (i = 0; i < ncdf; i++) z[i] = cdf_cpt[i].z;
 
-	GMT_sample_cpt (z, ncdf, continuous, reverse);
+	GMT_sample_cpt (z, -ncdf, continuous, reverse);	/* -ve to keep original colors */
 
 	GMT_free ((void *)cdf_cpt);
 	GMT_free ((void *)zdata);
