@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdclip.c	2.39  09/21/99
+ *	$Id: grdclip.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
    grdclip read a grdfile and sets all values < the user-supplied
@@ -24,6 +24,8 @@
    WHF Smith, 22 Aug 88
    Modified for v2.0 4/26/91 P.Wessel
    Modified for v3.1 4/29/98 P.Wessel
+   Modified for v3.3.5 06/29/2000 P.Wessel
+   Version:	3.4
  
  */
 
