@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdcontour.c	2.104  09/21/99
+ *	$Id: grdcontour.c,v 1.3 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdcontour reads a 2-D grd file and contours it. This algoritm handles
@@ -30,6 +30,9 @@
  * Version:	2.0	Based on old v1.x by Paul Wessel & Walter Smith
  * Revised:	21-JUL-1998 for GMT 3.1
  * Revised:	03-MAR-1999 for GMT 3.2
+ * Revised:	25-FEB-2000 for GMT 3.3.4
+ * Revised:	25-JUN-2000 for GMT 3.3.5
+ * Version:	3.4 Added Kaj Jancke's modification for transparent label boxes
  *
  */
 
@@ -57,10 +60,12 @@ struct SAVE {
 } *save;
 
 void dump_contour (double *xx, double *yy, int nn, double cval, int id, BOOLEAN interior, char *file);
-void draw_contour (double *xx, double *yy, int nn, double cval, char ctype, double cangle, int closed, double gap, int half_width, char *unit);
+void draw_contour (double *xx, double *yy, int nn, double cval, char ctype, double cangle, int closed, double gap, int half_width, char *unit, int box, int label_font_size);
 void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float *grd, double tick_gap, double tick_length, BOOLEAN tick_low, BOOLEAN tick_high, BOOLEAN tick_label, char *labels);
 void plot_labels (int size, int box, int rgb[]);
 
+int int_cont_count = 0, ext_cont_count = 0;
+
 main (int argc, char **argv)
 {
 	int i, j, k, n, nm, c, n_edges, label_font_size = 9, b_rgb[3], n_alloc;
@@ -72,7 +77,7 @@ main (int argc, char **argv)
 	BOOLEAN tick_high = FALSE, tick_low = FALSE, cpt_given = FALSE, tick_label = FALSE;
 	
 	char *grdfile = 0, dfile[BUFSIZ], line[BUFSIZ], *cont_type, *cont_do_tick;
-	char *cpt_file, *labels, c_unit[20], txt_a[32], txt_b[32];
+	char *cpt_file, *labels, c_unit[32], txt_a[32], txt_b[32];
 	
 	double c_int = 0.0, c_low, c_high, take_out, aval, a_int = 0.0, tick_gap = 0.2;
 	double anot_dist = 4.0, west = 0.0, east = 0.0, south = 0.0, north = 0.0, tick_length = 0.04;
@@ -150,6 +155,7 @@ main (int argc, char **argv)
 					if (argv[i][j] && GMT_getrgb (&argv[i][j+1], b_rgb)) bad++;
 					anot = TRUE;
 					if (strchr (argv[i], 'o')) box = 2;
+					if (strchr (argv[i], 't')) box = 0;
 					if (bad) {
 						fprintf (stderr, "%s: GMT SYNTAX ERROR -A option.  Correct syntax:\n", GMT_program);
 						fprintf (stderr, "\t-A[-][aint][f<fontsize>][a<fixedangle>][/<r/g/b>][o]\n");
@@ -157,7 +163,7 @@ main (int argc, char **argv)
 					}
 					break;
 				case 'C':
-					if ((fpc = GMT_fopen (&argv[i][2], "r")) == NULL)
+					if ((fpc = fopen (&argv[i][2], "r")) == NULL)
 						c_int = atof (&argv[i][2]);
 					else {
 						c_int = 1.0;
@@ -195,6 +201,7 @@ main (int argc, char **argv)
 					ncut = atoi (&argv[i][2]);
 					break;
 				case 'T':
+					do_ticks = TRUE;
 					if (argv[i][2]) {
 						if (argv[i][2] == '+')
 							tick_high = TRUE, j = 1;
@@ -202,25 +209,27 @@ main (int argc, char **argv)
 							tick_low = TRUE, j = 1;
 						else
 							tick_high = tick_low = TRUE, j = 0;
-						n = sscanf (&argv[i][2+j], "%[^/]/%s", txt_a, txt_b);
-						tick_gap = GMT_convert_units (txt_a, GMT_INCH);
-						tick_length = GMT_convert_units (txt_b, GMT_INCH);
-						if (n != 2 || tick_gap <= 0.0 || tick_length == 0.0) {
-							fprintf (stderr, "%s: GMT SYNTAX ERROR -T option.  Correct syntax:\n", GMT_program);
-							fprintf (stderr, "\t-T[+|-][<tick_gap>[c|i|m|p]/<tick_length>[c|i|m|p]][:LH], <tick_gap> must be > 0\n");
-							error = TRUE;
+						if (argv[i][2+j] != ':') {
+							n = sscanf (&argv[i][2+j], "%[^/]/%s", txt_a, txt_b);
+							if (n == 2) {
+								tick_gap = GMT_convert_units (txt_a, GMT_INCH);
+								tick_length = GMT_convert_units (txt_b, GMT_INCH);
+							}
 						}
-						do_ticks = TRUE;
 						for (j = 2; argv[i][j] && argv[i][j] != ':'; j++);
 						if (argv[i][j]) {
 							j++;
 							tick_label = TRUE;
 							labels = &argv[i][j];
 						}
-							
+						if (n == 1 || tick_gap <= 0.0 || tick_length == 0.0) {
+							fprintf (stderr, "%s: GMT SYNTAX ERROR -T option.  Correct syntax:\n", GMT_program);
+							fprintf (stderr, "\t-T[+|-][<tick_gap>[c|i|m|p]/<tick_length>[c|i|m|p]][:LH], <tick_gap> must be > 0\n");
+							error = TRUE;
+						}
 					}
 					else
-						do_ticks = tick_high = tick_low = TRUE;	/* Do default */
+						tick_high = tick_low = TRUE;	/* Do default */
 					break;
 				case 'W':
 					k = 2;
@@ -258,60 +267,65 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"grdcontour %s - Contouring of 2-D gridded data sets\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdcontour <grdfile> -C<cont_int> -J<params> [-A[-][<anot_int>][f<fontsize>][/r/g/b][a<angle>][o]]\n");
-		fprintf (stderr, "	[-B<tickinfo>] [-D<dumpfile>] [-Eaz/el] [-G<gap>[c|i|m|p]/<npoints>] [-K] [-L<Low/high>]\n");
-		fprintf (stderr, "	[-M[<flag>]] [-N[unit]] [-O] [-P] [-Q<cut>] [-R<west/east/south/north>] [-S<smooth>] [-T[+|-][<gap>[c|i|m|p]/<length>[c|i|m|p]][:LH]] [-U[<label>]] [-V]\n");
-		fprintf (stderr, "	[-W[+]<type><pen>] [-X<x_shift>] [-Y<y_shift>] [-Z[<fact>[/shift>]][p]] [-bo[s]] [-c<ncopies>]\n\n");
+		fprintf (stderr, "\t[-B<tickinfo>] [-D<dumpfile>] [-Eaz/el] [-G<gap>[c|i|m|p]/<npoints>] [-K] [-L<Low/high>]\n");
+		fprintf (stderr, "\t[-M[<flag>]] [-N[unit]] [-O] [-P] [-Q<cut>] [-R<west/east/south/north>] [-S<smooth>] [-T[+|-][<gap>[c|i|m|p]/<length>[c|i|m|p]][:LH]] [-U[<label>]] [-V]\n");
+		fprintf (stderr, "\t[-W[+]<type><pen>] [-X<x_shift>] [-Y<y_shift>] [-Z[<fact>[/shift>]][p]] [-bo[s]] [-c<ncopies>]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr, "	<grdfile> is 2-D netCDF grdfile to be contoured\n");
-		fprintf (stderr, "	-C Contours to be drawn can be specified in one of three ways:\n");
-		fprintf (stderr, "	   -Fixed contour interval\n");
-		fprintf (stderr, "	   -Name of file with contour levels in col 1 and C(ont) or A(not) in col 2\n");
-		fprintf (stderr, "	      [and optionally an individual annotation angle in col 3.]\n");
-		fprintf (stderr, "	   -Name of cpt-file\n");
-		fprintf (stderr, "	   If -T is used, only contours with upper case C or A is ticked\n");
-		fprintf (stderr, "	     [cpt-file contours are set to C unless last column has flags; Use -A to force all to A]\n");
+		fprintf (stderr, "\t<grdfile> is 2-D netCDF grdfile to be contoured\n");
+		fprintf (stderr, "\t-C Contours to be drawn can be specified in one of three ways:\n");
+		fprintf (stderr, "\t   -Fixed contour interval\n");
+		fprintf (stderr, "\t   -Name of file with contour levels in col 1 and C(ont) or A(not) in col 2\n");
+		fprintf (stderr, "\t      [and optionally an individual annotation angle in col 3.]\n");
+		fprintf (stderr, "\t   -Name of cpt-file\n");
+		fprintf (stderr, "\t   If -T is used, only contours with upper case C or A is ticked\n");
+		fprintf (stderr, "\t     [cpt-file contours are set to C unless last column has flags; Use -A to force all to A]\n");
 		GMT_explain_option ('j');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "	-A Annotation information. [Default is no annotation].\n");
-		fprintf (stderr, "	   Append - to disable all contour anotations\n");
-		fprintf (stderr, "	   Append f followed by desired font size in points [Default is 9].\n");
-		fprintf (stderr, "	   Append /r/g/b to change color of text box [Default is %d/%d/%d]\n", b_rgb[0], b_rgb[1], b_rgb[2]);
-		fprintf (stderr, "	   Append o to draw outline of text box [Default is no outline]\n");
-		fprintf (stderr, "	   Append a<angle> to force anotations at this fixed angle [Default follows contour]\n");
+		fprintf (stderr, "\t-A Annotation information. [Default is no annotation].\n");
+		fprintf (stderr, "\t   Append - to disable all contour anotations\n");
+		fprintf (stderr, "\t   Append f followed by desired font size in points [Default is 9].\n");
+		fprintf (stderr, "\t   Append /r/g/b to change color of text box [Default is %d/%d/%d]\n", b_rgb[0], b_rgb[1], b_rgb[2]);
+		fprintf (stderr, "\t   Append o to draw outline of text box [Default is no outline]\n");
+		fprintf (stderr, "\t   Append t for transparent text box [Default is opaque]\n");
+		fprintf (stderr, "\t   Append a<angle> to force anotations at this fixed angle [Default follows contour]\n");
 		GMT_explain_option ('b');
-		fprintf (stderr, "	-D to Dump contour lines to individual files (but see -M)\n");
-		fprintf (stderr, "	-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
-		fprintf (stderr, "	-G Gap between anotations in %s and number of (x,y) values used to find L2 label angle [Default = %lg/10]\n", GMT_unit_names[gmtdefs.measure_unit], anot_dist);
+		fprintf (stderr, "\t-D to Dump contour lines to individual files (but see -M)\n");
+		fprintf (stderr, "\t   Append file prefix [contour].  Files will be called <dumpfile>_<cont>_#[_i].xyz|b\n");
+		fprintf (stderr, "\t   where <cont> is the countour value and # is a segment counter.\n");
+		fprintf (stderr, "\t   _i is inserted for interior (closed) contours, with xyz (ascii) or b (binary) as extension.\n");
+		fprintf (stderr, "\t   However, if -D- is given then files are C#_e or C#_i plus extension, where # is a running number.\n");
+		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t-G Gap between anotations in %s and number of (x,y) values used to find L2 label angle [Default = %lg/10]\n", GMT_unit_names[gmtdefs.measure_unit], anot_dist);
 		GMT_explain_option ('K');
-		fprintf (stderr, "	-L only contour inside this range\n");
-		fprintf (stderr, "      -M Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
-		fprintf (stderr, "         whose first character is <flag> ['>'].  This header also has the contour level value\n");
-		fprintf (stderr, "	-N add z-unit from grdfile to contour labels; append unit to override name.\n");
-		fprintf (stderr, "	   Start unit with - for no space between anotation and unit\n");
+		fprintf (stderr, "\t-L only contour inside this range\n");
+		fprintf (stderr, "\t-M Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
+		fprintf (stderr, "\t   whose first character is <flag> ['>'].  This header also has the contour level value\n");
+		fprintf (stderr, "\t-N add z-unit from grdfile to contour labels; append unit to override name.\n");
+		fprintf (stderr, "\t   Start unit with - for no space between anotation and unit\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr, "	-Q Do not draw contours with less than <cut> points [Draw all contours]\n");
+		fprintf (stderr, "\t-Q Do not draw contours with less than <cut> points [Draw all contours]\n");
 		GMT_explain_option ('R');
-		fprintf (stderr, "	   [Default is extent of grdfile]\n");
-		fprintf (stderr, "	-S will Smooth contours by splining and resampling\n");
-		fprintf (stderr, "	   at approximately gridsize/<smooth> intervals\n");
-		fprintf (stderr, "	-T will embellish innermost, closed contours with ticks pointing in the downward direction\n");
-		fprintf (stderr, "	   User may specify to tick only highs (-T+) or lows (-T-) [-T means both]\n");
-		fprintf (stderr, "	   Append spacing/ticklength (in %s) to change defaults [%lg/%lg]\n", GMT_unit_names[gmtdefs.measure_unit], tick_gap, tick_length);
-		fprintf (stderr, "	   Append :LH to plot the characters L and H in the center of closed contours\n");
-		fprintf (stderr, "	   for local Lows and Highs (e.g, give :-+ to plot - and + signs)\n");
+		fprintf (stderr, "\t   [Default is extent of grdfile]\n");
+		fprintf (stderr, "\t-S will Smooth contours by splining and resampling\n");
+		fprintf (stderr, "\t   at approximately gridsize/<smooth> intervals\n");
+		fprintf (stderr, "\t-T will embellish innermost, closed contours with ticks pointing in the downward direction\n");
+		fprintf (stderr, "\t   User may specify to tick only highs (-T+) or lows (-T-) [-T means both]\n");
+		fprintf (stderr, "\t   Append spacing/ticklength (in %s) to change defaults [%lg/%lg]\n", GMT_unit_names[gmtdefs.measure_unit], tick_gap, tick_length);
+		fprintf (stderr, "\t   Append :LH to plot the characters L and H in the center of closed contours\n");
+		fprintf (stderr, "\t   for local Lows and Highs (e.g, give :-+ to plot - and + signs)\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
-		fprintf (stderr, "	-W sets pen attributes. <type> can be 'a' for anotated contours and\n");
-		fprintf (stderr, "	   'c' for regular contours.  The default settings are\n");
-		fprintf (stderr, "	   Contour pen:  width = %lgp, color = (%d/%d/%d), texture = solid\n", pen[0].width, pen[0].rgb[0], pen[0].rgb[1], pen[0].rgb[2]);
-		fprintf (stderr, "	   Annotate pen: width = %lgp, color = (%d/%d/%d), texture = solid\n", pen[1].width, pen[1].rgb[0], pen[1].rgb[1], pen[1].rgb[2]);
-		fprintf (stderr, "	   Use + to draw colored contours based on the cpt file\n");
+		fprintf (stderr, "\t-W sets pen attributes. <type> can be 'a' for anotated contours and\n");
+		fprintf (stderr, "\t   'c' for regular contours.  The default settings are\n");
+		fprintf (stderr, "\t   Contour pen:  width = %lgp, color = (%d/%d/%d), texture = solid\n", pen[0].width, pen[0].rgb[0], pen[0].rgb[1], pen[0].rgb[2]);
+		fprintf (stderr, "\t   Annotate pen: width = %lgp, color = (%d/%d/%d), texture = solid\n", pen[1].width, pen[1].rgb[0], pen[1].rgb[1], pen[1].rgb[2]);
+		fprintf (stderr, "\t   Use + to draw colored contours based on the cpt file\n");
 		GMT_explain_option ('X');
-		fprintf (stderr, "	-Z to subract <shift> and multiply data by <fact> before contouring [1/0].\n");
-		fprintf (stderr, "	   Append p for z-data that is periodic in 360 (i.e., phase data)\n");
+		fprintf (stderr, "\t-Z to subract <shift> and multiply data by <fact> before contouring [1/0].\n");
+		fprintf (stderr, "\t   Append p for z-data that is periodic in 360 (i.e., phase data)\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('c');
 		exit (EXIT_FAILURE);
@@ -330,6 +344,10 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -S option:  Smooth_factor must be > 0\n", GMT_program);
 		error++;
 	}
+	if (ncut < 0) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -Q option:  Value must be >= 0\n", GMT_program);
+		error++;
+	}
 	if (z_project.view_azimuth > 360.0 || z_project.view_elevation <= 0.0 || z_project.view_elevation > 90.0) {
 		fprintf (stderr, "%s: GMT SYNTAX ERROR -E option:  Enter azimuth in 0-360 range, elevation in 0-90 range\n", GMT_program);
 		error++;
@@ -447,26 +465,29 @@ main (int argc, char **argv)
 			cont_do_tick = (char *) GMT_memory ((void *)cont_do_tick, (size_t)n_contours, sizeof (char), GMT_program);
 			cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_contours, sizeof (double), GMT_program);
 		}
-		for (i = 0; i < GMT_n_colors; i++) {
-			contour[i] = GMT_lut[i].z_low;
+		for (i = c = 0; i < GMT_n_colors; i++) {
+			if (GMT_lut[i].skip) continue;
+			contour[c] = GMT_lut[i].z_low;
 			if (!use_cpt_anot)
-				cont_type[i] = 'C';
+				cont_type[c] = 'C';
 			else if (GMT_lut[i].anot)
-				cont_type[i] = 'A';
+				cont_type[c] = 'A';
 			else
-				cont_type[i] = (anot) ? 'A' : 'C';
-			cont_angle[i] = (fix_angle) ? label_angle : GMT_d_NaN;
-			cont_do_tick[i] = do_ticks;
+				cont_type[c] = (anot) ? 'A' : 'C';
+			cont_angle[c] = (fix_angle) ? label_angle : GMT_d_NaN;
+			cont_do_tick[c] = do_ticks;
+			c++;
 		}
-		contour[GMT_n_colors] = GMT_lut[GMT_n_colors-1].z_high;
+		contour[c] = GMT_lut[GMT_n_colors-1].z_high;
 		if (!use_cpt_anot)
-			cont_type[GMT_n_colors] = 'C';
+			cont_type[c] = 'C';
 		else if (GMT_lut[GMT_n_colors-1].anot & 2)
-			cont_type[GMT_n_colors] = 'A';
+			cont_type[c] = 'A';
 		else
-			cont_type[GMT_n_colors] = (anot) ? 'A' : 'C';
-		cont_angle[GMT_n_colors] = (fix_angle) ? label_angle : GMT_d_NaN;
-		cont_do_tick[GMT_n_colors] = do_ticks;
+			cont_type[c] = (anot) ? 'A' : 'C';
+		cont_angle[c] = (fix_angle) ? label_angle : GMT_d_NaN;
+		cont_do_tick[c] = do_ticks;
+		n_contours = c + 1;
 	}
 	else if (fpc != NULL) {	/* read contour info from file */
 		n_contours = 0;
@@ -485,7 +506,7 @@ main (int argc, char **argv)
 				cont_angle = (double *) GMT_memory ((void *)cont_angle, (size_t)n_alloc, sizeof (double), GMT_program);
 			}
 		}
-		GMT_fclose (fpc);
+		fclose (fpc);
 	}
 	else {	/* Set up contour intervals automatically from c_int and a_int */
 		min = floor (header.z_min / c_int) * c_int; if (!GMT_z_periodic && min < header.z_min) min += c_int;	
@@ -549,8 +570,8 @@ main (int argc, char **argv)
 				}
 				else
 					interior = FALSE;
-				if (dump) dump_contour (x, y, n, cval, section, interior, dfile);
-				if (n >= ncut) draw_contour (x, y, n, cval, cont_type[c], cont_angle[c], interior, anot_dist, half, c_unit);
+				if (dump && n >= ncut) dump_contour (x, y, n, cval, section, interior, dfile);
+				if (n >= ncut) draw_contour (x, y, n, cval, cont_type[c], cont_angle[c], interior, anot_dist, half, c_unit, box, label_font_size);
 				if (cont_do_tick[c] && interior && n >= ncut) {
 					save[n_save].x = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
 					save[n_save].y = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
@@ -639,28 +660,37 @@ void dump_contour (double *xx, double *yy, int nn, double cval, int id, BOOLEAN
 		}
 		else	/* Later we append to it */
 			fp = GMT_fopen (file, "a+");
-		fprintf (fp, "%c %lg contour\n", GMT_io.EOF_flag, cval);
+		sprintf (GMT_io.segment_header, "%c %lg contour\n\0", GMT_io.EOF_flag, cval);
+		GMT_write_segmentheader (fp, 3);
 	}
 	else {
-		if (interior)
-			sprintf (fname, "%s_%lg_%d_i.%s\0", file, cval, id, suffix);
-		else
-			sprintf (fname, "%s_%lg_%d.%s\0", file, cval, id, suffix);
+		if (interior) {
+			if (file[0] == '-')	/* Running numbers only */
+				sprintf (fname, "C%d_i.%s\0", int_cont_count++, suffix);
+			else
+				sprintf (fname, "%s_%lg_%d_i.%s\0", file, cval, id, suffix);
+		}
+		else {
+			if (file[0] == '-')	/* Running numbers only */
+				sprintf (fname, "C%d_e.%s\0", ext_cont_count++, suffix);
+			else
+				sprintf (fname, "%s_%lg_%d.%s\0", file, cval, id, suffix);
+		}
 		fp = GMT_fopen (fname, GMT_io.w_mode);
 	}
 	for (i = 0; i < nn; i++) {
-/*		GMT_xy_to_geo (&out[0], &out[1], xx[i], yy[i]); */
 		out[0] = xx[i];	out[1] = yy[i];
 		GMT_output (fp, 3, out);
 	}
 	GMT_fclose (fp);
 }
 
-void draw_contour (double *xx, double *yy, int nn, double cval, char ctype, double cangle, int closed, double gap, int half_width, char *unit)
+void draw_contour (double *xx, double *yy, int nn, double cval, char ctype, double cangle, int closed, double gap, int half_width, char *unit, int box, int label_font_size)
 {
-	int i, j;
+	int i, j, start = 0;
 	double dist, angle, dx, dy, width, sum_x2, sum_xy, one = 1.0;
-	char label[100], format[100];
+	double label_height, label_width, x, y;
+	char label[128], format[128];
 	struct LABEL *new_label;
 	
 	if (nn < 2) return;
@@ -669,11 +699,14 @@ void draw_contour (double *xx, double *yy, int nn, double cval, char ctype, doub
 	ps_comment (format);
 	
 	GMT_n_plot = GMT_geo_to_xy_line (xx, yy, nn);
-	GMT_plot_line (GMT_x_plot, GMT_y_plot, GMT_pen, GMT_n_plot);
+	if (box > 0) GMT_plot_line (GMT_x_plot, GMT_y_plot, GMT_pen, GMT_n_plot);	/* Draw entire contour for opaque label boxes */
 	
 	if (ctype == 'A' || ctype == 'a') {	/* Annotated contours */
 		GMT_get_format (cval, unit, format);
 		sprintf (label, format, cval);
+		label_height = label_width = label_font_size * GMT_u2u[GMT_PT][GMT_INCH] * 0.58;  /* Constants from GMT_textbox3d()KJ*/
+		label_width *= 0.81 * strlen(label) * 0.78;
+		label_width *= 0.66;  /* was still too wide KJ*/
 		dist = (closed) ? gap - one : 0.0;	/* Label closed contours longer than 1 inch */
 		for (i = 1; i < GMT_n_plot; i++) {
 			if (GMT_pen[i] == 3) continue;
@@ -714,15 +747,33 @@ void draw_contour (double *xx, double *yy, int nn, double cval, char ctype, doub
 				dist = 0.0;
 				old_label->next_label = new_label;
 				old_label = new_label;
+
+				if (box == 0) {	/* Transpartent label box */
+					/* new segments of the contour line from label to label KJ*/
+					j = 1;
+					x = GMT_x_plot[i];
+					y = GMT_y_plot[i];
+	   				while ((GMT_x_plot[i-j]> (x - label_width)) && (GMT_x_plot[i-j] < (x + label_width)) &&
+	                        		(GMT_y_plot[i-j] > (y - label_height)) && (GMT_y_plot[i-j] < (y+label_height))) j++;
+					GMT_plot_line (&GMT_x_plot[start], &GMT_y_plot[start], &GMT_pen[start], i-j-start); 
+					start = i + 1;
+	   				while ( (GMT_x_plot[start] > (x - label_width)) && (GMT_x_plot[start] < (x + label_width)) &&
+	                      	 	 (GMT_y_plot[start] > (y - label_height)) && (GMT_y_plot[start] < (y + label_height))) start++;
+				}
 			}
 			GMT_x_plot[i-1] = GMT_x_plot[i];	GMT_y_plot[i-1] = GMT_y_plot[i];
 		}
+		if (box == 0) GMT_plot_line (&GMT_x_plot[start], &GMT_y_plot[start], &GMT_pen[start], GMT_n_plot-start); /*KJ the last segment, supposed not to end with a label*/
+	}
+	else if (box == 0) {   /* just one line , no holes for labels */
+		GMT_plot_line (GMT_x_plot, GMT_y_plot, GMT_pen, GMT_n_plot);
 	}
 }
 
 void plot_labels (int size, int box, int rgb[])
 {
 	/* box = 1: white box only, box = 2: white box + draw outline */
+        /* box = 0: no box (append 't' instead of 'o' in -A parameter) */
 	double dx, dy;
 	struct LABEL *this, *old;
 	
@@ -733,7 +784,7 @@ void plot_labels (int size, int box, int rgb[])
 	
 	ps_setpaint (gmtdefs.basemap_frame_rgb);
 	
-	for (old = anchor; old->next_label; old = old->next_label) {	/* First draw boxes if not 3-D*/
+	for (old = anchor; box >= 0 && old->next_label; old = old->next_label) {	/* First draw boxes if not 3-D*/
 		this = old->next_label;
 		GMT_textbox3d (this->x, this->y, project_info.z_level, size, gmtdefs.anot_font, this->label, this->angle, 6, box, dx, dy, rgb);
 	}
@@ -765,7 +816,7 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 		txt[1][0] = labels[1];	txt[1][1] = 0;
 	}
 	
-	for (i = 0; i < n; i++) {
+	for (i = 0; i < n; i++) {	/* Mark polygons that have other polygons inside them */
 		np = save[i].n;
 		for (j = 0; save[i].do_it && j < n; j++) {
 			inside = GMT_non_zero_winding (save[j].x[0], save[j].y[0], save[i].x, save[i].y, np);
@@ -773,11 +824,15 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 		}
 	}
 	
+	/* Here, only the polygons that are innermost (containing the local max/min, will have do_it = TRUE */
+	
 	for (i = 0; i < n; i++) {
 		if (!save[i].do_it) continue;
 		np = save[i].n;
 		s = (double *) GMT_memory (VNULL, (size_t)np, sizeof (double), GMT_program);
 		
+		/* Get rectangular surrounding box for this polygon */
+		
 		xmin = xmax = save[i].x[0];
 		ymin = ymax = save[i].y[0];
 		for (j = 1; j < np; j++) {
@@ -787,6 +842,8 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 			ymax = MAX (ymax, save[i].y[j]);
 		}
 		
+		/* Find a point inside the polygon to test for high or low */
+		
 		ix = irint ((0.5 * (xmin + xmax) - h->x_min) / h->x_inc) + 1;
 		iy = irint ((h->y_max - ymax) / h->y_inc) + 1;
 		x0 = h->x_min + ix * h->x_inc;
@@ -799,6 +856,7 @@ void sort_and_plot_ticks (struct SAVE *save, int n, struct GRD_HEADER *h, float
 			else
 				iy++;
 		}
+		if (iy == h->ny) iy--;	/* Check to make sure we dont exceed array bounds in next line. However iy may thus be wrong */ 
 		save[i].high = (grd[iy*h->nx+ix] > save[i].cval);
 		
 		if (save[i].high && !tick_high) continue;	/* Dont tick highs */
