@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdedit.c	2.36  09/21/99
+ *	$Id: grdedit.c,v 1.3 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdedit reads an existing grdfile and takes command line
@@ -27,7 +27,8 @@
  *
  * Author:	Paul Wessel
  * Date:	9-SEP-1998
- * Version:	3.1
+ * Version:	3.4
+ * Modified:	19-FEB-2000 PW: Added -V option
  */
  
 #include "gmt.h"
@@ -38,11 +39,11 @@ main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, got_input = FALSE, got_region = FALSE, shift = FALSE, adjust = FALSE;
 	
-	int i, entry, one_or_zero;
+	int i, one_or_zero;
 
 	double w, e, s, n, shift_amount = 0.0;
 	
-	char *grdfile, input[BUFSIZ], *ptr;
+	char *grdfile, input[BUFSIZ];
 	
 	struct GRD_HEADER grd;
 	
@@ -58,6 +59,7 @@ main (int argc, char **argv)
 				/* Common parameters */
 			
 				case 'R':
+				case 'V':
 				case '\0':
 					error += GMT_get_common_args (argv[i], &w, &e, &s, &n);
 					if (argv[i][1] == 'R') got_region = TRUE;
@@ -88,17 +90,18 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "grdedit %s - Modifying the header in a netCDF grdfile\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdedit grdfile [-A] [-R<west>/<east>/<south>/<north>]\n");
-		fprintf (stderr, "	[-S] [-Dxunit/yunit/zunit/scale/offset/title/remark]\n");
+		fprintf (stderr, "\t[-S] [-Dxunit/yunit/zunit/scale/offset/title/remark] [-V]\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr, "	grdfile is file to be modified\n");
+		fprintf (stderr, "\tgrdfile is file to be modified\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "	-A will adjust dx/dy to be compatible with the files domain or -R\n");
-		fprintf (stderr, "	-D to enter information.  Specify '=' to get default value\n");
+		fprintf (stderr, "\t-A will adjust dx/dy to be compatible with the files domain or -R\n");
+		fprintf (stderr, "\t-D to enter information.  Specify '=' to get default value\n");
 		GMT_explain_option ('R');
-		fprintf (stderr, "	-S For global grids of 360 degree longitude range.\n");
-		fprintf (stderr, "	   Will rotate entire grid to coincide with new borders in -R\n");
+		fprintf (stderr, "\t-S For global grids of 360 degree longitude range.\n");
+		fprintf (stderr, "\t   Will rotate entire grid to coincide with new borders in -R\n");
+		GMT_explain_option ('V');
 		exit (EXIT_FAILURE);
 	}
 	
@@ -138,45 +141,21 @@ main (int argc, char **argv)
 		exit (EXIT_FAILURE);
 	}
 	
+	if (gmtdefs.verbose) fprintf (stderr, "%s: Editing parameters for file %s\n", GMT_program, grdfile);
+	
 	/* Decode grd information given, if any */
 	
 	if (got_input) {
-		ptr = strtok (input, "/");
-		entry = 0;
-		while (ptr) {
-			switch (entry) {
-				case 0:
-					if (ptr[0] != '=') strcpy (grd.x_units, ptr);
-					break;
-				case 1:
-					if (ptr[0] != '=') strcpy (grd.y_units, ptr);
-					break;
-				case 2:
-					if (ptr[0] != '=') strcpy (grd.z_units, ptr);
-					break;
-				case 3:
-					if (ptr[0] != '=') grd.z_scale_factor = atof (ptr);
-					break;
-				case 4:
-					if (ptr[0] != '=') grd.z_add_offset = atof (ptr);
-					break;
-				case 5:
-					if (ptr[0] != '=') strcpy (grd.title, ptr);
-					break;
-				case 6:
-					if (ptr[0] != '=') strcpy (grd.remark, ptr);
-					break;
-				default:
-					break;
-			}
-			ptr = strtok (CNULL, "/");
-			entry++;
-		}
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Decode and change attributes in file %s\n", GMT_program, grdfile);
+		GMT_decode_grd_h_info (input, &grd);
+
 	}
 	
 	if (!shift) {
 		one_or_zero = (grd.node_offset) ? 0 : 1;
 		if (got_region) {
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Reset region in file %s to %lg/%lg/%lg/%lg\n",
+				GMT_program, grdfile, w, e, s, n);
 			grd.x_min = w;	grd.x_max = e;
 			grd.y_min = s;	grd.y_max = n;
 			adjust = TRUE;	/* Must ensure -R -I compatibility */
@@ -184,6 +163,8 @@ main (int argc, char **argv)
 		if (adjust) {
 			grd.x_inc = (grd.x_max - grd.x_min) / (grd.nx - one_or_zero);
 			grd.y_inc = (grd.y_max - grd.y_min) / (grd.ny - one_or_zero);
+			if (gmtdefs.verbose) fprintf (stderr, "%s: Reset grid-spacing in file %s to %lg/%lg\n",
+				GMT_program, grdfile, grd.x_inc, grd.y_inc);
 		}
 		if (GMT_write_grd_info (grdfile, &grd)) {
 			fprintf (stderr, "%s: Error updating file %s\n", GMT_program, grdfile);
@@ -192,6 +173,7 @@ main (int argc, char **argv)
 	}
 	else {
 		shift_amount = w - grd.x_min;
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Shifting longitudes in file %s by %lg degrees\n", GMT_program, grdfile, shift_amount);
 		a = (float *) GMT_memory (VNULL, (size_t)(grd.nx * grd.ny), sizeof (float), GMT_program);
 		if (GMT_read_grd (grdfile, &grd, a, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) {
 			fprintf (stderr, "%s: Error reading file %s\n", GMT_program, grdfile);
@@ -204,6 +186,7 @@ main (int argc, char **argv)
 		}
 		GMT_free ((void *)a);
 	}
+	if (gmtdefs.verbose) fprintf (stderr, "%s: File %s updated.\n", GMT_program, grdfile);
 	
 	GMT_end (argc, argv);
 }
