@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdfft.c	2.52  09/21/99
+ *	$Id: grdfft.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  *  grdfft.c
@@ -30,6 +30,7 @@
  *   Note to myself:  users want azimuthal power spectra, and
  *   cross-spectra between two grids.  --whfs
  *
+ * Version:	3.4
  */
 
 #include "gmt.h"
@@ -297,6 +298,7 @@ main (int argc, char **argv)
 		fprintf (stderr,"\t-T Compute isostatic response.  Input file is topo load. Append elastic thickness,\n");
 		fprintf (stderr,"\t   and densities of load, mantle, water, and infill, all in SI units\n");
 		fprintf (stderr,"\t   It also implicitly sets -L\n");
+		GMT_explain_option ('V');
 		fprintf (stderr,"\tList operations in the order desired for execution.\n");
 		exit (EXIT_FAILURE);
 	}
@@ -810,7 +812,7 @@ int	parse_f_string(char *c)
 {
 	int	i, j, n_tokens, descending;
 	double	fourvals[4];
-	char	line[80], *p;
+	char	line[256], *p;
 	
 	strcpy(line, c);
 	i = j = 0;	/* j is Filter type:  r=0, x=1, y=2  */
@@ -956,7 +958,7 @@ void	do_spectrum(double *par, int give_wavelength)
 		freq = (k + 1) * delta_k;
 		if (give_wavelength) freq = 1.0/freq;
 		power[k] *= powfactor;
-		printf(format, freq, power[k], eps_pow * power[k]);
+		fprintf (GMT_stdout, format, freq, power[k], eps_pow * power[k]);
 	}
 }
 
