@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdfilter.c	2.54  10/18/99
+ *	$Id: grdfilter.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdfilter.c  reads a grdfile and creates filtered grd file
@@ -34,7 +34,8 @@
  *		PW: 03-Jun-1998 upgrade to GMT 3.1
  *		PW: 02-Jun-1999 upgrade to GMT 3.3 + added SINCOS option
  *		PW: 18-Oct-1999 Use sincos directly
- *
+ *		PW: 18-JUN-2000 3.3.5
+ * Version:	3.4
  */
  
 #include "gmt.h"
@@ -63,7 +64,7 @@ main (int argc, char **argv)
 	int	i_in, j_in, ii, jj, i, ij_in, ij_out, ij_wt, effort_level;
 	int	distance_flag, filter_type, one_or_zero = 1;
 	
-	BOOLEAN	error, new_range, new_increment, fast_way, shift, slow;
+	BOOLEAN	error, new_range, new_increment, fast_way, shift, slow, toggle = FALSE;
 	
 	double	west_new, east_new, south_new, north_new, dx_new, dy_new, offset;
 	double	filter_width, x_scale, y_scale, x_width, y_width;
@@ -126,9 +127,12 @@ main (int argc, char **argv)
 					GMT_getinc (&argv[i][2], &dx_new, &dy_new);
 					new_increment = TRUE;
 					break;
-				case 'N':	/* Pixel registration */
+				case 'N':	/* Pixel registration OBSOLTE but BACKWARD COMPATIBLE */
 					one_or_zero = 0;
 					break;
+				case 'T':	/* Toggle registration */
+					toggle = TRUE;
+					break;
 				default:
 					error = TRUE;
 					GMT_default_error (argv[i][1]);
@@ -142,7 +146,7 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "grdfilter %s - Filter a 2-D netCDF grdfile in the Time domain\n\n", GMT_VERSION);
 		fprintf(stderr,"usage: grdfilter input_file -D<distance_flag> -F<type><filter_width>\n");
-		fprintf(stderr,"\t-G<output_file> [-I<xinc>[m|c][/<yinc>[m|c]] ] [-N] [-R<west/east/south/north>] [-V]\n");
+		fprintf(stderr,"\t-G<output_file> [-I<xinc>[m|c][/<yinc>[m|c]] ] [-R<west/east/south/north>] [-T] [-V]\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
@@ -162,7 +166,7 @@ main (int argc, char **argv)
 		fprintf(stderr,"\t-I for new Increment of output grid; enter xinc, optionally xinc/yinc.\n");
 		fprintf(stderr,"\t   Default is yinc = xinc.  Append an m [or c] to xinc or yinc to indicate minutes [or seconds];\n");
 		fprintf(stderr,"\t   The new xinc and yinc should be divisible by the old ones (new lattice is subset of old).\n");
-		fprintf(stderr, "\t-N Force pixel registration for output grid [Default is gridline registration]\n");
+		fprintf(stderr, "\t-T Toggles between grid and pixel registration for output grid [Default is same as input registration]\n");
 		fprintf(stderr, "\t-R for new Range of output grid; enter <WESN> (xmin, xmax, ymin, ymax) separated by slashes.\n");
 		GMT_explain_option ('V');
 		exit (EXIT_FAILURE);
@@ -185,10 +189,14 @@ main (int argc, char **argv)
 		fprintf (stderr, "\t-FX<width>, with X one of bcgmp, width is filter fullwidth\n");
 		error++;
 	}
+	if (toggle && one_or_zero == 0) {	/* Both -N and -T set, not good */
+		fprintf (stderr, "%s: GMT SYNTAX ERROR -T option:  Not allowed with obsolete -N option\n", GMT_program);
+		error++;
+	}
 	if (error) exit (EXIT_FAILURE);
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands */
-	
+
 	if (project_info.region_supplied) new_range = TRUE;
 
 	if (GMT_read_grd_info (fin, &h)) {
@@ -197,6 +205,10 @@ main (int argc, char **argv)
 	}
 	GMT_grd_init (&h, argc, argv, TRUE);	/* Update command history only */
 
+	if (toggle) {	/* Make output grid of the opposite registration */
+		one_or_zero = (h.node_offset) ? 1 : 0;
+	}
+	
 	/* Read the input grd file and close it  */
 	
 	input = (float *) GMT_memory (VNULL, (size_t)(h.nx * h.ny), sizeof(float), GMT_program);
@@ -237,7 +249,7 @@ main (int argc, char **argv)
 
 	test_h.x_min = west_new;	test_h.x_max = east_new;	test_h.x_inc = dx_new;
 	test_h.y_min = south_new;	test_h.y_max = north_new;	test_h.y_inc = dy_new;
-	GMT_grd_RI_verify (&test_h);
+	GMT_grd_RI_verify (&test_h, 1);
 
 	/* We can save time by computing a weight matrix once [or once pr scanline] only
 	   if new grid spacing is multiple of old spacing */
