@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdgradient.c	2.54  09/21/99
+ *	$Id: grdgradient.c,v 1.4 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  *  grdgradient.c
@@ -40,10 +40,10 @@
  *		11 Sep 97 by PW now may pass average gradient along with sigma in -N
  *		22 Apr 98 by WHFS to add boundary conditions, switch sense of -S and 
  *			-D, and switch -Da to -Dc, for consistency of args.
+ * Version:	3.4
  */
  
 #include "gmt.h"
-#include "gmt_boundcond.h"
 
 float *data, *slp;
 
@@ -57,7 +57,7 @@ main (int argc, char **argv)
 	BOOLEAN find_directions = FALSE, do_cartesian = FALSE, do_orientations = FALSE, save_slopes = FALSE, add_ninety = FALSE;
 	BOOLEAN sigma_set = FALSE, offset_set = FALSE, exp_trans = FALSE, two_azims = FALSE;
 	
-	double	dx_grid, dy_grid, x_factor, y_factor, dzdx, dzdy, ave_gradient, norm_val = 1.0, sigma;
+	double	dx_grid, dy_grid, x_factor, y_factor, dzdx, dzdy, ave_gradient, norm_val = 1.0, sigma = 0.0;
 	double	azim, denom, max_gradient = 0.0, min_gradient = 0.0, rpi, m_pr_degree, lat, azim2;
 	double	x_factor2, y_factor2, dzdx2, dzdy2, dzds1, dzds2, offset;
 	
@@ -161,30 +161,30 @@ main (int argc, char **argv)
 		fprintf (stderr, "usage: grdgradient <infile> -G<outfile> [-A<azim>[/<azim2>]] [-D[a][o][n]] [-L<flag>]\n");
 		fprintf (stderr, "[-M] [-N[t_or_e][<amp>[/<sigma>[/<offset>]]]] [-S<slopefile>] [-V]\n\n");
 		if (GMT_quick) exit (EXIT_FAILURE);
-		fprintf (stderr,"       <infile> is name of input grdfile\n");
+		fprintf (stderr,"\t<infile> is name of input grdfile\n");
 		fprintf (stderr,"\n\tOPTIONS:\n");
-		fprintf (stderr, "	-A sets azimuth (0-360 CW from North (+y)) for directional derivatives\n");
-		fprintf (stderr, "	  -A<azim>/<azim2> will compute two directions and save the one larger in magnitude.\n");
-		fprintf (stderr, "	-D finds the direction of grad z.\n");
-		fprintf (stderr, "	   Append c to get cartesian angle (0-360 CCW from East (+x)) [Default:  azimuth]\n");
-		fprintf (stderr, "	   Append o to get bidirectional orientations [0-180] rather than directions [0-360]\n");
-		fprintf (stderr, "	   Append n to add 90 degrees to the values from c or o\n");
-		fprintf (stderr, "	-G output file for results from -A or -D\n");
-		fprintf (stderr, "	-L sets boundary conditions.  <flag> can be either\n");
-		fprintf (stderr, "	   g for geographic boundary conditions\n");
-		fprintf (stderr, "	   or one or both of\n");
-		fprintf (stderr, "	   x for periodic boundary conditions on x\n");
-		fprintf (stderr, "	   y for periodic boundary conditions on y\n");
-		fprintf (stderr, "	   [Default:  Natural conditions]\n");
-		fprintf (stderr, "	-M to use map units.  In this case, dx,dy of grdfile\n");
-		fprintf (stderr, "	   will be converted from degrees lon,lat into meters.\n");
-		fprintf (stderr, "	   Default computes gradient in units of data/grid_distance.\n");
-		fprintf (stderr, "	-N will normalize gradients so that max |grad| = <amp> [1.0]\n");
-		fprintf (stderr, "	  -Nt will make atan transform, then scale to <amp> [1.0]\n");
-		fprintf (stderr, "	  -Ne will make exp  transform, then scale to <amp> [1.0]\n");
-		fprintf (stderr, "	  -Nt<amp>/<sigma>[/<offset>] or -Ne<amp>/<sigma>[/<offset>] sets sigma\n");
-		fprintf (stderr, "	     (and offset) for transform. [sigma, offset estimated from data]\n");
-		fprintf (stderr, "	-S output file for |grad z|; requires -D\n");
+		fprintf (stderr, "\t-A sets azimuth (0-360 CW from North (+y)) for directional derivatives\n");
+		fprintf (stderr, "\t  -A<azim>/<azim2> will compute two directions and save the one larger in magnitude.\n");
+		fprintf (stderr, "\t-D finds the direction of grad z.\n");
+		fprintf (stderr, "\t   Append c to get cartesian angle (0-360 CCW from East (+x)) [Default:  azimuth]\n");
+		fprintf (stderr, "\t   Append o to get bidirectional orientations [0-180] rather than directions [0-360]\n");
+		fprintf (stderr, "\t   Append n to add 90 degrees to the values from c or o\n");
+		fprintf (stderr, "\t-G output file for results from -A or -D\n");
+		fprintf (stderr, "\t-L sets boundary conditions.  <flag> can be either\n");
+		fprintf (stderr, "\t   g for geographic boundary conditions\n");
+		fprintf (stderr, "\t   or one or both of\n");
+		fprintf (stderr, "\t   x for periodic boundary conditions on x\n");
+		fprintf (stderr, "\t   y for periodic boundary conditions on y\n");
+		fprintf (stderr, "\t   [Default:  Natural conditions]\n");
+		fprintf (stderr, "\t-M to use map units.  In this case, dx,dy of grdfile\n");
+		fprintf (stderr, "\t   will be converted from degrees lon,lat into meters.\n");
+		fprintf (stderr, "\t   Default computes gradient in units of data/grid_distance.\n");
+		fprintf (stderr, "\t-N will normalize gradients so that max |grad| = <amp> [1.0]\n");
+		fprintf (stderr, "\t  -Nt will make atan transform, then scale to <amp> [1.0]\n");
+		fprintf (stderr, "\t  -Ne will make exp  transform, then scale to <amp> [1.0]\n");
+		fprintf (stderr, "\t  -Nt<amp>/<sigma>[/<offset>] or -Ne<amp>/<sigma>[/<offset>] sets sigma\n");
+		fprintf (stderr, "\t     (and offset) for transform. [sigma, offset estimated from data]\n");
+		fprintf (stderr, "\t-S output file for |grad z|; requires -D\n");
 		GMT_explain_option ('V');
 		exit (EXIT_FAILURE);
 	}
@@ -346,6 +346,7 @@ main (int argc, char **argv)
 					denom = 0.0;
 					for (k = 0; k < nm; k++) if (!GMT_is_fnan (data[k])) denom += pow(data[k] - ave_gradient, 2.0);
 					denom = sqrt( (n_used - 1) / denom);
+					sigma = 1.0 / denom;
 				}
 				rpi = 2.0 * norm_val / M_PI;
 				for (k = 0; k < nm; k++) if (!GMT_is_fnan (data[k])) data[k] = (float)(rpi * atan((data[k] - ave_gradient)*denom));
@@ -394,10 +395,10 @@ main (int argc, char **argv)
 		else {
 			strcpy (header.title, "Directional derivative(s)");
 		}
-		sprintf (format, "\t%s\t%s\t%s\n\0", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
+		sprintf (format, "\t%s\t%s\t%s\t%s\n\0", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
 		if (gmtdefs.verbose) {
-			fprintf (stderr, "%s:  Min Mean Max intensities:", GMT_program);
-			fprintf (stderr, format, min_gradient, ave_gradient, max_gradient);
+			fprintf (stderr, "%s:  Min Mean Max sigma intensities:", GMT_program);
+			fprintf (stderr, format, min_gradient, ave_gradient, max_gradient, sigma);
 		}
 	}
 	else
