@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:   @(#)grdimage.c	2.87  11/29/99
+ *	$Id: grdimage.c,v 1.5 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdimage will read a grdfile and image the area using the PostScript
@@ -27,6 +27,8 @@
  * Ver:		3.1	based on old 3.0
  * Ver:		3.3.2	Added -S as in grdproject
  * Ver:		3.3.3	Added -T[s] for non-interpolated tiling
+ * Ver:		3.3.6	-T needed polygon clipping and wrapping (like psxy)
+ * Ver:		3.4
  */
  
 #include "gmt.h"
@@ -39,6 +41,7 @@ char *c_method[2] = {
 	"colortiles",
 };
 
+void GMT_limits (struct GRD_HEADER *r, struct GRD_HEADER *g);
 void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g);
 
 main (int argc, char **argv)
@@ -139,29 +142,29 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"grdimage %s - Plot grdfiles in 2-D\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdimage <grdfile> -C<cpt_file> -J<params> [-B<tickinfo>] [-E<dpi>] [-G[f|b]<rgb>]\n");
-		fprintf (stderr, "   [-I<intensity_file>] [-K] [-M] [-O] [-P] [-R<w/e/s/n>] [-S<radius>] [-T[s]] [-U] [-V]\n");
-		fprintf (stderr, "   [-X<x_shift>] [-Y<y_shift>] [-c<ncopies>]\n\n");
+		fprintf (stderr, "\t[-I<intensity_file>] [-K] [-M] [-O] [-P] [-R<w/e/s/n>] [-S<radius>] [-T[s]] [-U] [-V]\n");
+		fprintf (stderr, "\t[-X<x_shift>] [-Y<y_shift>] [-c<ncopies>]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr,"	<grdfile> is data set to be plotted\n");
-		fprintf (stderr,"	-C color palette file\n");
+		fprintf (stderr,"\t<grdfile> is data set to be plotted\n");
+		fprintf (stderr,"\t-C color palette file\n");
 		GMT_explain_option ('j');
 		fprintf (stderr,"\n\tOPTIONS:\n");
 		GMT_explain_option ('b');
-		fprintf(stderr, "	-E sets dpi for the projected grid which must be constructed\n");
-		fprintf(stderr, "	   if -Jx or -Jm is not selected [Default gives same size as input grid]\n");
-		fprintf (stderr,"	-G sets transparency color for images that\n");
-		fprintf (stderr,"	   otherwise would result in 1-bit images\n");
-		fprintf (stderr,"	-I use illumination.  Append name of intensity grd file\n");
+		fprintf(stderr, "\t-E sets dpi for the projected grid which must be constructed\n");
+		fprintf(stderr, "\t   if -Jx or -Jm is not selected [Default gives same size as input grid]\n");
+		fprintf (stderr,"\t-G sets transparency color for images that\n");
+		fprintf (stderr,"\t   otherwise would result in 1-bit images\n");
+		fprintf (stderr,"\t-I use illumination.  Append name of intensity grd file\n");
 		GMT_explain_option ('K');
-		fprintf (stderr, "	-M force monochrome image\n");
+		fprintf (stderr, "\t-M force monochrome image\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
 		GMT_explain_option ('R');
-		fprintf(stderr, "	-S sets the search radius in projected units [Default avoids aliasing]\n");
-		fprintf(stderr, "	-T will image the data without interpolation by painting polygonal tiles\n");
-		fprintf(stderr, "	Append s to skip tiles for nodes with z = NaN [Default paints all tiles]\n");
+		fprintf(stderr, "\t-S sets the search radius in projected units [Default avoids aliasing]\n");
+		fprintf(stderr, "\t-T will image the data without interpolation by painting polygonal tiles\n");
+		fprintf(stderr, "\t   Append s to skip tiles for nodes with z = NaN [Default paints all tiles]\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
 		GMT_explain_option ('X');
@@ -276,10 +279,11 @@ main (int argc, char **argv)
 		}
 		grid_type = (dpi > 0) ? 1 : g_head.node_offset;	/* Force pixel if dpi is set */
 		GMT_grdproject_init (&r_head, 0.0, 0.0, nx_proj, ny_proj, dpi, grid_type);
+		GMT_init_search_radius (&max_radius, &r_head, &g_head, FALSE);
 		nm2 = r_head.nx * r_head.ny;
 		map = (float *) GMT_memory (VNULL, (size_t)nm2, sizeof (float), "grdproject");
 
-		GMT_grd_forward (tmp1, &g_head, map, &r_head, max_radius, FALSE);
+		GMT_grd_forward (tmp1, &g_head, map, &r_head, max_radius);
 		
 		GMT_free ((void *)tmp1);
 		if (intens) {
@@ -292,7 +296,7 @@ main (int argc, char **argv)
 			}
 			GMT_grdproject_init (&j_head, 0.0, 0.0, nx_proj, ny_proj, dpi, grid_type);
 			intensity = (float *) GMT_memory (VNULL, (size_t)nm2, sizeof (float), "grdproject");
-			GMT_grd_forward (tmp2, &i_head, intensity, &j_head, max_radius, FALSE);
+			GMT_grd_forward (tmp2, &i_head, intensity, &j_head, max_radius);
 			GMT_free ((void *)tmp2);
 		}
 		nm = nm2;
@@ -319,8 +323,9 @@ main (int argc, char **argv)
 	GMT_map_clip_on (GMT_no_rgb, 3);
 
 	if (tiling) {	/* Plot image as polygonal pieces and then exit */
-		int np;
-		double *grd_x, grd_y, *xx, *yy, d, dx2, dy2, xpos, ypos;
+		int np, index, plot_n, n_use, start;
+		double *grd_x, grd_y, *xx, *yy, *xp, *yp, d, dx2, dy2, y_bot, y_top;
+		struct GMT_FILL *f;
 		
 		d = (off) ? 0.0 : 0.5;
 		dx2 = 0.5 * g_head.x_inc;
@@ -332,23 +337,61 @@ main (int argc, char **argv)
 		for (i = 0; i < g_head.nx; i++) grd_x[i] = g_head.x_min + (i + d) * g_head.x_inc;
 		for (j = node = 0; j < g_head.ny; j++) {
 			grd_y = g_head.y_max - (j + d) * g_head.y_inc;
+			y_bot = grd_y - dy2;
+			y_top = grd_y + dy2;
+			if (MAPPING) {
+				if (y_bot < -90.0) y_bot = -90.0;
+				if (y_top > +90.0) y_top = +90.0;
+			}
 			for (i = 0; i < g_head.nx; i++, node++) {	/* Compute rgb for each pixel */
 				if (GMT_is_fnan (map[node]) && tiling == 2) continue;
-				GMT_get_rgb24 (map[node], rgb);
+				index = GMT_get_rgb24 (map[node], rgb);
+				if (GMT_cpt_skip) continue;	/* Skip this z-slice */
 				if (intens) GMT_illuminate (intensity[node], rgb);
+				if (monochrome) rgb[0] = rgb[1] = rgb[2] = YIQ (rgb);
 				
-				np = GMT_graticule_path (&xx, &yy, 1, grd_x[i] - dx2, grd_x[i] + dx2, grd_y - dy2, grd_y + dy2);
+				np = GMT_graticule_path (&xx, &yy, 1, grd_x[i] - dx2, grd_x[i] + dx2, y_bot, y_top);
 				
-				/* Do map projection and plot */
-				
-				for (k = 0; k < np; k++) {
-					GMT_geo_to_xy (xx[k], yy[k], &xpos, &ypos);
-					xx[k] = xpos;	yy[k] = ypos;
-				}
-				ps_polygon (xx, yy, np, rgb, FALSE);
-
+				plot_n = GMT_clip_to_map (xx, yy, np, &xp, &yp);
 				GMT_free ((void *)xx);
 				GMT_free ((void *)yy);
+				if (plot_n == 0) continue;	/* Outside */
+
+				if ((*GMT_will_it_wrap) (xp, yp, plot_n, &start)) {	/* Polygon wraps */
+					
+					/* First truncate agains left border */
+						
+					GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, -1);
+					n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
+					if (index >= 0 && (f = GMT_lut[index].fill))
+						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, f, FALSE);
+					else if (index < 0 && (f = GMT_bfn.fill[index+3]))
+						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, f, FALSE);
+					else
+						ps_polygon (GMT_x_plot, GMT_y_plot, n_use, rgb, FALSE);
+													
+					/* Then truncate agains right border */
+						
+					GMT_n_plot = (*GMT_truncate) (xp, yp, plot_n, start, +1);
+					n_use = GMT_compact_line (GMT_x_plot, GMT_y_plot, GMT_n_plot, FALSE, 0);
+					if (index >= 0 && (f = GMT_lut[index].fill))
+						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, f, FALSE);
+					else if (index < 0 && (f = GMT_bfn.fill[index+3]))
+						GMT_fill (GMT_x_plot, GMT_y_plot, n_use, f, FALSE);
+					else
+						ps_polygon (GMT_x_plot, GMT_y_plot, n_use, rgb, FALSE);
+					
+				}
+				else {
+					if (index >= 0 && (f = GMT_lut[index].fill))
+						GMT_fill (xp, yp, plot_n, f, FALSE);
+					else if (index < 0 && (f = GMT_bfn.fill[index+3]))
+						GMT_fill (xp, yp, plot_n, f, FALSE);
+					else
+						ps_polygon (xp, yp, plot_n, rgb, FALSE);
+				}
+				GMT_free ((void *)xp);
+				GMT_free ((void *)yp);
 			}
 		}
 		
@@ -365,7 +408,10 @@ main (int argc, char **argv)
 		GMT_end (argc, argv);
 	}
 				
-	if (gmtdefs.verbose) fprintf (stderr, "%s: Evaluate pixel colors\n", GMT_program);
+	if (gmtdefs.verbose) {
+		if (GMT_cpt_pattern) fprintf (stderr, "%s: Warning: Patterns in cpt file only apply to -T\n", GMT_program);
+		fprintf (stderr, "%s: Evaluate pixel colors\n", GMT_program);
+	}
 	
 	if (monochrome || GMT_gray)
 		bitimage_8 = (unsigned char *) GMT_memory (VNULL, (size_t)nm, sizeof (char), GMT_program);
@@ -417,10 +463,11 @@ main (int argc, char **argv)
 	if (GMT_gray) for (k = 0, GMT_b_and_w = TRUE; GMT_b_and_w && k < nm; k++) if (!(bitimage_8[k] == 0 || bitimage_8[k] == 255)) GMT_b_and_w = FALSE;
 	
 	if (GMT_b_and_w) {	/* Can get away with 1 bit image */
-		int nx8, shift, byte, b_or_w, k8;
+		int nx8, shift, byte, b_or_w, k8, nx_pixels;
 		unsigned char *bit;
 		
 		nx8 = (int)ceil (r_head.nx / 8.0);
+		nx_pixels = nx8 * 8;
 		bit = (unsigned char *) GMT_memory (VNULL, (size_t)(nx8 * r_head.ny), sizeof (char), GMT_program);
 		
 		for (j = k = k8 = 0; j < r_head.ny; j++) {
@@ -449,14 +496,14 @@ main (int argc, char **argv)
 		}
 		GMT_free ((void *)bitimage_8);
 		
-		x_side = nx8 * 8.0 * dx;
+		x_side = nx_pixels * dx;
 		if (transparent) {
 			if (gmtdefs.verbose) fprintf (stderr, "[1-bit image mask painted %d/%d/%d]\n", f_rgb[0], f_rgb[1], f_rgb[2]);
-			ps_imagemask (x0, y0, x_side, y_side, bit, nx, r_head.ny, polarity, f_rgb);
+			ps_imagemask (x0, y0, x_side, y_side, bit, nx_pixels, r_head.ny, polarity, f_rgb);
 		}
 		else {
 			if (gmtdefs.verbose) fprintf (stderr, "[1-bit B/W image]\n");
-			ps_image (x0, y0, x_side, y_side, bit, nx, r_head.ny, 1);
+			ps_image (x0, y0, x_side, y_side, bit, nx_pixels, r_head.ny, 1);
 		}
 		GMT_free ((void *)bit);
 	}
@@ -531,6 +578,13 @@ void GMT_set_proj_limits (struct GRD_HEADER *r, struct GRD_HEADER *g)
 		r->y_min = MIN (r->y_min, y);	r->y_max = MAX (r->y_max, y);
 	}
 
+	if (AZIMUTHAL && all_lons) {	/* Full 360, use min/max for x */
+		r->x_min = project_info.xmin;	r->x_max = project_info.xmax;
+	}
+	if (AZIMUTHAL && all_lats) {	/* Full -90/+90, use min/max for y */
+		r->y_min = project_info.ymin;	r->y_max = project_info.ymax;
+	}
+
 	/* Then truncate, if neccesary, values to fit inside plotbox */
 	
 	r->x_min = MAX (r->x_min, project_info.xmin);	r->x_max = MIN (r->x_max, project_info.xmax);
