@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdinfo.c	2.52  10/20/99
+ *	$Id: grdinfo.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdinfo reads one or more grd file and [optionally] prints out various
@@ -24,6 +24,8 @@
  * Version:	3.1	Based on 3.0
  * 		3.3.2	Added -D option
  * 		3.3.3	Added -F option via M. Odegard
+ * 		3.3.5	12-JUL-2000
+ * Version:	3.4
  */
 
 #include "gmt.h"
@@ -54,6 +56,7 @@ main (int argc, char **argv)
 			switch (argv[i][1]) {
 				/* Common parameters */
 			
+				case 'V':
 				case '\0':
 					error += GMT_get_common_args (argv[i], 0, 0, 0, 0);
 					break;
@@ -100,15 +103,17 @@ main (int argc, char **argv)
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr, "	<grdfiles> may be one or more netCDF grdfiles\n");
+		fprintf (stderr, "\t<grdfiles> may be one or more netCDF grdfiles\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "	-C formats report in fields on a single line using the format\n");
-		fprintf (stderr, "	   file w e s n z0 z1 dx dy nx ny [x0 y0 x1 y1] [med scale] [mean std rms]\n");
-		fprintf (stderr, "	-D reports domain in dd:mm:ss format [Default is decimal]\n");
-		fprintf (stderr, "	-F reports domain in world mapping format [Default is generic]\n");
-		fprintf (stderr, "	-L1 reports median and L1-scale of data set\n");
-		fprintf (stderr, "	-L[2] reports mean, standard deviation, and rms of data set\n");
-		fprintf (stderr, "	-M searches for the global min and max locations (x0,y0) and (x1,y1)\n");
+		fprintf (stderr, "\t-C formats report in fields on a single line using the format\n");
+		fprintf (stderr, "\t   file w e s n z0 z1 dx dy nx ny [x0 y0 x1 y1] [med scale] [mean std rms] [n_nan]\n");
+		fprintf (stderr, "\t   (-M gives [x0 y0 x1 y1] and [n_nan]; -L1 gives [med scale]; -L2 gives [mean std rms])\n");
+		fprintf (stderr, "\t-D reports domain in dd:mm:ss format [Default is decimal]\n");
+		fprintf (stderr, "\t-F reports domain in world mapping format [Default is generic]\n");
+		fprintf (stderr, "\t-L1 reports median and L1-scale of data set\n");
+		fprintf (stderr, "\t-L[2] reports mean, standard deviation, and rms of data set\n");
+		fprintf (stderr, "\t-M searches for the global min and max locations (x0,y0) and (x1,y1)\n");
+		GMT_explain_option ('V');
 		exit (EXIT_FAILURE);
 	}
 	
@@ -124,7 +129,7 @@ main (int argc, char **argv)
 
 	GMT_put_history (argc, argv);	/* Update .gmtcommands */
 
-	a = (float *) GMT_memory (VNULL, (size_t)1, sizeof (float), "grdinfo");
+	a = (float *) GMT_memory (VNULL, (size_t)1, sizeof (float), GMT_program);
 
 	for (k = 1; k < argc; k++) {	/* Loop over arguments, skip options */
 	
@@ -133,22 +138,25 @@ main (int argc, char **argv)
 		strcpy (file, argv[k]);
 		for (j = 0; file[j]; j++) if (file[j] == '=') file[j] = 0;
 		if (strcmp (file, "=") && access (file, R_OK)) {
-			fprintf (stderr, "grdinfo:  File %s not found\n", file);
+			fprintf (stderr, "%s:  File %s not found\n", GMT_program, file);
 			continue;
 		}
 
 		GMT_grd_init (&grd, argc, argv, FALSE);
 
 		if (GMT_read_grd_info (argv[k], &grd)) {
-			fprintf (stderr, "grdinfo: Error opening file %s\n", file);
+			fprintf (stderr, "%s: Error opening file %s\n", GMT_program, file);
 			continue;
 		}
+		
+		if (gmtdefs.verbose) fprintf (stderr, "%s: Processing file %s\n", GMT_program, file);
+
 		if (grd.z_min == grd.z_max && GMT_grd_i_format) quick = FALSE, find_max = TRUE;
 		
 		if (!quick) {	/* Must determine the location of global min and max values */
 
 			nm = grd.nx * grd.ny;
-			a = (float *) GMT_memory ((void *)a, (size_t)nm, sizeof (float), "grdinfo");
+			a = (float *) GMT_memory ((void *)a, (size_t)nm, sizeof (float), GMT_program);
 			if (GMT_read_grd (argv[k], &grd, a, 0.0, 0.0, 0.0, 0.0, GMT_pad, FALSE)) continue;
 		
 			z_min = DBL_MAX;	z_max = -DBL_MAX;
@@ -201,95 +209,98 @@ main (int argc, char **argv)
 
 		if (columns) {
 			sprintf (format, "%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%s\t%d\t%d\0", file, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, grd.nx, grd.ny);
-			printf (format, grd.x_min, grd.x_max, grd.y_min, grd.y_max, grd.z_min, grd.z_max, grd.x_inc, grd.y_inc);
+			fprintf (GMT_stdout, format, grd.x_min, grd.x_max, grd.y_min, grd.y_max, grd.z_min, grd.z_max, grd.x_inc, grd.y_inc);
 			if (find_max) {
 				sprintf (format, "\t%s\t%s\t%s\t%s\0", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-				printf (format, x_min, y_min, x_max, y_max);
+				fprintf (GMT_stdout, format, x_min, y_min, x_max, y_max);
 			}
 			if (l1) {
 				sprintf (format, "\t%s\t%s\n\0", gmtdefs.d_format, gmtdefs.d_format);
-				printf (format, median, scale);
+				fprintf (GMT_stdout, format, median, scale);
 			}
 			if (l2) {
 				sprintf (format, "\t%s\t%s\t%s\0", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-				printf (format, mean, stdev, rms);
+				fprintf (GMT_stdout, format, mean, stdev, rms);
 			}
-			putchar ('\n');
+			if (find_max) fprintf (GMT_stdout, "\t%d", n_nan);
+			fprintf (GMT_stdout, "\n");
 		}
 		else {
-			printf ("%s: Title: %s\n", file, grd.title);
-			printf ("%s: Command: %s\n", file, grd.command);
-			printf ("%s: Remark: %s\n", file, grd.remark);
-			printf ("%s: %s node registation used\n", file, type[grd.node_offset]);
-			printf ("%s: grdfile format # %d\n", file, GMT_grd_i_format);
+			fprintf (GMT_stdout, "%s: Title: %s\n", file, grd.title);
+			fprintf (GMT_stdout, "%s: Command: %s\n", file, grd.command);
+			fprintf (GMT_stdout, "%s: Remark: %s\n", file, grd.remark);
+			fprintf (GMT_stdout, "%s: %s node registration used\n", file, type[grd.node_offset]);
+			fprintf (GMT_stdout, "%s: grdfile format # %d\n", file, GMT_grd_i_format);
 			if (geographic) {
 				char item1[80], item2[80], item3[80];
 				GMT_format_ddmmss (grd.x_min, item1);
 				GMT_format_ddmmss (grd.x_max, item2);
 				GMT_format_ddmmss (grd.x_inc, item3);
-				printf ("%s: x_min: %s x_max: %s x_inc: %s units: %s nx: %d\n\0", file, item1, item2, item3, grd.x_units, grd.nx);
+				fprintf (GMT_stdout, "%s: x_min: %s x_max: %s x_inc: %s units: %s nx: %d\n\0", file, item1, item2, item3, grd.x_units, grd.nx);
 				GMT_format_ddmmss (grd.y_min, item1);
 				GMT_format_ddmmss (grd.y_max, item2);
 				GMT_format_ddmmss (grd.y_inc, item3);
-				printf ("%s: y_min: %s y_max: %s y_inc: %s units: %s ny: %d\n\0", file, item1, item2, item3, grd.y_units, grd.ny);
+				fprintf (GMT_stdout, "%s: y_min: %s y_max: %s y_inc: %s units: %s ny: %d\n\0", file, item1, item2, item3, grd.y_units, grd.ny);
 			}
 			else if (world_form) {
 				if ((fabs (grd.x_min) < 500.0) && (fabs (grd.x_max) < 500.0) && (fabs (grd.y_min) < 500.0) && (fabs (grd.y_max) < 500.0)) {
-					printf("%s: x_min: %.7f\n", file, grd.x_min);
-					printf("%s: x_max: %.7f\n", file, grd.x_max);
-					printf("%s: x_inc: %.7f\n", file, grd.x_inc);
-					printf("%s: units: %s\n", file, grd.x_units);
-					printf("%s: nx: %d\n", file, grd.nx);
-					printf("%s: y_min: %.7f\n", file, grd.y_min);
-					printf("%s: y_max: %.7f\n", file, grd.y_max);
-					printf("%s: y_inc: %.7f\n", file, grd.y_inc);
-					printf("%s: units: %s\n", file, grd.y_units);
-					printf("%s: ny: %d\n", file, grd.ny);
+					fprintf (GMT_stdout, "%s: x_min: %.7f\n", file, grd.x_min);
+					fprintf (GMT_stdout, "%s: x_max: %.7f\n", file, grd.x_max);
+					fprintf (GMT_stdout, "%s: x_inc: %.7f\n", file, grd.x_inc);
+					fprintf (GMT_stdout, "%s: units: %s\n", file, grd.x_units);
+					fprintf (GMT_stdout, "%s: nx: %d\n", file, grd.nx);
+					fprintf (GMT_stdout, "%s: y_min: %.7f\n", file, grd.y_min);
+					fprintf (GMT_stdout, "%s: y_max: %.7f\n", file, grd.y_max);
+					fprintf (GMT_stdout, "%s: y_inc: %.7f\n", file, grd.y_inc);
+					fprintf (GMT_stdout, "%s: units: %s\n", file, grd.y_units);
+					fprintf (GMT_stdout, "%s: ny: %d\n", file, grd.ny);
 				}
 				else {
-					printf("%s: x_min: %.2f\n", file, grd.x_min);
-					printf("%s: x_max: %.2f\n", file, grd.x_max);
-					printf("%s: x_inc: %.2f\n", file, grd.x_inc);
-					printf("%s: units: %s\n", file, grd.x_units);
-					printf("%s: nx: %d\n", file, grd.nx);
-					printf("%s: y_min: %.2f\n", file, grd.y_min);
-					printf("%s: y_max: %.2f\n", file, grd.y_max);
-					printf("%s: y_inc: %.2f\n", file, grd.y_inc);
-					printf("%s: units: %s\n", file, grd.y_units);
-					printf("%s: ny: %d\n", file, grd.ny);
+					fprintf (GMT_stdout, "%s: x_min: %.2f\n", file, grd.x_min);
+					fprintf (GMT_stdout, "%s: x_max: %.2f\n", file, grd.x_max);
+					fprintf (GMT_stdout, "%s: x_inc: %.2f\n", file, grd.x_inc);
+					fprintf (GMT_stdout, "%s: units: %s\n", file, grd.x_units);
+					fprintf (GMT_stdout, "%s: nx: %d\n", file, grd.nx);
+					fprintf (GMT_stdout, "%s: y_min: %.2f\n", file, grd.y_min);
+					fprintf (GMT_stdout, "%s: y_max: %.2f\n", file, grd.y_max);
+					fprintf (GMT_stdout, "%s: y_inc: %.2f\n", file, grd.y_inc);
+					fprintf (GMT_stdout, "%s: units: %s\n", file, grd.y_units);
+					fprintf (GMT_stdout, "%s: ny: %d\n", file, grd.ny);
 				}
 			}
 			else {
 				sprintf (format, "%s: x_min: %s x_max: %s x_inc: %s units: %s nx: %d\n\0", file, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, grd.x_units, grd.nx);
-				printf (format, grd.x_min, grd.x_max, grd.x_inc);
+				fprintf (GMT_stdout, format, grd.x_min, grd.x_max, grd.x_inc);
 				sprintf (format, "%s: y_min: %s y_max: %s y_inc: %s units: %s ny: %d\n\0", file, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, grd.y_units, grd.ny);
-				printf (format, grd.y_min, grd.y_max, grd.y_inc);
+				fprintf (GMT_stdout, format, grd.y_min, grd.y_max, grd.y_inc);
 			}
 			
 			if (find_max) {
+				if (z_min == -DBL_MAX) z_min = GMT_d_NaN;
+				if (z_max == +DBL_MAX) z_max = GMT_d_NaN;
 				sprintf (format, "%s: z_min: %s at x = %s y = %s z_max: %s at x = %s y = %s units: %s\n\0", file, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, grd.z_units);
-				printf (format, z_min, x_min, y_min, z_max, x_max, y_max);
+				fprintf (GMT_stdout, format, z_min, x_min, y_min, z_max, x_max, y_max);
 			}
 			else if (world_form) {
-				printf ("%s: zmin: %lg\n", file, grd.z_min);
-				printf ("%s: zmax: %lg\n", file, grd.z_max);
-				printf ("%s: units: %s\n", file, grd.z_units);
+				fprintf (GMT_stdout, "%s: zmin: %lg\n", file, grd.z_min);
+				fprintf (GMT_stdout, "%s: zmax: %lg\n", file, grd.z_max);
+				fprintf (GMT_stdout, "%s: units: %s\n", file, grd.z_units);
 			}
 			else {
 				sprintf (format, "%s: z_min: %s z_max: %s units: %s\n\0", file, gmtdefs.d_format, gmtdefs.d_format, grd.z_units);
-				printf (format, grd.z_min, grd.z_max);
+				fprintf (GMT_stdout, format, grd.z_min, grd.z_max);
 			}
 
 			sprintf (format, "%s: scale_factor: %s add_offset: %s\n\0", file, gmtdefs.d_format, gmtdefs.d_format);
-			printf (format, grd.z_scale_factor, grd.z_add_offset);
-			if (n_nan) printf ("%s: %d nodes set to NaN\n", file, n_nan);
+			fprintf (GMT_stdout, format, grd.z_scale_factor, grd.z_add_offset);
+			if (n_nan) fprintf (GMT_stdout, "%s: %d nodes set to NaN\n", file, n_nan);
 			if (l1) {
 				sprintf (format, "%s: median: %s scale: %s\n\0", file, gmtdefs.d_format, gmtdefs.d_format);
-				printf (format, median, scale);
+				fprintf (GMT_stdout, format, median, scale);
 			}
 			if (l2) {
 				sprintf (format, "%s: mean: %s stdev: %s rms: %s\n\0", file, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-				printf (format, mean, stdev, rms);
+				fprintf (GMT_stdout, format, mean, stdev, rms);
 			}
 		}
 	}
