@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdlandmask.c	3.55  10/05/99
+ *	$Id: grdlandmask.c,v 1.3 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdlandmask defines a grid based on region and xinc/yinc values,
@@ -26,6 +26,8 @@
  * Version:	3.0
  * Modified:	24-JUN-1998, for GMT 3.1
  *		18-AUG-1999, for GMT 3.3.2
+ *		13-JUL-2000, for GMT 3.3.5
+ * Version:	3.4
  */
  
 #include "gmt.h"
@@ -49,7 +51,7 @@ main (int argc, char **argv)
 	double xinc2, yinc2, min_area = 0.0, i_dx, i_dy, edge = 0.0, del_off, dummy;
 	float out_edge_in[5];
 	
-	char *maskfile = CNULL, res = 'l', line[80], *ptr;
+	char *maskfile = CNULL, res = 'l', line[128], *ptr;
 	
 	struct GRD_HEADER header;
 	struct POL *p;
@@ -136,12 +138,12 @@ main (int argc, char **argv)
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-A features smaller than <min_area> (in km^2) or of levels (0-4) outside the min-max levels\n");
 		fprintf (stderr, "\t   will be skipped [0/4] (see pscoast for details)]\n");
-		fprintf (stderr, "	-D Choose one of the following resolutions:\n");
-		fprintf (stderr, "	   f - full resolution (may be very slow for large regions)\n");
-		fprintf (stderr, "	   h - high resolution (may be slow for large regions)\n");
-		fprintf (stderr, "	   i - intermediate resolution\n");
-		fprintf (stderr, "	   l - low resolution [Default]\n");
-		fprintf (stderr, "	   c - crude resolution, for tasks that need crude continent outlines only\n");
+		fprintf (stderr, "\t-D Choose one of the following resolutions:\n");
+		fprintf (stderr, "\t   f - full resolution (may be very slow for large regions)\n");
+		fprintf (stderr, "\t   h - high resolution (may be slow for large regions)\n");
+		fprintf (stderr, "\t   i - intermediate resolution\n");
+		fprintf (stderr, "\t   l - low resolution [Default]\n");
+		fprintf (stderr, "\t   c - crude resolution, for tasks that need crude continent outlines only\n");
 		fprintf (stderr, "\t-F Force pixel registration for output grid [Default is gridline orientation]\n");
 		fprintf (stderr, "\t-N gives values to use if a node is outside or inside a feature.\n");
 		fprintf (stderr, "\t   Append o to let feature boundary be considered outside [Default is inside].\n");
@@ -176,7 +178,7 @@ main (int argc, char **argv)
 		header.x_max += 360.0;
 	}
 
-	GMT_grd_RI_verify (&header);
+	GMT_grd_RI_verify (&header, 1);
 	
 	if (dry_wet_only) {
 		out_edge_in[3] = out_edge_in[1];
