@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdmask.c	2.61  09/21/99
+ *	$Id: grdmask.c,v 1.3 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdmask defines a grid based on region and xinc/yinc values,
@@ -25,6 +25,11 @@
  * Date:	23-May-1991
  * Version:	2.0
  * Modified:	PW: 12-MAY-1998, to GMT 3.1
+ * 		PW: 18-FEB-2000, to GMT 3.3.4: Handle polarcap polygons
+ *				and for -S check if points are inside -R.
+ *				Also added -L for geographic data
+ * 		PW: 13-JUL-2000, 3.3.5
+ * Version:	3.4
  */
  
 #include "gmt.h"
@@ -34,13 +39,13 @@ float *data;
 main (int argc, char **argv)
 {
 
-	int	i, j, k, ij, nm, n_path, ix, iy, side, fno, n_files = 0, n_alloc = GMT_CHUNK, n_args;
-	int	one_or_zero, di, dj, i0, j0, n_fields, n_read, n_expected_fields;
+	int	i, j, k, ij, nm, n_path, side, fno, n_files = 0, n_alloc = GMT_CHUNK, n_args;
+	int	one_or_zero, di, dj, i0, j0, n_fields, n_read, n_expected_fields, polar;
 	
 	BOOLEAN	error = FALSE, done, nofile = TRUE, pixel = FALSE;
 	BOOLEAN	do_coverage = FALSE, map_units = FALSE;
 	
-	double	*x, *y, xx, yy, xmin, xmax, ymin, ymax, offset, dx, dy;
+	double	*x, *y, xx, yy, xmin, xmax, ymin, ymax, offset, dx, dy, r, ca, sa, lon_sum, dlon;
 	double xinc2, yinc2, distance, radius = 0.0, shrink, km_pr_deg, idx, idy, x0, y0, *in;
 
 	float out_edge_in[3];
@@ -96,6 +101,9 @@ main (int argc, char **argv)
 				case 'I':
 					GMT_getinc (&argv[i][2], &header.x_inc, &header.y_inc);
 					break;
+				case 'L':
+					GMT_geographic_in = TRUE;
+					break;
 				case 'M':
 					GMT_multisegment (&argv[i][2]);
 					break;
@@ -117,7 +125,7 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "grdmask %s - Create mask grdfile from polygons or point coverage\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: grdmask [<xyfiles>] -G<mask_grd_file> -I<xinc[m|c]>[/<yinc>[m|c]]\n");
-		fprintf (stderr, "\t-R<west/east/south/north> [-H[<nrec>]] [-F] [-N<out>/<edge>/<in>] [-M[<flag>]] [-S<radius>[k] [-V] [-:] [-bi[s][<n>]]\n\n");
+		fprintf (stderr, "\t-R<west/east/south/north> [-F] [-H[<nrec>]] [-L] [-N<out>/<edge>/<in>] [-M[<flag>]] [-S<radius>[k] [-V] [-:] [-bi[s][<n>]]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
@@ -129,6 +137,7 @@ main (int argc, char **argv)
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		fprintf (stderr, "\t-F Force pixel registration for output grid [Default is gridline orientation]\n");
 		GMT_explain_option ('H');
+		fprintf (stderr, "\t-L means that x is longitude, i.e. assumed to be periodic in 360\n");
 		GMT_explain_option ('M');
 		fprintf (stderr, "\t-N sets values to use if point is outside, on the path, or inside.\n");
 		fprintf (stderr, "\t   NaN is a valid entry.  Default values are 0/0/1.\n");
@@ -176,7 +185,7 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
 	}
 
-	GMT_grd_RI_verify (&header);
+	GMT_grd_RI_verify (&header, 1);
 
 	if (pixel) {
 		one_or_zero = 0;
@@ -223,8 +232,6 @@ main (int argc, char **argv)
 	
 	for (ij = 0; ij < nm; ij++) data[ij] = out_edge_in[0];
 	
-	ix = (gmtdefs.xy_toggle) ? 1 : 0;	iy = 1 - ix;		/* Set up which columns have x and y */
-
 	if (n_files > 0)
 		nofile = FALSE;
 	else
@@ -232,7 +239,7 @@ main (int argc, char **argv)
 	n_args = (argc > 1) ? argc : 2;
 	
 	done = FALSE;
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 2;
 	
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 		if (!nofile && argv[fno][0] == '-') continue;
@@ -250,7 +257,7 @@ main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 		
-		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 		
 		n_read = 0;
 		n_fields = GMT_input (fp,  &n_expected_fields, &in);
@@ -259,22 +266,28 @@ main (int argc, char **argv)
 			while (GMT_io.status & GMT_IO_SEGMENT_HEADER && !(GMT_io.status & GMT_IO_EOF)) n_fields = GMT_input (fp, &n_expected_fields, &in);
 			if ((GMT_io.status & GMT_IO_EOF)) continue;	/* At EOF */
 
-			n_path = 0;
+			n_path = polar = 0;
 			xmin = ymin = 1.0e+100;
 			xmax = ymax = -1.0e+100;
+			lon_sum = 0.0;
 	
 			while (!(GMT_io.status & (GMT_IO_SEGMENT_HEADER | GMT_IO_EOF))) {	/* Keep going until FALSE  */
 				if (GMT_io.status & GMT_IO_MISMATCH) {
 					fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, n_read);
 					exit (EXIT_FAILURE);
 				}
-				x[n_path] = in[ix];
-				y[n_path] = in[iy];
+				x[n_path] = in[0];
+				y[n_path] = in[1];
 	
 				if (x[n_path] > xmax) xmax = x[n_path];
 				if (x[n_path] < xmin) xmin = x[n_path];
 				if (y[n_path] > ymax) ymax = y[n_path];
 				if (y[n_path] < ymin) ymin = y[n_path];
+				if (n_path > 0) {	/* Do longitude-difference check sum */
+					dlon = x[n_path] - x[n_path-1];
+					if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
+					lon_sum += dlon;
+				}
 				n_path++;
 		
 				if (n_path == (n_alloc-1)) {	/* n_alloc-1 since we may need 1 more to close polygon */
@@ -285,9 +298,19 @@ main (int argc, char **argv)
 				n_fields = GMT_input (fp, &n_expected_fields, &in);
 			}
 
-			if (do_coverage) {
+			if (do_coverage) {	/* Assign 'inside' to nodes within given distance of data constrains */
 				for (k = 0; k < n_path; k++) {
 
+					if (y[k] < header.y_min || y[k] > header.y_max) continue;
+					if (GMT_geographic_in) {	/* Make sure we are not off by 360 somewhere */
+						x[k] -= 360.0;
+						while (x[k] < header.x_min) x[k] += 360.0;
+						if (x[k] > header.x_max) continue;
+					}
+					else if (x[k] < header.x_min || x[k] > header.x_max) continue;
+					
+					/* OK, this point is within bounds */
+					
 					i0 = (int)floor (((x[k] - header.x_min) * idx) + offset);
 					j0 = (int)floor (((header.y_max - y[k]) * idy) + offset);
 					data[j0 * header.nx + i0] = out_edge_in[2];	/* This is the nearest node */
@@ -308,25 +331,56 @@ main (int argc, char **argv)
 					}
 				}
 			}
-			else {
+			else {	/* assign 'inside' to nodes if they are inside given polygon */
 				if (x[n_path-1] != x[0] || y[n_path-1] != y[0]) {
 					x[n_path] = x[0];
 					y[n_path] = y[0];
+					dlon = x[n_path] - x[n_path-1];
 					n_path++;
+					if (fabs (dlon) > 180.0) dlon = copysign (360.0 - fabs (dlon), -dlon);
+					lon_sum += dlon;
+				}
+				if (fabs (fabs (lon_sum) - 360.0) < GMT_CONV_LIMIT) {	/* Contains a pole, convert to polar x,y */
+					polar = (y[0] < 0.0) ? -1 : +1;	/* S or N pole */
+					for (j = 0; j < n_path; j++) {
+						r = 90.0 - fabs (y[j]);
+						sincos (x[j] * D2R, &sa, &ca);
+						x[j] = r * ca;
+						y[j] = r * sa;
+					}
 				}
 
 				for (j = 0; j < header.ny; j++) {
 		
 					yy = header.y_max - j * header.y_inc;
 					if (pixel) yy -= yinc2;
-					if (yy < ymin || yy > ymax) continue;	/* Point outside, no need to assign value */
+					if (!polar && (yy < ymin || yy > ymax)) continue;	/* Point outside, no need to assign value */
 			
 					for (i = 0; i < header.nx; i++) {
 						xx = header.x_min + i * header.x_inc;
 						if (pixel) xx += xinc2;
-						if (xx < xmin || xx > xmax) continue;	/* Point outside, no need to assign value */
+						if (GMT_geographic_in) {	/* Make sure we are not off by 360 somewhere */
+							xx -= 360.0;
+							while (xx < xmin) xx += 360.0;
+						}
+						if (!polar && (xx < xmin || xx > xmax)) continue;	/* Point outside, no need to assign value */
 				
-						if ((side = GMT_non_zero_winding (xx, yy, x, y, n_path)) == 0) continue;	/* Outside */
+						if (polar == -1 && yy <= 0.0) {	/* Point on S hemisphere, check */
+							r = 90.0 + yy;
+							sincos (xx * D2R, &sa, &ca);
+							side = GMT_non_zero_winding (r * ca, r * sa, x, y, n_path);
+						}
+						else if (polar == +1 && yy >= 0.0) {	/* Point on N hemisphere, check */
+							r = 90.0 - yy;
+							sincos (xx * D2R, &sa, &ca);
+							side = GMT_non_zero_winding (r * ca, r * sa, x, y, n_path);
+						}
+						else if (polar)
+							side = 0;
+						else
+							side = GMT_non_zero_winding (xx, yy, x, y, n_path);
+							
+						if (side == 0) continue;	/* Outside */
 				
 						/* Here, point is inside or on edge, we must assign value */
 				
