@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdpaste.c	2.38  09/21/99
+ *	$Id: grdpaste.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdpaste.c reads two grdfiles and writes a new file with
@@ -23,13 +23,13 @@
  * Date:	5 august, 1988
  * Updated to v2.0 20-May-1991 Paul Wessel
  * Updated to v3.1 23-May-1998 Paul Wessel
+ * 3.3.5 23-JUN-2000 Paul Wessel
+ * Version:	3.4
  *
  */
  
 #include "gmt.h"
 
-float	*c;
-
 main (int argc, char **argv)
 {
 	BOOLEAN	error = FALSE;
@@ -40,6 +40,8 @@ main (int argc, char **argv)
 
 	double x_noise, y_noise;
 
+	float	*c;
+
 	struct GRD_HEADER ha, hb, hc;
 	
 	argc = GMT_begin (argc, argv);
