@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdreformat.c	3.35  11/18/99
+ *	$Id: grdreformat.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,20 +13,18 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdreformat.c reads a grd file in one format and outputs it in another
  *
  * Author:	Paul Wessel
- * Date:	3-JAN-1991-1999
- * Version:	3.1
+ * Date:	3-JAN-1991-2000
+ * Version:	3.4
  */
 
 #include "gmt.h"
 
-float *z;
-
 main (int argc, char **argv)
 {
 	BOOLEAN error = FALSE, global = FALSE;
@@ -35,6 +33,8 @@ main (int argc, char **argv)
 
 	double w, e, s, n, scale = 1.0, offset = 0.0;
 
+	float *z;
+
 	char *file[2], fname[2][BUFSIZ], line[BUFSIZ];
 
 	struct GRD_HEADER grd;
@@ -87,12 +87,12 @@ main (int argc, char **argv)
 		
 		fprintf (stderr, "	The following formats are supported\n\n");
 		sprintf (line, "%s%cshare%cgmtformats.d\0", GMTHOME, DIR_DELIM, DIR_DELIM);
-		if ((fp = GMT_fopen (line, "r")) == NULL) {
+		if ((fp = fopen (line, "r")) == NULL) {
 			fprintf (stderr, "%s: GMT ERROR: Cannot read file %s\n", GMT_program, line);
 			exit (EXIT_FAILURE);
 		}
 		while (fgets (line, BUFSIZ, fp)) if (line[0] != '#') fprintf (stderr, "\t%s", line);
-		GMT_fclose (fp);
+		fclose (fp);
 		exit (EXIT_FAILURE);
 	}
 	
