@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdsample.c	2.50  09/21/99
+ *	$Id: grdsample.c,v 1.4 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdsample reads a grdfile and evaluates the grid at new grid positions
@@ -26,13 +26,10 @@
  * Date:	19-JUL-1989
  * Revised:	6-JAN-1990	PW: Updated to v.2.0
  * Revised:	16-JUN-1998	PW: Updated to v.3.1
+ * Version:	3.4
  */
 
 #include "gmt.h"
-#include "gmt_boundcond.h"
-#include "gmt_bcr.h"
-
-float *a, *b;
 
 main (int argc, char **argv)
 {
@@ -43,6 +40,8 @@ main (int argc, char **argv)
 	
 	double *lon, lat, dx2, dy2;
 	
+	float *a, *b;
+
 	char *infile, *outfile, format[BUFSIZ];
 	
 	struct GRD_HEADER grd_a, grd_b;
@@ -106,28 +105,28 @@ main (int argc, char **argv)
 	}
 	
 	if (argc == 1 || GMT_quick) {
-		fprintf(stderr, "grdsample %s - Resample a gridded file onto a new grid\n\n", GMT_VERSION);
-		fprintf(stderr, "usage: grdsample <old_grdfile> -G<new_grdfile> [-F] [-I<dx>[m|c][/<dy>[m|c]]]\n");
-		fprintf(stderr, "	[-L<flag>] [-N<nx/ny>] [-Q] [-R<west/east/south/north>] [-T] [-V]\n");
+		fprintf (stderr, "grdsample %s - Resample a gridded file onto a new grid\n\n", GMT_VERSION);
+		fprintf (stderr, "usage: grdsample <old_grdfile> -G<new_grdfile> [-F] [-I<dx>[m|c][/<dy>[m|c]]]\n");
+		fprintf (stderr, "\t[-L<flag>] [-N<nx/ny>] [-Q] [-R<west/east/south/north>] [-T] [-V]\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf(stderr, "	<old_grdfile> is data set to be resampled\n");
-		fprintf(stderr, "	-G sets the name of the interpolated output grdfile\n");
-		fprintf(stderr, "\n\tOPTIONS:\n");
-		fprintf(stderr, "	-F force pixel node registration  [Default is centered]\n");
-		fprintf(stderr, "	-I sets the grid spacing (dx, dy) for the new grid\n");
-		fprintf (stderr, "	-L sets boundary conditions.  <flag> can be either\n");
-		fprintf (stderr, "	   g for geographic boundary conditions\n");
-		fprintf (stderr, "	   or one or both of\n");
-		fprintf (stderr, "	   x for periodic boundary conditions on x\n");
-		fprintf (stderr, "	   y for periodic boundary conditions on y\n");
-		fprintf(stderr, "	-N specifies number of columns (nx) and rows (ny) of new grid\n");
-		fprintf(stderr, "	-Q do quick, bilinear interpolation [Default is bicubic]\n");
-		fprintf(stderr, "	-R specifies a subregion [Default is old region]\n");
-		fprintf(stderr, "	-T Toggles between grid registration and pixel registration\n");
+		fprintf (stderr, "\t<old_grdfile> is data set to be resampled\n");
+		fprintf (stderr, "\t-G sets the name of the interpolated output grdfile\n");
+		fprintf (stderr, "\n\tOPTIONS:\n");
+		fprintf (stderr, "\t-F force pixel node registration  [Default is centered]\n");
+		fprintf (stderr, "\t-I sets the grid spacing (dx, dy) for the new grid\n");
+		fprintf (stderr, "\t-L sets boundary conditions.  <flag> can be either\n");
+		fprintf (stderr, "\t   g for geographic boundary conditions\n");
+		fprintf (stderr, "\t   or one or both of\n");
+		fprintf (stderr, "\t   x for periodic boundary conditions on x\n");
+		fprintf (stderr, "\t   y for periodic boundary conditions on y\n");
+		fprintf (stderr, "\t-N specifies number of columns (nx) and rows (ny) of new grid\n");
+		fprintf (stderr, "\t-Q do quick, bilinear interpolation [Default is bicubic]\n");
+		fprintf (stderr, "\t-R specifies a subregion [Default is old region]\n");
+		fprintf (stderr, "\t-T Toggles between grid registration and pixel registration\n");
 		GMT_explain_option ('V');
-		fprintf(stderr, "	   One only of -N -I must be specified\n");
+		fprintf (stderr, "\t   One only of -N -I must be specified\n");
 		
 		exit (EXIT_FAILURE);
 	}
@@ -220,7 +219,7 @@ main (int argc, char **argv)
 		grd_b.y_inc = (grd_b.y_max - grd_b.y_min) / (grd_b.ny - one);
 	}
 	
-	GMT_grd_RI_verify (&grd_b);
+	GMT_grd_RI_verify (&grd_b, 1);
 
 	b = (float *) GMT_memory (VNULL, (size_t)(grd_b.nx * grd_b.ny), sizeof(float), GMT_program);
 	
