@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdtrend.c	2.37  09/21/99
+ *	$Id: grdtrend.c,v 1.3 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /* grdtrend <input.grd> -N<n_model>[r] [-T<trend.grd>] [-V]
 	[-W<weight.grd] [-D<differences.grd]
@@ -48,7 +48,7 @@ Median absolute deviation (MAD: Huber, 1982).  The
 
 Author:		W. H. F. Smith
 Date:		21 May, 1991.
-Version:	1.0, after experience with 1-D case.
+Version:	3.4
 Calls:		uses the QR solution of the Normal
 		equations furnished by Wm. Menke's
 		C routine "gauss".  We gratefully
@@ -179,21 +179,21 @@ main(int argc, char **argv)
 	}
 
 	if (argc == 1 || GMT_quick) {
-		fprintf(stderr, "grdtrend %s - Fit trend surface to gridded data\n\n", GMT_VERSION);
-		fprintf(stderr,"usage:  grdtrend <input.grd> -N[r]<n_model> [-D<diff.grd>]\n");
-		fprintf(stderr,"	[-T<trend.grd>] [-V] [-W<weight.grd>]\n\n");
+		fprintf (stderr, "grdtrend %s - Fit trend surface to gridded data\n\n", GMT_VERSION);
+		fprintf (stderr,"usage:  grdtrend <input.grd> -N[r]<n_model> [-D<diff.grd>]\n");
+		fprintf (stderr,"\t[-T<trend.grd>] [-V] [-W<weight.grd>]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf(stderr,"\t<input.grd> is name of grdfile to fit trend to.\n");
-		fprintf(stderr,"\t-N # model parameters to fit; integer in [1,10].  Insert r for robust fit.\n");
-		fprintf(stderr,"\n\tOPTIONS:\n");
-		fprintf(stderr,"\t-D Supply filename to write grdfile of differences (input - trend).\n");
-		fprintf(stderr,"\t-T Supply filename to write grdfile of trend.\n");
+		fprintf (stderr,"\t<input.grd> is name of grdfile to fit trend to.\n");
+		fprintf (stderr,"\t-N # model parameters to fit; integer in [1,10].  Insert r for robust fit.\n");
+		fprintf (stderr,"\n\tOPTIONS:\n");
+		fprintf (stderr,"\t-D Supply filename to write grdfile of differences (input - trend).\n");
+		fprintf (stderr,"\t-T Supply filename to write grdfile of trend.\n");
 		GMT_explain_option ('V');
-		fprintf(stderr,"\t-W Supply filename if you want to [read and] write grdfile of weights.\n");
-		fprintf(stderr,"\t   If <weight.grd> can be read at run, and if robust = FALSE, weighted problem will be solved.\n");
-		fprintf(stderr,"\t   If robust = TRUE, weights used for robust fit will be written to <weight.grd>.\n");
+		fprintf (stderr,"\t-W Supply filename if you want to [read and] write grdfile of weights.\n");
+		fprintf (stderr,"\t   If <weight.grd> can be read at run, and if robust = FALSE, weighted problem will be solved.\n");
+		fprintf (stderr,"\t   If robust = TRUE, weights used for robust fit will be written to <weight.grd>.\n");
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -293,7 +293,7 @@ main(int argc, char **argv)
 		load_gtg_and_gtd(data, head_d.nx, head_d.ny, xval, yval, pstuff, gtg, gtd, n_model, weight, weighted);
 		gauss(gtg, gtd, n_model, n_model, zero_test, &ierror, 1);
 		if (ierror) {
-			fprintf(stderr,"%s:  Gauss returns error code %d\n", GMT_program, ierror);
+			fprintf (stderr,"%s:  Gauss returns error code %d\n", GMT_program, ierror);
 			exit (EXIT_FAILURE);
 		}
 		compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, n_model, pstuff);
@@ -310,13 +310,13 @@ main(int argc, char **argv)
 				load_gtg_and_gtd(data, head_d.nx, head_d.ny, xval, yval, pstuff, gtg, gtd, n_model, weight, weighted);
 				gauss(gtg, gtd, n_model, n_model, zero_test, &ierror, 1);
 				if (ierror) {
-					fprintf(stderr,"%s:  Gauss returns error code %d\n", GMT_program, ierror);
+					fprintf (stderr,"%s:  Gauss returns error code %d\n", GMT_program, ierror);
 					exit (EXIT_FAILURE);
 				}
 				compute_trend(trend, head_d.nx, head_d.ny, xval, yval, gtd, n_model, pstuff);
 				compute_resid(data, trend, resid, nxy);
 				compute_chisq(resid, weight, nxy, &chisq, scale);
-				if (gmtdefs.verbose) fprintf(stderr, format, GMT_program, iterations, old_chisq, chisq);
+				if (gmtdefs.verbose) fprintf (stderr, format, GMT_program, iterations, old_chisq, chisq);
 				iterations++;
 			} while ( (old_chisq / chisq) > 1.0001);
 
@@ -499,7 +499,7 @@ void	compute_chisq(float *resid, float *weight, int nxy, double *chisq, double s
 
 void	compute_robust_weight(float *resid, float *weight, int nxy, double *scale)
 {
-	int	i, j;
+	int	i, j, j2;
 	double	r, mad;
 
 	for (i = j = 0; i < nxy; i++) {
@@ -510,11 +510,12 @@ void	compute_robust_weight(float *resid, float *weight, int nxy, double *scale)
 
 	qsort ((void *)weight, (size_t)j, sizeof(float), GMT_comp_float_asc);
 
+	j2 = j / 2;
 	if (j%2) {
-		mad = weight[j/2];
+		mad = weight[j2];
 	}
 	else {
-		mad = 0.5 *(weight[j/2] + weight[j/2 - 1]);
+		mad = 0.5 *(weight[j2] + weight[j2 - 1]);
 	}
 
 	/* Adjust mad to equal Gaussian sigma:  */
@@ -538,7 +539,7 @@ void	compute_robust_weight(float *resid, float *weight, int nxy, double *scale)
 void	write_model_parameters(double *gtd, int n_model)
 {
 	int	i;
-	char	pbasis[10][12];
+	char	pbasis[10][16];
 
 	sprintf(pbasis[0], "Mean\0");
 	sprintf(pbasis[1], "X\0");
@@ -552,7 +553,7 @@ void	write_model_parameters(double *gtd, int n_model)
 	sprintf(pbasis[9], "P3(y)\0");
 
 	sprintf(format, "Coefficient fit to %%s:  %s\n\0", gmtdefs.d_format);
-	for(i = 0; i < n_model; i++) fprintf(stderr, format, pbasis[i], gtd[i]);
+	for(i = 0; i < n_model; i++) fprintf (stderr, format, pbasis[i], gtd[i]);
 
 	return;
 }
