@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)grdvolume.c	3.24  11/10/99
+ *	$Id: grdvolume.c,v 1.4 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * grdvolume reads a 2d binary gridded grdfile, and calculates the volume
@@ -24,12 +24,11 @@
  * Author:	Paul Wessel
  * Date:	23-SEP-1997
  * Revised:	02-JUN-1999
+ * Version:	3.4
  */
 
 #include "gmt.h"
 
-float *f;
-
 void SW_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da);
 void NE_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da);
 void SE_triangle (float f[], int ij, int nx, BOOLEAN triangle, double *dv, double *da);
@@ -51,6 +50,8 @@ main (int argc, char **argv)
 	double take_out, west, east, south, north, dv, da, cval = 0.0, x_size, y_size, m_pr_deg, x_fact, shift = 0.0, fact = 1.0;
 	double *area, *vol, *height, clow, chigh, deltac, base, this_base, small;
 	
+	float *f;
+
 	char *grdfile = CNULL, format[BUFSIZ], map_units = '\0';
 	
 	struct GRD_HEADER grd;
@@ -102,10 +103,8 @@ main (int argc, char **argv)
 					break;
 			}
 		}
-		else if (access ((grdfile = argv[i]), R_OK)) {
-			fprintf (stderr, "%s: Cannot open file %s\n", GMT_program, grdfile);
-			exit (EXIT_FAILURE);
-		}
+		else
+			grdfile = argv[i];
 	}
 	
 	if (argc == 1 || GMT_quick) {
@@ -114,18 +113,18 @@ main (int argc, char **argv)
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr, "	<grdfile> is the name of the 2-D binary data set\n");
+		fprintf (stderr, "\t<grdfile> is the name of the 2-D binary data set\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "	-C find area and volume inside the <cval> contour\n");
-		fprintf (stderr, "	   OR search using all contours from low to high\n");
-		fprintf (stderr, "	   [Default returns entire area and volume of grid]\n");
-		fprintf (stderr, "	-L Add volume from <base> up to contour [Default is from contour and up only]\n");
-		fprintf (stderr, "	-S Convert degrees to m, append k for km [Default is Cartesian]\n");
-		fprintf (stderr, "	-T Use curvature rather than maximum to find best contour value\n");
+		fprintf (stderr, "\t-C find area and volume inside the <cval> contour\n");
+		fprintf (stderr, "\t   OR search using all contours from low to high\n");
+		fprintf (stderr, "\t   [Default returns entire area and volume of grid]\n");
+		fprintf (stderr, "\t-L Add volume from <base> up to contour [Default is from contour and up only]\n");
+		fprintf (stderr, "\t-S Convert degrees to m, append k for km [Default is Cartesian]\n");
+		fprintf (stderr, "\t-T Use curvature rather than maximum to find best contour value\n");
 		GMT_explain_option ('R');
 		GMT_explain_option ('V');
-		fprintf (stderr, "	   Append l for listing of all results (when contour search is selected)\n");
-		fprintf (stderr, "	-Z Subract <shift> and then multiply data by <fact> before processing [1/0]\n");
+		fprintf (stderr, "\t   Append l for listing of all results (when contour search is selected)\n");
+		fprintf (stderr, "\t-Z Subract <shift> and then multiply data by <fact> before processing [1/0]\n");
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -343,15 +342,15 @@ main (int argc, char **argv)
 
 	if (full) {
 		sprintf (format, "%s\t%s\t%s\t%s\n\0", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		for (c = 0; c < n_contours; c++) printf (format, clow + c * deltac, area[c], vol[c], height[c]);
+		for (c = 0; c < n_contours; c++) fprintf (GMT_stdout, format, clow + c * deltac, area[c], vol[c], height[c]);
 	}
 	else if (contour) {
         	sprintf (format, "%s\t%s\t%s\t%s\n\0", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		printf (format, clow + c * deltac, area[c], vol[c], height[c]);
+		fprintf (GMT_stdout, format, clow + c * deltac, area[c], vol[c], height[c]);
 	}
 	else {
         	sprintf (format, "%s\t%s\t%s\n\0", gmtdefs.d_format, gmtdefs.d_format, gmtdefs.d_format);
-		printf (format, area[c], vol[c], height[c]);
+		fprintf (GMT_stdout, format, area[c], vol[c], height[c]);
 	}
 
 	GMT_free ((void *)f);
