@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)nearneighbor.c	2.70  10/29/99
+ *	$Id: nearneighbor.c,v 1.5 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * Based on a specified grid size, nearneighbor reads an xyz file and
@@ -27,23 +27,20 @@
  * constraint on the other side of the periodic boundary.
  *
  * Author:	Paul Wessel
- * Date:	02-JUN-1999
- * Version:	3.3
+ * Date:	14-JUL-2000
+ * Version:	3.4
  */
  
 #include "gmt.h"
-#include "gmt_boundcond.h"
-
-float *grd;
 
 struct NODE {	/* Structure with point id and distance pairs for all sectors */
 	float *distance;	/* Distance of nearest datapoint to this node per sector */
 	int *datum;		/* Point id of this data point */
-} **grid_node;
+};
 
 struct POINT {	/* Structure with input data constraints */
 	float x, y, z, w;
-} *point;
+};
 
 struct NODE *add_new_node(int n);
 void assign_node (struct NODE **node, int n_sector, int sector, double distance, int id);
@@ -51,17 +48,18 @@ void assign_node (struct NODE **node, int n_sector, int sector, double distance,
 main (int argc, char **argv)
 {
 	int i, j, k, ij, i0, j0, *di, dj, n_sectors = 4, sector, n, n_alloc = 5 * GMT_CHUNK, n_fields, nx_2;
-	int ix, iy, n_set, n_almost, n_none, n_files = 0, n_args, fno, one_or_zero, n_expected_fields, pad[4], distance_flag = 0;
-	int max_di, actual_max_di, ii, jj, n_req;
+	int n_set, n_almost, n_none, n_files = 0, n_args, fno, one_or_zero, n_expected_fields, pad[4], distance_flag = 0;
+	int max_di, actual_max_di, ii, jj, n_req, x_wrap, y_wrap;
 
-	BOOLEAN go, error = FALSE, done = FALSE, first = TRUE, nofile = TRUE;
-	BOOLEAN set_empty = FALSE, weighted = FALSE, skip, wrap_180;
+	BOOLEAN go, error = FALSE, done = FALSE, nofile = TRUE;
+	BOOLEAN set_empty = FALSE, weighted = FALSE, wrap_180, replicate_x, replicate_y;
 
 	double radius = 0.0, weight, weight_sum, *x0, *y0, dx, dy, delta, distance, factor;
 	double *in, *shrink, km_pr_deg, x_left, x_right, y_top, y_bottom, offset, xinc2, yinc2, idx, idy;
-	double half_y_width, y_width, half_x_width, x_width;
+	double half_y_width, y_width, half_x_width, x_width, three_over_radius;
+
+	float empty = 0.0, *grd;
 
-	float empty = 0.0;
 	char *outfile = CNULL, line[BUFSIZ];
 
 	FILE *fp = NULL;
@@ -70,6 +68,10 @@ main (int argc, char **argv)
 	
 	struct GMT_EDGEINFO edgeinfo;
 
+	struct NODE **grid_node;
+	
+	struct POINT  *point;
+	
 	argc = GMT_begin (argc, argv);
 	
 	GMT_boundcond_init (&edgeinfo);
@@ -115,7 +117,12 @@ main (int argc, char **argv)
 					outfile = &argv[i][2];
 					break;
 				case 'L':
-					error += GMT_boundcond_parse (&edgeinfo, &argv[i][2]);
+					if (argv[i][2]) {
+						error += GMT_boundcond_parse (&edgeinfo, &argv[i][2]);
+						if (edgeinfo.gn) GMT_geographic_in = TRUE;
+					}
+					else
+						GMT_geographic_in = TRUE;
 					break;
 				case 'N':
 					n_sectors = atoi (&argv[i][2]);
@@ -141,31 +148,31 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "nearneighbor %s - A \"Nearest neighbor\" gridding algorithm\n\n", GMT_VERSION);
 		fprintf(stderr, "usage: nearneighbor [xyzfile(s)] -G<out_grdfile> -I<dx>[m|c][/<dy>[m|c]]\n");
-		fprintf(stderr, "	-N<sectors> -R<west/east/south/north> -S<radius>[m|c|k|K] [-E<empty>] [-F]\n");
-		fprintf(stderr, "	[-H ] [-L<flags>] [-V ] [-W] [-:] [-bi[s][<n>]]\n\n");
+		fprintf(stderr, "\t-N<sectors> -R<west/east/south/north> -S<radius>[m|c|k|K] [-E<empty>] [-F]\n");
+		fprintf(stderr, "\t[-H ] [-L[<flags>]] [-V ] [-W] [-:] [-bi[s][<n>]]\n\n");
 		if (GMT_quick) exit (EXIT_FAILURE);
-		fprintf(stderr, "	-G name of output grid.\n");
-		fprintf(stderr, "	-I sets the grid spacing for the grid.  Append m for minutes, c for seconds.\n");
-		fprintf(stderr, "	-N sets number of sectors. Default is quadrant search [4].\n");
+		fprintf(stderr, "\t-G name of output grid.\n");
+		fprintf(stderr, "\t-I sets the grid spacing for the grid.  Append m for minutes, c for seconds.\n");
+		fprintf(stderr, "\t-N sets number of sectors. Default is quadrant search [4].\n");
 		GMT_explain_option ('R');
-		fprintf(stderr, "	-S sets search radius in -R, -I units; append m or c for minutes or seconds.\n");
-		fprintf(stderr, "	   Append k for km (implies -R,-I in degrees), use flat Earth approximation.\n");
-		fprintf(stderr, "	   Append K for km (implies -R,-I in degrees), use great circle distances.\n");
-		fprintf(stderr, "	   \n");
+		fprintf(stderr, "\t-S sets search radius in -R, -I units; append m or c for minutes or seconds.\n");
+		fprintf(stderr, "\t   Append k for km (implies -R,-I in degrees), use flat Earth approximation.\n");
+		fprintf(stderr, "\t   Append K for km (implies -R,-I in degrees), use great circle distances.\n");
 		fprintf(stderr, "\n\tOPTIONS:\n");
-		fprintf(stderr, "	-E value to use for empty nodes [Default is NaN].\n");
-		fprintf(stderr, "	-F Force pixel registration [Default is gridline registration].\n");
+		fprintf(stderr, "\t-E value to use for empty nodes [Default is NaN].\n");
+		fprintf(stderr, "\t-F Force pixel registration [Default is gridline registration].\n");
 		GMT_explain_option ('H');
-		fprintf(stderr, "	-L sets boundary conditions.  <flags> can be either\n");
-		fprintf(stderr, "	   g for geographic boundary conditions, or one or both of\n");
-		fprintf(stderr, "	   x for periodic boundary conditions on x\n");
-		fprintf(stderr, "	   y for periodic boundary conditions on y\n");
+		fprintf(stderr, "\t-L sets boundary conditions.  <flags> can be either\n");
+		fprintf(stderr, "\t   g for geographic boundary conditions, or one or both of\n");
+		fprintf(stderr, "\t   x for periodic boundary conditions on x\n");
+		fprintf(stderr, "\t   y for periodic boundary conditions on y\n");
+		fprintf (stderr, "\t   If no flag is given, x is assumed to be longitude and periodic in 360.\n");
 		GMT_explain_option ('V');
-		fprintf(stderr, "	-W input file has observation weights in 4th column.\n");
+		fprintf(stderr, "\t-W input file has observation weights in 4th column.\n");
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf(stderr, "	   Default is 3 (or 4 if -W is set) columns\n");
+		fprintf(stderr, "\t   Default is 3 (or 4 if -W is set) columns\n");
 		GMT_explain_option ('.');
 		
 		exit (EXIT_FAILURE);
@@ -210,12 +217,10 @@ main (int argc, char **argv)
 		fprintf (stderr, "%s: Expects %d-column %s-precision binary data\n", GMT_program, GMT_io.ncol[0], type[GMT_io.single_precision[0]]);
 	}
 
-	GMT_grd_RI_verify (&header);
+	GMT_grd_RI_verify (&header, 1);
 
-	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : 3 + weighted;
 	
-        ix = (gmtdefs.xy_toggle);       iy = 1 - ix;
-        
         if (n_files > 0)
         	nofile = FALSE;
         else
@@ -281,6 +286,10 @@ main (int argc, char **argv)
 	half_x_width = 0.5 * x_width;			half_y_width = 0.5 * y_width;
 	nx_2 = edgeinfo.nxp / 2;
 	n = 0;
+	replicate_x = (edgeinfo.nxp && !header.node_offset);	/* Gridline registration has duplicate column */
+	replicate_y = (edgeinfo.nyp && !header.node_offset);	/* Gridline registration has duplicate row */
+	x_wrap = header.nx - 1;			/* Add to node index to go to right column */
+	y_wrap = (header.ny - 1) * header.nx;	/* Add to node index to go to bottom row */
 	
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over input files, if any */
 		if (!nofile && argv[fno][0] == '-') continue;
@@ -299,136 +308,103 @@ main (int argc, char **argv)
 		
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 		
-		if (gmtdefs.io_header) {
-			for (i = 0; i < gmtdefs.n_header_recs; i++) {
-				fgets (line, BUFSIZ, fp);
-				if (first) printf ("%s", line);
-			}
-			first = FALSE;
-		}
+		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 
-		n_fields = GMT_input (fp, &n_expected_fields, &in);
-			
-		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 	
-			skip = FALSE;
-
 			if (GMT_io.status & GMT_IO_MISMATCH) {
 				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
 				exit (EXIT_FAILURE);
 			}
 
-			if (in[ix] < x_left || in[ix] > x_right) skip = TRUE;
-			if (in[iy] < y_bottom || in[iy] > y_top) skip = TRUE;
+			if (in[0] < x_left || in[0] > x_right) continue;
+			if (in[1] < y_bottom || in[1] > y_top) continue;
 		
-			if (!skip) {
-				point[n].x = (float)in[ix];
-				point[n].y = (float)in[iy];
-				point[n].z = (float)in[2];
-				if (weighted) point[n].w = (float)in[3];
+			point[n].x = (float)in[0];
+			point[n].y = (float)in[1];
+			point[n].z = (float)in[2];
+			if (weighted) point[n].w = (float)in[3];
 		
-				/* Find indeces of the node closest to this data point */
+			/* Find indeces of the node closest to this data point */
 
-				i0 = (int)floor (((in[ix] - header.x_min) * idx) + offset);
-				j0 = (int)floor (((header.y_max - in[iy]) * idy) + offset);
+			i0 = (int)floor (((in[0] - header.x_min) * idx) + offset);
+			j0 = (int)floor (((header.y_max - in[1]) * idy) + offset);
 		
-				/* Loop over all nodes within radius of this node */
+			/* Loop over all nodes within radius of this node */
 
-				for (j = j0 - dj; j <= (j0 + dj); j++) {
+			for (j = j0 - dj; j <= (j0 + dj); j++) {
 					
-					wrap_180 = FALSE;
+				jj = j;
+				if (GMT_y_out_of_bounds (&jj, &header, &edgeinfo, &wrap_180)) continue;	/* Outside y-range */
 
-					if (j < 0) {	/* Depending on BC's we wrap around or skip */
-						if (edgeinfo.gn) {	/* N Polar condition */
-							jj = abs (j) - header.node_offset;
-							wrap_180 = TRUE;
-						}
-						else if (edgeinfo.nyp) {	/* Periodic in y */
-							jj = j + edgeinfo.nyp;
-						}
-						else
-							continue;
+				for (i = i0 - di[jj]; i <= (i0 + di[jj]); i++) {
+
+					ii = i;
+					if (GMT_x_out_of_bounds (&ii, &header, &edgeinfo, wrap_180)) continue;	/* Outside x-range */ 
+
+					/* Here, (ii,jj) is index of a node (k) inside the grid */
+
+					k = jj * header.nx + ii;
+					dx = in[0] - x0[ii];	dy = in[1] - y0[jj];
+
+					/* Check for wrap-around in x or y.  This should only occur if the
+					   search radius is larger than 1/2 the grid width/height so that
+					   the shortest distance is going through the periodic boundary.
+					   For longitudes the dx obviously cannot exceed 180 (half_x_width)
+					   since we could then go the other direction instead.
+					*/
+					if (edgeinfo.nxp && fabs (dx) > half_x_width) dx -= copysign (x_width, dx);
+					if (edgeinfo.nyp && fabs (dy) > half_y_width) dy -= copysign (y_width, dy);
+
+					switch (distance_flag) {	/* Take different action depending on how we want distances calculated */
+
+						case 0:		/* Cartesian distance */
+
+							distance = hypot (dx, dy);
+							break;
+						case 1:		/* Flat Earth Approximation */
+
+							distance = km_pr_deg * hypot (dx * shrink[jj], dy);
+							break;
+						case 2:		/* Full spherical calculation */
+
+							distance = km_pr_deg * GMT_great_circle_dist (x0[ii], y0[jj], in[0], in[1]);
+							break;
+						default:
+							break;
 					}
-					else if (j >= header.ny) {	/* Depending on BC's we wrap around or skip */
-						if (edgeinfo.gs) {	/* S Polar condition */
-							jj = j - 2 + header.node_offset;
-							wrap_180 = TRUE;
-						}
-						else if (edgeinfo.nyp) {	/* Periodic in y */
-							jj = j - edgeinfo.nyp;
-						}
-						else
-							continue;
+
+					if (distance > radius) continue;	/* Data constraint is too far from this node */
+
+					/* OK, this point should constrain this node.  Calculate which sector and assign the value */
+
+					sector = ((int)((d_atan2 (dy, dx) + M_PI) * factor)) % n_sectors;
+					assign_node (&grid_node[k], n_sectors, sector, distance, n);
+
+					/* With periodic, gridline-registrered grids there are duplicate rows and/or columns
+					   so we may have to assign the point to more than one node.  The next section deals
+					   with this situation.
+					*/
+
+					if (replicate_x) {	/* Must check if we have to replicate a column */
+						if (ii == 0) 	/* Must replicate left to right column */
+							assign_node (&grid_node[k+x_wrap], n_sectors, sector, distance, n);
+						else if (ii == edgeinfo.nxp)	/* Must replicate right to left column */
+							assign_node (&grid_node[k-x_wrap], n_sectors, sector, distance, n);
 					}
-					else
-						jj = j;
-
-					for (i = i0 - di[jj]; i <= (i0 + di[jj]); i++) {
-
-						if (i < 0) {	/* Depending on BC's we wrap around or skip */
-							if (edgeinfo.nxp) {	/* Periodic in x */
-								ii = abs (i) - header.node_offset;
-							}
-							else
-								continue;
-						}
-						else if (i >= header.nx) {	/* Depending on BC's we wrap around or skip */
-							if (edgeinfo.nxp) {	/* Periodic in x */
-								ii = i - edgeinfo.nxp;
-							}
-							else
-								continue;
-						}
-						else
-							ii = i;
-
-						if (wrap_180) ii = (ii + nx_2) % edgeinfo.nxp;
-						k = jj * header.nx + ii;
-						dx = in[ix] - x0[ii];	dy = in[iy] - y0[jj];
-
-						/* Check for wrap-around in x or y */
-						if (edgeinfo.nxp && fabs (dx) > half_x_width) dx -= copysign (x_width, dx);
-						if (edgeinfo.nyp && fabs (dy) > half_y_width) dy -= copysign (y_width, dy);
-
-						switch (distance_flag) {	/* Take different action depending on how we want distances calculated */
-
-							case 0:		/* Cartesian distance */
-
-								distance = hypot (dx, dy);
-								break;
-							case 1:		/* Flat Earth Approximation */
-
-								distance = km_pr_deg * hypot (dx * shrink[jj], dy);
-								break;
-							case 2:		/* Full spherical calculation */
-
-								distance = km_pr_deg * GMT_great_circle_dist (x0[ii], y0[jj], in[ix], in[iy]);
-								break;
-							default:
-								break;
-						}
-
-						if (distance > radius) continue;
-
-						sector = ((int)((d_atan2 (dy, dx) + M_PI) * factor)) % n_sectors;
-						assign_node (&grid_node[k], n_sectors, sector, distance, n);
-
-						if (edgeinfo.nxp && ii == 0) {	/* Must replicate to redundant column */
-							assign_node (&grid_node[k+header.nx-1], n_sectors, sector, distance, n);
-							if (edgeinfo.nyp && jj == 0)	/* Must replicate to redundant row */
-								assign_node (&grid_node[header.nx-1], n_sectors, sector, distance, n);
-						}
-						if (edgeinfo.nyp && jj == 0)	/* Must replicate to redundant row */
-							assign_node (&grid_node[ii], n_sectors, sector, distance, n);
+					if (replicate_y) {	/* Must check if we have to replicate a row */
+						if (jj == 0)	/* Must replicate top to bottom row */
+							assign_node (&grid_node[k+y_wrap], n_sectors, sector, distance, n);
+						else if (jj == edgeinfo.nyp)	/* Must replicate bottom to top row */
+							assign_node (&grid_node[k-y_wrap], n_sectors, sector, distance, n);
 					}
 				}
-				n++;
-				if (n == n_alloc) {
-					n_alloc += GMT_CHUNK;
-					point = (struct POINT *) GMT_memory ((void *)point, (size_t)n_alloc, sizeof (struct POINT), GMT_program);
-				}
 			}
-			n_fields = GMT_input (fp, &n_expected_fields, &in);
+			n++;
+			if (n == n_alloc) {
+				n_alloc += GMT_CHUNK;
+				point = (struct POINT *) GMT_memory ((void *)point, (size_t)n_alloc, sizeof (struct POINT), GMT_program);
+			}
 		}
 		if (fp != GMT_stdin) GMT_fclose (fp);
 	}
@@ -441,26 +417,30 @@ main (int argc, char **argv)
 	n_set = n_almost = n_none = 0;
 
 	if (!set_empty) empty = GMT_f_NaN;
+	three_over_radius = 3.0 / radius;
 	
 	for (j = ij = 0; j < header.ny; j++) {
 		for (i = 0; i < header.nx; i++, ij++) {
-			grd[ij] = empty;
 
-			if (!grid_node[ij]) {	/* No nearest neighbors */
+			if (!grid_node[ij]) {	/* No nearest neighbors, set to empty and goto next node */
 				n_none++;
+				grd[ij] = empty;
 				continue;
 			}
 
 			for (k = 0, go = TRUE; go && k < n_sectors; k++) if (grid_node[ij]->datum[k] < 0) go = FALSE;
-			if (!go) { 	/* Not full set of neighbors */
+			if (!go) { 	/* Not full set of neighbors in all sectors, set to empty and goto next node */
 				n_almost++;
+				grd[ij] = empty;
 				continue;
 			}
 
+			/* OK, here we have enough data and need to calculate the weighted value */
+
 			n_set++;
-			weight_sum = grd[ij] = 0.0;	/* Replace the empty so that we may compute a sum */
+			weight_sum = grd[ij] = 0.0;	/* Initialize sums */
 			for (k = 0; k < n_sectors; k++) {
-				delta = 3.0 * grid_node[ij]->distance[k] / radius;
+				delta = three_over_radius * grid_node[ij]->distance[k];
 				weight = 1.0 / (1.0 + delta * delta);	/* This is distance weight */
 				if (weighted) weight *= point[grid_node[ij]->datum[k]].w;	/* This is observation weight */
 				grd[ij] += (float)(weight * point[grid_node[ij]->datum[k]].z);
