@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)project.c	2.51  10/18/99
+ *	$Id: project.c,v 1.4 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * project.c
@@ -31,7 +31,7 @@
 				and binary i/o.  Old -M renamed -Q.
 		03-NOV-1998	PW: Can read any number of data columns; z in -Fz refers to
 				all these columns in the output.
-
+   Version:	3.4		PW: Fixed problem with small circle distances
 */
 
 #include "gmt.h"
@@ -40,14 +40,13 @@ struct DATA {
         double  a[6];
 	double *z;
 	char *t;
-} *p_data;
+};
 
-int     output_choice[7];
 int	compare_distances(const void *point_1, const void *point_2);
 int	do_flat_earth();
 int	solve_right_spherical_triangle();
 int	sphere_azim_dist();
-void	oblique_setup(double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads);
+double	oblique_setup(double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads);
 void	oblique_transform(double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c, int rads);
 void	make_euler_matrix(double *p, double *e, double *theta, int rads);
 void	matrix_3v(double *a, double *x, double *b);
@@ -56,15 +55,13 @@ void	sphere_project_setup(double alat, double alon, double *a, double blat, doub
 void	flat_project_setup(double alat, double alon, double blat, double blon, double plat, double plon, double *azim, double *e, int two_pts, BOOLEAN pole_set);
 void	copy_text_from_col3 (char *line, char *z_cols);
 
-
-int	output_choice[7];
-
 main (int argc, char **argv)
 {	
-	int	i, j, k, n_definitions, n_outputs, n_used, n_total_read, ix, iy, n_alloc = GMT_CHUNK;
+	int	i, j, k, n_definitions, n_outputs, n_used, n_total_read, n_alloc = GMT_CHUNK;
 	int	nc, ne, np, nl, nw, n_files = 0, fno, n_args, n_fields, n_expected_fields, n_total_used = 0, n_z = 0, n_items;
+	int     output_choice[7];
 
-	double	xx, yy, x_a, y_a, x_b, y_b, x_p, y_p, d_to_km, cos_theta, sin_theta;
+	double	xx, yy, x_a, y_a, x_b, y_b, x_p, y_p, d_to_km, cos_theta, sin_theta, sin_lat_to_pole = 1.0;
 	double	theta, d_inc = 0.0, d_along, *in, *out = (double *)NULL;
 	
 	double	azimuth = 0.0, l_min = 0.0, l_max = 0.0, w_min = 0.0, w_max = 0.0;
@@ -78,6 +75,8 @@ main (int argc, char **argv)
 	
 	char	modifier, record_str[BUFSIZ], heading[7][64], txt_a[32], txt_b[32];
 	
+	struct DATA *p_data;
+	
 	argc = GMT_begin (argc, argv);
 
 	check_length = check_width = dateline = error = find_new_point = flat_earth = generate = greenwich = FALSE;
@@ -86,6 +85,7 @@ main (int argc, char **argv)
 	x_a = x_b = x_p = y_a = y_b = y_p = 0.0;
 	n_definitions = 0;
 	n_outputs = 0;
+	GMT_geographic_in = GMT_geographic_out = TRUE;
 	j = 1;
 	for (i = 0; i < 7; i++) output_choice[i] = 0;
 
@@ -194,6 +194,7 @@ main (int argc, char **argv)
 					break;
 				case 'N':
 					flat_earth = TRUE;
+					GMT_geographic_in = GMT_geographic_out = FALSE;
 					break;
 				case 'S':
 					sort_output = TRUE;
@@ -381,7 +382,7 @@ main (int argc, char **argv)
 	}
 	else {
 		if (pole_set) {
-			oblique_setup(y_p, x_p, pole, y_a, x_a, center, pole_set, rads);
+			sin_lat_to_pole = oblique_setup(y_p, x_p, pole, y_a, x_a, center, pole_set, rads);
 		}
 		else {
 			sphere_project_setup(y_a, x_a, a, y_b, x_b, b, &azimuth, pole, center, two_points, rads);
@@ -446,7 +447,7 @@ main (int argc, char **argv)
 			yy = y_a;
 			GMT_geo_to_cart(&yy, &xx, x, rads);
 			for (i = 0; i < n_used; i++) {
-				theta = p_data[i].a[2];
+				theta = p_data[i].a[2] / sin_lat_to_pole;
 				make_euler_matrix(pole, e, &theta, rads);
 				matrix_3v(e,x,xt);
 				GMT_cart_to_geo(&yy, &xx, xt, rads);
@@ -469,7 +470,7 @@ main (int argc, char **argv)
 		/* Now output generated track */
 
 		if (!GMT_io.binary[1]) {
-			if (gmtdefs.io_header) printf("lon\tlat\tdist\n");
+			if (gmtdefs.io_header) fprintf (GMT_stdout, "lon\tlat\tdist\n");
 	
 			for (i = 0; i < n_used; i++) {
 				for (j = 0; j < n_outputs; j++) out[j] = p_data[i].a[output_choice[j]];
@@ -480,7 +481,6 @@ main (int argc, char **argv)
 	
 	else {	/* Must read input file */
 
-		ix = (gmtdefs.xy_toggle) ? 1 : 0;	iy = 1 - ix;		/* Set up which columns have x and y */
 		n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
 
 		if (n_files > 0)
@@ -508,14 +508,14 @@ main (int argc, char **argv)
 			if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
 		
 			if (gmtdefs.io_header) {
-				fgets (record_str, BUFSIZ, fp);
-				sscanf(record_str, "%s %s %s", heading[0], heading[1], heading[6]);
-				if (! (heading[6]) ) strcpy(heading[6],"Z");
-				strcpy(heading[2],"p");
-				strcpy(heading[3],"q");
-				strcpy(heading[4],"r");
-				strcpy(heading[5],"s");
-				for (i = 1; i < gmtdefs.n_header_recs; i++) fgets (record_str, BUFSIZ, fp);
+				GMT_fgets (record_str, BUFSIZ, fp);
+				sscanf (record_str, "%s %s %s", heading[0], heading[1], heading[6]);
+				if (! (heading[6]) ) strcpy (heading[6],"Z");
+				strcpy (heading[2],"p");
+				strcpy (heading[3],"q");
+				strcpy (heading[4],"r");
+				strcpy (heading[5],"s");
+				for (i = 1; i < gmtdefs.n_header_recs; i++) GMT_fgets (record_str, BUFSIZ, fp);
 			}
 	
 			n_fields = GMT_input (fp, &n_expected_fields, &in);
@@ -539,8 +539,8 @@ main (int argc, char **argv)
 						exit (EXIT_FAILURE);
 					}
 
-					xx = in[ix];
-					yy = in[iy];
+					xx = in[0];
+					yy = in[1];
 	
 					n_total_read ++;
 			
@@ -630,9 +630,9 @@ main (int argc, char **argv)
 					if (first && gmtdefs.io_header) {
 						for (j = 0; j < n_outputs; j++) {
 							if (output_choice[j] == -1)
-								printf("%s", heading[6]);
+								fprintf (GMT_stdout, "%s", heading[6]);
 							else
-								printf("%s", heading[output_choice[j]]);
+								fprintf (GMT_stdout, "%s", heading[output_choice[j]]);
 							(j == (n_outputs - 1)) ? fprintf (GMT_stdout, "\n") : fprintf (GMT_stdout, "\t");
 						}
 						first = FALSE;
@@ -704,7 +704,7 @@ int	compare_distances(const void *point_1, const void *point_2)
 		return (0);
 }
 
-void	oblique_setup(double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads)
+double	oblique_setup(double plat, double plon, double *p, double clat, double clon, double *c, int c_given, int rads)
 {
 	/* routine sets up a unit 3-vector p, the pole of an 
 	   oblique projection, given plat, plon, the position 
@@ -721,6 +721,7 @@ void	oblique_setup(double plat, double plon, double *p, double clat, double clon
 	   from degrees to radians.  */
 	
 	double	s[3];  /* s points to the south pole  */
+	double cp, sin_lat_to_pole;
 
 	s[0] = s[1] = 0.0;
 	s[2] = -1.0;
@@ -732,6 +733,9 @@ void	oblique_setup(double plat, double plon, double *p, double clat, double clon
 	}
 	GMT_cross3v(p, s, c);
 	GMT_normalize3v(c);
+	cp = GMT_dot3v (p, s);
+	sin_lat_to_pole = d_sqrt (1.0 - cp * cp);
+	return (sin_lat_to_pole);
 }
 
 void	oblique_transform(double xlat, double xlon, double *x_t_lat, double *x_t_lon, double *p, double *c, int rads)
@@ -825,7 +829,7 @@ void	matrix_2v(double *a, double *x, double *b)
 	b[1] = x[0]*a[2] + x[1]*a[3];
 }
 
-void	sphere_project_setup(double alat, double alon, double *a, double blat, double blon, double *b, double *azim, double *p, double *c, int two_pts, int rads)
+void sphere_project_setup(double alat, double alon, double *a, double blat, double blon, double *b, double *azim, double *p, double *c, int two_pts, int rads)
 {
 	/* routine to initialize a pole vector, p, and a central meridian 
 	   normal vector, c, for use in projecting points onto a great circle.
