@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)pscoast.c	3.79  11/29/99
+ *	$Id: pscoast.c,v 1.3 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * pscoast can draw shorelines, paint the landward and/or seaward side of
@@ -46,8 +46,8 @@
  * 
  *
  * Author:	Paul Wessel
- * Date:	24-MAR-1984
- * Version:	3.1	Added clipping and dumping as options
+ * Date:	24-JUN-2000
+ * Version:	3.4
  *
  */
 
@@ -57,11 +57,6 @@
 int debug_bin = -1;
 #endif
 
-struct GMT_SHORE c;
-struct GMT_BR b, r;
-
-char *shore_resolution[5] = {"full", "high", "intermediate", "low", "crude"};
-
 void recursive_painting (int k0, int np, struct POL *p, int level, int start_level, struct GMT_FILL fill[]);
 void recursive_clipping (int k0, int np, struct POL *p, int level, int start_level, int *n_pieces);
 
@@ -75,7 +70,7 @@ main (int argc, char **argv)
 	BOOLEAN	error = FALSE, set_lake_fill = FALSE, draw_river = FALSE, shift = FALSE, need_coast_base, recursive;
 	BOOLEAN	greenwich = FALSE, draw_coast = FALSE, fill_ocean = FALSE, fill_land = FALSE, possibly_donut_hell = FALSE;
 	BOOLEAN clobber_background, draw_border = FALSE, paint_polygons = FALSE, donut, fill_in_use = FALSE;
-	BOOLEAN draw_scale = FALSE, gave_xy = FALSE, fancy = FALSE, donut_hell = FALSE;
+	BOOLEAN draw_scale = FALSE, gave_xy = FALSE, fancy = FALSE, donut_hell = FALSE, world_map_save, did_clip = FALSE;
 	BOOLEAN clip_ocean = FALSE, clip_land = FALSE, end_of_clip = FALSE, clipping = FALSE, dumping = FALSE;
 	
 	double	west = 0.0, east = 0.0, south = 0.0, north = 0.0, bin_x[5], bin_y[5];
@@ -83,11 +78,13 @@ main (int argc, char **argv)
 	double *xtmp, *ytmp, min_area = 0.0, x0, y0, scale_lat, length, step = 0.01;
 	double anti_x, anti_y, x_0, y_0, x_c, y_c, dist, out[2];
 	
-	char res = 'l', key[5], *string, measure = '\0';
-	
+	char res = 'l', key[5], *string, measure = '\0', comment[128];
+	char *shore_resolution[5] = {"full", "high", "intermediate", "low", "crude"};
+
 	struct GMT_PEN coast, pen, bpen[N_BLEVELS], rpen[N_RLEVELS];
 	struct GMT_FILL fill[5];	/* Colors for (0) water, (1) land, (2) lakes, (3) islands in lakes, (4) lakes in islands in lakes */
-	
+	struct GMT_SHORE c;
+	struct GMT_BR b, r;
 	struct POL *p;
 		
 	GMT_init_fill (&fill[0], 255, 255, 255);	/* Default Ocean color */
@@ -313,9 +310,9 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {	/* Display usage */
 		fprintf (stderr,"pscoast %s - Plot continents, shorelines, rivers, and borders on maps\n\n", GMT_VERSION);
 		fprintf (stderr,"usage: pscoast -J<params> -R<west>/<east>/<south>/<north> [-A<min_area>[/<min_level>/<max_level>]] [-B<tickinfo>]\n");
-		fprintf (stderr, "	 [-C[<fill>]] [-D<resolution>] [-Eaz/el] [-G[<fill>]] [-I<feature>[/<pen>]]\n");
-		fprintf (stderr, "	 [-K] [-L[f][x]<lon0>/<lat0>/<slat>/<length>[m|n|k]] [-M[<flag>]] [-N<feature>[/<pen>]] [-O] [-P] [-Q]\n");
-		fprintf (stderr, "	 [-S<fill>] [-U[dx/dy/][label]] [-V] [-W[<pen>]] [-X<x_shift>] [-Y<y_shift>] [-bo[s]] [-c<ncopies>]\n");
+		fprintf (stderr, "\t[-C[<fill>]] [-D<resolution>] [-Eaz/el] [-G[<fill>]] [-I<feature>[/<pen>]]\n");
+		fprintf (stderr, "\t[-K] [-L[f][x]<lon0>/<lat0>/<slat>/<length>[m|n|k]] [-M[<flag>]] [-N<feature>[/<pen>]] [-O] [-P] [-Q]\n");
+		fprintf (stderr, "\t[-S<fill>] [-U[dx/dy/][label]] [-V] [-W[<pen>]] [-X<x_shift>] [-Y<y_shift>] [-bo[s]] [-c<ncopies>]\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
@@ -483,6 +480,8 @@ main (int argc, char **argv)
 
 	GMT_map_setup (west, east, south, north);
 
+	world_map_save = GMT_world_map;
+	
 	if (need_coast_base && GMT_init_shore (res, &c, project_info.w, project_info.e, project_info.s, project_info.n))  {
 		fprintf (stderr, "%s: %s resolution shoreline data base not installed\n", GMT_program, shore_resolution[base]);
 		need_coast_base = FALSE;
@@ -504,7 +503,7 @@ main (int argc, char **argv)
 	}
 
 	if (dumping) {
-		if (!GMT_io.binary[1]) printf ("# Data from the %s resolution GMT shoreline, borders, and rivers database\n", shore_resolution[base]);
+		if (!GMT_io.binary[1]) fprintf (GMT_stdout, "# Data from the %s resolution GMT shoreline, borders, and rivers database\n", shore_resolution[base]);
 	}
 	else {
 		ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
@@ -580,7 +579,7 @@ main (int argc, char **argv)
 			GMT_fill (xtmp, ytmp, n, &fill[0], FALSE);
 		GMT_free ((void *)xtmp);
 		GMT_free ((void *)ytmp);
-		fill_ocean = FALSE;
+		fill_ocean = FALSE; 
 	}
 	if (clipping) {
 		start_direction = (clip_land) ? 1 : -1;
@@ -599,8 +598,10 @@ main (int argc, char **argv)
 	}
 
 	if (west < 0.0 && east > 0.0 && project_info.projection > 5) greenwich = TRUE;
-	if ((360.0 - fabs (project_info.e - project_info.w) ) < c.bsize)
+	if ((360.0 - fabs (project_info.e - project_info.w) ) < c.bsize) {
 		GMT_world_map = TRUE;
+		if (project_info.projection == GNOMONIC) GMT_world_map = FALSE;
+	}
 	if (GMT_world_map && greenwich)
 		edge = project_info.central_meridian;
 	else if (!GMT_world_map && greenwich) {
@@ -630,7 +631,10 @@ main (int argc, char **argv)
 		GMT_get_shore_bin (ind, &c, min_area, min_level, max_level);
 		
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Working on block # %5d\r", GMT_program, bin);
-		if (!dumping) printf ("%% Bin # %d\n", bin);
+		if (!dumping) {
+			sprintf (comment, "Bin # %d\0", bin);
+			ps_comment (comment);
+		}
 
 		if (GMT_world_map && greenwich) {
 			left = c.bsize * (bin % c.bin_nx);	right = left + c.bsize;
@@ -664,6 +668,7 @@ main (int argc, char **argv)
 			if (clipping) {
 				for (k = level_to_be_painted; k < MAX_LEVEL - 1; k++) {
 					recursive_clipping (-1, np_new, p, k, k, &n_pieces);
+					did_clip = TRUE;
 				}
 				
 				for (k = 0; k < np_new; k++) {	/* Do any remaining interior polygons */
@@ -739,7 +744,7 @@ main (int argc, char **argv)
 	}
 	if (need_coast_base) GMT_shore_cleanup (&c);
 
-	if (clipping) ps_clipon (p[k].lon, p[k].lat, 0, GMT_no_rgb, 2);	/* Dummy to end path */
+	if (clipping && did_clip) ps_clipon (p[k].lon, p[k].lat, 0, GMT_no_rgb, 2);	/* Dummy to end path */
 
 	if (gmtdefs.verbose) fprintf (stderr, "\n");
 	
@@ -869,6 +874,7 @@ main (int argc, char **argv)
 		if (frame_info.plot) {
 			/* project_info.w = west_border;
 			project_info.e = east_border; */
+			GMT_world_map = world_map_save;
 			GMT_map_basemap ();
 		}
 	
@@ -884,7 +890,7 @@ main (int argc, char **argv)
 void recursive_painting (int k0, int np, struct POL p[], int level, int start_level, struct GMT_FILL fill[]) {
 	int k, flag;
 	BOOLEAN go;
-	char text[50];
+	char text[64];
 	if (level > MAX_LEVEL) return;
 	flag = (level == start_level) ? 1 : 0;
 	for (k = k0 + 1; k < np; k++) {
@@ -909,7 +915,7 @@ void recursive_clipping (int k0, int np, struct POL *p, int level, int start_lev
 {
 	int k, flag;
 	BOOLEAN go;
-	char text[50];
+	char text[64];
 	if (level > MAX_LEVEL) return;
 	for (k = k0 + 1; k < np; k++) {
 		if (p[k].n == 0 || p[k].level < level) continue;
