@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:   @(#)pscontour.c	2.69  11/09/99
+ *	$Id: pscontour.c,v 1.4 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /* pscontour will read a file of points in the plane, performs the
  * Delaunay triangulation, and contours these triangles.  As an option
@@ -22,9 +22,7 @@
  *
  * Author:	Paul Wessel
  * Date:	1-JAN-1995
- * Version:	3.0
- * Revised:	13-MAY-1998, for GMT 3.1
- *
+ * Version:	3.4
  */
 
 #include "gmt.h"
@@ -35,6 +33,7 @@ int delta, sum;
 #define get_direction(node_1, node_2) (((delta = (node_2) - (node_1)) == 1 || delta == -2) ? 1 : -1)
 /* Returns the id of the node common to the two edges */
 #define get_node_index(edge_1, edge_2) (((sum = (edge_1) + (edge_2)) == 1) ? 1 : ((sum == 2) ? 0 : 2))
+#define get_other_node(node1, node2) (((sum = (node1 + node2)) == 3) ? 0 : ((sum == 2) ? 1 : 2))	/* The other node needed */
 
 struct PSCONTOUR {
 	int n_alloc, nl;
@@ -63,24 +62,25 @@ struct PT {
 struct LABEL {
 	double x, y;
 	double angle;
-	char label[32];
+	char label[64];
 	struct LABEL *next_label, *prev_label;
 } *anchor, *old_label;
 	
 int get_triangle_crossings(double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, int **v, int **cindex);
 void draw_contour(double *xx, double *yy, int nn, double cval, char ctype, double cangle, int closed, double gap), plot_labels(int size, int box, int rgb[]);
 void dump_contour(double *xx, double *yy, int nn, double cval, int id, BOOLEAN interior, char *file);
+void paint_it (double x[], double y[], int n, double z);
 
 main(int argc, char **argv)
 {
-	int n, np, nx, i, j, ij, k, kk, k2, k3, way1, way2, node, n1, n2, c, rgb[3];
-	int n_alloc, box = 1, section = 0, ix, iy, *ind, *vert, *cind, n_fields, n_contours;
-	int add, close, b_rgb[3], label_font_size = 9, bad, n_expected_fields;
+	int n, np, nx, i, j, ij, k, k2, k3, node1, node2, c, rgb[3];
+	int n_alloc, box = 1, section = 0, *ind, *vert, *cind, n_fields, n_contours;
+	int add, close, b_rgb[3], label_font_size = 9, bad, n_expected_fields, low, high, last_entry, last_exit;
 	
 	BOOLEAN error = FALSE, image_them = FALSE, draw_contours = FALSE, dump = FALSE, clip = TRUE;
 	BOOLEAN fix_angle = FALSE, more, t_set = FALSE, draw_mesh = FALSE, use_cpt_colors = FALSE, use_cpt_anot = TRUE;
 	
-	double xx[3], yy[3], zz[3], xout[4], yout[4], *in, west, east, south, north, z_min;
+	double xx[3], yy[3], zz[3], xout[5], yout[5], *in, west, east, south, north;
 	double *xc, *yc, *zc, *x, *y, *z, *xp, *yp, anot_dist, label_angle = 0.0, current_contour = -DBL_MAX;
 	
 	char line[BUFSIZ], *cpt_file = CNULL, *t_file = CNULL, dfile[BUFSIZ];
@@ -215,48 +215,48 @@ main(int argc, char **argv)
 	
 	if (GMT_quick || argc == 1) {	/* Display usage */
 		fprintf (stderr,"pscontour %s - Contour xyz-data by triangulation\n\n", GMT_VERSION);
-		fprintf(stderr,"usage: pscontour <xyzfile> -C<cpt_file> -J<params> -R<west>/<east>/<south>/<north>\n");
-		fprintf (stderr, "	[-A[-][f<fontsize>][/r/g/b][a<angle>][o]] [-B<tickinfo>] [-D<dumpfile>] [-E<az>/<el>] [-G<gap>]\n");
-		fprintf (stderr, "	[-H[<nrec>]] [-I] [-K] [-L<pen>] [-M[<flag>]] [-N] [-O] [-P] [-T<indexfile>]\n");
-		fprintf (stderr, "	[-U] [-V] [-W[+]<pen>] [-X<x_shift>] [-Y<y_shift>] [-c<ncopies>] [-:] [-bi[s][<n>]] [-bo[s]]\n\n");
+		fprintf (stderr,"usage: pscontour <xyzfile> -C<cpt_file> -J<params> -R<west>/<east>/<south>/<north>\n");
+		fprintf (stderr, "\t[-A[-][f<fontsize>][/r/g/b][a<angle>][o]] [-B<tickinfo>] [-D<dumpfile>] [-E<az>/<el>] [-G<gap>]\n");
+		fprintf (stderr, "\t[-H[<nrec>]] [-I] [-K] [-L<pen>] [-M[<flag>]] [-N] [-O] [-P] [-T<indexfile>]\n");
+		fprintf (stderr, "\t[-U] [-V] [-W[+]<pen>] [-X<x_shift>] [-Y<y_shift>] [-c<ncopies>] [-:] [-bi[s][<n>]] [-bo[s]]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf(stderr,"	-C Color palette table\n");
+		fprintf (stderr,"\t-C Color palette table\n");
 		GMT_explain_option ('j');
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "	-A Annotation format information.\n");
-		fprintf (stderr, "	   Append - to disable all contour anotations\n");
-		fprintf (stderr, "	   Append f followed by desired font size in points [Default is 9].\n");
-		fprintf (stderr, "	   Append /r/g/b to change color of text box [Default is %d/%d/%d]\n", b_rgb[0], b_rgb[1], b_rgb[2]);
-		fprintf (stderr, "	   Append o to draw outline of text box [Default is no outline]\n");
-		fprintf (stderr, "	   Append a<angle> to force anotations at this fixed angle [Default follows contour]\n");
+		fprintf (stderr, "\t-A Annotation format information.\n");
+		fprintf (stderr, "\t   Append - to disable all contour anotations\n");
+		fprintf (stderr, "\t   Append f followed by desired font size in points [Default is 9].\n");
+		fprintf (stderr, "\t   Append /r/g/b to change color of text box [Default is %d/%d/%d]\n", b_rgb[0], b_rgb[1], b_rgb[2]);
+		fprintf (stderr, "\t   Append o to draw outline of text box [Default is no outline]\n");
+		fprintf (stderr, "\t   Append a<angle> to force anotations at this fixed angle [Default follows contour]\n");
 		GMT_explain_option ('b');
-		fprintf (stderr, "	-D to Dump contour lines to individual files (but see -M)\n");
-		fprintf (stderr, "	-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
-		fprintf (stderr, "	-G Gap between anotations in %s [Default = %lg]\n", GMT_unit_names[gmtdefs.measure_unit], anot_dist);
+		fprintf (stderr, "\t-D to Dump contour lines to individual files (but see -M)\n");
+		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t-G Gap between anotations in %s [Default = %lg]\n", GMT_unit_names[gmtdefs.measure_unit], anot_dist);
 		GMT_explain_option ('H');
-		fprintf (stderr, "	-I Color triangles using the cpt file\n");
+		fprintf (stderr, "\t-I Color triangles using the cpt file\n");
 		GMT_explain_option ('K');
-		fprintf (stderr, "	-L draws the triangular mesh with the specified pen\n");
-		fprintf (stderr, "	-M Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
-		fprintf (stderr, "         whose first character is <flag> ['>'].  This header also has the contour level value\n");
-		fprintf(stderr,"	-N do NOT clip contours/image at the border [Default clips]\n");
+		fprintf (stderr, "\t-L draws the triangular mesh with the specified pen\n");
+		fprintf (stderr, "\t-M Used with -D.   Create a single multiple segment file where contours are separated by a record\n");
+		fprintf (stderr, "\t   whose first character is <flag> ['>'].  This header also has the contour level value\n");
+		fprintf (stderr,"\t-N do NOT clip contours/image at the border [Default clips]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf(stderr,"	-T file with triplets of point indeces for each triangle\n");
-		fprintf(stderr,"	   [Default performs the Delauney triangulation on xyz-data]\n");
+		fprintf (stderr,"\t-T file with triplets of point indeces for each triangle\n");
+		fprintf (stderr,"\t   [Default performs the Delauney triangulation on xyz-data]\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
-		fprintf (stderr, "	-W selects contouring and sets contour pen attributes\n");
-		fprintf (stderr, "	   Use + to draw colored contours based on the cpt file\n");
+		fprintf (stderr, "\t-W selects contouring and sets contour pen attributes\n");
+		fprintf (stderr, "\t   Use + to draw colored contours based on the cpt file\n");
 		GMT_explain_option ('X');
 		GMT_explain_option ('c');
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf(stderr,"	   Default is 3 input columns.\n");
+		fprintf (stderr,"\t   Default is 3 input columns.\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
@@ -308,7 +308,7 @@ main(int argc, char **argv)
 		error++;
 	}
 	if (t_set) {
-		if ((fp_d = GMT_fopen (t_file, GMT_io.r_mode)) == NULL) {
+		if ((fp_d = fopen (t_file, GMT_io.r_mode)) == NULL) {
 		fprintf (stderr, "%s: Could not open index file %s\n", GMT_program, t_file);
 			exit (EXIT_FAILURE);
 		}
@@ -349,12 +349,10 @@ main(int argc, char **argv)
 	y = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	z = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 	
-	ix = (gmtdefs.xy_toggle) ? 1 : 0;        iy = 1 - ix;              /* Set up which columns have x and y */
-
 	n = 0;
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 3;
 	n_fields = GMT_input (fp, &n_expected_fields, &in);
 		
 	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
@@ -366,8 +364,8 @@ main(int argc, char **argv)
 
 		if (!GMT_is_dnan (in[2])) {	/* Unless z = NaN */
 		
-			x[n] = in[ix];
-			y[n] = in[iy];
+			x[n] = in[0];
+			y[n] = in[1];
 			z[n] = in[2];
 			n++;
 		
@@ -416,7 +414,7 @@ main(int argc, char **argv)
 				more = (fgets (line, BUFSIZ, fp_d) != CNULL);
 		}
 		ind = (int *) GMT_memory ((void *)ind, (size_t)ij, sizeof (int), GMT_program);
-		GMT_fclose (fp_d);
+		fclose (fp_d);
 	}
 	else	/* Do Delauney triangulation */
 			
@@ -457,7 +455,7 @@ main(int argc, char **argv)
 	GMT_setpen (&pen);
 
 	for (i = ij = 0; i < np; i++, ij += 3) {	/* For all triangles */
-	
+			
 		k = ij;
 		xx[0] = x[ind[k]];	yy[0] = y[ind[k]];	zz[0] = z[ind[k++]];
 		xx[1] = x[ind[k]];	yy[1] = y[ind[k]];	zz[1] = z[ind[k++]];
@@ -467,48 +465,105 @@ main(int argc, char **argv)
 		
 		if (image_them) {	/* Must color the triangle slices according to cpt file */
 		
-			/* First paint background color for entire triangle */
-			
-			z_min = MIN (zz[0], MIN (zz[1], zz[2]));
-			GMT_get_rgb24 (z_min, rgb);
-			if (project_info.three_D) {
-				for (k = 0; k < 3; k++) GMT_xy_do_z_to_xy (xx[k], yy[k], project_info.z_level, &xout[k], &yout[k]);
-				ps_patch (xout, yout, 3, rgb, FALSE);
+			if (nx == 0) {	/* No contours go through - easy */
+				paint_it (xx, yy, 3, zz[0]);	/* Any z will do */
 			}
-			else
-				ps_patch (xx, yy, 3, rgb, FALSE);
-				
-			/* Then loop over contours and paint the part that is higher up in the color scheme */
+			else {	/* Must paint all those slices separately */
 			
-			for (k = k2 = 0, k3 = 1; k < nx; k++, k2 += 2, k3 += 2) {
-				xout[0] = xc[k2];	yout[0] = yc[k2];
-				xout[1] = xc[k3];	yout[1] = yc[k3];
-				node = get_node_index (vert[k2], vert[k3]);
+				/* Find vertecies with the lowest and highest values */
 				
-				/* We either add one vertix and paint that triangle or we must add the other
-				   two vertices and paint a polygon.  The if-test will tell us which one */
-				   
-				if (zz[node] > zc[k2]) {	/* Add this single vertex to path */
-					xout[2] = xx[node];
-					yout[2] = yy[node];
-					GMT_get_rgb24 (zc[k2], rgb);
-					if (project_info.three_D)
-						for (kk = 0; kk < 3; kk++) GMT_xy_do_z_to_xy (xout[kk], yout[kk], project_info.z_level, &xout[kk], &yout[kk]);
-					ps_patch (xout, yout, 3, rgb, FALSE);
+				for (k = 1, low = high = 0; k < 3; k++) {
+					if (zz[k] < zz[low])   low = k;
+					if (zz[k] > zz[high]) high = k;
 				}
-				else {	/* Must add the other two vertices */
-					n1 = (node + 1) % 3;
-					n2 = (node + 2) % 3;
-					way1 = get_direction (vert[k2], vert[k3]);
-					way2 = get_direction (n1, n2);
-					if (way1 * way2 < 0) i_swap (n1, n2);
-					xout[2] = xx[n1];	yout[2] = yy[n1];
-					xout[3] = xx[n2];	yout[3] = yy[n2];
-					GMT_get_rgb24 (zc[k2], rgb);
-					if (project_info.three_D)
-						for (kk = 0; kk < 4; kk++) GMT_xy_do_z_to_xy (xout[kk], yout[kk], project_info.z_level, &xout[kk], &yout[kk]);
-					ps_patch (xout, yout, 4, rgb, FALSE);
+				
+				/* Paint the piece delimited by the low node and the first contour */
+				
+				xout[0] = xx[low];	yout[0] = yy[low];
+				node1 = get_node_index (vert[0], vert[1]);	/* Find single vertex opposing this contour segment */
+				if (node1 == low) {	/* Contour and low node make up a triangle */
+					xout[1] = xc[0];	yout[1] = yc[0];
+					xout[2] = xc[1];	yout[2] = yc[1];
+					n = 3;
 				}
+				else {	/* Need twe other two vertices to form a 4-sided polygon */
+					node2 = get_other_node (node1, low);	/* The other node needed */
+					xout[1] = xx[node2];	yout[1] = yy[node2];
+					if (low == vert[0] || node2 == vert[1]) {	/* Add segment in opposite order */
+						xout[2] = xc[1];	yout[2] = yc[1];
+						xout[3] = xc[0];	yout[3] = yc[0];
+					}
+					else  {	/* Add in regular order */
+						xout[2] = xc[0];	yout[2] = yc[0];
+						xout[3] = xc[1];	yout[3] = yc[1];
+					}
+					n = 4;
+				}
+				paint_it (xout, yout, n, 0.5 * (zz[low] + zc[1]));	/* z is contour value */
+
+				/* Then loop over contours and paint the part between contours */
+			
+				for (k = 1, k2 = 2, k3 = 3; k < nx; k++, k2 += 2, k3 += 2) {
+					xout[0] = xc[k2-2];	yout[0] = yc[k2-2];
+					xout[1] = xc[k3-2];	yout[1] = yc[k3-2];
+					n = 2;
+					last_entry = vert[k2-2];
+					last_exit  = vert[k3-2];
+					if (last_exit == vert[k2]) {
+						xout[n] = xc[k2];	yout[n] = yc[k2];	n++;
+						xout[n] = xc[k3];	yout[n] = yc[k3];	n++;
+						if (vert[k3] != last_entry) {	/* Need to add an intervening corner */
+							node1 = get_node_index (last_entry, vert[k3]);	/* Find corner id */
+							xout[n] = xx[node1];	yout[n] = yy[node1];	n++;
+						}
+					}
+					else if (last_exit == vert[k3]) {
+						xout[n] = xc[k3];	yout[n] = yc[k3];	n++;
+						xout[n] = xc[k2];	yout[n] = yc[k2];	n++;
+						if (vert[k2] != last_entry) {	/* Need to add an intervening corner */
+							node1 = get_node_index (last_entry, vert[k2]);	/* Find corner id */
+							xout[n] = xx[node1];	yout[n] = yy[node1];	n++;
+						}
+					}
+					else if (last_entry == vert[k2]) {
+						node1 = get_node_index (last_exit, vert[k3]);	/* Find corner id */
+						xout[n] = xx[node1];	yout[n] = yy[node1];	n++;
+						xout[n] = xc[k3];	yout[n] = yc[k3];	n++;
+						xout[n] = xc[k2];	yout[n] = yc[k2];	n++;
+					}
+					else {
+						node1 = get_node_index (last_exit, vert[k2]);	/* Find corner id */
+						xout[n] = xx[node1];	yout[n] = yy[node1];	n++;
+						xout[n] = xc[k2];	yout[n] = yc[k2];	n++;
+						xout[n] = xc[k3];	yout[n] = yc[k3];	n++;
+					}
+					paint_it (xout, yout, n, 0.5 * (zc[k2]+zc[k2-2]));
+				}
+			
+				/* Add the last piece between last contour and high node */
+				
+				k2 -= 2;	k3 -= 2;
+				xout[0] = xx[high];	yout[0] = yy[high];
+				node1 = get_node_index (vert[k2], vert[k3]);	/* Find corner id */
+				if (node1 == high) {	/* Cut off a triangular piece */
+					xout[1] = xc[k2];	yout[1] = yc[k2];
+					xout[2] = xc[k3];	yout[2] = yc[k3];
+					n = 3;
+				}
+				else {	/* Need a 4-sided polygon */
+					node2 = get_other_node (node1, high);	/* The other node needed */
+					xout[1] = xx[node2];	yout[1] = yy[node2];
+					if (high == vert[0] || node2 == vert[1]) {	/* On same side, start here */
+						xout[2] = xc[k3];	yout[2] = yc[k3];
+						xout[3] = xc[k2];	yout[3] = yc[k2];
+					}
+					else  {	/* On same side, start here */
+						xout[2] = xc[k2];	yout[2] = yc[k2];
+						xout[3] = xc[k3];	yout[3] = yc[k3];
+					}
+					n = 4;
+				}
+				paint_it (xout, yout, n, 0.5 * (zz[high] + zc[k2]));	/* z is contour value */
 			}
 		}
 		
@@ -683,6 +738,10 @@ main(int argc, char **argv)
 
 int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **xc, double **yc, double **zc, int **v, int **cindex)
 {
+	/* This routine finds all the contour crossings for this triangle.  Each contour consists of
+	 * linesegments made up of two points, with coordinates xc, yc, and contour level zc.
+	 */
+	 
 	int i, j, k, k2, i1, nx, n_alloc, *vout, *cind;
 	double xx[3], yy[3], zz[3], zmin, zmax, dz, frac, *xout, *yout, *zout;
 	
@@ -690,14 +749,14 @@ int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **
 	xx[1] = x[ind[1]];	yy[1] = y[ind[1]];	zz[1] = z[ind[1]];
 	xx[2] = x[ind[2]];	yy[2] = y[ind[2]];	zz[2] = z[ind[2]];
 
-	zmin = MIN (zz[0], MIN (zz[1], zz[2]));
-	zmax = MAX (zz[0], MAX (zz[1], zz[2]));
+	zmin = MIN (zz[0], MIN (zz[1], zz[2]));	/* Min z vertex */
+	zmax = MAX (zz[0], MAX (zz[1], zz[2]));	/* Max z vertex */
 	
 	i = 0;	j = GMT_n_colors - 1;
 	while (GMT_lut[i].z_low <= zmin && i < GMT_n_colors) i++;
 	while (GMT_lut[j].z_high >= zmax && j > 0) j--;
 	
-	nx = j - i + 2;
+	nx = j - i + 2;	/* Total number of contours */
 	
 	if (nx <= 0) return (0);
 	
@@ -707,6 +766,8 @@ int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **
 	zout = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	vout = (int *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (int), GMT_program);
 	cind = (int *) GMT_memory (VNULL, (size_t)nx, sizeof (int), GMT_program);
+
+	/* Fill out array zout which holds the nx contour levels */
 	
 	k = k2 = 0;
 	while (i <= j) {
@@ -717,9 +778,11 @@ int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **
 	}
 	zout[k2] = zout[k2+1] = GMT_lut[j].z_high;
 	cind[k] = j + 1;
-		
+
+	/* Loop over the contour levels and determine the line segments */
+	
 	for (k = k2 = j = 0; k < nx; k++, k2 += 2) {
-		for (i = 0; i < 3; i++) {
+		for (i = 0; i < 3; i++) {	/* Try each side in turn 0-1, 1-2, 2-0 */
 			i1 = (i == 2) ? 0 : i + 1;
 			if ((zout[k2] >= zz[i] && zout[k2] < zz[i1]) || (zout[k2] <= zz[i] && zout[k2] > zz[i1])) {
 				dz = zz[i1] - zz[i];
@@ -731,7 +794,7 @@ int get_triangle_crossings (double *x, double *y, double *z, int *ind, double **
 					xout[j] = xx[i] + frac * (xx[i1] - xx[i]);
 					yout[j] = yy[i] + frac * (yy[i1] - yy[i]);
 				}
-				vout[j++] = i;
+				vout[j++] = i;	/* Keep track of the side number */
 			}
 		}
 		if (j%2) j--;	/* Contour went through a single vertice only, skip this */
@@ -750,7 +813,7 @@ void draw_contour (double *xx, double *yy, int nn, double cval, char ctype, doub
 {
 	int i;
 	double dist, angle, dx, dy, width;
-	char label[100], format[50];
+	char label[128], format[64];
 	struct LABEL *new_label;
 	
 	if (nn < 2) return;
@@ -850,7 +913,8 @@ void dump_contour (double *xx, double *yy, int nn, double cval, int id, BOOLEAN
 		}
 		else	/* Later we append to it */
 			fp = GMT_fopen (file, "a+");
-		fprintf (fp, "%c %lg contour\n", GMT_io.EOF_flag, cval);
+		sprintf (GMT_io.segment_header, "%c %lg contour\0", GMT_io.EOF_flag, cval);
+		GMT_write_segmentheader (fp, 3);
 	}
 	else {
 		if (interior)
@@ -866,3 +930,22 @@ void dump_contour (double *xx, double *yy, int nn, double cval, int id, BOOLEAN
 	GMT_fclose (fp);
 }
 
+void paint_it (double x[], double y[], int n, double z) {
+	int index, k, rgb[3];
+	struct GMT_FILL *f;
+	
+	if (n < 3) return;	/* Need at least 3 points to make a polygon */
+	
+	index = GMT_get_rgb24 (z, rgb);
+	if (GMT_cpt_skip) return;	/* Skip this z-slice */
+	
+	/* Now we must paint, with colors or patterns */
+	
+	if (project_info.three_D)
+		for (k = 0; k < n; k++) GMT_xy_do_z_to_xy (x[k], y[k], project_info.z_level, &x[k], &y[k]);
+	if ((index >= 0 && (f = GMT_lut[index].fill)) || (index < 0 && (f = GMT_bfn.fill[index+3]))) {
+		GMT_fill (x, y, n, f, FALSE);	/* Contours drawn separately later if desired */
+	}
+	else
+		ps_patch (x, y, n, rgb, FALSE);	/* Contours drawn separately later if desired */
+}
