@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)pshistogram.c	2.72  09/21/99
+ *	$Id: pshistogram.c,v 1.4 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * pshistogram.c -- a program for plotting histograms
@@ -27,7 +27,7 @@
  * Updated to v3.1 13-Jun-1998 Paul Wessel
  * Updated to v3.3 23-Apr-1999 Paul Wessel
  *		     Now uses two passes to get accurate mean and stdev
- *
+ * Version:	3.4
  */
 
 #include "gmt.h"
@@ -61,7 +61,6 @@ int	second = FALSE, hist_type = COUNTS;
 struct GMT_PEN pen;
 struct GMT_FILL fill;
 	
-void getmad(float *x, int n, float *location, float *scale);
 int read_data(void);
 int fill_boxes(void);
 int plot_boxes(BOOLEAN stairs, BOOLEAN flip_to_y);
@@ -180,49 +179,49 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"pshistogram %s - Calculate and plot histograms\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pshistogram [file] -Jx|X<scale> -W<width> [-2] [-B<tickinfo>] [-C] [-Eaz/el] [-G<fill>]\n");
-		fprintf (stderr, "	[-H[<nrec>]] [-I[o]] [-K] [-L<pen>] [-O] [-P] [-Q] [-Rw/e/s/n] [-S] [-U[text]] [-V]\n");
-		fprintf (stderr, "	[-X<x_shift>] [-Y<y_shift>] [-Z[type]] [-c<ncopies>] [-bi[s][<n>]]\n\n");
+		fprintf (stderr, "\t[-H[<nrec>]] [-I[o]] [-K] [-L<pen>] [-O] [-P] [-Q] [-Rw/e/s/n] [-S] [-U[text]] [-V]\n");
+		fprintf (stderr, "\t[-X<x_shift>] [-Y<y_shift>] [-Z[type]] [-c<ncopies>] [-bi[s][<n>]]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
-		fprintf (stderr, "	-Jx|X for linear projection.  Scale in %s/units (or width in %s)\n", GMT_unit_names[gmtdefs.measure_unit], GMT_unit_names[gmtdefs.measure_unit]);
-		fprintf (stderr, "	    Use / to specify separate x/y scaling.\n");
-		fprintf (stderr, "	    If -JX is used then give axes lengths in %s rather than scales\n", GMT_unit_names[gmtdefs.measure_unit]);
+		fprintf (stderr, "\t-Jx|X for linear projection.  Scale in %s/units (or width in %s)\n", GMT_unit_names[gmtdefs.measure_unit], GMT_unit_names[gmtdefs.measure_unit]);
+		fprintf (stderr, "\t    Use / to specify separate x/y scaling.\n");
+		fprintf (stderr, "\t    If -JX is used then give axes lengths in %s rather than scales\n", GMT_unit_names[gmtdefs.measure_unit]);
 
-		fprintf (stderr, "	-W sets the bin width\n");
+		fprintf (stderr, "\t-W sets the bin width\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "	-2 use second rather than first column\n");
+		fprintf (stderr, "\t-2 use second rather than first column\n");
 		GMT_explain_option ('b');
-		fprintf (stderr, "	-A will plot horizontal bars [Default is vertical]\n");
-		fprintf (stderr, "	-C will center the bins\n");
-		fprintf (stderr, "	-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
-		fprintf (stderr, "	-G Specify color or pattern for columns. fill can be either\n");
-		fprintf (stderr, "	   1) <r/g/b> (each 0-255) for color or <gray> (0-255) for gray-shade [0]\n");
-		fprintf (stderr, "	   2) p[or P]<dpi>/<pattern> for predefined patterns (1-90).\n");
+		fprintf (stderr, "\t-A will plot horizontal bars [Default is vertical]\n");
+		fprintf (stderr, "\t-C will center the bins\n");
+		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t-G Specify color or pattern for columns. fill can be either\n");
+		fprintf (stderr, "\t   1) <r/g/b> (each 0-255) for color or <gray> (0-255) for gray-shade [0]\n");
+		fprintf (stderr, "\t   2) p[or P]<dpi>/<pattern> for predefined patterns (1-90).\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "	-I will inquire about min/max x and y.  No plotting is done\n");
-		fprintf (stderr, "	   Append o to output the resulting x, y data\n");
+		fprintf (stderr, "\t-I will inquire about min/max x and y.  No plotting is done\n");
+		fprintf (stderr, "\t   Append o to output the resulting x, y data\n");
 		GMT_explain_option ('K');
-		fprintf (stderr, "	-L <pen> is penwidth. Specify r/g/b for colored line.\n");
+		fprintf (stderr, "\t-L <pen> is penwidth. Specify r/g/b for colored line.\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr, "	-Q plot a cumulative histogram\n");
+		fprintf (stderr, "\t-Q plot a cumulative histogram\n");
 		GMT_explain_option ('r');
-		fprintf (stderr, "	   If neither -R nor -I are set, w/e/s/n will be based on input data\n");
-		fprintf (stderr, "	-S Draws a stairs-step diagram instead of histogram.\n");
+		fprintf (stderr, "\t   If neither -R nor -I are set, w/e/s/n will be based on input data\n");
+		fprintf (stderr, "\t-S Draws a stairs-step diagram instead of histogram.\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
 		GMT_explain_option ('X');
-		fprintf (stderr, "	-Z to choose type of vertical axis.  Select from\n");
-		fprintf (stderr, "	   0 - Counts [Default]\n");
-		fprintf (stderr, "	   1 - Frequency percent\n");
-		fprintf (stderr, "	   2 - Log (1+counts)\n");
-		fprintf (stderr, "	   3 - Log (1+frequency percent)\n");
-		fprintf (stderr, "	   4 - Log10 (1+counts)\n");
-		fprintf (stderr, "	   5 - Log10 (1+frequency percent)\n");
+		fprintf (stderr, "\t-Z to choose type of vertical axis.  Select from\n");
+		fprintf (stderr, "\t   0 - Counts [Default]\n");
+		fprintf (stderr, "\t   1 - Frequency percent\n");
+		fprintf (stderr, "\t   2 - Log (1+counts)\n");
+		fprintf (stderr, "\t   3 - Log (1+frequency percent)\n");
+		fprintf (stderr, "\t   4 - Log10 (1+counts)\n");
+		fprintf (stderr, "\t   5 - Log10 (1+frequency percent)\n");
 		GMT_explain_option ('c');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "	   Default is 2 input columns\n");
+		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('.');
 
 		exit (EXIT_FAILURE);
@@ -337,7 +336,7 @@ main (int argc, char **argv)
 					yy = d_log101p( 100.0 * boxh[ibox] / n_counted );
 				else
 					yy = boxh[ibox];
-				printf (format, xx, yy);
+				fprintf (GMT_stdout, format, xx, yy);
 			}
 		}
 		GMT_free ((void *) x);
@@ -375,7 +374,7 @@ main (int argc, char **argv)
 	}
 
 	if (flip_to_y) {
-		char buffer[100];
+		char buffer[128];
 		d_swap (frame_info.anot_int[0], frame_info.anot_int[1]);
 		d_swap (frame_info.frame_int[0], frame_info.frame_int[1]);
 		d_swap (frame_info.grid_int[0], frame_info.grid_int[1]);
@@ -423,9 +422,9 @@ int	read_data (void) {
 	
 	n = 0;
 	xmin = DBL_MAX;	xmax = -DBL_MAX;
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 1 + second;
 	
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (buffer, BUFSIZ, fp_in);
+	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp_in);
 	
 	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 	
@@ -609,48 +608,38 @@ int	plot_boxes (BOOLEAN stairs, BOOLEAN flip_to_y)
 }
 
 int	get_loc_scl (float *x, int n, float *stats)
-     	             	/* L2, L1, LMS location, L2, L1, LMS scale  */
-   	  
 {
+	/* Returns stats[] = L2, L1, LMS location, L2, L1, LMS scale  */
 
-	int	i, j, istop, multiplicity = 1;
-	double	xsum, x2sum, dx, mid_point_sum = 0.0, length, short_length = 1.0e+30;
+	int	i, j;
+	double	xsum, x2sum, dx, median, mode, mad, lmsscl;
 
 	if (n < 3) return(-1);
 	
 	qsort ((void *)x, (size_t)n, sizeof(float), GMT_comp_float_asc);
 
+	/* Get median */
 	j = n/2;
-	
-	if (n%2)
-		stats[1] = x[j];
-	else
-		stats[1] = (float)(0.5 * (x[j] + x[j-1]));
+	median = (n%2) ? x[j] : (0.5 * (x[j] + x[j-1]));
+	stats[1] = (float)median;
 
-	istop = n - j;
-	for (i = 0; i < istop; i++) {
-		length = x[i + j] - x[i];
-		if (length == short_length) {
-			multiplicity++;
-			mid_point_sum += (0.5 * (x[i + j]  + x[i]) );
-		}
-		else if (length < short_length) {
-			multiplicity = 1;
-			mid_point_sum = (0.5 * (x[i + j]  + x[i]) );
-			short_length = length;
-		}
-	}
-	if (multiplicity > 1) mid_point_sum /=  multiplicity;
-	stats[2] = (float)mid_point_sum;
+	/* Get mode */
+	GMT_mode_f (x, n, j, 0, &mode);
+	stats[2] = (float)mode;
 	
-	getmad (x, n, &stats[1], &stats[4]);
-	getmad (x, n, &stats[2], &stats[5]);
+	/* Get MAD for L1 */
+	GMT_getmad_f (x, n, median, &mad);
+	stats[4] = (float)mad;
+	
+	/* Get LMSscale for mode */
+	GMT_getmad_f (x, n, mode, &lmsscl);
+	stats[5] = (float)lmsscl;
 
 	/* Calculate mean and stdev in two passes to minimize risk of overflow */
 
 	xsum = x2sum = 0.0;
 	for (i = 0; i < n; i++) xsum += x[i];
-	xsum /= n;	/* xsum is not the mean value */
+	xsum /= n;	/* xsum is now the mean value */
 	for (i = 0; i < n; i++) {
 		dx = x[i] - xsum;
 		x2sum += (dx * dx);
@@ -658,68 +647,5 @@ int	get_loc_scl (float *x, int n, float *stats)
 	stats[0] = (float)xsum;
 	stats[3] = (float)sqrt (x2sum / (n - 1));
 
-	return(0);
-}
-
-void	getmad (float *x, int n, float *location, float *scale)
-{
-	double	e_low, e_high, error, last_error;
-	int	i_low, i_high, n_dev, n_dev_stop;
-	
-	i_low = 0;
-	while (i_low < n && x[i_low] <= *location) i_low++;
-	i_low--;
-	
-	i_high = n - 1;
-	while (i_high >= 0 && x[i_high] >= *location) i_high--;
-	i_high++;
-
-	n_dev_stop = n / 2;
-	error = 0.0;
-	n_dev = 0;
-
-
-	while (n_dev < n_dev_stop) {
-	
-		last_error = error;
-
-		if (i_low < 0) {
-			error = x[i_high] - *location;
-			i_high++;
-			n_dev++;
-		}
-		
-		else if (i_high == n) {
-			error = *location - x[i_low];
-			i_low--;
-			n_dev++;
-		}
-		else {
-			e_low = *location - x[i_low];
-			e_high = x[i_high] - *location;
-			
-			if (e_low < e_high) {
-				error = e_low;
-				i_low--;
-				n_dev++;
-			}
-			else if (e_high < e_low) {
-				error = e_high;
-				i_high++;
-				n_dev++;
-			}
-			else {
-				error = e_high;
-				i_low--;
-				i_high++;
-				if ( !(n_dev)) n_dev--; /* First time count only once  */
-				n_dev += 2;
-			}
-		}
-	}
-	
-	if (n%2) 
-		*scale = (float)(1.4826 * error);
-	else
-		*scale = (float)(0.7413 * (error + last_error));
+	return (0);
 }
