@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)psmask.c	2.73  11/24/99
+ *	$Id: psmask.c,v 1.4 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * psmask tries to achieve masking using one of two different approachs:
@@ -32,8 +32,8 @@
  * the default with the -T switch.  Then, masking is achieved using tiling
  *
  * Author:	Paul Wessel
- * Date:	21-OCT-1998
- * Version:	3.1
+ * Date:	01-JUL-2000
+ * Version:	3.4
  *
  */
 
@@ -51,16 +51,16 @@ int *edge;
 double *x, *y;	/* Arrays holding the contour xy values */
 double *x_grid, *y_grid;
 
-void draw_clip_contours(double *xx, double *yy, int nn, int rgb[], int id, int flag);
-void dump_clip_countours(double *xx, double *yy, int nn, int id, char *file);
-void shrink_clip_contours(double *x, double *y, int n);
-int trace_clip_contours(char *grd, int nx, int *edge, double west, double north, double dx, double dy, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, int *max);
-int clip_contours(char *grd, int nx, int ny, double west, double north, double dx, double dy, double xinc2, double yinc2, int *edge, int first, double **x, double **y, int *max);
+void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int flag);
+void dump_clip_countours (double *xx, double *yy, int nn, int id, char *file);
+void shrink_clip_contours (double *x, double *y, int n);
+int trace_clip_contours (char *grd, int nx, int *edge, double west, double north, double dx, double dy, double xinc2, double yinc2, double **xx, double **yy, int i, int j, int kk, int *max);
+int clip_contours (char *grd, int nx, int ny, double west, double north, double dx, double dy, double xinc2, double yinc2, int *edge, int first, double **x, double **y, int *max);
 
 main (int argc, char **argv)
 {
 	int i, j, ij, n, nm, n_edges, di, dj, ii, jj, n_expected_fields;
-	int nx, ny, section, n_alloc, ix, iy, n_read, n_fields, one_or_zero = 1;
+	int nx, ny, section, n_alloc, n_read, n_fields, one_or_zero = 1;
 	
 	BOOLEAN error = FALSE, first = TRUE, dump = FALSE, invert = FALSE;
 	BOOLEAN end_of_clip = FALSE, find_paths = TRUE, use_poly;
@@ -162,38 +162,38 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"psmask %s - Clipping of 2-D data sets\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: psmask <xyz-file> -I<xinc>[m|c][/<yinc>[m|c]] -J<params> -R<west/east/south/north>\n");
-		fprintf (stderr, "	[-B<tickinfo>] [-C] -D<file> [-Eaz/el] [-F] [-G<fill>] [-H[<nrec>]] [-K] [-M[<flag>]]\n");
-		fprintf (stderr, "	[-N] [-O] [-P] [-S<radius>[k]] [-T] [-U[<label>]] [-V] [-X<x_shift>] [-Y<y_shift>]\n");
-		fprintf (stderr, "	[-c<ncopies>][ -:] [-bi[s][<n>]] [-bo[s]]\n\n");
+		fprintf (stderr, "\t[-B<tickinfo>] [-C] -D<file> [-Eaz/el] [-F] [-G<fill>] [-H[<nrec>]] [-K] [-M[<flag>]]\n");
+		fprintf (stderr, "\t[-N] [-O] [-P] [-S<radius>[k]] [-T] [-U[<label>]] [-V] [-X<x_shift>] [-Y<y_shift>]\n");
+		fprintf (stderr, "\t[-c<ncopies>][ -:] [-bi[s][<n>]] [-bo[s]]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr, "	<xyz-file> is the datafile.  If not given, read standard input\n");
-		fprintf (stderr, "	-I sets grid increments; enter xinc, optionally xinc/yinc.\n");
-		fprintf (stderr, "	   Default is yinc = xinc.  Append an m [or s] to xinc or yinc to indicate minutes [or seconds]\n");
+		fprintf (stderr, "\t<xyz-file> is the datafile.  If not given, read standard input\n");
+		fprintf (stderr, "\t-I sets grid increments; enter xinc, optionally xinc/yinc.\n");
+		fprintf (stderr, "\t   Default is yinc = xinc.  Append an m [or s] to xinc or yinc to indicate minutes [or seconds]\n");
 		GMT_explain_option ('j');
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		GMT_explain_option ('b');
-		fprintf (stderr, "	-C means stop existing clip-path.  No other options required\n");
-		fprintf (stderr, "	-D dumps the clip-paths to files using the prefix <file>_ [mask_]\n");
-		fprintf (stderr, "	   Ignored if -T is specified\n");
-		fprintf (stderr, "	-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
-		fprintf (stderr, "	-F Force pixel registration [Default is gridline registration]\n");
-		fprintf (stderr, "	-G sets fill color r/g/b [Default is no fill]\n");
+		fprintf (stderr, "\t-C means stop existing clip-path.  No other options required\n");
+		fprintf (stderr, "\t-D dumps the clip-paths to files using the prefix <file>_ [mask_]\n");
+		fprintf (stderr, "\t   Ignored if -T is specified\n");
+		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t-F Force pixel registration [Default is gridline registration]\n");
+		fprintf (stderr, "\t-G sets fill color r/g/b [Default is no fill]\n");
 		GMT_explain_option ('H');
 		GMT_explain_option ('K');
 		GMT_explain_option ('M');
-		fprintf (stderr, "	-N will invert the sense of the clipping [or tiling]\n");
+		fprintf (stderr, "\t-N will invert the sense of the clipping [or tiling]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr, "	-S Search radius: Nodes inside circles of <radius> centered on\n");
-		fprintf (stderr, "	   the input data poins are considered to be reliable estimates of the surface\n");
-		fprintf (stderr, "	   Default is -S0, i.e., only the nearest node is considered reliable\n");
-		fprintf (stderr, "	   Append k to indicate km (hence -R -I are in degrees)\n");
-		fprintf (stderr, "	-T will paint tiles.  [Default will trace data outline]\n");
-		fprintf (stderr, "	   If set you must also specify a color/fill with -G\n");
-		fprintf (stderr, "	   !May only be used with linear, Mercator, and basic cylindrical projections!\n");
+		fprintf (stderr, "\t-S Search radius: Nodes inside circles of <radius> centered on\n");
+		fprintf (stderr, "\t   the input data poins are considered to be reliable estimates of the surface\n");
+		fprintf (stderr, "\t   Default is -S0, i.e., only the nearest node is considered reliable\n");
+		fprintf (stderr, "\t   Append k to indicate km (hence -R -I are in degrees)\n");
+		fprintf (stderr, "\t-T will paint tiles.  [Default will trace data outline]\n");
+		fprintf (stderr, "\t   If set you must also specify a color/fill with -G\n");
+		fprintf (stderr, "\t   !May only be used with linear, Mercator, and basic cylindrical projections!\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
 		GMT_explain_option ('X');
@@ -201,7 +201,7 @@ main (int argc, char **argv)
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "	   Default is 2 input columns\n");
+		fprintf (stderr, "\t   Default is 2 input columns\n");
 		GMT_explain_option ('o');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
@@ -278,8 +278,6 @@ main (int argc, char **argv)
 
 		if (gmtdefs.verbose) fprintf (stderr, "%s: Allocate memory, read and process data file\n", GMT_program);
 	
-		ix = (gmtdefs.xy_toggle) ? 1 : 0;        iy = 1 - ix;              /* Set up which columns have x and y */
-		
 		/* Enlarge region by 1 row/column */
 
 		west -= dx;	east += dx;	south -= dy;	north += dy;
@@ -309,9 +307,9 @@ main (int argc, char **argv)
 #endif
 		}
 		
-		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 
-		n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+		n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 2;
 		n_read = 0;
 		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 
@@ -328,9 +326,9 @@ main (int argc, char **argv)
 
 			/* Determine the node closest to the data point */
 
-			i = (int)floor (((in[ix] - west) * idx) + offset);
+			i = (int)floor (((in[0] - west) * idx) + offset);
 			if (i < 0 || i >= nx) continue;
-			j = (int)floor (((north - in[iy]) * idy) + offset);
+			j = (int)floor (((north - in[1]) * idy) + offset);
 			if (j < 0 || j >= ny) continue;
 
 			if (node_only) {
@@ -463,6 +461,7 @@ void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int
 {
 	int i;
 	double x, y;
+	char comment[64];
 	
 	if (nn < 2 && flag < 2) return;
 	
@@ -474,10 +473,15 @@ void draw_clip_contours (double *xx, double *yy, int nn, int rgb[], int id, int
 	
 	if (project_info.three_D) GMT_2D_to_3D (xx, yy, nn);
 	
-	if (nn > 0) printf ("%% Start of clip path sub-segment %d\n", id);
+	if (nn > 0) {
+		sprintf (comment, "Start of clip path sub-segment %d\0", id);
+		ps_comment (comment);
+	}
 	ps_clipon (xx, yy, nn, rgb, flag);
-	if (nn > 0) printf ("%% End of clip path sub-segment %d\n", id);
-		
+	if (nn > 0) {
+		sprintf (comment, "End of clip path sub-segment %d\0", id);
+		ps_comment (comment);
+	}		
 }
 
 void dump_clip_countours (double *xx, double *yy, int nn, int id, char *file)
@@ -509,7 +513,7 @@ int clip_contours (char *grd, int nx, int ny, double west, double north, double
 {
 	/* The routine finds the zero-contour in the grd dataset.  it assumes that
 	 * no node has a value exactly == 0.0.  If more than max points are found
-	 * trace_clip_contours will try to allocate more memory in blocks of 1000 points
+	 * trace_clip_contours will try to allocate more memory in blocks of GMT_CHUNK points
 	 */
 	 
 	 static int i0, j0;
@@ -522,7 +526,8 @@ int clip_contours (char *grd, int nx, int ny, double west, double north, double
 	 
 	 /* Reset edge-flags to zero, if necessary */
 	 if (first) {
-	 	i0 = j0 = 0;
+	 	i0 = 0;	/* Begin with upper left bin which is i = 0 and j = 1 */
+	 	j0 = 1;
 		p[0] = p[4] = 0;	p[1] = 1;	p[2] = 1 - nx;	p[3] = -nx;
 		i_off[0] = i_off[2] = i_off[3] = i_off[4] = 0;	i_off[1] =  1;
 		j_off[0] = j_off[1] = j_off[3] = j_off[4] = 0;	j_off[2] = -1;
@@ -534,7 +539,7 @@ int clip_contours (char *grd, int nx, int ny, double west, double north, double
 		
 	for (j = j0; go_on && j < ny; j++) {
 		ij = i0 + j * nx;
-		for (i = i0; go_on && i < nx-1; i++, ij++) {
+		for (i = i0; go_on && i < nx-1; i++, ij++) {	/* nx-1 since the last bin starts at nx-2 and ends at nx-1 */
 			edge_word = ij / 32 + offset;
 			edge_bit = ij % 32;
 			if (!(edge[edge_word] & bit[edge_bit]) && ((grd[ij]+grd[ij-nx]) == 1)) { /* Start tracing countour */
@@ -544,10 +549,10 @@ int clip_contours (char *grd, int nx, int ny, double west, double north, double
 				n = trace_clip_contours (grd, nx, edge, west, north, dx, dy, xinc2, yinc2, x, y, i, j, 3, max);
 				go_on = FALSE;
 				i0 = i + 1;
-				j0 = j;
+				j0 = j;	/* Return to finish this row later */
 			}
 		}
-		if (!go_on) i0 = 1;
+		if (go_on) i0 = 0;	/* Go to start of next row unless we found something */
 	}
 	
 	return (n);
@@ -613,8 +618,8 @@ int trace_clip_contours (char *grd, int nx, int *edge, double west, double north
 		}
 		
 		if (n > m) {	/* Must try to allocate more memory */
-			*max += 1000;
-			m += 1000;
+			*max += GMT_CHUNK;
+			m += GMT_CHUNK;
 			*xx = (double *) GMT_memory ((void *)*xx, (size_t)(*max), sizeof (double), "trace_clip_contours");
 			*yy = (double *) GMT_memory ((void *)*yy, (size_t)(*max), sizeof (double), "trace_clip_contours");
 		}
