@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)psrose.c	2.59  09/21/99
+ *	$Id: psrose.c,v 1.4 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * psrose reads a file [or standard input] with azimuth and length information
@@ -27,10 +27,8 @@
  * south/north is -90,90 for halfcircle and 0,360 for full circle
  *
  * Author:	Paul Wessel
- * Date:	20-JAN-1995
- * Version:	3.0
- * Revised:	21-JUN-1998, for GMT 3.1
- *		21-JUN-1999, For GMT 3.3.1
+ * Date:	20-JUN-2000
+ * Version:	3.4
  */
  
 #include "gmt.h"
@@ -49,8 +47,7 @@ void arc3d(double x, double y, double r, double a1, double a2, int status);
 main (int argc, char **argv)
 {
 	int i, n_bins, bin, n_anot, n = 0, a_rgb[3];
-	int status, n_alpha, n_modes, ix, iy;
-	int n_fields, n_expected_fields;
+	int status, n_alpha, n_modes, n_fields, n_expected_fields;
 	
 	BOOLEAN error = FALSE, inquire = FALSE, outline = FALSE, find_mean = FALSE, normalize = FALSE;
 	BOOLEAN half_only = FALSE, rose = FALSE, windrose = TRUE, sector_plot = FALSE;
@@ -59,7 +56,7 @@ main (int argc, char **argv)
 	double scale = 3.0, max = 0.0, sector = 0.0, radius, az, x_origin, y_origin, tmp;
 	double angle1, angle2, angle, x, y, mean_theta, mean_radius, xr = 0.0, yr = 0.0;
 	double x1, x2, y1, y2, total = 0.0, total_arc, off, max_radius, az_offset, *in;
-	double tailwidth = 0.03, headlength = 0.12, headwidth = 0.1, one_or_two = 1.0;
+	double tailwidth = 0.03, headlength = 0.12, headwidth = 0.1, one_or_two = 1.0, s, c;
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, asize, lsize, zscale = 1.0;
 	
 	char text[BUFSIZ], lw[10], le[10], ln[10], *file = CNULL, format[BUFSIZ];
@@ -126,7 +123,7 @@ main (int argc, char **argv)
 					break;
 				case 'C':		/* Read mode file and plot directions */
 					draw_modes = TRUE;
-					if ((fpm = GMT_fopen (&argv[i][2], "r")) == NULL) find_mean = TRUE;
+					if ((fpm = fopen (&argv[i][2], "r")) == NULL) find_mean = TRUE;
 					break;
 				case 'E':
 					sscanf (&argv[i][2], "%lf/%lf", &z_project.view_azimuth, &z_project.view_elevation);
@@ -196,45 +193,45 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"psrose %s - Polar histogram (rose diagram) plotter\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: psrose <infile> [-A<sector_angle>[r]] [-B<tickinfo>] \n");
-		fprintf (stderr, "	[-C[<modes>]] [-Eaz/el] [-G<fill>] [-H[<nrec>]] [-I] [-K] [-M<parameters>]\n");
-		fprintf (stderr, "	[-N] [-O] [-P] [-R<r0/r1/theta0/theta1>] [-Sscale[n]] [-T] [-U[label]] [-V] [-:]\n");
-		fprintf (stderr, "	[-Wpen] [-Xx_shift] [-Yy_shift] [-Zscale] [-cncopies] [-bi[s][<n>]]\n\n");
+		fprintf (stderr, "\t[-C[<modes>]] [-Eaz/el] [-G<fill>] [-H[<nrec>]] [-I] [-K] [-M<parameters>]\n");
+		fprintf (stderr, "\t[-N] [-O] [-P] [-R<r0/r1/theta0/theta1>] [-Sscale[n]] [-T] [-U[label]] [-V] [-:]\n");
+		fprintf (stderr, "\t[-Wpen] [-Xx_shift] [-Yy_shift] [-Zscale] [-cncopies] [-bi[s][<n>]]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr, "	<infile> (in ASCII or binary) has (length,azimuth) pairs.  If not given read standard input\n");
+		fprintf (stderr, "\t<infile> (in ASCII or binary) has (length,azimuth) pairs.  If not given read standard input\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "	-A sector width in degrees for sector diagram [Default is windrose]\n");
-		fprintf (stderr, "	   append r to get rose diagram\n");
+		fprintf (stderr, "\t-A sector width in degrees for sector diagram [Default is windrose]\n");
+		fprintf (stderr, "\t   append r to get rose diagram\n");
 		GMT_explain_option ('B');
-		fprintf (stderr, "	   (Remember: radial is x-direction, azimuthal is y-direction)\n");
-		fprintf (stderr, "	-C plot vectors listed in the <modes> file.  If no file, use mean direction\n");
-		fprintf (stderr, "	-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
-		fprintf (stderr, "	-G <fill> specify color for diagram.  [Default is no fill]\n");
+		fprintf (stderr, "\t   (Remember: radial is x-direction, azimuthal is y-direction)\n");
+		fprintf (stderr, "\t-C plot vectors listed in the <modes> file.  If no file, use mean direction\n");
+		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t-G <fill> specify color for diagram.  [Default is no fill]\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "	-I for inquire.  Only compute statistics - no plot is created\n");
+		fprintf (stderr, "\t-I for inquire.  Only compute statistics - no plot is created\n");
 		GMT_explain_option ('K');
-		fprintf (stderr, "	-M Append tailwidth/headlength/headwidth/r/g/b to set arrow attributes [0.03i/0.12i/0.1i/0/0/0\n");
-		fprintf (stderr, "	-N normalizes rose plots for area, i.e. takes sqrt(r) before plotting [FALSE]\n");
-		fprintf (stderr, "	   Only applicable if normalization has been specified with -S<radius>n\n");
+		fprintf (stderr, "\t-M Append tailwidth/headlength/headwidth/r/g/b to set arrow attributes [0.03i/0.12i/0.1i/0/0/0\n");
+		fprintf (stderr, "\t-N normalizes rose plots for area, i.e. takes sqrt(r) before plotting [FALSE]\n");
+		fprintf (stderr, "\t   Only applicable if normalization has been specified with -S<radius>n\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr, "	-R specifies the region.  (r0 = 0, r1 = max_radius.  For azimuth:\n");
-		fprintf (stderr, "	   Specify theta0/theta1 = -90/90 (half-circle) or 0/360 only)\n");
-		fprintf (stderr, "	   If r0 = r1 = 0, psrose will compute reasonable a r1 value.\n");
-		fprintf (stderr, "	-S specifies the radius of the unit circle in %s [%lg]. Normalize r if n is appended\n", GMT_unit_names[gmtdefs.measure_unit], scale);
-		fprintf (stderr, "	-T indicates that the vectors are oriented (two-headed), not directed [Default]\n");
+		fprintf (stderr, "\t-R specifies the region.  (r0 = 0, r1 = max_radius.  For azimuth:\n");
+		fprintf (stderr, "\t   Specify theta0/theta1 = -90/90 (half-circle) or 0/360 only)\n");
+		fprintf (stderr, "\t   If r0 = r1 = 0, psrose will compute reasonable a r1 value.\n");
+		fprintf (stderr, "\t-S specifies the radius of the unit circle in %s [%lg]. Normalize r if n is appended\n", GMT_unit_names[gmtdefs.measure_unit], scale);
+		fprintf (stderr, "\t-T indicates that the vectors are oriented (two-headed), not directed [Default]\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
-		fprintf (stderr, "	-W sets pen attributes for outline of rose. [Default is no outline]\n");
+		fprintf (stderr, "\t-W sets pen attributes for outline of rose. [Default is no outline]\n");
 		GMT_explain_option ('X');
-		fprintf (stderr, "	-Z multiply the radii by scale before plotting\n");
+		fprintf (stderr, "\t-Z multiply the radii by scale before plotting\n");
 		GMT_explain_option ('c');
-		fprintf (stderr, "	-: Expect (azimuth,radius) input rather than (radius,azimuth) [%s]\n", choice[gmtdefs.xy_toggle]);
+		fprintf (stderr, "\t-: Expect (azimuth,radius) input rather than (radius,azimuth) [%s]\n", choice[gmtdefs.xy_toggle]);
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "	   Default is 2 input columns.\n");
-		fprintf (stderr, "	(See man page for gmtdefaults to set these and other defaults to ON or OFF)\n");
+		fprintf (stderr, "\t   Default is 2 input columns.\n");
+		fprintf (stderr, "\t(See man page for gmtdefaults to set these and other defaults to ON or OFF)\n");
 		exit (EXIT_FAILURE);
 	}
 
@@ -316,13 +313,12 @@ main (int argc, char **argv)
 	azimuth = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	length = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
 	
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (text, BUFSIZ, fp);
+	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (text, BUFSIZ, fp);
 		
 	/* Read data and do some stats */
 	
 	n = 0;
-	ix = gmtdefs.xy_toggle;	iy = 1 - ix;
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 2;
 	
 	n_fields = GMT_input (fp,  &n_expected_fields, &in);
 
@@ -333,8 +329,8 @@ main (int argc, char **argv)
 			exit (EXIT_FAILURE);
 		}
 
-		length[n] = in[ix];
-		azimuth[n] = in[iy];
+		length[n] = in[0];
+		azimuth[n] = in[1];
 		
 		if (zscale != 1.0) length[n] *= zscale;
 		
@@ -350,8 +346,9 @@ main (int argc, char **argv)
 		
 		/* Double angle to find mean azimuth */
 		
-		xr += length[n] * cos (one_or_two * azimuth[n] * D2R);
-		yr += length[n] * sin (one_or_two * azimuth[n] * D2R);
+		sincos (one_or_two * azimuth[n] * D2R, &s, &c);
+		xr += length[n] * c;
+		yr += length[n] * s;
 		
 		total += length[n];
 		
@@ -485,10 +482,12 @@ main (int argc, char **argv)
 		angle2 = (half_only) ? 0.0 : 90.0;
 		angle1 *= D2R;
 		angle2 *= D2R;
-		x1 = (sum[0] * scale) * cos (angle1);
-		y1 = (sum[0] * scale) * sin (angle1);
-		x2 = (sum[n_bins-1] * scale) * cos (angle2);
-		y2 = (sum[n_bins-1] * scale) * sin (angle2);
+		sincos (angle1, &s, &c);
+		x1 = (sum[0] * scale) * c;
+		y1 = (sum[0] * scale) * s;
+		sincos (angle2, &s, &c);
+		x2 = (sum[n_bins-1] * scale) * c;
+		y2 = (sum[n_bins-1] * scale) * s;
 		plot3d (x1, y1, 3);
 		plot3d (x2, y2, 2);
 		for (bin = n_bins-1; bin >= 0; bin--) {
@@ -520,7 +519,7 @@ main (int argc, char **argv)
 					mode_length = (double *) GMT_memory ((void *)mode_length, (size_t)n_alloc, sizeof (double), "psrose");
 				}
 			}
-			GMT_fclose (fpm);
+			fclose (fpm);
 			mode_direction = (double *) GMT_memory ((void *)mode_direction, (size_t)n_modes, sizeof (double), "psrose");
 			mode_length = (double *) GMT_memory ((void *)mode_length, (size_t)n_modes, sizeof (double), "psrose");
 		}
@@ -528,8 +527,9 @@ main (int argc, char **argv)
 			if (eq_area) mode_length[i] = sqrt (mode_length[i]);
 			if (half_only && mode_direction[i] > 90.0 && mode_direction[i] <= 270.0) mode_direction[i] -= 180.0;
 			angle = D2R * (90.0 - mode_direction[i]);
-			xr = scale * mode_length[i] * cos (angle);
-			yr = scale * mode_length[i] * sin (angle);
+			sincos (angle, &s, &c);
+			xr = scale * mode_length[i] * c;
+			yr = scale * mode_length[i] * s;
 			GMT_vector3d (0.0, 0.0, xr, yr, project_info.z_level, tailwidth, headlength, headwidth, gmtdefs.vector_shape, a_rgb, TRUE);
 		}
 		
@@ -541,8 +541,9 @@ main (int argc, char **argv)
 		n_alpha = (frame_info.grid_int[1] > 0.0) ? irint (total_arc / frame_info.grid_int[1]) : -1;
 		for (i = 0; i <= n_alpha; i++) {
 			angle = D2R * i * frame_info.grid_int[1];
-			x = max_radius * scale * cos (angle);
-			y = max_radius * scale * sin (angle);
+			sincos (angle, &s, &c);
+			x = max_radius * scale * c;
+			y = max_radius * scale * s;
 			plot3d (0.0, 0.0, 3);
 			plot3d (x, y, 2);
 		}
@@ -557,7 +558,11 @@ main (int argc, char **argv)
 		GMT_get_format (frame_info.anot_int[0], frame_info.unit[0], format);
 		
 		if (half_only) {
-			strcpy (lw, "90\\312W");	strcpy (le, "90\\312E");	strcpy (ln, "0\\312");
+			int which;
+			which = (gmtdefs.degree_format >= 10);	/* 0 is small [Default], 1 is large */
+			sprintf (lw, "90%sW\0", GMT_degree_symbol[which]);
+			sprintf (le, "90%sE\0", GMT_degree_symbol[which]);
+			sprintf (ln, "0%s\0", GMT_degree_symbol[which]);
 			y = -(3.0 * gmtdefs.anot_offset + GMT_font_height[gmtdefs.anot_font] * asize);
 			if (frame_info.label[0][0]) GMT_text3d (0.0, y, project_info.z_level, gmtdefs.label_font_size, gmtdefs.label_font, frame_info.label[0], 0.0, 10, 0);
 			y = -(5.0 * gmtdefs.anot_offset + GMT_font_height[gmtdefs.anot_font] * lsize + GMT_font_height[gmtdefs.label_font] * lsize);
@@ -614,7 +619,7 @@ main (int argc, char **argv)
 void arc3d (double x, double y, double r, double a1, double a2, int status)
 {
 	int i, n;
-	double da, a, *xx, *yy;
+	double da, a, *xx, *yy, s, c;
 	
 	if (project_info.three_D) {	/* Must project and draw line segments */
 		n = (int)ceil (a2 - a1);
@@ -622,11 +627,14 @@ void arc3d (double x, double y, double r, double a1, double a2, int status)
 		xx = (double *) GMT_memory (VNULL, (size_t)(n + 1), sizeof (double), GMT_program);
 		yy = (double *) GMT_memory (VNULL, (size_t)(n + 1), sizeof (double), GMT_program);
 		
+		a1 *= D2R;
+		a2 *= D2R;
 		da = (a2 - a1) / n;
 		for (i = 0; i <= n; i++) {
 			a = (i == n) ? a2 : a1 + da * i;
-			xx[i] = x + r * cosd (a);
-			yy[i] = y + r * sind (a);
+			sincos (a, &s, &c);
+			xx[i] = x + r * c;
+			yy[i] = y + r * s;
 		}
 		n++;	
 		GMT_2D_to_3D (xx, yy, n);
@@ -647,7 +655,7 @@ void plot3d (double x, double y, int pen)
 void pie3d (double x, double y, double rad, double a1, double a2, int rgb[], BOOLEAN outline)
 {
 	int i, n;
-	double da, a, *xx, *yy;
+	double da, a, *xx, *yy, s, c;
 	
 	if (project_info.three_D) {	/* Must project and draw line segments */
 		n = (int)ceil (a2 - a1);
@@ -655,11 +663,14 @@ void pie3d (double x, double y, double rad, double a1, double a2, int rgb[], BOO
 		xx = (double *) GMT_memory (VNULL, (size_t)(n + 2), sizeof (double), GMT_program);
 		yy = (double *) GMT_memory (VNULL, (size_t)(n + 2), sizeof (double), GMT_program);
 		
+		a1 *= D2R;
+		a2 *= D2R;
 		da = (a2 - a1) / n;
 		for (i = 0; i <= n; i++) {
 			a = (i == n) ? a2 : a1 + da * i;
-			xx[i] = x + rad * cosd (a);
-			yy[i] = y + rad * sind (a);
+			sincos (a, &s, &c);
+			xx[i] = x + rad * c;
+			yy[i] = y + rad * s;
 		}
 		xx[i] = x;	yy[i] = y;
 		n += 2;	
