@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)psscale.c	2.79  09/21/99
+ *	$Id: psscale.c,v 1.4 2001/03/19 18:15:09 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * psscale draws a grayscale or colorscale either vertically or
@@ -28,12 +28,16 @@
  * Modified:	01-AUG-1998
  *		3.3   13-APR-1999. PW: Fixed bad -X -Y behavior
  *		3.3.2 15-SEP-1999. PW: Fixed bad loop for -Z
+ *		3.3.4 21-JAN-2000. PW: Now paint rectangles for discrete cpt files
+ *		      08-FEB-2000. PW: Also understand patterns for discrete cpt files
+ * Version:	3.4
  *
  */
 
 #include "gmt.h"
 
 int bit_dpi = 300;
+BOOLEAN B_set = FALSE;
 
 void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN horizontal, BOOLEAN intens, double max_intens, BOOLEAN skip_lines, BOOLEAN extend, double e_length, BOOLEAN monochrome);
 void fix_format (char *unit, char *format);
@@ -49,7 +53,7 @@ main(int argc, char **argv)
 	double *z_width, e_length = 0.0, x_origin, y_origin;
 	
 	char flag, *cpt_file = NULL, *z_file = NULL, line[BUFSIZ];
-	char txt_a[32], txt_b[32], txt_c[32], txt_d[32], text[50];
+	char txt_a[32], txt_b[32], txt_c[32], txt_d[32], text[128];
 	
 	FILE *fp = NULL;
 	
@@ -66,6 +70,7 @@ main(int argc, char **argv)
 				/* Common parameters */
 
 				case 'B':
+					if (! (argv[i][2] == ':' || argv[i][2] == '/')) B_set = TRUE;
 				case 'K':
 				case 'O':
 				case 'P':
@@ -133,37 +138,37 @@ main(int argc, char **argv)
 	}
 	
 	if (argc == 1 || GMT_quick) {
-		fprintf (stderr,"%s %s - To create grayscale or colorscale for maps\n\n", GMT_program, GMT_VERSION);
-		fprintf(stderr,"usage: psscale -D<xpos/ypos/length/width[h]> [-C<cpt_file>] [-E[<length>]] [-B<tickinfo>] [-I[<intens>]\n");
-		fprintf(stderr,"	[-K] [-L] [-M] [-N<dpi>] [-O] [-P] [-S] [-U] [-V] [-X<x_shift>] [-Y<y_shift>] [-Z<zfile>] [-c<ncopies>]\n\n");
+		fprintf (stderr, "%s %s - To create grayscale or colorscale for maps\n\n", GMT_program, GMT_VERSION);
+		fprintf(stderr, "usage: psscale -D<xpos/ypos/length/width[h]> [-C<cpt_file>] [-E[<length>]] [-B<tickinfo>] [-I[<intens>]\n");
+		fprintf(stderr, "\t[-K] [-L] [-M] [-N<dpi>] [-O] [-P] [-S] [-U] [-V] [-X<x_shift>] [-Y<y_shift>] [-Z<zfile>] [-c<ncopies>]\n\n");
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr,"	-D set mid-point position and length/width for scale.\n");
-		fprintf (stderr,"	   Give negative length to reverse the scalebar\n");
-		fprintf (stderr,"	   Append h for horizontal scale\n");
-		fprintf(stderr,"\n\tOPTIONS:\n");
-		fprintf (stderr,"	-B Set scale anotation interval and label. Use y-label to set unit label\n");
-		fprintf (stderr,"	   If no anotation interval is set it is taken from the cpt file\n");
-		fprintf (stderr,"	-C Color palette file. If not set, stdin is read.\n");
-		fprintf (stderr,"	   By default all color changes are anotated (but see -B).  To use a subset,\n");
-		fprintf (stderr,"	   add an extra column to the cpt-file with a L, U, or B\n");
-		fprintf (stderr,"	   to anotate Lower, Upper, or Both color segment boundaries\n");
-		fprintf (stderr,"	-E add sidebar triangles for back- and forground colors.\n");
-		fprintf (stderr,"	   Optionally, append triangle height [Default scales with barwidth]\n");
-		fprintf (stderr,"	-I add illumination for +- <max_intens> [1.0]\n");
+		fprintf (stderr, "\t-D set mid-point position and length/width for scale.\n");
+		fprintf (stderr, "\t   Give negative length to reverse the scalebar\n");
+		fprintf (stderr, "\t   Append h for horizontal scale\n");
+		fprintf (stderr, "\n\tOPTIONS:\n");
+		fprintf (stderr, "\t-B Set scale anotation interval and label. Use y-label to set unit label\n");
+		fprintf (stderr, "\t   If no anotation interval is set it is taken from the cpt file\n");
+		fprintf (stderr, "\t-C Color palette file. If not set, stdin is read.\n");
+		fprintf (stderr, "\t   By default all color changes are anotated (but see -B).  To use a subset,\n");
+		fprintf (stderr, "\t   add an extra column to the cpt-file with a L, U, or B\n");
+		fprintf (stderr, "\t   to anotate Lower, Upper, or Both color segment boundaries\n");
+		fprintf (stderr, "\t-E add sidebar triangles for back- and forground colors.\n");
+		fprintf (stderr, "\t   Optionally, append triangle height [Default scales with barwidth]\n");
+		fprintf (stderr, "\t-I add illumination for +- <max_intens> [1.0]\n");
 		GMT_explain_option ('K');
-		fprintf (stderr,"	-L For equal-sized color rectangles\n");
-		fprintf (stderr, "	-M force monochrome colorbar using YIQ transformation\n");
-		fprintf (stderr,"	-N effective dots-per-inch for color scale [300]\n");
+		fprintf (stderr, "\t-L For equal-sized color rectangles\n");
+		fprintf (stderr, "\t-M force monochrome colorbar using YIQ transformation\n");
+		fprintf (stderr, "\t-N effective dots-per-inch for color scale [300]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr,"	-S Skip drawing color boundary lines on color scale [Default draws lines]\n");
+		fprintf (stderr, "\t-S Skip drawing color boundary lines on color scale [Default draws lines]\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
 		GMT_explain_option ('X');
-		fprintf (stderr,"	-Z give colorbar-width (in %s) per color entry\n", GMT_unit_names[gmtdefs.measure_unit]);
-		fprintf (stderr,"	   By default, width of entry is scaled to color range\n");
-		fprintf (stderr,"	   i.e., z = 0-100 gives twice the width as z = 100-150.\n");
+		fprintf (stderr, "\t-Z give colorbar-width (in %s) per color entry\n", GMT_unit_names[gmtdefs.measure_unit]);
+		fprintf (stderr, "\t   By default, width of entry is scaled to color range\n");
+		fprintf (stderr, "\t   i.e., z = 0-100 gives twice the width as z = 100-150.\n");
 		GMT_explain_option ('c');
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
@@ -207,7 +212,7 @@ main(int argc, char **argv)
 
 	z_width = (double *) GMT_memory (VNULL, (size_t)GMT_n_colors, sizeof (double), GMT_program);
 	
-	if (z_file && (fp = GMT_fopen (z_file, "r")) == NULL) {
+	if (z_file && (fp = fopen (z_file, "r")) == NULL) {
 		fprintf (stderr, "%s: Unable to open file %s\n", GMT_program, z_file);
 		exit (EXIT_FAILURE);
 	}
@@ -219,9 +224,13 @@ main(int argc, char **argv)
 				exit (EXIT_FAILURE);
 			}
 			if (line[0] == '#' || line[0] == '\n') continue;	/* Skip comments and blank lines */
-			sscanf (line, "%lf", &z_width[i++]);
+			n = sscanf (line, "%lf", &z_width[i++]);
+			if (n != 1) {
+				fprintf (stderr, "%s: Error reading file %s near line %d\n", GMT_program, z_file, i);
+				exit (EXIT_FAILURE);
+			}
 		}
-		GMT_fclose (fp);
+		fclose (fp);
 		if (i < GMT_n_colors) {
 			fprintf (stderr, "%s: -Z file %s has fewer slices than -C file %s!\n", GMT_program, z_file, cpt_file);
 			exit (EXIT_FAILURE);
@@ -255,17 +264,16 @@ main(int argc, char **argv)
 	if (horizontal) {
 		gmtdefs.x_origin = xpos - 0.5 * fabs (length);
 		gmtdefs.y_origin = ypos - width;
-		if (frame_info.anot_int[0] == 0.0) frame_info.plot = TRUE;
 		frame_info.side[1] = frame_info.side[2] = frame_info.side[3] = 0;
 	}
 	else {
 		gmtdefs.x_origin = xpos;
 		gmtdefs.y_origin = ypos - 0.5 * fabs (length);
-		if (frame_info.anot_int[0] == 0.0) frame_info.plot = TRUE;
 		frame_info.side[0] = frame_info.side[2] = frame_info.side[3] = 0;
 		d_swap (z_project.xmin, z_project.ymin);
 		d_swap (z_project.xmax, z_project.ymax);
 	}
+	if (frame_info.anot_int[0] == 0.0) frame_info.plot = TRUE;
 	gmtdefs.page_orientation |= 8;	/* Ensure absolute coordinates */
 
 /*-----------------end of kludge-------------------------------------------*/
@@ -275,18 +283,25 @@ main(int argc, char **argv)
 		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
 		
 	GMT_echo_command (argc, argv);
-	if (gmtdefs.unix_time) GMT_timestamp (argc, argv);
 	
 	ps_transrotate (x_origin, y_origin, 0.0);
 
 	ps_setpaint (gmtdefs.basemap_frame_rgb);
 	
+	if (gmtdefs.unix_time) {	/* Extra shift/unshift since psscale does things differently */
+		ps_transrotate (-gmtdefs.x_origin, -gmtdefs.y_origin, 0.0);
+		GMT_timestamp (argc, argv);
+		ps_transrotate (gmtdefs.x_origin, gmtdefs.y_origin, 0.0);
+	}
+	
 	GMT_draw_colorbar (length, width, z_width, horizontal, intens, max_intens, skip_lines, extend, e_length, monochrome);
 
 	if (must_shift_back) ps_transrotate (-x_origin, -y_origin, 0.0);
 
 	ps_plotend (gmtdefs.last_page);
 	
+	GMT_free ((void *)z_width);
+	
 	GMT_end (argc, argv);
 }
 
@@ -294,11 +309,14 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 {
 	int i, ii, j, ndec = -1, dec, rgb[3], rrggbb[3];
 	int nx, ny, nm, barmem, k;
-	BOOLEAN reverse, all = TRUE;
+	BOOLEAN reverse, all = TRUE, use_image;
 	char format[64], text[64], test[64];
 	unsigned char *bar;
-	double off, anot_off, label_off, len, len2, size, x0, x1, dx, xx, xt[3], yt[3];
-	double z, xleft, xright, inc_i, inc_j, get_z(double x, double *width, int n);
+	double off, anot_off, label_off, len, len2, size, x0, x1, dx, xx;
+	double z, xleft, xright, inc_i, inc_j;
+	double get_z (double x, double *width, int n);
+	double xp[4], yp[4];
+	struct GMT_FILL *f;
 	
 	ps_setfont (gmtdefs.anot_font);
 
@@ -306,7 +324,8 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 
 	/* Find max decimals needed */
 	
-	if (frame_info.anot_int[0] > 0.0) {     /* Used -B */
+
+	if (B_set) {
 		ndec = GMT_get_format (frame_info.anot_int[0], frame_info.unit[0], format);
 	}
 	else {
@@ -339,36 +358,38 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 	reverse = (length < 0.0);
 	length = fabs (length);
 	xright = length;
+
+	use_image = (GMT_continuous || intens);
 	
-	/* Make bitimage for colorbar using bit_dpi */
+	if (use_image) {	/* Make bitimage for colorbar using bit_dpi */
+		nx = irint (length * bit_dpi);
+		ny = irint (width * bit_dpi);
+		nm = nx * ny;
+		inc_i = 1.0 / bit_dpi;
+		inc_j = (intens) ? 2.0 * max_intens / (ny - 1) : 0.0;
+		barmem = (monochrome || GMT_gray) ? nm : 3 * nm;
+		bar = (unsigned char *) GMT_memory (VNULL, (size_t)barmem, sizeof (char), GMT_program);
 	
-	nx = irint (length * bit_dpi);
-	ny = irint (width * bit_dpi);
-	nm = nx * ny;
-	inc_i = 1.0 / bit_dpi;
-	inc_j = (intens) ? 2.0 * max_intens / (ny - 1) : 0.0;
-	barmem = (monochrome || GMT_gray) ? nm : 3 * nm;
-	bar = (unsigned char *) GMT_memory (VNULL, (size_t)barmem, sizeof (char), GMT_program);
+		/* Load bar image */
 	
-	/* Load bar image */
-	
-	for (i = 0; i < nx; i++) {
-		z = get_z (i * inc_i, z_width, GMT_n_colors);
-		GMT_get_rgb24 (z, rrggbb);
-		ii = (reverse) ? nx - i - 1 : i;
-		for (j = 0; j < ny; j++) {
-			for (k = 0; k < 3; k++) rgb[k] = rrggbb[k];
-			k = j * nx + ii;
-			if (intens) GMT_illuminate (max_intens - j * inc_j, rgb);
-			if (GMT_gray)	/* All gray, pick red */
-				bar[k] =  (unsigned char) rgb[0];
-			else if (monochrome)	/* Convert to gray using the YIQ transformation */
-				bar[k] =  (unsigned char) YIQ (rgb);
-			else {
-				k *= 3;
-				bar[k++] = (unsigned char) rgb[0];
-				bar[k++] = (unsigned char) rgb[1];
-				bar[k++] = (unsigned char) rgb[2];
+		for (i = 0; i < nx; i++) {
+			z = get_z (i * inc_i, z_width, GMT_n_colors);
+			GMT_get_rgb24 (z, rrggbb);
+			ii = (reverse) ? nx - i - 1 : i;
+			for (j = 0; j < ny; j++) {
+				for (k = 0; k < 3; k++) rgb[k] = rrggbb[k];
+				k = j * nx + ii;
+				if (intens) GMT_illuminate (max_intens - j * inc_j, rgb);
+				if (GMT_gray)	/* All gray, pick red */
+					bar[k] =  (unsigned char) rgb[0];
+				else if (monochrome)	/* Convert to gray using the YIQ transformation */
+					bar[k] =  (unsigned char) YIQ (rgb);
+				else {
+					k *= 3;
+					bar[k++] = (unsigned char) rgb[0];
+					bar[k++] = (unsigned char) rgb[1];
+					bar[k++] = (unsigned char) rgb[2];
+				}
 			}
 		}
 	}
@@ -378,18 +399,42 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 	/* Current point (0,0) is now at lower left location of bar */
 
 	if (horizontal) {
-		if (monochrome || GMT_gray)
-			ps_image (0.0, 0.0, length, width, bar, nx, ny, 8);
-		else 
-			GMT_color_image (0.0, 0.0, length, width, bar, nx, ny);
-
+		if (use_image) {	/* Must plot as image */
+			if (monochrome || GMT_gray)
+				ps_image (0.0, 0.0, length, width, bar, nx, ny, 8);
+			else 
+				GMT_color_image (0.0, 0.0, length, width, bar, nx, ny);
+		}
+		else {			/* Plot as rectangles */
+			x0 = x1 = 0.0;
+			yp[0] = yp[1] = 0.0;	yp[2] = yp[3] = width;
+			for (i = 0; i < GMT_n_colors; i++) {
+				x1 += z_width[i];
+				if ((f = GMT_lut[i].fill)) {	/* Using pattern fills */
+					xp[0] = xp[3] = x0;	xp[1] = xp[2] = x1;
+					ps_imagefill (xp, yp, 4, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+				}
+				else
+					ps_rect (x0, 0.0, x1, width, GMT_lut[i].rgb_low, FALSE);
+				x0 = x1;
+			}
+		}
+		
 		if (extend) {	/* Add color triangles for fore- and background */
-			xt[0] = xt[1] = xleft;	xt[2] = xt[0] - e_length;
-			yt[0] = width;	yt[1] = 0.0;	yt[2] = 0.5 * width;
-			ps_polygon (xt, yt, 3, GMT_bfn.background_rgb, 1);
-			xt[0] = xt[1] = xright;	xt[2] = xt[0] + e_length;
-			yt[0] = width;	yt[1] = 0.0;	yt[2] = 0.5 * width;
-			ps_polygon (xt, yt, 3, GMT_bfn.foreground_rgb, 1);
+			xp[0] = xp[1] = xleft;	xp[2] = xp[0] - e_length;
+			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
+			if ((f = GMT_bfn.fill[1]))
+				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+			else
+				ps_polygon (xp, yp, 3, GMT_bfn.background_rgb, FALSE);
+			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
+			xp[0] = xp[1] = xright;	xp[2] = xp[0] + e_length;
+			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
+			if ((f = GMT_bfn.fill[0]))
+				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+			else
+				ps_polygon (xp, yp, 3, GMT_bfn.foreground_rgb, FALSE);
+			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
 		}
 		
 		ps_plot (xleft, 0.0, 3);	ps_plotr (length, 0.0, 2);
@@ -397,8 +442,8 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 		ps_plot (xleft, 0.0, 3);	ps_plotr (0.0, width, 2);
 		ps_plot (xright, 0.0, 3);	ps_plotr (0.0, width, 2);
 
-		if (fabs (frame_info.anot_int[0]) > 0.0) {	/* Used -B */
-			GMT_x_axis (xleft, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, frame_info.anot_int[0], frame_info.frame_int[0], frame_info.label[0], frame_info.unit[0], LINEAR, 0, TRUE, 1);
+		if (B_set) {	/* Used -B */
+			GMT_x_axis (xleft, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, frame_info.anot_int[0], frame_info.frame_int[0], frame_info.label[0], frame_info.unit[0], LINEAR, 0, TRUE, frame_info.side[0]-1);
 			dx = fabs (frame_info.grid_int[0]); 
 			if (dx > 0.0) GMT_linearx_grid (GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, 0.0, width, dx);
 		}
@@ -460,10 +505,26 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 	}
 	else {	/* Vertical scale */
 		ps_transrotate (width, 0.0, 90.0);
-		if (monochrome || GMT_gray)
-			ps_image (0.0, 0.0, length, width, bar, nx, ny, 8);
-		else 
-			GMT_color_image (0.0, 0.0, length, width, bar, nx, ny);
+		if (use_image) {	/* Must plot with image */
+			if (monochrome || GMT_gray)
+				ps_image (0.0, 0.0, length, width, bar, nx, ny, 8);
+			else 
+				GMT_color_image (0.0, 0.0, length, width, bar, nx, ny);
+		}
+		else {			/* Plot as rectangles */
+			x0 = x1 = 0.0;
+			yp[0] = yp[1] = 0.0;	yp[2] = yp[3] = width;
+			for (i = 0; i < GMT_n_colors; i++) {
+				x1 += z_width[i];
+				if ((f = GMT_lut[i].fill)) {	/* Using pattern fills */
+					xp[0] = xp[3] = x0;	xp[1] = xp[2] = x1;
+					ps_imagefill (xp, yp, 4, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+				}
+				else
+					ps_rect (x0, 0.0, x1, width, GMT_lut[i].rgb_low, FALSE);
+				x0 = x1;
+			}
+		}
 		sprintf (text, "%d\0", (int)fabs (floor (GMT_lut[0].z_low)));
 		sprintf (test, "%d\0", (int)fabs (ceil (GMT_lut[GMT_n_colors-1].z_high)));
 		off = ((MAX ((int)strlen (text), (int)strlen (test)) + ndec) * 0.49 + ((ndec > 0) ? 0.3 : 0.0) + ((GMT_lut[0].z_low < 0.0) ? 0.3 : 0.0))
@@ -471,22 +532,30 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 		anot_off = -(((len > 0.0) ? len : 0.0) + gmtdefs.anot_offset + off);
 		label_off = anot_off - 0.7 * gmtdefs.anot_font_size * GMT_u2u[GMT_PT][GMT_INCH] - 1.5 * gmtdefs.anot_offset;
 		if (extend) {	/* Add color triangles for fore- and background */
-			xt[0] = xt[1] = xleft;	xt[2] = xt[0] - e_length;
-			yt[0] = width;	yt[1] = 0.0;	yt[2] = 0.5 * width;
-			ps_polygon (xt, yt, 3, GMT_bfn.background_rgb, 1);
-			xt[0] = xt[1] = xright;	xt[2] = xt[0] + e_length;
-			yt[0] = width;	yt[1] = 0.0;	yt[2] = 0.5 * width;
-			ps_polygon (xt, yt, 3, GMT_bfn.foreground_rgb, 1);
+			xp[0] = xp[1] = xleft;	xp[2] = xp[0] - e_length;
+			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
+			if ((f = GMT_bfn.fill[1]))
+				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+			else
+				ps_polygon (xp, yp, 3, GMT_bfn.background_rgb, FALSE);
+			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
+			xp[0] = xp[1] = xright;	xp[2] = xp[0] + e_length;
+			yp[0] = width;	yp[1] = 0.0;	yp[2] = 0.5 * width;
+			if ((f = GMT_bfn.fill[0]))
+				ps_imagefill (xp, yp, 3, f->pattern_no, f->pattern, f->inverse, f->dpi, FALSE, f->colorize, f->f_rgb, f->b_rgb);	/* Contours drawn separately below if desired */
+			else
+				ps_polygon (xp, yp, 3, GMT_bfn.foreground_rgb, FALSE);
+			ps_polygon (xp, yp, 3, GMT_no_rgb, 1);
 		}
 		ps_plot (xleft, 0.0, 3);	ps_plotr (length, 0.0, 2);
 		ps_plot (xleft, width, 3);	ps_plotr (length, 0.0, 2);
 		ps_plot (xleft, 0.0, 3);	ps_plotr (0.0, width, 2);
 		ps_plot (xright, 0.0, 3);	ps_plotr (0.0, width, 2);
-		if (fabs (frame_info.anot_int[0]) > 0.0) {	/* Used -B */
+		if (B_set) {	/* Used -B */
 			dx = fabs (frame_info.grid_int[0]); 
 			if (dx > 0.0) GMT_linearx_grid (GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, 0.0, width, dx);
 			ps_transrotate (0.0, 0.0, -90.0);
-			GMT_y_axis (0.0, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, frame_info.anot_int[0], frame_info.frame_int[0], CNULL, CNULL, LINEAR, 0, FALSE, 1);
+			GMT_y_axis (0.0, 0.0, length, GMT_lut[0].z_low, GMT_lut[GMT_n_colors-1].z_high, frame_info.anot_int[0], frame_info.frame_int[0], CNULL, CNULL, LINEAR, 0, FALSE, frame_info.side[0]-1);
 			ps_rotatetrans (0.0, 0.0, 90.0);
 		}
 		else {
@@ -549,6 +618,7 @@ void GMT_draw_colorbar (double length, double width, double *z_width, BOOLEAN ho
 		ps_rotatetrans (-width, 0.0, -90.0);
 
 	}
+	if (use_image) GMT_free ((void *)bar);
 }
 
 double get_z (double x, double *width, int n)
