@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)pstext.c	2.82  11/08/99
+ *	$Id: pstext.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * pstext will read (x, y, angle, size, fontno, justify, text) from GMT_stdin
@@ -22,10 +22,12 @@
  * one or more text paragraphs to be typeset.
  *
  * Author:	Paul Wessel
- * Date:	21-JAN-1991-1999
+ * Date:	21-JAN-1991-2000
  * Version:	2.0 based on old v1.x
  * Version:	3.1 based on old 3.0
  * Version:	3.3: PW: Introducing paragraph mode -M
+ * Version:	3.3.4: PW: Input x,y can be in dd:mm[:ss] format
+ * Version:	3.4
  *
  */
 
@@ -54,17 +56,17 @@ void GMT_putwords (double x, double y, char **text, int n_words, struct GMT_TEXT
 
 main(int argc, char **argv)
 {
-	int i, j, last_font, fno, n_files = 0, n_args, n_words = 0, nscan;
+	int i, j, k, last_font, fno, n_files = 0, n_args, n_words = 0, nscan;
 	int ix, iy, use_font[N_FONTS], n_processed = 0, n_read = 0, n_paragraphs = 0;
 	size_t n_alloc;
 	
 	double xy[2], west = 0., east = 0.0, south = 0.0, north = 0.0, plot_x, plot_y;
 	double new_z_level = 0.0, z_level = 0.0;
 	
-	BOOLEAN error = FALSE, nofile = TRUE, outline = FALSE, set_z = FALSE, master_record = FALSE;
+	BOOLEAN error = FALSE, nofile = TRUE, outline = FALSE, set_z = FALSE, master_record = FALSE, offset_outward = FALSE;
 	BOOLEAN solid = TRUE, do_paragraphs = FALSE, list = FALSE, done = FALSE, clip = TRUE, old_GMT_world_map, add;
 	
-	char text[BUFSIZ], line[BUFSIZ], this_font[80], just_key[5], txt_a[32], txt_b[32];
+	char text[BUFSIZ], line[BUFSIZ], this_font[80], just_key[5], txt_a[32], txt_b[32], txt_x[32], txt_y[32];
 	char *p, **word, pjust;
 	
 	FILE *fp = NULL;
@@ -119,7 +121,9 @@ main(int argc, char **argv)
 					T.y_space = GMT_convert_units (txt_b, GMT_INCH);
 					break;
 				case 'D':
-					for (j = 0; argv[i][j] && argv[i][j] != 'v'; j++);
+					k = 2;
+					if (argv[i][k] == 'j') offset_outward = TRUE, k++;
+					for (j = k; argv[i][j] && argv[i][j] != 'v'; j++);
 					if (argv[i][j] == 'v') {
 						T.boxflag |= 32;
 						if (argv[i][j+1] && GMT_getpen (&argv[i][j+1], &T.vecpen)) {
@@ -128,7 +132,7 @@ main(int argc, char **argv)
 						}
 						argv[i][j] = 0;
 					}
-					sscanf (&argv[i][2], "%[^/]/%s", txt_a, txt_b);
+					sscanf (&argv[i][k], "%[^/]/%s", txt_a, txt_b);
 					T.x_offset = GMT_convert_units (txt_a, GMT_INCH);
 					T.y_offset = GMT_convert_units (txt_b, GMT_INCH);
 					break;
@@ -179,6 +183,10 @@ main(int argc, char **argv)
 						}
 						T.boxflag |= 2;
 					}
+					if (!(T.boxflag | 31)) {
+						fprintf (stderr, "%s: GMT SYNTAX ERROR -W option: Must give arguments\n", GMT_program);
+						error++;
+					}
 					break;
 				case 'Z':
 					if (argv[i][2]) {
@@ -200,28 +208,28 @@ main(int argc, char **argv)
 	}
 	
 	if (argc == 1 || GMT_quick) {
-		fprintf (stderr,"pstext %s - To plot text on maps\n\n", GMT_VERSION);
-		fprintf(stderr,"usage: pstext <txtfile> -J<params> -R<west>/<east>/<south>/<north>\n");
-		fprintf (stderr, "	[-B<tickinfo>] [-C<dx>/<dy>] [-D<dx>/<dy>[v[<pen>]] [-Eaz/el] [-G<color>]\n");
-		fprintf (stderr, "	[-H[<nrec>]] [-K] [-L] [-M[<flag>]] [-N] [-O] [-P] [-S<pen>] [-U] [-V] [-W<fill>[o|O|c[<pen>]]]\n");
-		fprintf (stderr, "	[-X<x_shift>] [-Y<y_shift>] [-Z<zlevel>] [-c<ncopies>] [-:]\n\n");
-		fprintf(stderr,"	Reads (x,y,size,angle,fontno,justify,text) from <txtfile> [or stdin]\n");
-		fprintf(stderr,"	OR (with -M) one or more text paragraphs with formatting info in the segment header.\n");
-		fprintf(stderr,"	justify is of the form [T|M|B][L|C|R] (top/middle/bottom/left/center/right)\n");
-		fprintf(stderr,"	Built-in escape sequences:\n");
-		fprintf(stderr,"		@~ toggles between current font and Symbol font\n");
-		fprintf(stderr,"		@%%<no>%% switches to font number <no>; @%%%% resets font\n");
-		fprintf(stderr,"		@+ toggles between normal and superscript mode\n");
-		fprintf(stderr,"		@- toggles between normal and subscript mode\n");
-		fprintf(stderr,"		@# toggles between normal and Small Caps mode\n");
-		fprintf(stderr,"		@!<char1><char2> makes one composite character\n");
-		fprintf(stderr,"		@@ prints the @ sign itself\n");
-		fprintf(stderr,"		@e, @o, @a, @E, @O, @A give the accented Scandinavian characters\n");
-		fprintf(stderr,"		In addition, paragraph text (-M) also understands these sequences:\n");
-		fprintf(stderr,"		@:<size>: switches font size; @:: resets font size\n");
-		fprintf(stderr,"		@;<r/g/b>; switches font color; @;; resets font color\n");
-		fprintf(stderr,"		@_ toggles between normal and underlined text\n");
-		fprintf(stderr,"	   (See manual page for more information)\n");
+		fprintf (stderr, "pstext %s - To plot text on maps\n\n", GMT_VERSION);
+		fprintf (stderr, "usage: pstext <txtfile> -J<params> -R<west>/<east>/<south>/<north>\n");
+		fprintf (stderr, "\t[-B<tickinfo>] [-C<dx>/<dy>] [-D[j]<dx>/<dy>[v[<pen>]] [-Eaz/el] [-G<color>]\n");
+		fprintf (stderr, "\t[-H[<nrec>]] [-K] [-L] [-M[<flag>]] [-N] [-O] [-P] [-S<pen>] [-U] [-V] [-W[<fill>][o|O|c[<pen>]]]\n");
+		fprintf (stderr, "\t[-X<x_shift>] [-Y<y_shift>] [-Z<zlevel>] [-c<ncopies>] [-:]\n\n");
+		fprintf (stderr, "\tReads (x,y,size,angle,fontno,justify,text) from <txtfile> [or stdin]\n");
+		fprintf (stderr, "\tOR (with -M) one or more text paragraphs with formatting info in the segment header.\n");
+		fprintf (stderr, "\tjustify is of the form [T|M|B][L|C|R] (top/middle/bottom/left/center/right)\n");
+		fprintf (stderr, "\tBuilt-in escape sequences:\n");
+		fprintf (stderr, "\t   @~ toggles between current font and Symbol font\n");
+		fprintf (stderr, "\t   @%%<no>%% switches to font number <no>; @%%%% resets font\n");
+		fprintf (stderr, "\t   @+ toggles between normal and superscript mode\n");
+		fprintf (stderr, "\t   @- toggles between normal and subscript mode\n");
+		fprintf (stderr, "\t   @# toggles between normal and Small Caps mode\n");
+		fprintf (stderr, "\t   @!<char1><char2> makes one composite character\n");
+		fprintf (stderr, "\t   @@ prints the @ sign itself\n");
+		fprintf (stderr, "\t   @e, @o, @a, @E, @O, @A give the accented Scandinavian characters\n");
+		fprintf (stderr, "\t   In addition, paragraph text (-M) also understands these sequences:\n");
+		fprintf (stderr, "\t   @:<size>: switches font size; @:: resets font size\n");
+		fprintf (stderr, "\t   @;<r/g/b>; switches font color; @;; resets font color\n");
+		fprintf (stderr, "\t   @_ toggles between normal and underlined text\n");
+		fprintf (stderr, "\t(See manual page for more information)\n");
 		
 		if (list) {	/* List fonts */
 			fprintf (stderr, "\n\tFont #	Font Name\n");
@@ -234,34 +242,35 @@ main(int argc, char **argv)
 		
 		GMT_explain_option ('j');
 		GMT_explain_option ('R');
-		fprintf(stderr,"\n\tOPTIONS:\n");
+		fprintf (stderr, "\n\tOPTIONS:\n");
 		GMT_explain_option ('b');
-		fprintf(stderr,"	-C sets the clearance between characters and surrounding box [0.05i/0.05i]\n");
-		fprintf(stderr,"	   Only used if -W has been set\n");
-		fprintf(stderr,"	-D adds <add_x>,<add_y> to the text origin AFTER projecting with -J. [0/0]\n");
-		fprintf(stderr,"	   Append v[<pen>] to draw line from text to original point (-M only)\n");
-		fprintf (stderr, "	-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
-		fprintf (stderr, "	-G set the color (red/green/blue (0-255)) for solid text [0/0/0]\n");
+		fprintf (stderr, "\t-C sets the clearance between characters and surrounding box [0.05i/0.05i]\n");
+		fprintf (stderr, "\t   Only used if -W has been set\n");
+		fprintf (stderr, "\t-D adds <add_x>,<add_y> to the text origin AFTER projecting with -J. [0/0]\n");
+		fprintf (stderr, "\t   Use -Dj to move text origin away from point (direction determined by text's justification)\n");
+		fprintf (stderr, "\t   Append v[<pen>] to draw line from text to original point (-M only)\n");
+		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t-G set the color (red/green/blue (0-255)) for solid text [0/0/0]\n");
 		GMT_explain_option ('H');
 		GMT_explain_option ('K');
 		GMT_explain_option ('M');
 		fprintf (stderr, "\t   Expects (x y size angle fontno justify linespace parwidth parjust) in segment header\n");
 		fprintf (stderr, "\t   followed by lines with one or more paragraphs of text.\n");
 		fprintf (stderr, "\t   parjust is one of (l)eft, (c)enter, (r)ight, or (j)ustified.\n");
-		fprintf (stderr, "	-N Do Not clip text that exceeds the map boundaries [Default will clip]\n");
+		fprintf (stderr, "\t-N Do Not clip text that exceeds the map boundaries [Default will clip]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr, "	-S draw outline of characters.  Append pen attributes\n");
+		fprintf (stderr, "\t-S draw outline of characters.  Append pen attributes\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
-		fprintf (stderr, "	-W paints a colored rectangle underneath the text.  [Default is no rectangle]\n");
-		fprintf (stderr, "	   Append colors [255/255/255].   To draw outline, append o, O, or c, and optionally a pen [No outline]\n");
-		fprintf (stderr, "	   Lower case o will draw rectangular outline\n");
-		fprintf (stderr, "	   Upper case O will draw rectangle with rounded corners (-M only)\n");
-		fprintf (stderr, "	   Lower case c will draw concave rectangle (-M only)\n");
-		fprintf (stderr, "	   Upper case C will draw convex rectangle (-M only)\n");
+		fprintf (stderr, "\t-W paints and outlines a rectangle underneath the text.  [Default is no rectangle]\n");
+		fprintf (stderr, "\t   Append fill color [none].  To draw outline, append o, O, or c, and optionally a pen [No outline]\n");
+		fprintf (stderr, "\t   Lower case o will draw rectangular outline\n");
+		fprintf (stderr, "\t   Upper case O will draw rectangle with rounded corners (-M only)\n");
+		fprintf (stderr, "\t   Lower case c will draw concave rectangle (-M only)\n");
+		fprintf (stderr, "\t   Upper case C will draw convex rectangle (-M only)\n");
 		GMT_explain_option ('X');
-		fprintf (stderr, "      -Z For 3-D plots: Set the z-level of map [0]\n");
+		fprintf (stderr, "\t-Z For 3-D plots: Set the z-level of map [0]\n");
 		GMT_explain_option ('c');
 		GMT_explain_option (':');
 		GMT_explain_option ('.');
@@ -308,6 +317,7 @@ main(int argc, char **argv)
 	GMT_put_history (argc, argv);	/* Update .gmtcommands */
 
 	add = !(T.x_offset == 0.0 && T.y_offset == 0.0);
+	if (add && offset_outward) T.boxflag |= 64;
 	if (n_files > 0)
 		nofile = FALSE;
 	else
@@ -361,7 +371,7 @@ main(int argc, char **argv)
 			fp = GMT_stdin;
 			done = TRUE;
 		}
-		else if ((fp = GMT_fopen (argv[fno], "r")) == NULL) {
+		else if ((fp = fopen (argv[fno], "r")) == NULL) {
 			fprintf (stderr, "psxy: Cannot open file %s\n", argv[fno]);
 			continue;
 		}
@@ -371,6 +381,8 @@ main(int argc, char **argv)
 
 		while (fgets (line, BUFSIZ, fp)) {
 
+			if (line[0] == '\0' || (line[0] == '#' && GMT_io.EOF_flag != '#')) continue;	/* Skip blank lines or # comments */
+			
 			if (do_paragraphs) {	/* Paragraph mode */
 				if (line[0] == GMT_io.EOF_flag) {
 
@@ -380,7 +392,12 @@ main(int argc, char **argv)
 						n_paragraphs++;
 					}
 
-					nscan = sscanf (&line[2], "%lf %lf %d %lf %s %s %s %s %c\n", &xy[ix], &xy[iy], &T.font_size, &T.paragraph_angle, this_font, just_key, txt_a, txt_b, &pjust);
+					nscan = sscanf (&line[2], "%s %s %d %lf %s %s %s %s %c\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, txt_a, txt_b, &pjust);
+					if (!(GMT_scanf (txt_x, &xy[ix]) && GMT_scanf (txt_y, &xy[iy]))) {
+						fprintf (stderr, "%s: Record %d had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
+						exit (EXIT_FAILURE);
+					}
+					
 					if (nscan != 9) {
 						fprintf (stderr, "%s: Record %d had incomplete paragraph information, must exit)\n", GMT_program, n_read);
 						exit (EXIT_FAILURE);
@@ -447,23 +464,50 @@ main(int argc, char **argv)
 				n_read++;
 			}
 			else {	/* Old-style pstext input */
-				sscanf (line, "%lf %lf %d %lf %s %s %[^\n]\n", &xy[ix], &xy[iy], &T.font_size, &T.paragraph_angle, this_font, just_key, text);
+				nscan = sscanf (line, "%s %s %d %lf %s %s %[^\n]\n", txt_x, txt_y, &T.font_size, &T.paragraph_angle, this_font, just_key, text);
+				if (!(GMT_scanf (txt_x, &xy[ix]) && GMT_scanf (txt_y, &xy[iy]))) {
+					fprintf (stderr, "%s: Record %d had bad x and/or y coordinates, must exit)\n", GMT_program, n_read);
+					exit (EXIT_FAILURE);
+				}
+				if (nscan != 7) {
+					fprintf (stderr, "%s: Record %d is incomplete (skipped)\n", GMT_program, n_read);
+					continue;
+				}
 				n_read++;
 				GMT_geo_to_xy (xy[0], xy[1], &plot_x, &plot_y);
 				if (clip) {
 					GMT_map_outside (xy[0], xy[1]);
 					if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
 				}
-				if (add) {
-					plot_x += T.x_offset;
-					plot_y += T.y_offset;
-				}
 			
 				T.block_justify = (isdigit ((int)just_key[0])) ? atoi (just_key) : GMT_just_decode (just_key);
 				if (T.block_justify == -99) {
 					fprintf (stderr, "%s: Record %d had bad justification info (skipped)\n", GMT_program, n_read);
 					continue;
 				}
+				if (add) {
+					if (offset_outward) {	/* Smart offset according to justification (from Dave Huang) */
+						switch (T.block_justify & 3) {
+							case 1:
+								plot_x += T.x_offset;
+								break;
+							case 3:
+								plot_x -= T.x_offset;
+								break;
+						}
+						switch (T.block_justify >> 2) {
+							case 0:
+								plot_y += T.y_offset;
+								break;
+							case 2:
+								plot_y -= T.y_offset;
+								break;
+						}
+					} else {	/* Default hard offset */
+						plot_x += T.x_offset;
+						plot_y += T.y_offset;
+					}
+				}
 				if (this_font[0] >= '0' && this_font[0] <= '9')	/* Gave a font # */
 					T.paragraph_font = atoi (this_font);
 				else {	/* Gave a font name */
@@ -488,14 +532,14 @@ main(int argc, char **argv)
 					GMT_textbox3d (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, (T.boxflag & 1), T.x_space, T.y_space, T.boxfill.rgb);
 					GMT_setpen (&T.txtpen);
 				}
-				if (solid) {
-					ps_setpaint (T.txtfill.rgb);
-					GMT_text3d (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, FALSE);
-				}
 				if (outline) {
 					GMT_setpen (&T.txtpen);
 					GMT_text3d (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, TRUE);
 				}
+				if (solid) {
+					ps_setpaint (T.txtfill.rgb);
+					GMT_text3d (plot_x, plot_y, z_level, T.font_size, T.paragraph_font, text, T.paragraph_angle, T.block_justify, FALSE);
+				}
 			}
 		} /* Go read next line */
 
@@ -504,7 +548,7 @@ main(int argc, char **argv)
 			n_processed = n_words = 0;
 			n_paragraphs++;
 		}
-		if (fp != GMT_stdin) GMT_fclose (fp);
+		if (fp != GMT_stdin) fclose (fp);
 	}
 	
  	if (do_paragraphs) GMT_free ((void *)word);
