@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)pswiggle.c	2.69  11/29/99
+ *	$Id: pswiggle.c,v 1.4 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * pswiggle reads x,y,z from GMT_stdin and plots a wiggleplot using the
@@ -26,8 +26,8 @@
  * anomaly, and trackline.  Plotting of the outline and track is optional.
  *
  * Author:	Paul Wessel
- * Date:	20-NOV-1999
- * Version:	3.3.3
+ * Date:	20-JUN-2000
+ * Version:	3.4
  *
  */
  
@@ -43,13 +43,13 @@ void plot_wiggle (double *x, double *y, double *z, int np, double zscale, BOOLEA
 
 main (int argc, char **argv)
 {
-	int i, j, n, k, ix, iy, n_files = 0, fno, n_alloc = GMT_CHUNK, n_args, n_expected_fields, n_fields;
+	int i, j, n, k, n_files = 0, fno, n_alloc = GMT_CHUNK, n_args, n_expected_fields, n_fields;
 	
 	BOOLEAN error = FALSE, nofile = TRUE, outline = FALSE, track = FALSE, paint_wiggle, draw_scale = FALSE;
 	BOOLEAN do_fill = FALSE, negative = FALSE, fix = FALSE, done, gave_xy = FALSE;
 	
 	double west = 0.0, east = 360.0, south = 0.0, north = 0.0, x_2, y_2, bias = 0.0, fix_az;
-	double zscale = 0.0, dx, dy, dz, ds, x0, y0, length, dist_gap = DBL_MAX, *in, pref_dir = 0.0;
+	double zscale = 0.0, dx, dy, dz, ds, x0, y0, length, dist_gap = DBL_MAX, *in, pref_az = 0.0;
 	
 	char line[BUFSIZ], *units, txt_a[32], txt_b[32], unit = 0;
 	
@@ -95,10 +95,10 @@ main (int argc, char **argv)
 					error += GMT_io_selection (&argv[i][2]);
 					break;
 				case 'A':
-					pref_dir = atof (&argv[i][2]);
+					pref_az = atof (&argv[i][2]);
 					break;
 				case 'D':
-					dist_gap = atof (&argv[i][2]) * 1000.0;
+					dist_gap = atof (&argv[i][2]);
 					break;
 				case 'E':
 					sscanf (&argv[i][2], "%lf/%lf", &z_project.view_azimuth, &z_project.view_elevation);
@@ -173,45 +173,45 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr,"pswiggle %s - Plot xyz-series along tracks\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: pswiggle <xyz-files> -J<params> -R<west/east/south/north> -Z<scale>\n");
-		fprintf (stderr, "	[-A<azimuth>] [-B<tickinfo>] [-C<center>] [-D<gap>] [-E<az/el>] [-G<fill>] [-H[<nrec>]]\n");
-		fprintf (stderr, "	[-I<az>] [-K] [-M[<flag>]] [-N] [-O] [-P] [-S[x]<lon0>/<lat0>/<length>/<units>] [-T<trackpen>]\n");
-		fprintf (stderr, "	[-U] [-V] [-W<outlinepen>] [-X<x_shift>] [-Y<y_shift>] [-c<ncopies>] [-:] [-bi[s][<n>]\n");
+		fprintf (stderr, "\t[-A<azimuth>] [-B<tickinfo>] [-C<center>] [-D<gap>] [-E<az/el>] [-G<fill>] [-H[<nrec>]]\n");
+		fprintf (stderr, "\t[-I<az>] [-K] [-M[<flag>]] [-N] [-O] [-P] [-S[x]<lon0>/<lat0>/<length>/<units>] [-T<trackpen>]\n");
+		fprintf (stderr, "\t[-U] [-V] [-W<outlinepen>] [-X<x_shift>] [-Y<y_shift>] [-c<ncopies>] [-:] [-bi[s][<n>]\n");
 
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr, "	<xyz_files> is one or more files.  If none, read standard input\n");
+		fprintf (stderr, "\t<xyz_files> is one or more files.  If none, read standard input\n");
 		GMT_explain_option ('j');
 		GMT_explain_option ('R');
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf (stderr, "	-A set azimuth for preferred positive wiggle orientation [0.0 (north)]\n");
+		fprintf (stderr, "\t-A set azimuth for preferred positive wiggle orientation [0.0 (north)]\n");
 		GMT_explain_option ('b');
-		fprintf (stderr, "	-C sets center value to be removed from z before plotting. [0]\n");
-		fprintf (stderr, "	-D means there is a datagap if 2 points are more than <gap> distance units apart\n");
-		fprintf (stderr, "	   gap is in km for map projections, user-units otherwise\n");
-		fprintf (stderr, "	-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
-		fprintf (stderr, "	-G Specify color or pattern for positive areas. fill can be either\n");
-		fprintf (stderr, "	   1) <r/g/b> (each 0-255) for color or <gray> (0-255) for gray-shade [0]\n");
-		fprintf (stderr, "	   2) p[or P]<dpi>/<pattern> for predefined patterns (1-90).\n");
+		fprintf (stderr, "\t-C sets center value to be removed from z before plotting. [0]\n");
+		fprintf (stderr, "\t-D means there is a datagap if 2 points are more than <gap> distance units apart\n");
+		fprintf (stderr, "\t   gap is in km for map projections, user-units otherwise\n");
+		fprintf (stderr, "\t-E set azimuth and elevation of viewpoint for 3-D perspective [180/90]\n");
+		fprintf (stderr, "\t-G Specify color or pattern for positive areas. fill can be either\n");
+		fprintf (stderr, "\t   1) <r/g/b> (each 0-255) for color or <gray> (0-255) for gray-shade [0]\n");
+		fprintf (stderr, "\t   2) p[or P]<dpi>/<pattern> for predefined patterns (1-90).\n");
 		GMT_explain_option ('H');
-		fprintf (stderr, "	-I set fixed projection azimuths for wiggles\n");
+		fprintf (stderr, "\t-I set fixed projection azimuths for wiggles\n");
 		GMT_explain_option ('K');
 		GMT_explain_option ('M');
-		fprintf (stderr, "	-N Fill negative wiggles instead [Default is positive]\n");
+		fprintf (stderr, "\t-N Fill negative wiggles instead [Default is positive]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr, "      -S draws a simple vertical scale centered on <lon0>/<lat0>.  Use -Sx to specify cartesian coordinates instead.\n");
-		fprintf (stderr, "         <length> is in z-units, append unit name for labeling\n");
-		fprintf (stderr, "	-T specifies track pen attributes. [Default is no track]\n");
+		fprintf (stderr, "\t-S draws a simple vertical scale centered on <lon0>/<lat0>.  Use -Sx to specify cartesian coordinates instead.\n");
+		fprintf (stderr, "\t   <length> is in z-units, append unit name for labeling\n");
+		fprintf (stderr, "\t-T specifies track pen attributes. [Default is no track]\n");
 		GMT_explain_option ('U');
 		GMT_explain_option ('V');
-		fprintf (stderr, "	-W specifies outline pen attributes. [Default is no outline]\n");
+		fprintf (stderr, "\t-W specifies outline pen attributes. [Default is no outline]\n");
 		GMT_explain_option ('X');
-		fprintf (stderr, "	-Z gives the wiggle scale in data-units per %s\n", GMT_unit_names[gmtdefs.measure_unit]);
+		fprintf (stderr, "\t-Z gives the wiggle scale in data-units per %s\n", GMT_unit_names[gmtdefs.measure_unit]);
 		GMT_explain_option ('c');
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "	Default is 3 input columns\n");
+		fprintf (stderr, "\t   Default is 3 input columns\n");
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -253,6 +253,8 @@ main (int argc, char **argv)
 
 	GMT_map_setup (west, east, south, north);
 
+	if (MAPPING) dist_gap *= 1000.0;    /* Distance in meters */
+	
 	ps_plotinit (CNULL, gmtdefs.overlay, gmtdefs.page_orientation, gmtdefs.x_origin, gmtdefs.y_origin,
 		gmtdefs.global_x_scale, gmtdefs.global_y_scale, gmtdefs.n_copies,
 		gmtdefs.dpi, GMT_INCH, gmtdefs.paper_width, gmtdefs.page_rgb, GMT_epsinfo (argv[0]));
@@ -280,15 +282,13 @@ main (int argc, char **argv)
 			zscale *= GMT_u2u[GMT_PT][GMT_INCH];
 			break;
 		default:
+			zscale *= GMT_u2u[gmtdefs.measure_unit][GMT_INCH];
 			break;
 	}
-
-	if (fix) fix_az = (90.0 - fix_az) - 90.0;	/* fiz_az is given in azimuth, convert to direction, then sub 90 to get equivalent trackline direction*/
-	pref_dir = 90.0 - pref_dir;		/* pref_dir is given in azimuth, convert to direction */
 	
 	/* Now convert angles to radians and keep it that way */
-	start_az = (pref_dir - 90.0) * D2R;
-	stop_az  = (pref_dir + 90.0) * D2R;
+	start_az = (pref_az - 90.0) * D2R;
+	stop_az  = (pref_az + 90.0) * D2R;
 	fix_az *= D2R;
 	
 	GMT_map_clip_on (GMT_no_rgb, 3);
@@ -323,18 +323,16 @@ main (int argc, char **argv)
 		
 		if (!nofile && gmtdefs.verbose) {
 			fprintf (stderr, "%s: Working on file %s\n", GMT_program, argv[fno]);
-			printf ("%% file %s\n", argv[fno]);
+			sprintf (line, "File %s\0", argv[fno]);
+			ps_comment (line);
 		}
 		
-		ix = (gmtdefs.xy_toggle);	iy = 1 - ix;
-		
-		
-		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 		if (GMT_io.multi_segments) {
-			fgets (line, BUFSIZ, fp);
+			GMT_fgets (line, BUFSIZ, fp);
 			if (gmtdefs.verbose) ps_comment (line);
 		}
-		n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+		n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 3;
 
 		n_fields = GMT_input (fp, &n_expected_fields, &in);
 		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
@@ -352,8 +350,8 @@ main (int argc, char **argv)
 					exit (EXIT_FAILURE);
 				}
 
-				lon[n] = in[ix];
-				lat[n] = in[iy];
+				lon[n] = in[0];
+				lat[n] = in[1];
 				z[n] = in[2] - bias;
 				n++;
 				if (n >= n_alloc) {
@@ -376,7 +374,7 @@ main (int argc, char **argv)
 			for (i = 1; i < n; i++) {	/* Convert to inches/cm and get distance increments */
 				dx = lon[i] - lon[i-1];
 				dy = lat[i] - lat[i-1];
-				if (project_info.projection > 5) {	/* Distance in meters */
+				if (MAPPING) {	/* Distance in meters */
 					dx *= cos (0.5 * (lat[i] + lat[i-1]) * D2R);
 					ds = hypot (dx, dy) * project_info.M_PR_DEG;
 				}
@@ -447,10 +445,8 @@ void plot_wiggle (double *x, double *y, double *z, int np, double zscale, BOOLEA
 {
 	double dx, dy, len, az = 0.0, s, c, x_inc, y_inc;
 	int n, i;
-	static char *name[2] = {"Positive", "Negative"};
+	static char *name[2] = {"Positive", "Negative"}, comment[32];
 
-	/* Despite variable names, calculations here use directions, not azimuths */
-	
 	if (fixed) {
 		az = fix_az;
 		sincos (az, &s, &c);
@@ -465,14 +461,14 @@ void plot_wiggle (double *x, double *y, double *z, int np, double zscale, BOOLEA
 			if (!fixed && i < np-1) {
 				dx = x[i+1] - x[i];
 				dy = y[i+1] - y[i];
-				if (!(dx == 0.0 && dy == 0.0)) az = d_atan2 (dy, dx) - TWO_PI;
+				if (!(dx == 0.0 && dy == 0.0)) az = -d_atan2 (dy, dx) - TWO_PI;	/* Azimuth of normal to track */
 				while (az < start_az) az += TWO_PI;
 				if (az > stop_az) az -= M_PI;
 			}
 			if (fabs (z[i]) > 0.0) {
 				if (!fixed) sincos (az, &s, &c);
 				len = zscale * z[i];
-				x_inc = -len * s;
+				x_inc = len * s;
 				y_inc = len * c;
 			}
 			else
@@ -499,19 +495,22 @@ void plot_wiggle (double *x, double *y, double *z, int np, double zscale, BOOLEA
 	
 	
 	if (paint_wiggle) { /* First shade wiggles */
-		printf ("%% %s wiggle\n", name[negative]);
+		sprintf (comment, "%s wiggle\0", name[negative]);
+		ps_comment (comment);
 		GMT_fill (GMT_x_plot, GMT_y_plot, n, fill, FALSE);
 	}
 	
 	if (outline) { /* Then draw wiggle outline */
-		printf ("%% Wiggle line\n");
+		sprintf (comment, "Wiggle line\0");
+		ps_comment (comment);
 		GMT_setpen (pen_o);
 		ps_line (&GMT_x_plot[1], &GMT_y_plot[1], np, 3, FALSE, TRUE);
 	}
 	
 	if (track) {	/* Finally draw track line */
 		if (project_info.three_D) GMT_2D_to_3D (x, y, np);
-		printf ("%% Track line\n");
+		sprintf (comment, "Track line\0");
+		ps_comment (comment);
 		GMT_setpen (pen_t);
 		ps_line (x, y, np, 3, FALSE, TRUE);
 	}
@@ -520,7 +519,7 @@ void plot_wiggle (double *x, double *y, double *z, int np, double zscale, BOOLEA
 void GMT_draw_z_scale (double x0, double y0, double length, double zscale, BOOLEAN gave_xy, char *units)
 {
 	double dy, a, b, off;
-	char txt[80];
+	char txt[128];
 	
 	GMT_setpen (&gmtdefs.tick_pen);
 
