@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)psxyz.c	2.120  11/24/99
+ *	$Id: psxyz.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * psxyz will read <x,y,z> triplets from GMT_stdin and plot
@@ -22,12 +22,12 @@
  * size is the symbol size.  Several map projections are supported.
  * For linear projections a 3-D basemap is provided.  All symbols are
  * projected onto the xy plane (so that circles becomes ellipses) except
- * BAR and CUBE which is fully 3-D.
+ * COLUMN and CUBE which are fully 3-D.
  * PostScript code is then written to GMT_stdout.
  *
  * Author:    Paul Wessel
- * Date:      24-NOV-1999
- * Version:   3.3.3
+ * Date:      22-AUG-2000
+ * Version:   3.4
  *
  */
 
@@ -51,13 +51,36 @@
 #define ELLIPSE		13
 #define VECTOR		14
 #define VECTOR2		15
-#define TEXT		16
-#define FAULT		-17
+#define PIE		16
+#define TEXT		17
+#define FRONT		-18
+
+/* FRONT symbols */
+
+#define F_FAULT		0
+#define F_TRIANGLE	1
+#define F_SLIP		2
+#define F_CIRCLE	3
+#define F_BOX		4
+
+/* Direction of front symbols: */
+
+#define F_LEFT		+1
+#define F_CENTERED	0
+#define F_RIGHT		-1
 
 #define POINTSIZE 0.005
 
 double *xp, *yp;
 
+struct FRONTLINE {		/* A sub-symbol for symbols along a front */
+	double f_gap;		/* Gap between front symbols in inches */
+	double f_len;		/* Length of front symbols in inches */
+	double f_off;		/* Offset of first symbol from start of front in inches */
+	int f_sense;		/* Draw symbols to left (+1), centered (0), or right (-1) of line */
+	int f_symbol;		/* Which symbol to draw along the front line */
+};
+
 struct PSXYZ_SYMBOL {
 	int symbol;		/* Symbol id */
 	int n_required;		/* Number of additional columns necessary to decode chosen symbol */
@@ -87,10 +110,7 @@ struct PSXYZ_SYMBOL {
 
 	char string[64];	/* Character code to plot (could be octal) */
 
-	double f_gap;		/* Gap between fault ticks in inches */
-	double f_len;		/* Length of fault ticks in inches */
-	int f_sense;		/* Draw tick to left (+1), centered (0), or right (-1) of line */
-	BOOLEAN f_triangle;	/* TRUE if teeth should be drawn, default is ticks */
+	struct FRONTLINE f;	/* parameters needed for a front */
 } S;
 
 
@@ -118,20 +138,21 @@ void hexagon3D(double x, double y, double z, double size, int *rgb, int outline)
 void star3D(double x, double y, double z, double size, int *rgb, int outline);
 void diamond3D(double x, double y, double z, double size, int *rgb, int outline);
 void ellipse3D (double x, double y, double z, double direction, double major, double minor, int rgb[], int outline);
+void pie3D (double x, double y, double z, double size, double dir1, double dir2, int rgb[], int outline);
 void sort_on_distance(struct DATA1 *data, int n);
 
 int decode_symbol_option (char *text, struct PSXYZ_SYMBOL *p, BOOLEAN cmd);
 void GMT_plot_ellipse (double lon, double lat, double z, double major, double minor, double azimuth, struct GMT_FILL fill, int outline);
-void GMT_draw_fence (double x[], double y[], int n, double gap, double len, int sense, int triangle, int rgb[], BOOLEAN outline);
+void GMT_draw_fence (double x[], double y[], int n, struct FRONTLINE *f, struct GMT_FILL *g, BOOLEAN outline);
 
 main (int argc, char **argv)
 {
-	int 	i, j, n, ix = 0, iy = 1, n_files = 0, fno, rgb[9], n_cols_start = 3, n_fields;
+	int 	i, j, n, n_files = 0, fno, rgb[9], n_cols_start = 3, n_fields;
 	int	n_alloc = GMT_CHUNK, n_args, three, four, five, n_required = 3;
 	int	n_total_read = 0, font_size, n_expected = 0;
 	
 	BOOLEAN	error = FALSE, nofile = TRUE, polygon = FALSE, done, close_polygon = FALSE;
-	BOOLEAN	fill_set = FALSE, outline = FALSE, get_rgb = FALSE, read_symbol, set_z = FALSE;
+	BOOLEAN	fill_set = FALSE, outline = FALSE, get_rgb = FALSE, read_symbol, set_z = FALSE, clip_set = FALSE;
 	BOOLEAN skip_if_outside = TRUE, default_outline = FALSE, default_polygon = FALSE, cpt_given = FALSE;
 	
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, new_z_level = 0.0;
@@ -147,6 +168,8 @@ main (int argc, char **argv)
 	
 	argc = GMT_begin (argc, argv);
 	
+	GMT_3D_mode = 1;	/* Only do background axis first; do foreground at end */
+	
 	GMT_init_pen (&pen, GMT_PENWIDTH);
 	GMT_init_fill (&fill, -1, -1, -1);      /* Default is no fill */
 
@@ -254,63 +277,70 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {	/* Display usage */
 		fprintf (stderr,"psxyz %s - Plot lines, polygons, and symbols in 3-D\n\n", GMT_VERSION);
 		fprintf(stderr,"usage: psxyz <xyzfiles> -R<west/east/south/north/zmin/zmax> -J<params>\n");
-		fprintf(stderr, "	-Jz<params> [-B<tickinfo>] [-C<cpt>] [-E<az/el>] [-G<fill>] [-H[<nrec>]] [-K] [-L]\n");
-		fprintf(stderr, "	[-M[<flag>]] [-N] [-O] [-P] [-S<symbol><size>[/size_y]] [-U] [-V] [-W[<pen>]] [-X<x_shift>]\n");
-		fprintf(stderr, "	[-Y<y_shift>] [-:] [-c<ncopies>] [-bi[s][<n>]]\n");
+		fprintf(stderr, "\t-Jz<params> [-B<tickinfo>] [-C<cpt>] [-E<az/el>] [-G<fill>] [-H[<nrec>]] [-K] [-L]\n");
+		fprintf(stderr, "\t[-M[<flag>]] [-N] [-O] [-P] [-S<symbol><size>[/size_y]] [-U] [-V] [-W[<pen>]] [-X<x_shift>]\n");
+		fprintf(stderr, "\t[-Y<y_shift>] [-:] [-c<ncopies>] [-bi[s][<n>]]\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr, "	<xyzfiles> is one or more files.  If none, read standard input\n");
+		fprintf (stderr, "\t<xyzfiles> is one or more files.  If none, read standard input\n");
 		GMT_explain_option ('j');
 		GMT_explain_option ('Z');
 		GMT_explain_option ('r');
 		fprintf (stderr, "\n\tOPTIONS:\n");
 		GMT_explain_option ('b');
-		fprintf (stderr, "	-C Use cpt-file to assign symbol colors based on t-value in 4th column (requires -S)\n");
-		fprintf (stderr, "	   Without -S, psxyz expects -M polygons and looks for -Z<tval> strings in each multiheader\n");
-		fprintf (stderr, "	-E set user viewpoint azimuth and elevation [180/90].\n");
-		fprintf (stderr, "	-G Specify color (for symbols/polygons) or pattern (for polygons). fill can be either\n");
-		fprintf (stderr, "	   1) <r/g/b> (each 0-255) for color or <gray> (0-255) for gray-shade [0].\n");
-		fprintf (stderr, "	   2) p[or P]<dpi>/<pattern> for predefined patterns (1-90).\n");
-		fprintf (stderr, "	   If -G is specified but not -S, psxyz draws a filled polygon.\n");
-		fprintf (stderr, "	   Default is no fill (transparent symbols or polygons)\n");
+		fprintf (stderr, "\t-C Use cpt-file to assign symbol colors based on t-value in 4th column (requires -S)\n");
+		fprintf (stderr, "\t   Without -S, psxyz expects -M polygons and looks for -Z<tval> strings in each multiheader\n");
+		fprintf (stderr, "\t-E set user viewpoint azimuth and elevation [180/90].\n");
+		fprintf (stderr, "\t-G Specify color (for symbols/polygons) or pattern (for polygons). fill can be either\n");
+		fprintf (stderr, "\t   1) <r/g/b> (each 0-255) for color or <gray> (0-255) for gray-shade [0].\n");
+		fprintf (stderr, "\t   2) p[or P]<dpi>/<pattern> for predefined patterns (1-90).\n");
+		fprintf (stderr, "\t   If -G is specified but not -S, psxyz draws a filled polygon.\n");
+		fprintf (stderr, "\t   Default is no fill (transparent symbols or polygons)\n");
 		GMT_explain_option ('H');
 		GMT_explain_option ('K');
-		fprintf (stderr, "	-L Force closed polygons\n");
+		fprintf (stderr, "\t-L Force closed polygons\n");
 		GMT_explain_option ('M');
-		fprintf (stderr, "	-N Do Not skip symbols that fall outside map border [Default will ignore those outside]\n");
+		fprintf (stderr, "\t-N Do Not skip symbols that fall outside map border [Default will ignore those outside]\n");
 		GMT_explain_option ('O');
 		GMT_explain_option ('P');
-		fprintf (stderr, "	-S to select symbol type and symbol size (in %s).  Choose between\n", GMT_unit_names[gmtdefs.measure_unit]);
-		fprintf (stderr, "	   st(a)r, (b)ar, (c)ircle, (d)iamond, (e)llipse, (f)ault, (h)exagon, (i)nvtriangle\n");
-		fprintf (stderr, "	   (l)etter, c(o)lumn, (p)oint, (s)quare, (t)riangle, c(u)be, (v)ector, (x)cross\n");
-		fprintf (stderr, "	   If no size is specified, psxyz expects the 4th column to have sizes.\n");
-		fprintf (stderr, "	   If no symbol is specified, psxyz expects the last column to have symbol code.\n");
-		fprintf (stderr, "	   [Note: if -C is selected then 4th means 5th column, etc.]\n");
-		fprintf (stderr, "	   column and cube are true 3-D objects (give size as xsize/ysize), the rest is 2-D perspective only.\n");
-		fprintf (stderr, "	   Symbols A, C, D, H, I, S, T are adjusted to have same area of a circle of given size\n");
-		fprintf (stderr, "	   Bar (or Column): Append b<base> to give the y- (or z-) value of the base [Default = 0 (1 for log-scales)]\n");
-		fprintf (stderr, "	   Ellipses: the direction, major, and minor axis must be in input columns 4, 5 and 6.\n");
-		fprintf (stderr, "	     If -SE rather than -Se is selected, psxyz will expect azimuth, and axes in km\n");
-		fprintf (stderr, "	     and convert azimuths based on the chosen map projection\n");
-		fprintf (stderr, "	   Faults: Give tickgap/ticklen[type], where type is:\n");
-		fprintf (stderr, "	     l - draw tick to left, r - draw tick to right [Default is centered]\n");
-		fprintf (stderr, "	     Upper case L or R gives filled triangles instead of tick\n");
-		fprintf (stderr, "	   Letter: append /<string> after symbol size, and optionally %%<font>\n");
-		fprintf (stderr, "         Vectors: the direction and length must be in input columns 4 and 5.\n");
-		fprintf (stderr, "	     Furthermore, <size> means arrowwidth/headlength/headwith [Default is %lg/%lg/%lg]\n", S.v_width, S.h_length, S.h_width);
-		fprintf (stderr, "           If -SV rather than -Sv is selected, psxyz will expect azimuth and length\n");
-		fprintf (stderr, "           and convert azimuths based on the chosen map projection\n");		GMT_explain_option ('U');
+		fprintf (stderr, "\t-S to select symbol type and symbol size (in %s).  Choose between\n", GMT_unit_names[gmtdefs.measure_unit]);
+		fprintf (stderr, "\t   st(a)r, (b)ar, (c)ircle, (d)iamond, (e)llipse, (f)ront, (h)exagon, (i)nvtriangle\n");
+		fprintf (stderr, "\t   (l)etter, c(o)lumn, (p)oint, (s)quare, (t)riangle, c(u)be, (v)ector, (w)edge, (x)cross\n");
+		fprintf (stderr, "\t   If no size is specified, psxyz expects the 4th column to have sizes.\n");
+		fprintf (stderr, "\t   If no symbol is specified, psxyz expects the last column to have symbol code.\n");
+		fprintf (stderr, "\t   [Note: if -C is selected then 4th means 5th column, etc.]\n");
+		fprintf (stderr, "\t   column and cube are true 3-D objects (give size as xsize/ysize), the rest is 2-D perspective only.\n");
+		fprintf (stderr, "\t   Symbols A, C, D, H, I, S, T are adjusted to have same area of a circle of given size\n");
+		fprintf (stderr, "\t   Bar (or Column): Append b<base> to give the y- (or z-) value of the base [Default = 0 (1 for log-scales)]\n");
+		fprintf (stderr, "\t   Ellipses: the direction, major, and minor axis must be in input columns 4, 5 and 6.\n");
+		fprintf (stderr, "\t     If -SE rather than -Se is selected, psxyz will expect azimuth, and axes in km\n");
+		fprintf (stderr, "\t     and convert azimuths based on the chosen map projection\n");
+		fprintf (stderr, "\t   Fronts: Give tickgap/ticklen[dir][type][:offset], where\n");
+		fprintf (stderr, "\t     dir    = Plot symbol to the l(eft) or r(ight) of front [Default is centered]\n");
+		fprintf (stderr, "\t     type   =  b(ox), c(ircle), f(ault), s(lip), t(riangle) [Default is f]\n");
+		fprintf (stderr, "\t       box      : square when centered, half-square otherwise.\n"); 
+		fprintf (stderr, "\t       circle   : full when centered, half-circle otherwise.\n"); 
+		fprintf (stderr, "\t       fault    : centered cross-tick or tick only in the <dir> direction.\n"); 
+		fprintf (stderr, "\t       slip     : left-lateral or right-lateral strike-slip arrows (centered is not defined).\n"); 
+		fprintf (stderr, "\t       triangle : diagonal square when centered, directed triangle otherwise.\n"); 
+		fprintf (stderr, "\t     offset = start plotting symbols when along-track distance equals offset [0].\n"); 
+		fprintf (stderr, "\t   Letter: append /<string> after symbol size, and optionally %%<font>\n");
+		fprintf (stderr, "\t   Vectors: the direction and length must be in input columns 4 and 5.\n");
+		fprintf (stderr, "\t     Furthermore, <size> means arrowwidth/headlength/headwith [Default is %lg/%lg/%lg]\n", S.v_width, S.h_length, S.h_width);
+		fprintf (stderr, "\t     If -SV rather than -Sv is selected, psxyz will expect azimuth and length\n");
+		fprintf (stderr, "\t     and convert azimuths based on the chosen map projection\n");		GMT_explain_option ('U');
+		fprintf (stderr, "\t   Wedges: the start and stop directions of pie wedge must be in input columns 3, 4.\n");
 		GMT_explain_option ('V');
-		fprintf (stderr, "	-W sets pen attributes [width = %lgp, color = (%d/%d/%d), solid line].\n", 
+		fprintf (stderr, "\t-W sets pen attributes [width = %lgp, color = (%d/%d/%d), solid line].\n", 
 			pen.width, pen.rgb[0], pen.rgb[1], pen.rgb[2]);
-		fprintf (stderr, "	   Implicitly draws symbol outline with this pen.\n");
+		fprintf (stderr, "\t   Implicitly draws symbol outline with this pen.\n");
 		GMT_explain_option ('X');
 		GMT_explain_option ('c');
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "	   Default is the required number of columns\n");
+		fprintf (stderr, "\t   Default is the required number of columns\n");
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
 	}
@@ -347,7 +377,7 @@ main (int argc, char **argv)
 	read_symbol = (S.symbol == UNSPECIFIED);
 
 	if (S.symbol == 0 && (fill_set || close_polygon)) polygon = TRUE;
-	if (S.symbol == FAULT) polygon = FALSE;
+	if (S.symbol == FRONT) polygon = FALSE;
 
 	default_pen = pen;
 	default_fill = fill;
@@ -411,7 +441,11 @@ main (int argc, char **argv)
 
 	if (frame_info.plot) GMT_map_basemap ();
 
-	if (project_info.z_pars[0] == 0.0 && skip_if_outside) GMT_map_clip_on (GMT_no_rgb, 3);
+	if (project_info.z_pars[0] == 0.0 && skip_if_outside) {
+		GMT_map_clip_on (GMT_no_rgb, 3);
+		clip_set = TRUE;
+	}
+	if (S.symbol == ELLIPSE) skip_if_outside = FALSE;
 
 	GMT_setpen (&pen);
 
@@ -421,7 +455,6 @@ main (int argc, char **argv)
 		ps_setfont (S.font_no);			/* Set the required font */
 	}
 	
-	ix = (gmtdefs.xy_toggle);	iy = 1 - ix;
 	done = FALSE;
 	for (fno = 1; !done && fno < n_args; fno++) {	/* Loop over all input files */
 		if (!nofile && argv[fno][0] == '-') continue;
@@ -442,7 +475,7 @@ main (int argc, char **argv)
 			sprintf (line, "File: %s\0", argv[fno]);
 			ps_comment (line);
 		}
-		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 		
 		if (S.symbol > 0) {	/* symbol part */
 		
@@ -455,18 +488,22 @@ main (int argc, char **argv)
 
 				while ((GMT_io.status & GMT_IO_SEGMENT_HEADER) && !(GMT_io.status & GMT_IO_EOF)) {	/* Skip segment headers */
 					if (gmtdefs.verbose) ps_comment (GMT_io.segment_header);
-					if ((p = strstr (GMT_io.segment_header, " -G"))) {
-						strcpy (line, &p[3]);
+					if ((p = strstr (GMT_io.segment_header, "-G"))) {
+						strcpy (line, &p[2]);
 						for (i = 0; line[i]; i++) if (line[i] == ' ') line[i] = '\0';
 						if (GMT_getfill (line, &fill)) {
 							fprintf (stderr, "%s: Trouble decoding fill info from header near line %d\n", GMT_program,  n_total_read);
 						}
 					}
+					else if (cpt_given && (p = strstr (GMT_io.segment_header, "-Z"))) {	/* Set symbol r/g/b via cpt-lookup */
+						z = (p[2] == 'N') ? GMT_d_NaN : atof (&p[2]);
+						GMT_get_rgb24 (z, fill.rgb);
+					}
 					else
 						fill = default_fill;
 
-					if ((p = strstr (GMT_io.segment_header, " -W"))) {
-						strcpy (line, &p[3]);
+					if ((p = strstr (GMT_io.segment_header, "-W"))) {
+						strcpy (line, &p[2]);
 						for (i = 0; line[i]; i++) if (line[i] == ' ') line[i] = '\0';
 						if (GMT_getpen (line, &pen)) {
 							fprintf (stderr, "%s: Trouble decoding pen info from header near line %d\n", GMT_program,  n_total_read);
@@ -495,7 +532,7 @@ main (int argc, char **argv)
 				S.size_x = S.size_y = 0.0;
 				if (S.read_size) {
 					S.size_x = in[three] * GMT_u2u[S.u][GMT_INCH];
-					S.size_y = ((S.symbol == COLUMN || S.symbol == CUBE) ? in[four] : in[three]) * GMT_u2u[S.u][GMT_INCH];
+					S.size_y = (S.symbol == COLUMN) ? in[four] * GMT_u2u[S.u][GMT_INCH] : S.size_x;
 				}
 
 				if (read_symbol) {
@@ -510,8 +547,8 @@ main (int argc, char **argv)
 				}
 
 				data1[n].flag = 0;
-				data1[n].lon = in[ix];
-				data1[n].lat = in[iy];
+				data1[n].lon = in[0];
+				data1[n].lat = in[1];
 				if (S.read_vector) {
 					data1[n].x_size = (float)in[three];	/* direction */
 					data1[n].y_size = (float)(in[four] * GMT_u2u[S.u][GMT_INCH]);	/* length */
@@ -531,6 +568,11 @@ main (int argc, char **argv)
 						data1[n].z_size *= (float)GMT_u2u[S.u][GMT_INCH];
 					}
 				}
+				else if (S.symbol == PIE) {
+					data1[n].x_size = (float)S.size_x;
+					data1[n].y_size = (float)(D2R * in[three+S.read_size]);	/* Stop direction in radians */
+					data1[n].z_size = (float)(D2R * in[four+S.read_size]);	/* Stop direction in radians */
+				}
 				else {
 					data1[n].x_size = (float)S.size_x;
 					data1[n].y_size = (float)S.size_y;
@@ -539,17 +581,26 @@ main (int argc, char **argv)
 
 				/* Skip zero-size symbols */
 				
-				if (S.symbol != POINT && S.symbol < ELLIPSE && S.size_x <= 0.0) continue;
-				if (S.read_vector && data1[n].y_size <= 0.0) continue;
-				if (S.symbol == ELLIPSE && (data1[n].y_size <= 0.0 || data1[n].z_size <= 0.0)) continue;
+				if (S.symbol != POINT && S.symbol < ELLIPSE && S.size_x <= 0.0) {
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: Symbol size <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					continue;
+				}
+				if (S.read_vector && data1[n].y_size <= 0.0) {
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: vector length <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					continue;
+				}
+				if (S.symbol == ELLIPSE && (data1[n].y_size <= 0.0 || data1[n].z_size <= 0.0)) {
+					if (gmtdefs.verbose) fprintf (stderr, "%s: Warning: ellipse axes <= 0.0 near line %d (skipped)\n", GMT_program, n_total_read);
+					continue;
+				}
 
 				if (skip_if_outside) {
 					if (in[2] < project_info.z_bottom || in[2] > project_info.z_top) continue;
-					GMT_map_outside (in[ix], in[iy]);
+					GMT_map_outside (in[0], in[1]);
 					if ( abs (GMT_x_status_new) > 1 || abs (GMT_y_status_new) > 1) continue;
 				}
 
-				GMT_project3D (in[ix], in[iy], in[2], &data1[n].x, &data1[n].y, &data1[n].z);
+				GMT_project3D (in[0], in[1], in[2], &data1[n].x, &data1[n].y, &data1[n].z);
 				if (get_rgb) data1[n].value = in[3];
 
 				if (S.symbol == COLUMN) {
@@ -594,6 +645,8 @@ main (int argc, char **argv)
 						}
 					}
 				}
+				
+				if (GMT_cpt_skip) continue;	/* Skip this because cpt says so */
 
 				switch (data1[i].symbol) {
 					case NONE:
@@ -697,6 +750,9 @@ main (int argc, char **argv)
 						else	/* Leave as specified */
 							GMT_vector3d (data1[i].x, data1[i].y, x2, y2, data1[i].z, S.v_width, S.h_length, S.h_width, gmtdefs.vector_shape, fill.rgb, outline);
 						break;
+					case PIE:
+						pie3D (data1[i].x, data1[i].y, data1[i].z, (double)data1[i].x_size, (double)data1[i].y_size, (double)data1[i].z_size, fill.rgb, outline);
+						break;
 				}
 			}
 			GMT_free ((void *)data1);
@@ -709,16 +765,16 @@ main (int argc, char **argv)
 
 				while (GMT_io.status & GMT_IO_SEGMENT_HEADER) {
 					if (gmtdefs.verbose) ps_comment (GMT_io.segment_header);
-					if ((p = strstr (GMT_io.segment_header, " -G"))) {
-						strcpy (line, &p[3]);
+					if ((p = strstr (GMT_io.segment_header, "-G"))) {
+						strcpy (line, &p[2]);
 						for (i = 0; line[i]; i++) if (line[i] == ' ') line[i] = '\0';
 						if (GMT_getfill (line, &fill)) {
 							fprintf (stderr, "%s: Trouble decoding fill info from header near line %d\n", GMT_program,  n_total_read);
 						}
 						polygon = TRUE;
 					}
-					else if ((p = strstr (GMT_io.segment_header, " -Z"))) {	/* Set polygon r/g/b via cpt-lookup */
-						z = atof (&p[3]);
+					else if (cpt_given && (p = strstr (GMT_io.segment_header, "-Z"))) {	/* Set polygon r/g/b via cpt-lookup */
+						z = (p[2] == 'N') ? GMT_d_NaN : atof (&p[2]);
 						GMT_get_rgb24 (z, fill.rgb);
 						fill.use_pattern = FALSE;
 						polygon = TRUE;
@@ -727,8 +783,8 @@ main (int argc, char **argv)
 						polygon = default_polygon;
 						fill = default_fill;
 					}
-					if ((p = strstr (GMT_io.segment_header, " -W"))) {
-						strcpy (line, &p[3]);
+					if ((p = strstr (GMT_io.segment_header, "-W"))) {
+						strcpy (line, &p[2]);
 						for (i = 0; line[i]; i++) if (line[i] == ' ') line[i] = '\0';
 						if (GMT_getpen (line, &pen)) {
 							fprintf (stderr, "%s: Trouble decoding pen info from header near line %d\n", GMT_program,  n_total_read);
@@ -755,7 +811,7 @@ main (int argc, char **argv)
 						exit (EXIT_FAILURE);
 					}
 
-					data2[n].x = in[ix];	data2[n].y = in[iy];
+					data2[n].x = in[0];	data2[n].y = in[1];
 					data2[n].z = in[2];
 					n++;
 					if (n == n_alloc) {
@@ -774,22 +830,24 @@ main (int argc, char **argv)
 				data2 = (struct DATA2 *) GMT_memory ((void *)data2, (size_t)n, sizeof (struct DATA2), GMT_program);
 				n_alloc = n;
 				
-				xp = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
-				yp = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
-				for (i = 0; i < n; i++) GMT_geoz_to_xy (data2[i].x, data2[i].y, data2[i].z, &xp[i], &yp[i]);
+				if (!GMT_cpt_skip) {
+					xp = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
+					yp = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
+					for (i = 0; i < n; i++) GMT_geoz_to_xy (data2[i].x, data2[i].y, data2[i].z, &xp[i], &yp[i]);
 				
-				if (polygon)
-					GMT_fill (xp, yp, n, &fill, outline);
-				else
-					ps_line (xp, yp, n, 3, FALSE, TRUE);
+					if (polygon)
+						GMT_fill (xp, yp, n, &fill, outline);
+					else
+						ps_line (xp, yp, n, 3, FALSE, TRUE);
 
-				if (S.symbol == FAULT) {
-					for (i = 0; i < n; i++) GMT_geo_to_xy (data2[i].x, data2[i].y, &xp[i], &yp[i]);
-					GMT_draw_fence (xp, yp, n, S.f_gap, S.f_len, S.f_sense, S.f_triangle, fill.rgb, outline); 	/* Must draw fault crossbars */
-				}
+					if (S.symbol == FRONT) {
+						for (i = 0; i < n; i++) GMT_geo_to_xy (data2[i].x, data2[i].y, &xp[i], &yp[i]);
+						GMT_draw_fence (xp, yp, n, &S.f, &fill, outline); 	/* Must draw fault crossbars */
+					}
 		
-				GMT_free ((void *)xp);
-				GMT_free ((void *)yp);
+					GMT_free ((void *)xp);
+					GMT_free ((void *)yp);
+				}
 				n = 0;
 			}
 			GMT_free ((void *)data2);
@@ -798,9 +856,10 @@ main (int argc, char **argv)
 		
 	}
 	
-	if (project_info.z_pars[0] == 0.0 && skip_if_outside) GMT_map_clip_off();
+	if (clip_set) GMT_map_clip_off();
 
 	if (pen.texture) ps_setdash (CNULL, 0);
+	if (project_info.three_D) GMT_vertical_axis (2);	/* Draw background axis */
 
 	ps_rotatetrans (z_project.xmin, z_project.ymin, 0.0);
 	ps_plotend (gmtdefs.last_page);
@@ -933,14 +992,15 @@ void circle3D (double x, double y, double z, double size, int rgb[], int outline
 {
 	/* Must plot a squashed circle */
 	int i;
-	double xx, yy, a, da, plot_x[51], plot_y[51];
+	double xx, yy, a, da, s, c, plot_x[51], plot_y[51];
 	
-	da = 2.0 * M_PI / 50.0;
+	da = TWO_PI / 50.0;
 	size *= 0.5;
 	for (i = 0; i <= 50; i++) {
 		a = i * da;
-		xx = x + size * cos (a);
-		yy = y + size * sin (a);
+		sincos (a, &s, &c);
+		xx = x + size * c;
+		yy = y + size * s;
 		GMT_xyz_to_xy (xx, yy, z, &plot_x[i], &plot_y[i]);
 	}
 	ps_polygon (plot_x, plot_y, 51, rgb, outline);
@@ -950,16 +1010,15 @@ void ellipse3D (double x, double y, double z, double direction, double major, do
 {
 	/* Must plot a squashed circle */
 	int i;
-	double dx, dy, a, da, sin_direction, cos_direction, x_prime, y_prime, plot_x[51], plot_y[51];
+	double dx, dy, a, da, s, c, sin_direction, cos_direction, x_prime, y_prime, plot_x[51], plot_y[51];
 	
-	direction *= D2R;
-	sin_direction = sin (direction);
-	cos_direction = cos (direction);
-	da = 2.0 * M_PI / 50.0;
+	sincos (direction * D2R, &sin_direction, &cos_direction);
+	da = TWO_PI / 50.0;
 	for (i = 0; i <= 50; i++) {
 		a = i * da;
-		dx = major * cos (a);
-		dy = minor * sin (a);
+		sincos (a, &s, &c);
+		dx = major * c;
+		dy = minor * s;
 		x_prime = x + dx * cos_direction - dy * sin_direction;
 		y_prime = y + dx * sin_direction + dy * cos_direction;
 		GMT_xyz_to_xy (x_prime, y_prime, z, &plot_x[i], &plot_y[i]);
@@ -967,6 +1026,28 @@ void ellipse3D (double x, double y, double z, double direction, double major, do
 	ps_polygon (plot_x, plot_y, 51, rgb, outline);
 }
 
+void pie3D (double x, double y, double z, double size, double dir1, double dir2, int rgb[], int outline) {
+	/* Must plot a squashed pie wedge */
+	int i, j, n;
+	double arc, xx, yy, a, da, s, c, plot_x[52], plot_y[52];
+	
+	arc = (dir2 - dir1);
+	while (arc > TWO_PI) arc -= TWO_PI;
+	da = TWO_PI / 50.0;	/* Standard step length for full circle */
+	n = irint (arc / da);	/* But we are not doing a full circle so use less than 50 points */
+	da = arc / n;		/* Step length for this pie wedge */
+	size *= 0.5;
+	GMT_xyz_to_xy (x, y, z, &plot_x[0], &plot_y[0]);	/* Start from center */
+	for (i = 0, j = 1; i <= n; i++, j++) {
+		a = dir1 + i * da;
+		sincos (a, &s, &c);
+		xx = x + size * c;
+		yy = y + size * s;
+		GMT_xyz_to_xy (xx, yy, z, &plot_x[j], &plot_y[j]);
+	}
+	ps_polygon (plot_x, plot_y, j, rgb, outline);
+}
+
 void triangle3D (double x, double y, double z, double size, int rgb[], int outline)
 {
 	int i;
@@ -1099,8 +1180,8 @@ int dist_compare (const void *point_1, const void *point_2)
 int decode_symbol_option (char *text, struct PSXYZ_SYMBOL *p, BOOLEAN cmd)
 {
 	int decode_error = 0, bset = 0, j, n, k, len, slash = 0;
-	BOOLEAN check = TRUE;
-	char symbol_type, txt_a[32], txt_b[32], txt_c[32], text_cp[80], *c;
+	BOOLEAN check = TRUE, old_style;
+	char symbol_type, txt_a[32], txt_b[32], txt_c[32], text_cp[128], *c;
 
 	p->n_required = 0;
 
@@ -1144,13 +1225,13 @@ int decode_symbol_option (char *text, struct PSXYZ_SYMBOL *p, BOOLEAN cmd)
 			decode_error = (n != 3);
 		}
 	}
-	else if (strchr ("aAbCcDdeEfHhIipoSsTtuVvx", (int) text[0]) && strchr ("CcIiMmPp", (int) text[1])) {	/* Symbol, but no size given (size assumed given on command line), only unit information */
+	else if (strchr ("aAbCcDdeEfHhIipoSsTtuVvwx", (int) text[0]) && strchr ("CcIiMmPp", (int) text[1])) {	/* Symbol, but no size given (size assumed given on command line), only unit information */
 		n = sscanf (text, "%c", &symbol_type);
 		if (p->size_x == 0.0) p->size_x = p->given_size_x;
 		if (p->size_y == 0.0) p->size_y = p->given_size_y;
 		if (text[1] && (p->u = GMT_get_unit (text[1])) < 0) decode_error = TRUE;
 	}
-	else if (strchr ("aAbCcDdeEfHhIipoSsTtuVvx", (int) text[0]) && (text[1] == '\n' || text[1] == '\0')) {	/* Symbol, but no size given (size assumed given on command line) */
+	else if (strchr ("aAbCcDdeEfHhIipoSsTtuVvwx", (int) text[0]) && (text[1] == '\n' || text[1] == '\0')) {	/* Symbol, but no size given (size assumed given on command line) */
 		n = sscanf (text, "%c", &symbol_type);
 		if (p->size_x == 0.0) p->size_x = p->given_size_x;
 		if (p->size_y == 0.0) p->size_y = p->given_size_y;
@@ -1239,30 +1320,87 @@ int decode_symbol_option (char *text, struct PSXYZ_SYMBOL *p, BOOLEAN cmd)
 			p->n_required = 3;
 			check = FALSE;
 			break;
-		case 'f':
-			p->symbol = FAULT;
-			len = strlen(text);
-			switch (text[len-1]) {
-				case 'L':
-					p->f_triangle = TRUE;
-				case 'l':
-					p->f_sense = 1;
+
+		case 'f':	/* Fronts:   -Sf<spacing>/<size>[dir][type][:<offset>]	*/
+			p->symbol = FRONT;
+			p->f.f_off = 0.0;
+			strcpy (text_cp, text);
+			if ((c = strchr (text_cp, ':'))) {	/* Gave :<offset>, set it and strip it off */
+				c++;	/* SKip over the colon */
+				p->f.f_off = GMT_convert_units (c, GMT_INCH);
+				c--;	/* Go back to colon */
+				*c = 0;	/* Effectively chops off the offset modifier */
+			}
+			len = strlen (text_cp) - 1;
+			
+			old_style = FALSE;
+			switch (text_cp[len]) {
+				case 'f':	/* Fault front */
+					p->f.f_symbol = F_FAULT;
 					len--;
 					break;
-				case 'R':
-					p->f_triangle = TRUE;
-				case 'r':
-					p->f_sense = -1;
+				case 't':	/* Triangle front */
+					p->f.f_symbol = F_TRIANGLE;
 					len--;
 					break;
+				case 's':	/* Strike-slip front */
+					p->f.f_symbol = F_SLIP;
+					len--;
+					break;
+				case 'c':	/* [half-]circle front */
+					p->f.f_symbol = F_CIRCLE;
+					len--;
+					break;
+				case 'b':	/* [half-]square front */
+					p->f.f_symbol = F_BOX;
+					len--;
+					break;
+					
+				/* Old style (backward compatibility) */
+				
+				case 'L':	/* Left triangle */
+					p->f.f_symbol = F_TRIANGLE;
+				case 'l':	/* Left ticked fault */
+					p->f.f_sense = F_LEFT;
+					old_style = TRUE;
+					break;
+				case 'R':	/* Right triangle */
+					p->f.f_symbol = F_TRIANGLE;
+				case 'r':	/* Right ticked fault */
+					p->f.f_sense = F_RIGHT;
+					old_style = TRUE;
+					break;
 				default:
-					p->f_sense = 0;
+					p->f.f_sense = F_CENTERED;
 					break;
 			}
-			text[len] = 0;	/* Chop off the lL/rR if present */
-			sscanf (&text[1], "%[^/]/%s", txt_a, txt_b);
-			p->f_gap = GMT_convert_units (txt_a, GMT_INCH);
-			p->f_len = GMT_convert_units (txt_b, GMT_INCH);
+
+			if (!old_style) {
+				switch (text_cp[len]) {	/* Get sense - default is centered */
+					case 'l':
+						p->f.f_sense = F_LEFT;
+						break;
+					case 'r':
+						p->f.f_sense = F_RIGHT;
+						break;
+					default:
+						len++;
+						p->f.f_sense = F_CENTERED;
+						if (p->f.f_symbol == F_SLIP) {
+							fprintf (stderr, "%s: Error in Option -Sf: Must specify (l)eft-lateral or (r)ight-lateral slip\n", GMT_program);
+							exit (EXIT_FAILURE);
+						}
+						break;
+				}
+			}
+
+			text_cp[len] = 0;	/* Gets rid of the [dir][type] flags, if present */
+			
+			/* Pull out and get spacing and size */
+			
+			sscanf (&text_cp[1], "%[^/]/%s", txt_a, txt_b);
+			p->f.f_gap = GMT_convert_units (txt_a, GMT_INCH);
+			p->f.f_len = GMT_convert_units (txt_b, GMT_INCH);
 			break;
 		case 'H':
 			p->size_x *= 1.09963611078;	/* To equal area of circle with same size */
@@ -1349,6 +1487,10 @@ int decode_symbol_option (char *text, struct PSXYZ_SYMBOL *p, BOOLEAN cmd)
 			p->n_required = 2;
 			check = FALSE;
 			break;
+		case 'w':
+			p->symbol = PIE;
+			p->n_required = 2;
+			break;
 		case 'x':
 			p->symbol = CROSS;
 			break;
@@ -1427,7 +1569,8 @@ void GMT_plot_ellipse (double lon, double lat, double z, double major, double mi
 		else
 			px[i] = lon +
 				R2D * d_atan2 (x_prime * sin_c, (rho * cosp * cos_c - y_prime * sinp * sin_c));
-		if ((px[i]) <= -180) (px[i]) += 360.0;
+		while ((px[i] - project_info.central_meridian) < -180.0) px[i] += 360.0;
+		while ((px[i] - project_info.central_meridian) > 180.0) px[i] -= 360.0;
 	}
 
 	if ((plot_n = GMT_clip_to_map (px, py, GMT_ELLIPSE_APPROX, &xp, &yp)) == 0) return;
@@ -1465,30 +1608,40 @@ void GMT_plot_ellipse (double lon, double lat, double z, double major, double mi
 	GMT_free ((void *)py);
 }
 
-void GMT_draw_fence (double x[], double y[], int n, double gap, double len, int sense, int triangle, int rgb[], BOOLEAN outline)
+void GMT_draw_fence (double x[], double y[], int n, struct FRONTLINE *f, struct GMT_FILL *g, BOOLEAN outline)
 {
 	int i, ngap;
-	double *s, xx[3], yy[3], dist = 0.0, frac, dx, dy, angle;
-	double x0, y0, len2, len3, cosa, sina;
+	BOOLEAN skip;
+	double *s, xx[4], yy[4], dist, w, frac, dx, dy, angle, dir1, dir2;
+	double gap, x0, y0, xp, yp, len2, len3, cosa, sina, sa, ca, offx, offy;
 	
 	if (n < 2) return;
 
 	s = (double *) GMT_memory (VNULL, (size_t)n, sizeof (double), GMT_program);
-	for (i = 1, s[0] = 0.0; i < n; i++) s[i] = s[i-1] + hypot (x[i] - x[i-1], y[i] - y[i-1]);
+	for (i = 1, s[0] = 0.0; i < n; i++) {
+		/* Watch out for longitude wraps */
+		dx = x[i] - x[i-1];
+		w = GMT_half_map_width (y[i]);
+		if (GMT_world_map && dx > w) dx = copysign (2 * w - fabs (dx), -dx);
+		s[i] = s[i-1] + hypot (dx, y[i] - y[i-1]);
+	}
 		
-	if (gap > 0.0) {
-		ngap = irint (s[n-1] / gap);
-		gap = s[n-1] / (double) ngap;
+	if (f->f_gap > 0.0) {
+		ngap = irint (s[n-1] / f->f_gap);
+		gap = s[n-1] / (double)ngap;
+		dist = f->f_off;
 		ngap++;
 	}
 	else {
-		ngap = (int) fabs (gap);
+		ngap = (int) fabs (f->f_gap);
 		gap = s[n-1] / (ngap - 1);
 		if (ngap == 1) dist = 0.5 * s[n-1];
 	}
 	
-	len2 = 0.5 * len;
-	len3 = 0.866025404 * len;			
+	len2 = 0.5 * f->f_len;
+	len3 = 0.866025404 * f->f_len;	
+	if (f->f_sense == F_CENTERED) len3 = len2;
+	
 	i = 0;
 	while (i < n) {
 		while ((s[i] - dist) > -SMALL) {	/* Time for tick */
@@ -1510,38 +1663,144 @@ void GMT_draw_fence (double x[], double y[], int n, double gap, double len, int
 				y0 = y[i] - dy * frac;
 			}
 			angle = d_atan2 (dy, dx);
-						
-			if (triangle) {
-				if (sense == -1) angle += M_PI;
-				sincos (angle, &sina, &cosa);
-				xx[0] = x0 + len2 * cosa;
-				yy[0] = y0 + len2 * sina;
-				xx[1] = x0 - len3 * sina;
-				yy[1] = y0 + len3 * cosa;
-				xx[2] = x0 - len2 * cosa;
-				yy[2] = y0 - len2 * sina;
-				if (project_info.three_D) GMT_2D_to_3D (xx, yy, 3);
-				ps_patch (xx, yy, 3, rgb, outline);
+			skip = (GMT_world_map && fabs (dx) > GMT_half_map_width (y[i]));	/* Dont do ticks on jumps */
+			if (skip) {
+				dist += gap;
+				i++;
+				continue;
 			}
-			else {
-				xx[0] = xx[1] = x0;	yy[0] = yy[1] = y0;
-				if (sense == 0) {
-					angle -= M_PI_2;
+			
+			switch (f->f_symbol) {
+				case F_TRIANGLE:	/* Triangle */	
+					switch (f->f_sense) {
+						case F_CENTERED:
+							sincos (angle, &sina, &cosa);
+							xx[0] = x0 + len2 * cosa;
+							yy[0] = y0 + len2 * sina;
+							xx[1] = x0 - len3 * sina;
+							yy[1] = y0 + len3 * cosa;
+							xx[2] = x0 - len2 * cosa;
+							yy[2] = y0 - len2 * sina;
+							xx[3] = x0 + len3 * sina;
+							yy[3] = y0 - len3 * cosa;
+                              				if (project_info.three_D) GMT_2D_to_3D (xx, yy, 4);
+							ps_patch (xx, yy, 4, g->rgb, outline);
+							break;
+						case F_RIGHT:
+							angle += M_PI;
+						case F_LEFT:
+							sincos (angle, &sina, &cosa);
+							xx[0] = x0 + len2 * cosa;
+							yy[0] = y0 + len2 * sina;
+							xx[1] = x0 - len3 * sina;
+							yy[1] = y0 + len3 * cosa;
+							xx[2] = x0 - len2 * cosa;
+							yy[2] = y0 - len2 * sina;
+                              				if (project_info.three_D) GMT_2D_to_3D (xx, yy, 3);
+							ps_patch (xx, yy, 3, g->rgb, outline);
+							break;
+					}
+					break;
+				
+				case F_CIRCLE:	/* Circles */	
+					switch (f->f_sense) {
+						case F_CENTERED:
+							circle3D (x0, y0, 0.0, f->f_len, g->rgb, outline);
+							break;
+						case F_RIGHT:
+							angle += M_PI;
+						case F_LEFT:
+							dir1 = angle;
+							dir2 = angle + M_PI;
+							if (dir1 > dir2) dir1 -= TWO_PI;
+							pie3D (x0, y0, 0.0, f->f_len, dir1, dir2, g->rgb, outline);
+							break;
+					}
+					break;
+				
+				case F_BOX:	/* Squares */	
+					switch (f->f_sense) {
+						case F_CENTERED:	/* Full square centered on line */
+							sincos (angle, &sina, &cosa);
+							xx[0] = x0 + len2 * (cosa + sina);	/* LR */
+							yy[0] = y0 + len2 * (sina - cosa);
+							xx[1] = x0 + len2 * (cosa - sina);	/* UR */
+							yy[1] = y0 + len2 * (sina + cosa);
+							xx[2] = x0 + len2 * (-cosa - sina);	/* UL */
+							yy[2] = y0 + len2 * (-sina + cosa);
+							xx[3] = x0 + len2 * (-cosa + sina);	/* LL */
+							yy[3] = y0 + len2 * (-sina - cosa);
+							break;
+						case F_RIGHT:
+							angle += M_PI;
+						case F_LEFT:
+							/* Half square on the chosen side */
+							sincos (angle, &sina, &cosa);
+							xx[0] = x0 + len2 * (cosa);	/* LR */
+							yy[0] = y0 + len2 * (sina);
+							xx[1] = x0 + len2 * (cosa - sina);	/* UR */
+							yy[1] = y0 + len2 * (sina + cosa);
+							xx[2] = x0 + len2 * (-cosa - sina);	/* UL */
+							yy[2] = y0 + len2 * (-sina + cosa);
+							xx[3] = x0 + len2 * (-cosa);	/* LL */
+							yy[3] = y0 + len2 * (-sina);
+							break;
+					}
+                              		if (project_info.three_D) GMT_2D_to_3D (xx, yy, 4);
+					ps_patch (xx, yy, 4, g->rgb, outline);
+					break;
+				
+				case F_SLIP: /* draw strike-slip arrows */
 					sincos (angle, &sina, &cosa);
-					xx[0] += len2 * cosa;
-					yy[0] += len2 * sina;
-					xx[1] -= len2 * cosa;
-					yy[1] -= len2 * sina;
-				}
-				else {
-					angle += (sense * M_PI_2);
-					sincos (angle, &sina, &cosa);
-					xx[1] += len * cosa;
-					yy[1] += len * sina;
-				}
-				if (project_info.three_D) GMT_2D_to_3D (xx, yy, 2);
-				ps_plot (xx[0], yy[0], 3);
-				ps_plot (xx[1], yy[1], 2);
+				        offx = gmtdefs.anot_offset * sina; /* get offsets from front line */
+					offy = gmtdefs.anot_offset * cosa;
+					/* sense == F_LEFT == left-lateral, R_RIGHT = right lateral */
+					/* arrow "above" line */
+					sincos (angle + (f->f_sense * 30.0 * D2R), &sa, &ca);
+					xp = x0 + f->f_sense * offx;
+					yp = y0 - f->f_sense * offy;
+					xx[0] = xp - len2 * cosa;
+					yy[0] = yp - len2 * sina;
+					xx[1] = xp + len2 * cosa;
+					yy[1] = yp + len2 * sina;
+					xx[2] = xx[1] - len2 * ca;
+					yy[2] = yy[1] - len2 * sa;
+                              		if (project_info.three_D) GMT_2D_to_3D (xx, yy, 3);
+					ps_line (xx, yy, 3, 3, FALSE, TRUE);
+
+					/* arrow "below" line */
+					sincos (angle - (f->f_sense * 150.0 *D2R), &sa, &ca);
+					xp = x0 - f->f_sense * offx;
+					yp = y0 + f->f_sense * offy;
+					xx[0] = xp + len2 * cosa;
+					yy[0] = yp + len2 * sina;
+					xx[1] = xp - len2 * cosa;
+					yy[1] = yp - len2 * sina;
+					xx[2] = xx[1] - len2 * ca;
+					yy[2] = yy[1] - len2 * sa;
+                              		if (project_info.three_D) GMT_2D_to_3D (xx, yy, 3);
+					ps_line (xx, yy, 3, 3, FALSE, TRUE);
+					break;
+					
+				case F_FAULT:	/* Normal fault ticks */	
+					xx[0] = xx[1] = x0;	yy[0] = yy[1] = y0;
+					if (f->f_sense == F_CENTERED) {
+						angle -= M_PI_2;
+						sincos (angle, &sina, &cosa);
+						xx[0] += len2 * cosa;
+						yy[0] += len2 * sina;
+						xx[1] -= len2 * cosa;
+						yy[1] -= len2 * sina;
+					}
+					else {
+						angle += (f->f_sense * M_PI_2);
+						sincos (angle, &sina, &cosa);
+						xx[1] += len2 * cosa;
+						yy[1] += len2 * sina;
+					}
+                              		if (project_info.three_D) GMT_2D_to_3D (xx, yy, 2);
+					ps_line (xx, yy, 2, 3, FALSE, TRUE);
+					break;
 			}
 			dist += gap;
 		}
