@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system: 	@(#)spectrum1d.c	2.46  09/21/99
+ *	$Id: spectrum1d.c,v 1.3 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  *
@@ -50,8 +50,8 @@
  *	-b	input and/or output data are in binary form
  *
  *	Author:		W. H. F. Smith
- *	Date:		11 April 1991-1999
- *	Revised:	5 June 1991-1999 to add W and N options in prep for
+ *	Date:		11 April 1991-2000
+ *	Revised:	5 June 1991-2000 to add W and N options in prep for
  *			GMT v2.0 release.
  *			PW: Upgrade to GMT 3.1 w/ -b
  *			BCH-J: Upgraded -C: optional output flags
@@ -65,6 +65,7 @@
  *			modified periodograms", IEEE Transactions on
  *			Audio and Electroacoustics, Vol AU-15, No 2,
  *			June, 1967.
+ *  Version:	3.4
  */
 
 #include "gmt.h"
@@ -313,9 +314,9 @@ void	read_data(void)
 	n_data = 0;
 	x = (float *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(float), GMT_program);
 	if (y_given) y = (float *)GMT_memory (VNULL, (size_t)n_alloc, sizeof(float), GMT_program);
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 2;
 
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (buffer, BUFSIZ, fp);
+	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
@@ -457,7 +458,7 @@ void	write_output(char *output_choice, int n_outputs)
 {
 	int	i, j;
 	double	delta_f, eps_pow, out[3], *f_or_w;
-	char	fname[120];
+	char	fname[128];
 	FILE	*fpout;
 	
 	delta_f = 1.0 / (window * dt);
@@ -592,16 +593,16 @@ void	write_output(char *output_choice, int n_outputs)
 	  }
 	}
 
-	free ((void *)f_or_w);
+	GMT_free ((void *)f_or_w);
 
 }
 
 void	free_space (void)
 {
-	free ((void *)spec);
-	free ((void *)datac);
-	free ((void *)x);
-	if (y_given) free ((void *)y);
+	GMT_free ((void *)spec);
+	GMT_free ((void *)datac);
+	GMT_free ((void *)x);
+	if (y_given) GMT_free ((void *)y);
 }
 
 void	detrend_and_hanning(void)
