@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)splitxyz.c	2.49  09/21/99
+ *	$Id: splitxyz.c,v 1.2 2001/03/01 22:08:26 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * 
@@ -21,7 +21,8 @@
  * and split it into profile segments.
  * 
  * Author:	W. H. F. Smith
- * Date:	24 July, 1991-1999
+ * Date:	24 July, 1991-2000
+ * Version:	3.4
  */
 
 #include "gmt.h"
@@ -39,7 +40,7 @@ struct  DATA    {
 
 main(int argc, char **argv)
 {
-	int     i, ndata, n_alloc, nprofiles, begin, end, ix, iy, n_expected_fields, n_out, n_fields;
+	int     i, ndata, n_alloc, nprofiles, begin, end, n_expected_fields, n_out, n_fields;
 	int	n_required_fields;
 	
 	BOOLEAN	error = FALSE, dh_supplied = FALSE, ok, hilow, map_units = FALSE;
@@ -110,7 +111,7 @@ main(int argc, char **argv)
 					}
                                         break;
 				case 'M':
-                                        map_units = TRUE;
+                                        map_units = GMT_geographic_in = GMT_geographic_out = TRUE;
                                         break;
 				case 'N':
                                         strcpy(basename, &argv[i][2]);
@@ -244,13 +245,10 @@ main(int argc, char **argv)
 	i = 0;
 
         data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
-	ix = (gmtdefs.xy_toggle);	iy = 1 - ix;
 
-	if (!GMT_io.binary[0] && gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (buffer, BUFSIZ, fp);
+	if (!GMT_io.binary[0] && gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
 
-	n_fields = GMT_input (fp, &n_expected_fields, &in);
-				
-	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
 		if (GMT_io.status & GMT_IO_MISMATCH) {
 			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields,  n_expected_fields, i);
@@ -263,8 +261,8 @@ main(int argc, char **argv)
                         data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
                 }
 		if (!dh_supplied) {
-			data[ndata].x = in[ix];
-			data[ndata].y = in[iy];
+			data[ndata].x = in[0];
+			data[ndata].y = in[1];
 			if (!xy_only) data[ndata].z = in[2];
 			if (ndata) {
 				dy = (data[ndata].y - data[ndata-1].y);
@@ -288,14 +286,13 @@ main(int argc, char **argv)
 			ndata++;
 		}
 		else {
-			data[ndata].x = in[ix];
-			data[ndata].y = in[iy];
+			data[ndata].x = in[0];
+			data[ndata].y = in[1];
 			data[ndata].z = in[2];
 			data[ndata].d = in[3];
 			data[ndata].a = D2R * (90.0 - in[4]);
 			ndata++;
 		}
-		n_fields = GMT_input (fp, &n_expected_fields, &in);
 	}
 	
 	data = (struct DATA *) GMT_memory ((void *)data, (size_t)ndata, sizeof(struct DATA), GMT_program);
@@ -395,7 +392,7 @@ main(int argc, char **argv)
 						}
 					}
 					else
-						printf("> Start profile # %d\n", nprofiles);
+						fprintf (GMT_stdout, "> Start profile # %d\n", nprofiles);
 
 					for (i = begin; i < end; i++) {
 						out[0] = data[i].x;
