@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)surface.c	2.71  09/21/99
+ *	$Id: surface.c,v 1.4 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * surface.c:  a gridding program.
@@ -60,7 +60,7 @@
  *	work; simple testing today suggests it makes no difference, but I will
  *	leave it in.
  *
- * V 4.2 -- 4 June, 1991-1999:	P. Wessel upgraded to GMT v2.0 grdfile i/o.
+ * V 4.2 -- 4 June, 1991-2000:	P. Wessel upgraded to GMT v2.0 grdfile i/o.
  *	Added feature to constrain solution to be within lower and/or
  *	upper bounds.  Bounds can be min/max input data value, another
  *	value outside the input data range, or be provided by a grdfile
@@ -74,6 +74,10 @@
  *
  * 21-JUN-1998 PW: Upgraded to GMT 3.1 with -b option
  *
+ * 24-JAN-2000 PW/WHFS: Fixed bug related to use of -L option for "near-node" constraints.
+ * 10-JUL-2000 3.3.5 PW: Added -L plain.
+ * Version:	3.4
+ *
  *
  */
 
@@ -167,9 +171,9 @@ main(int argc, char **argv)
 
 	void	suggest_sizes_for_surface(int nx, int ny);
 
-	int	i, error = FALSE, size_query = FALSE;
+	int	i, error = FALSE, size_query = FALSE, n_files = 0;
 
-	char	modifier, *grdfile, low[100], high[100];
+	char	modifier, *grdfile, low[128], high[128];
 
 	struct GRD_HEADER h;
 
@@ -253,9 +257,8 @@ main(int argc, char **argv)
 								high_limit = atof (&argv[i][3]);
 							}
 							break;
-						default:
-							fprintf(stderr,"%s: GMT SYNTAX ERROR -L option: Unrecongized modifier %c\n", GMT_program, argv[i][2]);
-							error = TRUE;
+						default:	/* 360-periodicity option */
+							GMT_geographic_in = TRUE;
 							break;
 					}
 					break;
@@ -300,6 +303,7 @@ main(int argc, char **argv)
 				fprintf (stderr, "surface: cannot open input data file %s\n", argv[i]);
 				exit (EXIT_FAILURE);
 			}
+			n_files++;
 		}
 	}
 	
@@ -307,12 +311,12 @@ main(int argc, char **argv)
 		fprintf (stderr, "surface %s - Adjustable tension continuous curvature surface gridding\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: surface [xyz-file] -G<output_grdfile_name> -I<xinc>[m|c][/<yinc>[m|c]]\n");
 		fprintf (stderr, "\t-R<west>/<east>/<south>/<north> [-A<aspect_ratio>] [-C<convergence_limit>]\n");
-		fprintf (stderr, "\t[-H[<nrec>]] [-Ll<limit>] [-Lu<limit>] [-N<n_iterations>] ] [-S<search_radius>[m]]\n");
+		fprintf (stderr, "\t[-H[<nrec>]] [-Ll<limit>] [-L[u<limit>]] [-N<n_iterations>] ] [-S<search_radius>[m]]\n");
 		fprintf (stderr, "\t[-T<tension>[i][b] ] [-Q] [-V[l]] [-Z<over_relaxation_parameter>] [-:] [-bi[s][<n>]]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 		
-		fprintf (stderr, "\tsurface will read from standard input or [xyz-file].\n\n");
+		fprintf (stderr, "\tsurface will read from standard input or a signel <xyz-file>.\n\n");
 		fprintf (stderr, "\tRequired arguments to surface:\n");
 		fprintf (stderr, "\t-G sets output grd File name\n");
 		fprintf (stderr, "\t-I sets the Increment of the grid; enter xinc, optionally xinc/yinc.\n");
@@ -328,6 +332,7 @@ main(int argc, char **argv)
 		fprintf (stderr, "\t\tdefault will choose 0.001 of the range of your z data (1 ppt precision).\n");
 		fprintf (stderr, "\t\tEnter your own convergence limit in same units as z data.\n");
 		GMT_explain_option ('H');
+		fprintf (stderr, "\t-L without any modifiers indicate that x is longitude and periodic in 360:\n");
 		fprintf (stderr, "\t-L constrain the range of output values:\n");
 		fprintf (stderr, "\t\t-Ll<limit> specifies lower limit; forces solution to be >= <limit>.\n");
 		fprintf (stderr, "\t\t-Lu<limit> specifies upper limit; forces solution to be <= <limit>.\n");
@@ -391,6 +396,11 @@ main(int argc, char **argv)
 		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Binary input data (-bi) must have at least 3 columns\n", GMT_program);
 		error++;
 	}
+	if (n_files > 1) {
+		fprintf (stderr, "%s: GMT SYNTAX ERROR.  Only one input file allowed.  Preprocess you files with blockmean/median/mode first\n", GMT_program);
+		error++;
+	}
+	
 	
 	if (error) exit (EXIT_FAILURE);
 
@@ -408,7 +418,7 @@ main(int argc, char **argv)
 	h.x_inc = xinc;
 	h.y_inc = yinc;
 
-	GMT_grd_RI_verify (&h);
+	GMT_grd_RI_verify (&h, 1);
 
 	if (tension != 0.0) {
 		boundary_tension = tension;
@@ -735,9 +745,10 @@ void set_index (void) {
 }
 
 void find_nearest_point(void) {
-	int i, j, k, last_index, block_i, block_j, iu_index, briggs_index;
+	int i, j, ij_v2, k, last_index, block_i, block_j, iu_index, briggs_index;
 	double x0, y0, dx, dy, xys, xy1, btemp;
 	double b0, b1, b2, b3, b4, b5;
+	float z_at_node;
 	
 	last_index = -1;
 	small = 0.05 * ((grid_xinc < grid_yinc) ? grid_xinc : grid_yinc);
@@ -757,9 +768,24 @@ void find_nearest_point(void) {
 	 		y0 = ymin + block_j*grid_yinc;
 	 		dx = (data[k].x - x0)*r_grid_xinc;
 	 		dy = (data[k].y - y0)*r_grid_yinc;
-	 		if (fabs(dx) < small && fabs(dy) < small) {
+	 		if (fabs(dx) < small && fabs(dy) < small) {	/* Close enough to assign value to node */
 	 			iu[iu_index] = 5;
-	 			u[iu_index] = data[k].z;
+	 			/* v3.3.4: NEW CODE
+	 			 * Since point is basically moved from (dx, dy) to (0,0) we must adjust for
+	 			 * the small change in the planar trend between the two locations, and then
+	 			 * possibly clip the range if constraining surfaces were given.  Note that
+	 			 * dx, dy is in -1/1 range normalized by (grid * x|y_inc) so to recover the
+	 			 * dx,dy in final grid fractions we must scale by grid */
+	 			 
+	 			z_at_node = data[k].z + (float) (r_z_scale * grid * (plane_c1 * dx + plane_c2 * dy));
+	 			if (constrained) {
+					ij_v2 = (ny - block_j * grid - 1) * nx + block_i * grid;
+					if (set_low  && !GMT_is_fnan (lower[ij_v2]) && z_at_node < lower[ij_v2])
+						z_at_node = lower[ij_v2];
+					else if (set_high && !GMT_is_fnan (upper[ij_v2]) && z_at_node > upper[ij_v2])
+						z_at_node = upper[ij_v2];
+	 			}
+	 			u[iu_index] = z_at_node;
 	 		}
 	 		else {
 	 			if (dx >= 0.0) {
@@ -908,61 +934,45 @@ void new_initialize_grid(void)
 
 void read_data(void)
 {
-	int	i, j, k, kmax, kmin, ix, iy, n_fields, skip, n_expected_fields;
-	double	*in, zmin = 1.0e38, zmax = -1.0e38;
+	int	i, j, k, kmax, kmin, n_fields, n_expected_fields;
+	double	*in, zmin = DBL_MAX, zmax = -DBL_MAX;
 	char	buffer[BUFSIZ];
 
 	data = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 	
 	/* Read in xyz data and computes index no and store it in a structure */
 	
-	ix = (gmtdefs.xy_toggle) ? 1 : 0;        iy = 1 - ix;              /* Set up which columns have x and y */
 	k = 0;
 	z_mean = 0;
-	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : BUFSIZ;
-	if (!GMT_io.binary[0] && gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (buffer, BUFSIZ, fp_in);
-	
-	n_fields = GMT_input (fp_in, &n_expected_fields, &in);
-		
-	while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : 3;
+	if (!GMT_io.binary[0] && gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp_in);
+			
+	while ((n_fields = GMT_input (fp_in, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 
-		skip = FALSE;
-		
 		if (GMT_io.status & GMT_IO_MISMATCH) {
 			fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, k);
 			exit (EXIT_FAILURE);
 		}
 
-		if (GMT_is_dnan (in[2])) skip = TRUE;
+		if (GMT_is_dnan (in[2])) continue;
 		
-		if (!skip) {
-			i = (int)floor(((in[ix]-xmin)*r_grid_xinc) + 0.5);
-			if (i < 0 || i >= block_nx) skip = TRUE;
-			j = (int)floor(((in[iy]-ymin)*r_grid_yinc) + 0.5);
-			if (j < 0 || j >= block_ny) skip = TRUE;
+		i = (int)floor(((in[0]-xmin)*r_grid_xinc) + 0.5);
+		if (i < 0 || i >= block_nx) continue;
+		j = (int)floor(((in[1]-ymin)*r_grid_yinc) + 0.5);
+		if (j < 0 || j >= block_ny) continue;
+
+		data[k].index = i * block_ny + j;
+		data[k].x = (float)in[0];
+		data[k].y = (float)in[1];
+		data[k].z = (float)in[2];
+		if (zmin > in[2]) zmin = in[2], kmin = k;
+		if (zmax < in[2]) zmax = in[2], kmax = k;
+		k++;
+		z_mean += in[2];
+		if (k == n_alloc) {
+			n_alloc += GMT_CHUNK;
+			data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 		}
-		if (!skip) {
-			data[k].index = i * block_ny + j;
-			data[k].x = (float)in[ix];
-			data[k].y = (float)in[iy];
-			data[k].z = (float)in[2];
-			if (zmin > in[2]) {
-				zmin = in[2];
-				kmin = k;
-			}
-			if (zmax < in[2]) {
-				zmax = in[2];
-				kmax = k;
-			}
-			k++;
-			z_mean += in[2];
-			if (k == n_alloc) {
-				n_alloc += GMT_CHUNK;
-				data = (struct DATA *) GMT_memory ((void *)data, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
-			}
-		}
-		
-		n_fields = GMT_input (fp_in, &n_expected_fields, &in);
 	}
 	
 	if (fp_in != GMT_stdin) GMT_fclose (fp_in);
@@ -1515,6 +1525,12 @@ void	rescale_z_values(void)
 	/* Set z_scale = rms(z):  */
 	
 	z_scale = sqrt (ssz / npoints);
+	
+	if (z_scale == 0.0) {
+		fprintf(stderr,"%s: Input data lie exactly on a plane - no solution (aborting).\n", GMT_program);
+		exit (EXIT_FAILURE);
+	}
+	
 	r_z_scale = 1.0 / z_scale;
 
 	for (i = 0; i < npoints; i++) data[i].z *= (float)r_z_scale;
@@ -1900,4 +1916,3 @@ int	gcd_euclid(int a, int b)
 	}
 	return(u);
 }
-
