@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	09/21/99  @(#)trend1d.c	2.43
+ *	$Id: trend1d.c,v 1.3 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * trend1d [<xy[w]file>] -F<output_flags> -N[f]<n_m_parameters>[r] 
@@ -96,9 +96,11 @@
  * all these reasons I use Chebyshev polynomials.
  *
  * Author:	W. H. F. Smith
- * Date:	25 February 1991-1999.
- * Revised:	11 June, 1991-1999 for v2.0 of GMT-SYSTEM.
+ * Date:	25 February 1991-2000.
+ * Revised:	11 June, 1991-2000 for v2.0 of GMT-SYSTEM.
  *		13-JUN-1998, for GMT 3.1 (PW)
+ *		13-JUL-2000, for GMT 3.3.5 (PW)
+ * Version:	3.4
  */
 
 #include "gmt.h"
@@ -450,7 +452,7 @@ main(int argc, char **argv)
 void	read_data(struct DATA **data, int *n_data, double *xmin, double *xmax, int weighted_input, double **work, FILE *fp)
 {
 
-	int	i, ix, iy, n_alloc = GMT_CHUNK, n_expected_fields, n_fields;
+	int	i, n_alloc = GMT_CHUNK, n_expected_fields, n_fields;
 	double	*in;
 	char	buffer[BUFSIZ];
 
@@ -462,10 +464,9 @@ void	read_data(struct DATA **data, int *n_data, double *xmin, double *xmax, int
 	}
 	(*data) = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
 	
-	ix = (gmtdefs.xy_toggle) ? 1 : 0;	iy = 1 - ix;		/* Set up which columns have x and y */
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (buffer, BUFSIZ, fp);
+	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
 	i = 0;
-	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : 2 + weighted_input;
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 
@@ -474,8 +475,8 @@ void	read_data(struct DATA **data, int *n_data, double *xmin, double *xmax, int
 			exit (EXIT_FAILURE);
 		}
 
-		(*data)[i].x = in[ix];
-		(*data)[i].y = in[iy];
+		(*data)[i].x = in[0];
+		(*data)[i].y = in[1];
 		(*data)[i].w = (weighted_input) ? in[2] : 1.0;
 
 		if (i) {
