@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:	@(#)trend2d.c	2.41  09/21/99
+ *	$Id: trend2d.c,v 1.3 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * trend2d [<xyz[w]file>] -F<output_flags> -N<n_m_parameters>[r] 
@@ -59,8 +59,10 @@
  *
  *
  * Author:	W. H. F. Smith
- * Date:	17 June 1991-1999.
+ * Date:	17 June 1991-2000.
  * Revised:	12-JUN-1998 for GMT 3.1 (PW)
+ *		10-JUL-2000 for GMT 3.3.5 (PW) Added -L option
+ * Version:	3.4
  */
 
 #include "gmt.h"
@@ -168,6 +170,9 @@ main(int argc, char **argv)
 					increment = TRUE;
 					confid = (argv[i][2]) ? atof(&argv[i][2]) : 0.51;
 					break;
+				case 'L':
+					GMT_geographic_in = GMT_geographic_out = TRUE;
+					break;
 				case 'N':
 					if (argv[i][strlen (argv[i]) - 1] == 'r') robust = TRUE;
 					n_model_max = (argv[i][2]) ? atoi(&argv[i][2]) : 0;
@@ -191,7 +196,8 @@ main(int argc, char **argv)
 
 	if (argc == 1 || GMT_quick) {
 		fprintf(stderr,"trend2d %s - Fit a [weighted] [robust] polynomial for z = f(x,y) to ascii xyz[w]\n\n", GMT_VERSION);
-		fprintf(stderr,"usage:  trend2d -F<xymrw> -N<n_model>[r] [<xyz[w]file>] [-C<condition_#>] [-H[<nrec>]] [-I[<confidence>]] [-V] [-W] [-:] [-bi[s][<n>]] [-bo[s]]\n\n");
+		fprintf(stderr,"usage:  trend2d -F<xymrw> -N<n_model>[r] [<xyz[w]file>] [-C<condition_#>] [-H[<nrec>]] [-I[<confidence>]]\n\n");
+		fprintf(stderr,"	[-L] [-V] [-W] [-:] [-bi[s][<n>]] [-bo[s]]\n\n");
 		
 		if (GMT_quick) exit (EXIT_FAILURE);
 
@@ -205,6 +211,7 @@ main(int argc, char **argv)
 		fprintf(stderr,"\t-I Iteratively Increase # model parameters, to a max of <n_model> so long as the\n");
 		fprintf(stderr,"\t   reduction in variance is significant at the <confidence> level.\n");
 		fprintf(stderr,"\t   Give -I without a number to default to 0.51 confidence level.\n");
+		fprintf (stderr, "\t-L means that x is longitude, i.e. assumed to be periodic in 360\n");
 		GMT_explain_option ('V');
 		fprintf(stderr,"\t-W Weighted input given, weights in 4th column.  [Default is unweighted].\n");
 		GMT_explain_option (':');
@@ -259,7 +266,7 @@ main(int argc, char **argv)
 #ifdef SET_IO_MODE
 	GMT_setmode (1);
 #endif
-
+	project_info.w = 0;	project_info.e = 360.0;	/* For -L not to cause trouble in GMT_input */
 	np = n_model_max;	/* Row dimension for matrices gtg and v  */
 	allocate_array_space(np, &gtg, &v, &gtd, &lambda, &workb, &workz, &c_model, &o_model, &w_model);
 
@@ -406,7 +413,7 @@ main(int argc, char **argv)
 void	read_data(struct DATA **data, int *n_data, double *xmin, double *xmax, double *ymin, double *ymax, int weighted_input, double **work, FILE *fp)
 {
 
-	int	i, ix, iy, n_alloc = GMT_CHUNK, n_expected_fields, n_fields;
+	int	i, n_alloc = GMT_CHUNK, n_expected_fields, n_fields;
 	char	buffer[BUFSIZ];
 	double	*in;
 
@@ -417,11 +424,10 @@ void	read_data(struct DATA **data, int *n_data, double *xmin, double *xmax, doub
 #endif
 	}
 	(*data) = (struct DATA *) GMT_memory (VNULL, (size_t)n_alloc, sizeof(struct DATA), GMT_program);
-	ix = (gmtdefs.xy_toggle) ? 1 : 0;	iy = 1 - ix;		/* Set up which columns have x and y */
 
-	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (buffer, BUFSIZ, fp);
+	if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (buffer, BUFSIZ, fp);
 	i = 0;
-	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.binary[0]) ? GMT_io.ncol[0] : 3 + weighted_input;
 
 	while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {
 
@@ -430,8 +436,8 @@ void	read_data(struct DATA **data, int *n_data, double *xmin, double *xmax, doub
 			exit (EXIT_FAILURE);
 		}
 
-		(*data)[i].x = in[ix];
-		(*data)[i].y = in[iy];
+		(*data)[i].x = in[0];
+		(*data)[i].y = in[1];
 		(*data)[i].z = in[2];
 		(*data)[i].w = (weighted_input) ? in[3] : 1.0;
 
