@@ -1,7 +1,7 @@
 /*--------------------------------------------------------------------
- *    The GMT-system:   @(#)triangulate.c	2.51  09/21/99
+ *	$Id: triangulate.c,v 1.3 2001/04/11 23:07:29 pwessel Exp $
  *
- *	Copyright (c) 1991-1999 by P. Wessel and W. H. F. Smith
+ *	Copyright (c) 1991-2001 by P. Wessel and W. H. F. Smith
  *	See COPYING file for copying and redistribution conditions.
  *
  *	This program is free software; you can redistribute it and/or modify
@@ -13,7 +13,7 @@
  *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *	GNU General Public License for more details.
  *
- *	Contact info: www.soest.hawaii.edu/gmt
+ *	Contact info: gmt.soest.hawaii.edu
  *--------------------------------------------------------------------*/
 /*
  * triangulate reads one or more files (or GMT_stdin) with x,y[,whatever] and
@@ -31,6 +31,9 @@
  * Version:     2.0
  * Revised:	13-AUG-1998 for GMT 3.1
  * Revised:	02-MAR-1999 for GMT 3.2
+ * Revised:	29-MAR-2000 for GMT 3.3.5
+ * Modified:	10 Jul 2000 by PW to add -L
+ * Version:	3.4
  *
  */
  
@@ -38,12 +41,12 @@
 
 struct EDGE {
 	int begin, end;
-} *edge;
+};
 	
 main (int argc, char **argv)
 {
 	int i, j, ij, ij1, ij2, ij3, np, k, fno, n = 0, n_alloc = 0, n_files = 0, n_edge;
-	int n_expected_fields, x, y, n_args, *link, n_fields, compare_edge(const void *p1, const void *p2);
+	int n_expected_fields, n_args, *link, n_fields, compare_edge(const void *p1, const void *p2);
 	int one_or_zero, i_min, i_max, j_min, j_max, p;
 	
 	BOOLEAN error = FALSE, map_them = FALSE, inc_set = FALSE, do_ddx = FALSE, do_ddy = FALSE;
@@ -51,16 +54,18 @@ main (int argc, char **argv)
 	
 	double west = 0.0, east = 0.0, south = 0.0, north = 0.0, zj, zk, zl, zlj, zkj, *in;
 	double *xx, *yy, *zz, vx[4], vy[4], xinc2, yinc2, idx, idy, xp, yp, a, b, c, f;
-	double xkj, xlj, ykj, ylj;
+	double xkj, xlj, ykj, ylj, out[2];
 	
 	float *grd, empty = 0.0;
 	
-	char line[BUFSIZ], format[BUFSIZ], *outfile = CNULL;
+	char line[BUFSIZ], *outfile = CNULL;
 	
 	FILE *fp = NULL;
 	
 	struct GRD_HEADER header;
 
+	struct EDGE *edge;
+	
 	argc = GMT_begin (argc, argv);
 
 	GMT_grd_init (&header, argc, argv, FALSE);
@@ -102,6 +107,9 @@ main (int argc, char **argv)
 					GMT_getinc (&argv[i][2], &header.x_inc, &header.y_inc);
 					inc_set = TRUE;
 					break;
+				case 'L':
+					GMT_geographic_in = TRUE;
+					break;
 				case 'M':
 					GMT_multisegment (&argv[i][2]);
 					break;
@@ -118,28 +126,29 @@ main (int argc, char **argv)
 	if (argc == 1 || GMT_quick) {
 		fprintf (stderr, "triangulate %s - Optimal (Delaunay) triangulation of xyz-data\n\n", GMT_VERSION);
 		fprintf (stderr, "usage: triangulate <infiles> [-Dx|y] [-E<empty>] [-F] [-G<grdfile> [-H[<nrec>]] [-I<dx>[m|c][/<dy>[m|c]]] [-J<parameters>]\n");
-		fprintf (stderr, "	[-M[<flag>]] [-R<west/east/south/north>] [-V] [-:] [-bi[s][<n>]] [-bo]\n\n");
+		fprintf (stderr, "\t[-L] [-M[<flag>]] [-R<west/east/south/north>] [-V] [-:] [-bi[s][<n>]] [-bo]\n\n");
                 
 		if (GMT_quick) exit (EXIT_FAILURE);
                 
-		fprintf (stderr, "      infiles (in ASCII) has 2 or more columns.  If no file(s) is given, standard input is read.\n");
+		fprintf (stderr, "\tinfiles (in ASCII) has 2 or more columns.  If no file(s) is given, standard input is read.\n");
 		fprintf (stderr, "\n\tOPTIONS:\n");
-		fprintf(stderr, "	-Dx or -Dy takes derivative in that direction (only with -G) [Default is z value].\n");
-		fprintf(stderr, "	-E value to use for empty nodes [Default is NaN].\n");
-		fprintf(stderr, "	-F Force pixel registration (only with -G) [Default is gridline registration]\n");
-		fprintf(stderr, "	-G Grid data. Give name of output gridfile and specify -R -I\n");
+		fprintf (stderr, "\t-Dx or -Dy takes derivative in that direction (only with -G) [Default is z value].\n");
+		fprintf (stderr, "\t-E value to use for empty nodes [Default is NaN].\n");
+		fprintf (stderr, "\t-F Force pixel registration (only with -G) [Default is gridline registration]\n");
+		fprintf (stderr, "\t-G Grid data. Give name of output gridfile and specify -R -I\n");
 		GMT_explain_option ('H');
-		fprintf(stderr, "	-I sets the grid spacing for the grid.  Append m for minutes, c for seconds\n");
+		fprintf (stderr, "\t-I sets the grid spacing for the grid.  Append m for minutes, c for seconds\n");
 		GMT_explain_option ('J');   
-		fprintf (stderr, "      -M output triangle edges as multiple segments separated by a record\n");
-		fprintf (stderr, "         whose first character is <flag> ['>']\n");
-		fprintf (stderr, "         [Default is to output the indeces of vertices for each Delaunay triangle]\n");
+		fprintf (stderr, "\t-L means that x is longitude, i.e. assumed to be periodic in 360\n");
+		fprintf (stderr, "\t-M output triangle edges as multiple segments separated by a record\n");
+		fprintf (stderr, "\t   whose first character is <flag> ['>']\n");
+		fprintf (stderr, "\t   [Default is to output the indeces of vertices for each Delaunay triangle]\n");
 		GMT_explain_option ('R');
 		GMT_explain_option ('V');
 		GMT_explain_option (':');
 		GMT_explain_option ('i');
 		GMT_explain_option ('n');
-		fprintf (stderr, "         Default is 2 input columns\n");
+		fprintf (stderr, "\t   Default is 2 input columns\n");
 		fprintf (stderr, "\t-bo writes binary integer index table; ignored if -M is selected [Default is ASCII i/o]\n");
 		GMT_explain_option ('.');
 		exit (EXIT_FAILURE);
@@ -178,7 +187,7 @@ main (int argc, char **argv)
 	if (do_grid) {
 		header.x_min = west;	header.x_max = east;
 		header.y_min = south;	header.y_max = north;
-		GMT_grd_RI_verify (&header);
+		GMT_grd_RI_verify (&header, 1);
 	}
 
 	if (west != east && project_info.projection > 0) { /* Gave -R -J */
@@ -188,14 +197,12 @@ main (int argc, char **argv)
 
 	/* Now we are ready to take on some input values */
 	
-	x = (gmtdefs.xy_toggle) ? 1 : 0;	y = 1 - x;		/* Set up which columns have x and y */
-	
 	if (n_files > 0)
 		nofile = FALSE;
 	else
 		n_files = 1;
 	n_args = (argc > 1) ? argc : 2;
-	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : BUFSIZ;
+	n_expected_fields = (GMT_io.ncol[0]) ? GMT_io.ncol[0] : 2 + do_grid;
 	
 	n_alloc = GMT_CHUNK;
 	xx = (double *) GMT_memory (VNULL, (size_t)n_alloc, sizeof (double), GMT_program);
@@ -220,19 +227,17 @@ main (int argc, char **argv)
 
 		if (!nofile && gmtdefs.verbose) fprintf (stderr, "%s: Reading file %s\n", GMT_program, argv[fno]);
 		
-		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) fgets (line, BUFSIZ, fp);
+		if (gmtdefs.io_header) for (i = 0; i < gmtdefs.n_header_recs; i++) GMT_fgets (line, BUFSIZ, fp);
 
-		n_fields = GMT_input (fp, &n_expected_fields, &in);
-			
-		while (! (GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
+		while ((n_fields = GMT_input (fp, &n_expected_fields, &in)) >= 0 && !(GMT_io.status & GMT_IO_EOF)) {	/* Not yet EOF */
 					
 			if (GMT_io.status & GMT_IO_MISMATCH) {
 				fprintf (stderr, "%s: Mismatch between actual (%d) and expected (%d) fields near line %d\n", GMT_program, n_fields, n_expected_fields, n);
 				exit (EXIT_FAILURE);
 			}
 
-			xx[n] = in[x];
-			yy[n] = in[y];
+			xx[n] = in[0];
+			yy[n] = in[1];
 			if (do_grid) zz[n] = in[2];
 			n++;
 			
@@ -242,8 +247,6 @@ main (int argc, char **argv)
 				yy = (double *) GMT_memory ((void *)yy, (size_t)n_alloc, sizeof (double), GMT_program);
 				if (do_grid) zz = (double *) GMT_memory ((void *)zz, (size_t)n_alloc, sizeof (double), GMT_program);
 			}
-			
-			n_fields = GMT_input (fp, &n_expected_fields, &in);
 		}
 		
 		if (fp != GMT_stdin) GMT_fclose (fp);
@@ -312,11 +315,26 @@ main (int argc, char **argv)
 			b = -f * (zkj * xlj - xkj * zlj);
 			c = -a * vx[1] - b * vy[1] + zk;
 
+			/* Compute grid indices the current triangle may cover, assuming all triangles are
+			   in the -R region (header.x_min/x_max etc.)  Always, i_min <= i_max, j_min <= j_max.
+			 */
+
 			i_min = (int)floor ((MIN (MIN (vx[0], vx[1]), vx[2]) - header.x_min + xinc2) * idx);
 			i_max = (int)ceil ((MAX (MAX (vx[0], vx[1]), vx[2]) - header.x_min + xinc2) * idx);
 			j_min = (int)floor ((header.y_max - MAX (MAX (vy[0], vy[1]), vy[2]) + yinc2) * idy);
 			j_max = (int)ceil ((header.y_max - MIN (MIN (vy[0], vy[1]), vy[2]) + yinc2) * idy);
 			
+			/* Adjustments for triangles outside -R region. */
+			/* Triangle to the left or right. */
+			if ((i_max < 0) || (i_min >= header.nx)) continue;
+			/* Triangle Above or below */
+			if ((j_max < 0) || (j_min >= header.ny)) continue;
+
+			/* Triangle covers boundary, left or right. */
+			if (i_min < 0) i_min = 0;       if (i_max >= header.nx) i_max = header.nx - 1;
+			/* Triangle covers boundary, top or bottom. */
+			if (j_min < 0) j_min = 0;       if (j_max >= header.ny) j_max = header.ny - 1;
+
 			for (j = j_min; j <= j_max; j++) {
 				yp = header.y_max - j * header.y_inc - yinc2;
 				p = j * header.nx + i_min;
@@ -359,18 +377,19 @@ main (int argc, char **argv)
 	
 		if (gmtdefs.verbose) fprintf (stderr, "%s: %d unique triangle edges\n", GMT_program, n_edge);
 		
-		sprintf (format, "%s\t%s\n\0", gmtdefs.d_format, gmtdefs.d_format);
 		for (i = 0; i < n_edge; i++) {
-			printf ("%c Edge %d-%d\n", GMT_io.EOF_flag, edge[i].begin, edge[i].end);
-			printf (format, xx[edge[i].begin], yy[edge[i].begin]);
-			printf (format, xx[edge[i].end], yy[edge[i].end]);
+			fprintf (GMT_stdout, "%c Edge %d-%d\n", GMT_io.EOF_flag, edge[i].begin, edge[i].end);
+			out[0] = xx[edge[i].begin];	out[1] = yy[edge[i].begin];
+			GMT_output (GMT_stdout, 2, out);
+			out[0] = xx[edge[i].end];	out[1] = yy[edge[i].end];
+			GMT_output (GMT_stdout, 2, out);
 		}
 		GMT_free ((void *)edge);
 	}
 	else if (GMT_io.binary[1])
 		fwrite ((void *)link, sizeof (int), (size_t)(3*np), GMT_stdout);
 	else
-		for (i = ij = 0; i < np; i++, ij += 3) printf ("%d\t%d\t%d\n", link[ij], link[ij+1], link[ij+2]);
+		for (i = ij = 0; i < np; i++, ij += 3) fprintf (GMT_stdout, "%d\t%d\t%d\n", link[ij], link[ij+1], link[ij+2]);
 	
 	GMT_free ((void *) xx);
 	GMT_free ((void *) yy);
